function mklsgridcontinents(mode,elevation,maxiter);
% mklsgridcontinents.......triangular grid demo showing continents
%
% call: mklsgridcontinents(mode,elevation,maxiter);
%
%       mode: (string)
%             defines which demo to show
%             available are:
%             'continents': produce triangular grid that is locally refined on land
%                           and coarse on water.
%             'coasts':     produce triangular grid that islocally refined
%                           at coasts, but not on water and not inside
%                           continents.
%    elevation (number) [m]
%             The routine adapts the grid to be fine at areas with
%             elevation at or above this level.
%    maxiter: (number)
%             max number of iterations to be applied
%
% result: none
%
% The program writes some statistics to the screen:
% - iteration number (0 means grid initialization)
% - number of triangles checked for being on land or water, which is the
%   number of triangls in the grid.
% - number of triangles that will be iterated
% - number of nodes AFTER iteration
% - time spent for checking, defining which triangles to iterate and for
%   the iteration itself
%
% For the zero-th iteration, the number of "checking tri." is the number of
% triangles in the initial grid, and the number of nodes is the number of
% nodes in the initial grid.
%
% Martin Knapmeyer, 29.09.2004, 24.01.2005, 08.05.2008

initstep=[]; % to collect times needed for initialization
toctime=[]; % to collect times needed to work

%%% start timer
tic;

disp(['MKLSGRIDCONTINENTS: program start ' mkclock]);
disp(' ');

%%% table head for statistics output
disp(' it. |  checking tri. |   iterating tri.  |   Nodes   |  time [s]');
%      1234 123456789012345  1234567890123456789 12345678901 1234567890
disp('+----+----------------+-------------------+-----------+----------+');

%%% load topo data
%%% this is just the 1x1 deg grid shipped with matlab
topo=load('topo');
minalt=min(topo.topo(:));
maxalt=max(topo.topo(:));


%%% higher resolution topography, works only on my machine MK08052008
hirestopopath='/user/neptun/knapmeye/planets/erde/topo/etopo20.mat';
if exist(hirestopopath)
   topo=load(hirestopopath);
   minalt=min(topo.topo(:));
   maxalt=max(topo.topo(:));
end; % if exist


%%% construct colormap for grid and map plot
%%% the 291 is a magic number which derives from the altitudes in topo
%%% and the colormap length
%rgb=mktopocolor(minalt:abs(maxalt-minalt)/511:maxalt); 
%rgb=jet(512);
%rgb=[flipud(rgb(1:290,:))' flipud(rgb(291:512,:))']';
rgb=fliplr(1-cool);

%%% plot topo array
% figure(1);
% surf(topo.topo*0,topo.topo);
% shading flat
% view(2);


%%% plot topography map
figure(1);
clf;
lonwidth=size(topo.topo,2);
latwidth=size(topo.topo,1);
gridresolution=360/lonwidth;
range=[-180+gridresolution/2 180-gridresolution/2,...
        -90+gridresolution/2  90-gridresolution/2]; %[-180 179 -90 89]; %[0 359 -90 89];
dlonrange=abs(range(2)-range(1))/(lonwidth-1);
dlatrange=abs(range(4)-range(3))/(latwidth-1);
mapopt=mkmapopt(...
                'proj_type','hammer',...
                'proj_range',range,...
                'topo_mode','off',...
                'grid_mode','on',...
                'grid_range',range,...
                'grid_lonmesh',30,...
                'grid_latmesh',30,...
                'grid_style',':'...
                );
[lon,lat]=meshgrid(range(1):dlonrange:(range(2)),...
                   range(3):dlatrange:(range(4)));
[x,y]=mklonlat2xy(lon,lat,mapopt,'forward');
mkmakemap(mapopt);
colormap(rgb); %topo.topomap1);
cax=[minalt maxalt]*mapopt.proj_scale*mapopt.topo_exag;
caxis(cax);
hold on;
surf(x,y,topo.topo-maxalt,topo.topo);
hold off
shading flat;
view(2);
title('Node Map');




%%% 3D coordinates for plotting topo as globe
lon=mod(lon+180,360);
[tx,ty,tz]=sph2cart(lon*pi/180,lat*pi/180,lon*0+6000+50*topo.topo/1000);


%%% first part of initialization done
initstep=[initstep toc];


%%% generate grid
%%% This grid will have only one layer, and each node will have only
%%% one .fit-element.
lsopt=mklsopt(...
              'grid_type','triangular',...
              'grid_base','icosahedron',...
              'grid_init_dr',7000,...
              'grid_init_wdt',8000,...
              'iter_circmin',0,...
              'verbosity',0,...
              'mapopt',mapopt...
             );
 mygrid=mklsgridnew(lsopt);
 initialtrianz=mklsconstants(['Triangles' lsopt.grid_base]);
 
 %%% grid initialization done
 initstep=[initstep toc]; 
 
 %%% do some blind iterations:
 %%% if the starting grid is to coarse, there will be too few nodes
 %%% on the desired areas to get a meaningful result grid
 %initialiterations=0;
 %grid=mklsgridmodify(lsopt,grid,[],'iterate',initialiterations);
 
 

 %%% plot initial grid
 %%% Grid Map
 figure(1);
 %%% plot mesh grid (note that this doesnt apply the corrdinate
 %%% transofrmation due to topo orientation and hammer projection!)
 hold on
 oldhandle=mklsgridplot(lsopt,mygrid,'cloudmap');
 hold off
 axis auto
 axis normal
 axis equal
 %%% Grid 3D
 figure(2);
 clf
 meshhandle=mklsgridplot(lsopt,mygrid,'mesh');
 hold on
 surfhandle=surf(tx,ty,tz,topo.topo);
%      set(surfhandle,'EdgeLighting','flat');
%      set(surfhandle,'EdgeColor','flat');
%      set(surfhandle,'FaceLighting','flat');
%      set(surfhandle,'FaceColor','flat');
 colormap(rgb);
 caxis([minalt maxalt]); % topo range in m
 light('position',[0 -1 1]);
 lighting phong
 material dull
 shading interp
 set(meshhandle,'EdgeColor','w');
 hold off
 drawnow;
 
 
 
 %%% initialization done.
 %toctime=toc; disp(['MKLSGRIDCONTINENTS: Initialized. elapsed time so far: ' num2str(toctime) 'sec.']);
 initstep=[initstep toc]; % time wasted for initialization
 
 %%% statistics output to screen
 line=sprintf('%4.0f ,%13.0f   ,%17.0f  ,%9.0f  ,%10.4f',...
                 0,...
                 length(mygrid.layers.tri),...
                 NaN,...
                 size(mygrid.layers.nodes,1),...
                 initstep(2)-initstep(1));
 disp(line);
     
     
 
 %%% loop through grid to apply local refinement
 %%% each triangle, which has at least one node on land is refined
 %%% a maximum number of MAXITER iterations is applied
 checktri=[];
 iteratetri=[];
 nodeanz=[];
 for iter=1:maxiter;
     trianz=length(mygrid.layers.tri);
     %disp(['MKLSGRIDCONTINENTS: checking  ' int2str(trianz) ' triangles.']);
     checktri=[checktri trianz];
     triangles=[]; % to collect triangle IDs for deactivation
     nodeplotx=[]; % to collect node x plot coordinates
     nodeploty=[]; % to collect node y plot coordinate
     
     
     
     %%%%%% Determine if nodes are on land or on water
     %%% loop over all triangles
     for indy=1:trianz
         
         %%% work only on acitve triangles!
         if mygrid.layers.tri(indy).active==1
             childtype=(mygrid.layers.tri(indy).childtype);
             switch childtype

                 case {'node'}
                    %%% first determine if one node of triangle is above sea
                    %%% level, if not: append triangle ID to list for triangle
                    %%% deactivation
                    
                    for j=1:3 % loop over all three nodes of triangle

                        %%% get node position
                        id=mygrid.layers.tri(indy).nodes(j);
                        [rownum,nodes]=mklsgridgetnodetri(id,mygrid.layers);

                        %%% transform node coordinates into index for topo
                        %%% array
                        %lonindex=min([359 fix(nodes(2))+1]);
                        %latindex=max([1 fix(nodes(3))+90]);
                        lonindex=fix(fix(nodes(2))*(lonwidth-1)/360)+1; % MK08052008
                        latindex=fix((fix(nodes(3))+90)*(latwidth-1)/180)+1;

                        %%% compute plot node coordinates for map

                        %%% rotate longitude by 180 deg because of orientation
                        %%% of topo array
                        nodes(2)=mod(nodes(2)+180,360);
                        %%% for hammer projection, we have to make longitude
                        %%% to cover -180...180
                        if nodes(2)>180
                            nodes(2)=nodes(2)-360;
                        end; % if nodes(2)


                        %%% Is Node above sea level? If yes: iterate triangle!
                        if topo.topo(latindex,lonindex)>=elevation
                            %%% elevation at node is above sea level
                            
                            %%% write fitness list
                            %%% In this demo, a fitness value of 1 means
                            %%% that a node is on land, whereas a fitness
                            %%% of 0 means that the node is in water.
                            mygrid.layers.fit(rownum,2)=1;
                            
                            
                            %%% add to list of points-on-land
                            nodeplotx=[nodeplotx nodes(2)];
                            nodeploty=[nodeploty nodes(3)];
                            
                            %%% control plot: plot array indexes
%                             figure(1);
%                             hold on
%                             plot(lonindex,latindex,'yo');
%                             hold off
                        end;% if topo.topo

                        %%% mark index positions in array
%                         figure(1);
%                         hold on
%                         plot(lonindex,latindex,'r.');
%                         hold off

                    end; % for j

                 case {'triangle'}
                     %%% nothing happens
                 otherwise
                     error(['MKLSGRIDCONTINENTS: unknown child type ' upper(childtype)]);
             end; % switch childtype
         end; % if grid.layers.tri(indy).active==1
     end; % for indy
     %%%%%% Land/Water determination for each node done.
     
     
     %%%%%% determine which triangles to iterate
     %%% produce a layer-struct in which IDs are also indexes to the arrays
     fxslayer=mklsgridfxslayertri(mygrid.layers);
     %%% produce ID list of the initial triangles
     trilist=zeros(1,initialtrianz);
     for indy=1:initialtrianz
         trilist(indy)=mygrid.layers.tri(indy).id;
     end; % for indy
     %%% loop over all triangles, again
     for indy=1:trianz
         
         %%% work only on acitve triangles!
         if mygrid.layers.tri(indy).active==1
             childtype=(mygrid.layers.tri(indy).childtype);
             switch childtype

                 case {'node'}
                    %%% first determine how many nodes are above sea level
                    %%% here we need the FXS layer the first time
                    %%% fitness is 1 for land-nodes and 0 for water nodes.
                    %%% So we just have to sum the fitness for all three
                    %%% nodes of the triangle to know how many are on land.
                    cnt=sum(fxslayer.fit(fxslayer.tri(indy).nodes,2));

                    switch mode
                        case {'continents'}
                            if cnt>0
                                %%% Triangle has at least one point above sea level
                                %%% and will be iterated
                                triangles=[triangles mygrid.layers.tri(indy).id];

                                %%% additionally, all its neighbour triangles will
                                %%% be iterated 
                                %%% but only if current triangle does not
                                %%% have all nodes on land
                                if cnt<3
                                    %neighbours=mklsgridneighbourtri(fxslayer,fxslayer.tri(indy).id,indy);
                                    neighbours=mklsgridoverlapccrtri(fxslayer,indy,trilist);
                                    triangles=[triangles neighbours];
                                    %disp(['MKLSGRIDCONTIONENTS: '... %triangle ' int2str(indy) ': '...
                                    %      int2str(length(unique(neighbours))) ' neighbours to iterate.']);
                                end; % if cnt<3
                            end; % if cnt>0
                        case {'coasts'}
                            if (cnt==1)|(cnt==2)
                                %%% Triangle has not all points onland and
                                %%% not all points at sea level - which
                                %%% means, it is at a coast. MK24012005
                                %%% and will be iterated
                                triangles=[triangles mygrid.layers.tri(indy).id];

                                %%% additionally, all its neighbour triangles will
                                %%% be iterated 
                                %%% but only if current triangle has not all nodes on
                                %%% land or water
                                if (cnt<3)&(cnt>0)
                                    %neighbours=mklsgridneighbourtri(fxslayer,fxslayer.tri(indy).id,indy);
                                    neighbours=mklsgridoverlapccrtri(fxslayer,indy,trilist);
                                    triangles=[triangles neighbours];
                                    %disp(['MKLSGRIDCONTIONENTS: '... %triangle ' int2str(indy) ': '...
                                    %      int2str(length(unique(neighbours))) ' neighbours to iterate.']);
                                end; % if cnt<3
                            end; % if cnt>0
                        otherwise
                            error(['MKLSGRIDCONTINENTS: unknown mode ' upper(mode)]);
                    end; % switch mode

                 case {'triangle'}
                     %%% nothing happens
                 otherwise
                     error(['MKLSGRIDCONTINENTS: unknown child type ' upper(childtype)]);
             end; % switch childtype
         end; % if grid.layers.tri(indy).active==1
         
     end; % for indy
     %%%%%% triangles to iterate determined
     
     
     %%% iterate triangles with collected IDs
     %%% but first make triangle list unique
     triangles=unique(triangles);
     %disp(['MKLSGRIDCONTINENTS: iterating ' int2str(length(triangles)) ' triangles.']);
     iteratetri=[iteratetri length(triangles)];
     idlist.layer=mygrid.layers.id;
     idlist.triangle=triangles;
     [mygrid,modifycnt]=mklsgridmodify(lsopt,mygrid,idlist,'iterate');
     %disp(['MKLSGRIDCONTINENTS: grid has  ' int2str(length(grid.layers.nodes)) ' nodes.']);
     %disp(['MKLSGRIDCONTINENTS: ' int2str(modifycnt) ' triangles were modified.']);
     nodeanz=[nodeanz size(mygrid.layers.nodes,1)];
     
%      %%% plot grid as map of nodes
%      [nodeplotx,nodeploty]=mklonlat2xy(nodeplotx,nodeploty,mapopt,'forward');
%      figure(1);
%      %%% plot points on land (from previous grid!)
%      %hold on
%      %plot(nodeplotx,nodeploty,'yo');
%      %hold off
%      %%% plot mesh grid (not that this doesnt apply the corrdinate
%      %%% transformation due to topo orientation and hammer projection!
%      delete(oldhandle); % removes grid from last plot
%      hold on
%      oldhandle=mklsgridplot(lsopt,mygrid,'cloudmap');
%      %set(oldhandle,'EdgeColor','y');
%      hold off
%      axis auto
%      axis normal
%      axis equal
     
     
     %%% plot refined grid in 3D
     figure(2);
     meshhandle=mklsgridplot(lsopt,mygrid,'mesh');
     hold on
     surfhandle=surf(tx,ty,tz,topo.topo);
%      set(surfhandle,'EdgeLighting','flat');
%      set(surfhandle,'EdgeColor','flat');
%      set(surfhandle,'FaceLighting','flat');
%      set(surfhandle,'FaceColor','flat');
     colormap(rgb);
     caxis([minalt maxalt]); % topo range in m
     light('position',[0 -1 1]);
     lighting phong
     material dull
     shading interp
     set(meshhandle,'EdgeColor','w');
     hold off
     drawnow;
     
     %pause
     
     %toctime=toc; disp(['MKLSGRIDCONTINENTS: elapsed time so far: ' num2str(toctime) 'sec.']);
     toctime=[toctime toc];
     
     
     %%% statistics output to screen
     if iter<2
         spenttime=toctime(1)-initstep(3);
     else
         spenttime=toctime(iter)-toctime(iter-1);
     end; % if iter
     line=sprintf('%4.0f ,%13.0f   ,%17.0f  ,%9.0f  ,%10.4f',...
                     iter,...
                     checktri(iter),...
                     iteratetri(iter),...
                     nodeanz(iter),...
                     spenttime);
     disp(line);
     
 end; % for iter
 
 %toctime=toc; disp(['MKLSGRIDCONTINENTS: elapsed time so far: ' num2str(toctime) 'sec.']);
 toctime=[toctime toc];
 
 %%% table footer
 disp('+----+----------------+-------------------+-----------+----------+');
 
 disp(' ');
 disp(['MKLSGRIDCONTINENTS: program finished: ' mkclock ' (' num2str(toctime(end)) 'sec)']);