function mklsgriddemo(varargin);
% mklsgriddemo.......demo program for grid handling
%
% call: mklsgriddemo('iteration',miniter,maxiter);
%       mklsgriddemo('iteration',gridbase,maxiter);
%       mklsgriddemo('resolution',gridbase,gridwdt,minr,maxr,layerdr);
%       mklsgriddemo('continents',level,maxiter);
%       mklsgriddemo('coasts',level,maxiter);
%
%    The first parameter defines which demo to show:
%
%    'iteration': shows the behaviour of tetrahedron, octahedron and
%                  icsoahedron upon iteration. The user can define which
%                  iteration levels to show, these are then plotted in a
%                  matrix-like display with one columnt per body and lines
%                  corresponding to iteration level.
%
%                  additional parameters:
%
%                  miniter: (number)
%                           minimum iteration level
%                  maxiter: (number)
%                           maximum iteration level
%
%    'iteration2': shows the n-th iteration for one of the base bodies.
%
%                  additional parameters:
%
%                  gridbase: (string)
%                        name of the base body to be used. This may be one
%                        of the names allowed for the grid_base field in
%                        the LSOPT structure.
%
%                  maxiter: (number)
%                             iteration level. The base body will be
%                             iterated to this level.
%
%    'resolution': shows the spatial and vertical resolution of the initial
%                  grid, depending on layer spacing and mesh width. layer
%                  spacing can be set directly. Mesh width is controlled
%                  via tha max size of riangle circumcircles, the grid is
%                  iterated so fasr that all circumcircles fall below a
%                  given threshold.
%                  Draws 3D cutoff views of multilayer grids, statistics of
%                  circumcircle distibutions and plane views of the
%                  equatorial nodes of all layers.
%
%                  additional parameters:
%
%                  gridbase: (string)
%                            base body to use for grid construction
%                  gridwdt:  (number) [km]
%                            desired lateral grid resolution
%                     minr:  (number) [km]
%                            radius of innermost layer
%                     maxr:  (number) [km]
%                            Radius of outermost layer
%                  layerdr:  (number) [km]
%                            layer spacing
%
%    'continents': a single grid layer is refined locally in order to
%                  produce a grid that is fine on land and coarse on
%                  oceans.
%                  Draws a node map and a nice 3D view.
%
%                  additional parameters:
%
%                  level: (number) [m]
%                         elevation level in meters that is used as boundary
%                  maxiter: (number)
%                           max number of iteraions to be applied
%
%
%    'coasts':     similar to 'continents', but this time the grid is
%                  refined at coasts only.
%
%                  additional parameters:
%
%                  As in 'continents' demo
%
%
% result: none
%
% Martin Knapmeyer, 29.09.2004, 06.05.2008


%%% determine demo mode
[nin1,nin2]=size(varargin); % nin2 is number of input parameters
mode=varargin{1};
mode=lower(mode);


%%% switch to repsective demo routines
switch mode
    case {'continents','coasts'}
        if nin2~=3
            error('MKLSGRIDDEMO: illegal number of parameters for continents demo!');
        end; % if nin2~=3
        level=varargin{2};
        if isstr(level)
            error('MKLSGRIDDEMO: LEVEL must be a number!');
        end; % if isstring(level)
        maxiter=varargin{3};
        mklsgridcontinents(mode,level,maxiter);
    case {'resolution'}
        if nin2~=6
            error('MKLSGRIDDEMO: illegal number of parameters for resolution demo!');
        end; % if nin2~=3
        gridbase=varargin{2};
        if ~isstr(gridbase)
            error('MKLSGRIDDEMO: LEVEL must be a string!');
        end; % if isstring(level)
        gridwdt=varargin{3};
        minr=varargin{4};
        maxr=varargin{5};
        layerdr=varargin{6};
        mklsgridwdtdemotri(gridbase,gridwdt,minr,maxr,layerdr);
    case {'iteration'}
        miniter=varargin{2};
        maxiter=varargin{3};
        mklsgriditerdemotri(miniter,maxiter);
    case {'iteration2'}
      
        %%% decompose input
        gridbase=varargin{2};
        maxiter=varargin{3};
        
        %%% define LSOPT
        lsopt=mklsopt('planet_radius',1,...
                  'grid_base',gridbase,...
                  'grid_init_minr',0,...
                  'grid_init_maxr',1,...
                  'grid_init_dr',7000,...
                  'grid_init_wdt',10,...
                  'iter_circmin',0);
               
         %%% generate grid
         mygrid=mklsgridnew(lsopt); 
         
         %%% iterate grid
         if maxiter>0
            mygrid=mklsgridmodify(lsopt,mygrid,[],'iterate',maxiter);
         end; % if iter
         
         
         %%% plot iteration result
         clf;
         handle=mklsgridplot(lsopt,mygrid,'surface');
         set(handle,'LineWidth',0.1); % edge line width
         alpha(1);
         view(-12,21);
         set(gcf,'Color','w');
         drawnow;
            
            
    otherwise
        error(['MKLSGRIDDEMO: unknown demo mode ' upper(mode)]);
end; % switch mode