function mklsgridwdtdemotri(gridbase,meshwdt,minr,maxr,dr);
% mklsgridwdtdemotri.......demo of radius dependency of triangular grid mesh width
%
% call: mklsgridwdtdemotri(gridbase,meshwdt,minr,maxr,dr);
%
%       gridbase: (string)
%                string denoting the base body to be used for grid
%                construction. This must be one of the values alloewd for
%                the lsopt.grid_base field.
%
%       meshwdt: (number) [km]
%                desired mesh width of the generated grid, in terms of
%                triangle circumcircle radius, measured in Kilometer.
%                The generated grid will have no triangles with
%                circumcircles greater than this.
%
%          minr: (number) [km]
%                Radius of the innermost layer.
%
%          maxr: (number) [km]
%                Radius of the outermost layer.
%     
%            dr: (number) [km]
%                delta radius of the grid layers.
%                The generated grid will have a radius of the outermost
%                layer of 6371km (earth radius). The deeper layers will
%                have radii of 6371-n*dr, n=1...6371/dr.
%
% result: none
%
% This routine creates a multilayer triangular grid and iterates each layer
% until its largest triangle's circumcircle is smaller than the predefined
% mesh width.
%
% Produces three plots:
%
% Figure 1:
% A 3D cutaway plot of the layers. Coloring is chosen according to the
% number of iterations applied to each layer.
%
% Figure 2:
% Distributions of circumcircle radii in each layer, plotted against layer
% radius. A Black line shows the maximum circumcircle radius in each layer.
% A red line shos the desired mesh width. The red line should be above
% the black line. Coloring of radius markers according to colors in Fig. 1
%
% Figure 3:
% A plane view of all points in the equator plane. Each layer is
% represented by a circle with a dot at the positions of grid nodes.
% If a layer has no nodes in the equator plane, only a circle (not dots) is
% drawn. Coloring of radius according to colors in Figure 1.
%
% Martin Knapmeyer, 26.01.2005

%%% a useful constant, once more...
radian=pi/180;

%%%%%%%%%%% Construct Grid

%%% declare LSOPT structure
%gridbase='icosahedron';
pradius=6371; % km, planetary radius
lsopt=mklsopt('planet_radius',pradius,...
              'grid_type','triangular',...
              'grid_base',gridbase,...
              'grid_init_minr',minr,...
              'grid_init_maxr',maxr,...
              'grid_init_dr',dr,...
              'grid_init_wdt',meshwdt,...
              'iter_circmin',0,...
              'verbosity',1);
          
              
 %%% generate grid
 mygrid=mklsgridnew(lsopt);
 layeranz=size(mygrid.layers,2);
 
 %%%%%%%%%%% grid construction complete
 
 
 
 
 %%%%%%%%%%% Here follows only plotting stuff
 
 
 %%% kill some triangles to let us look in
 switch gridbase
     case {'tetrahedron'}
         killcnt=1;
     case {'octahedron'}
         killcnt=2;
     case {'icosahedron'}
         killcnt=9;
     otherwise
         killcnt=2;
 end; % switch gridbase
 idlist.layer=[];
 idlist.triangle=[];
 for indy=1:layeranz
     idlist(indy).layer=mygrid.layers(indy).id;
     for j=1:killcnt
        idlist(indy).triangle=[idlist(indy).triangle mygrid.layers(indy).tri(j).id];
     end;
 end; % for indy
 mygrid=mklsgridmodifytri(lsopt,mygrid,idlist,'kill');
 
 
 %%% plot grid
 figure(1);
 plotmode='surface';
 handles=mklsgridplot(lsopt,mygrid,plotmode);
 alpha(1);
 
 
 %%% the FaceColor is set depending on iteration depth of the layer
 maxiter=zeros(1,layeranz);
 for indy=1:layeranz
     iter=mklsgridgetmaxitertri(mygrid.layers(indy));
     maxiter(indy)=max(iter);
 end; % for indy
 rgb=jet(max(maxiter)+1);
 rgb=flipud(rgb);
 for indy=1:layeranz
     set(handles(indy),'FaceColor',rgb(1+maxiter(indy),:));
 end; % for indy
 
 
 %%% do some statistics to prove claims
 figure(2);
 plot([0 pradius],[1 1]*meshwdt,'r-');
 maxccr=zeros(1,layeranz);
 layrrad=maxccr;
 for indy=1:layeranz
     layrrad(indy)=mygrid.layers(indy).radius;
     reslist=mklsgridstattrilayer(lsopt,mygrid.layers(indy),'circle');
     reslist=reslist*radian*mygrid.layers(indy).radius; % from degrees to km
     maxccr(indy)=max(reslist(:));
     %disp(['Layer #' int2str(indy) ', ',...
     %      'radius=' num2str(mygrid.layers(indy).radius) 'km, ',...
     %      'maxCCR=' num2str(maxccr(indy)) 'km']);
     hold on
     h=plot(reslist*0+layrrad(indy),reslist,'o');
     set(h,'MarkerFaceColor',rgb(1+maxiter(indy),:));
     set(h,'MarkerEdgeColor',rgb(1+maxiter(indy),:));
     hold off
 end; % for indy
 hold on
 plot(layrrad,maxccr,'k-');
 hold off
 xlabel('Layer Radius [km]');
 ylabel('Circumcircle Radii [km]');
 ax=axis;
 ax(4)=ax(4)*1.1;
 axis([0 pradius 0 ax(4)]);
 grid on
 title('Circumcircle Radii Distributions');
 xlabel('Layer Radius [km]');
 ylabel('Circumcircle Radius [km]');
 
 
 %%% plot an equatorial cut
 figure(3);
 clf;
 for indy=1:layeranz
     %%% find nodes which are at the equator
     indies=find(mygrid.layers(indy).nodes(:,3)==0);
     %%% get longitudes of these nodes
     eqlons=sort(mygrid.layers(indy).nodes(indies,2));
     if ~isempty(eqlons)
         %%% nodes exist on equator - prepare to plot them
         eqlons=[eqlons; eqlons(1)];
         eqlons=eqlons*radian;
         linestyle='.-';
     else
         %%% no nodes exist on equator - prepare bare circle
         eqlons=(0:360)*radian;
         linestyle='-';
     end; % if ~isempty(eqlons)
     %%% plot these nodes into a plane
     x=mygrid.layers(indy).radius*cos(eqlons);
     y=mygrid.layers(indy).radius*sin(eqlons);
     hold on
     h=plot(x,y,linestyle);
     hold off
     set(h,'MarkerFaceColor',rgb(1+maxiter(indy),:));
     set(h,'MarkerEdgeColor',rgb(1+maxiter(indy),:));
     set(h,'Color',rgb(1+maxiter(indy),:));
 end; % for indy
 axis equal
 axis off
 title('Equator Plane View');
 