function [lsopt,mygrid,seisdat]=mklslocationdemo(slon,slat,selv,hlon,hlat,hdep,dbaz,bazbias,dt,tbias,phase,initwdt);
% mklslocationdemo...........demonstrate LOCSMITH location with artifical data
%
% call: 
%
%       [lsopt,mygrid,seisdat]=mklslocationdemo;
%       [lsopt,mygrid,seisdat]=mklslocationdemo('demo1');
%       [lsopt,mygrid,seisdat]=mklslocationdemo('demo2');
%
%       [lsopt,mygrid,seisdat]=mklslocationdemo(slon);
%       [lsopt,mygrid,seisdat]=mklslocationdemo(slon,slat);
%       [lsopt,mygrid,seisdat]=mklslocationdemo(slon,slat,selv);
%       [lsopt,mygrid,seisdat]=mklslocationdemo(slon,slat,selv,hlon);
%       [lsopt,mygrid,seisdat]=mklslocationdemo(slon,slat,selv,hlon,hlat);
%       [lsopt,mygrid,seisdat]=mklslocationdemo(slon,slat,selv,hlon,hlat,hdep);
%       [lsopt,mygrid,seisdat]=mklslocationdemo(slon,slat,selv,hlon,hlat,hdep,dbaz);
%       [lsopt,mygrid,seisdat]=mklslocationdemo(slon,slat,selv,hlon,hlat,hdep,dbaz,bazbias);
%       [lsopt,mygrid,seisdat]=mklslocationdemo(slon,slat,selv,hlon,hlat,hdep,dbaz,bazbias,dt,tbias,phase);
%       [lsopt,mygrid,seisdat]=mklslocationdemo(slon,slat,selv,hlon,hlat,hdep,dbaz,bazbias,dt,tbias,phase,initwdt);
%
%       (sorry for not having a property name / property value technology here)
%
%       'demo1': a predefined demo, using the lunar apollo network
%                configuration and an arbitrary surface event.
%                This demonstrates the "interior" search method.
%                Only Azimuth data is used.
%
%       'demo2': a predefined demo, using a tetrahedron configuration and a
%                surface event close to one of the stations. This demo shows the
%                "Attention Creep Effect": The initial grid points to an area
%                which is far from the real solution, but using the
%                neighborhood iteration method, the attention of the
%                program creeps into right direction.
%                I found this configuration by trying some random inputs,
%                therefore it is not a perfect example (takes a bit long to
%                compute).
%                This also demonstrates the boundary search method.
%                Only Azimuth data is used.
%
%       'demo3': a predefined demo, using the Apollo network configuration
%                and an arbitrary deep event.
%                Only Travel Time data is used.
%
%       slon: (numeric array) [deg]
%             vector of station longitudes in 0...360deg
%       slat: (numeric array) [deg]
%             vector of station latitudes
%       selv: (numeric array) [km]
%             vector of station elevations, positive upwards
%
%       hlon: (number) [deg]
%             Hypocenter longitude in 0...360deg
%       hlat: (number) [deg]
%             Hypocenter latitude
%       hdep: (number) [km]
%             Hypocenter depth, positive downwards
%
%       dbaz: (numeric array) [deg]
%             back azimuth uncertainty of each station, in degrees
%             This defines how wide the belt around the true BAZ is, which
%             is used in location. Simulates BAZ measurement error.
%             Use the empty martrix [] to switch off azimuth evaluation
%       bazbias: (numeric array) [deg]
%             A station specific bias which is added to the theoretical BAZ
%             values. Also simluates BAZ measurement error.
%
%       dt:  (numeric array) [s]
%            arrival time uncertainty of each station, in seconds
%       tbias: (numeric array) [s]
%            arrival time biases of all stations, in seconds
%
%       phase: (string array)
%            seismic phases assumed to be detected at all stations
%            arrival times are computed for these phases.
%
%       initwdt: (number) [km]
%            initial mesh width on the outermost layer of the grid
%            
%
%      DEFAULT values are returned by MKLSCONSTANTS('LocationDemo1').
%
% result: lsopt: (struct)
%                The LSOPT structure used for location
%
%         mygrid: (struct)
%                 The grid strucure resulting from the location.
%
%         seisdat: (struct)
%                 The LSD structure used for location.
%
% This routine computes azimuth and arrival time data for the given
% hypocenter and station network and then runs MKLOCSMITH to locate the
% hypocenter using the given data.
% If you don't want azimuth data to be generated, use the empty matrix []
% as input for DBAZ.
% This is a demonstration and test environment for MKLOCSMITH.
%
% The routine generates 
% figure(1): a map of the Epicenter, station network and
%               Azimuthal gap (which is a widely used proxy for achievable 
%               location accuracy)
% figure(2): view of the search grid evolution during the location process
%
% figure(3): 3D view of the final grid
%
% Martin Knapmeyer, 17.03.2005, 08.03.2007


%%% say hello
disp(['MKLSLOCATIONDEMO: started at ' mkclock]);

%%% default search method, might be changed in predefined demos
init_circmin=2000;
init_searchmode='interior';

%%% understand input
%%% (do not write email - I already know that this is a horrible way to implement this!)
nin=nargin;
parms=mklsconstants('LocationDemo1');
%%% cannot be defined as input parameter:
lsopt=parms.lsopt;
switch nin
    case{0}
        slon=parms.slon;
        slat=parms.slat;
        selv=parms.selv;
        hlon=parms.hlon;
        hlat=parms.hlat;
        hdep=parms.hdep;
        dbaz=parms.dbaz;
        bazbias=parms.bazbias;
        phase=parms.phase;
        ot=parms.ot;
        dt=parms.dt;
        tbias=parms.tbias;
    case {1}
        if isstr(slon)
            %%% input argument is a string denoting one of the predefined demos
            slon=lower(slon);
            switch slon
                case {'demo1'}
                    parms=mklsconstants('LocationDemo1');
                case {'demo2'}
                    parms=mklsconstants('LocationDemo2');
                    init_circmin=20;
                    init_searchmode='boundary';
                case {'demo3'}
                    parms=mklsconstants('LocationDemo3');
                case {'demo4'}
                    parms=mklsconstants('LocationDemo4');
                case {'demo5'}
                    parms=mklsconstants('LocationDemo5');
                case {'demo6'}
                    parms=mklsconstants('LocationDemo6');
                case {'demo7'}
                    parms=mklsconstants('LocationDemo7');
                otherwise
                    error(['MKLSLOCATIONDEMO: demo named ' upper(slon) ' not known!']);
            end; % switch slon
            slon=parms.slon;
        end; % if isstr(slon)
        slat=parms.slat;
        selv=parms.selv;
        hlon=parms.hlon;
        hlat=parms.hlat;
        hdep=parms.hdep;
        dbaz=parms.dbaz;
        bazbias=parms.bazbias;
        phase=parms.phase;
        ot=parms.ot;
        dt=parms.dt;
        tbias=parms.tbias;
    case {2}
        selv=parms.selv;
        hlon=parms.hlon;
        hlat=parms.hlat;
        hdep=parms.hdep;
        dbaz=parms.dbaz;
        bazbias=parms.bazbias;
        phase=parms.phase;
        ot=parms.ot;
        dt=parms.dt;
        tbias=parms.tbias;
    case {3}
        hlon=parms.hlon;
        hlat=parms.hlat;
        hdep=parms.hdep;
        dbaz=parms.dbaz;
        bazbias=parms.bazbias;
        phase=parms.phase;
        ot=parms.ot;
        dt=parms.dt;
        tbias=parms.tbias;
    case {4}
        hlat=parms.hlat;
        hdep=parms.hdep;
        dbaz=parms.dbaz;
        bazbias=parms.bazbias;
        phase=parms.phase;
        ot=parms.ot;
        dt=parms.dt;
        tbias=parms.tbias;
    case {5}
        hdep=parms.hdep;
        dbaz=parms.dbaz;
        bazbias=parms.bazbias;
        phase=parms.phase;
        ot=parms.ot;
        dt=parms.dt;
        tbias=parms.tbias;
    case {6}
        dbaz=parms.dbaz;
        bazbias=parms.bazbias;
        phase=parms.phase;
        ot=parms.ot;
        dt=parms.dt;
        tbias=parms.tbias;
    case {7}
        bazbias=parms.bazbias;
        phase=parms.phase;
        ot=parms.ot;
        dt=parms.dt;
        tbias=parms.tbias;
    case {8}
        phase=parms.phase;
        ot=parms.ot;
        dt=parms.dt;
        tbias=parms.tbias;
    case {11}
        ot=parms.ot;
        lsopt.iter_circmin=100;
        lsopt.verbosity_plot_mode='none';
    case {12}
        ot=parms.ot;
        lsopt.grid_init_wdt=initwdt;
        lsopt.iter_circmin=initwdt;
        lsopt.verbosity_plot_mode='none';
    otherwise
        error('MKLSLOCATIONDEMO: illegal number of input arguments!');
end; % switch nin

%%% fine velocity sampling
%lsopt.tt_options.interpdz=10;


%%% write Hypocenter coordinates to screen
disp(' ');
disp(['MKLSLOCATIONDEMO: True 4D Hypocenter at: ']);
disp(['                  ' num2str(hlon) ' lon']);
disp(['                  ' num2str(hlat) ' lat']);
disp(['                  ' num2str(hdep) ' depth']);
disp(['                  ' num2str(ot) ' sec after LOCSMITH reference time']);
disp(' ');



%%% test input plausibility
if (length(hlon)>1)|(length(hlat)>1)|(length(hdep)>1)
    error('MKLSLOCATIONDEMO: please define only one epicenter!');
end; % if length(hlon)
if ~isempty(dbaz)
    if length(dbaz)~=length(slon)
        error('MKLSLOCATIONDEMO: please define individual DBAZ for each station!');
    end; % if length(dbaz
    if length(bazbias)~=length(slon)
        error('MKLSLOCATIONDEMO: please define individual BAZBIAS for each station!');
    end; % if length(bazbias
end; % if ~isempty(dbaz)


%%% correct station longitudes if necessary
slon=mklslon180to360(slon);

%%% correct hypocenter longitude if necessary
hlon=mklslon180to360(hlon);

%%% plot station network and Epicenter in a mapping of the Azimuthal Gap
figure(1);
mklsgapmap(slon,slat);
gap=mklsgapmap(mklslon360to180(slon),slat,mklslon360to180(hlon),hlat);
title(['Azimuthal Gap at Epicenter: ' num2str(round(gap*10)/10) 'deg']);
hold on
h=plot3(mklslon360to180(hlon),hlat,400,'p');
set(h,'MarkerSize',18);
set(h,'MarkerFaceColor','w');
set(h,'MarkerEdgeColor','k');
for indy=1:length(slon)
    text(mklslon360to180(slon(indy)),slat(indy),410,['S' int2str(indy)],...
        'HorizontalAlignment','center');
end; % for indy
hold off



%%% set LOCSMITH options
range=[0 360 -90 90];
mapopt=mkmapopt(...
                'planet_radius',lsopt.planet_radius,...
                'proj_type','platecarree',...
                'proj_range',range,...
                'topo_mode','off',...
                'frame_mode','on',...
                'grid_mode','on',...
                'grid_range',range,...
                'grid_lonmesh',30,...
                'grid_latmesh',30,...
                'grid_style',':'...
                );
lsopt.mapopt=mapopt;




%%% construct LSD input struct for MKLOCSMITH
phaselist=phase;
arrivals=size(phaselist,1); % number of different arrivals per station to be evaluated
seisdat=mklsemptylsd(length(slon),...
                     1,...
                     length(slon)*arrivals);
%%% seisdat is now an empty LSD structure of the needed size.
%%% copy station locations into seisdat
for indy=1:seisdat.statcnt
    seisdat.station(indy).id=['S' int2str(indy)];
    seisdat.station(indy).lon=slon(indy);
    seisdat.station(indy).lat=slat(indy);
    seisdat.station(indy).elev=selv(indy);
end; % for indy
%%% define events (there is only one)
seisdat.event.id='Quake';
seisdat.event.info='A user defined hypocenter.';
%%% define arrival data
arrivalcnt=1;
disp('MKLSLOCATIONDEMO: generating synthetic data...');
for statindy=1:seisdat.statcnt
    for arrivalindy=1:arrivals
        %%% station, event, and phase reference
        seisdat.arrival(arrivalcnt).id=int2str(arrivalcnt);
        seisdat.arrival(arrivalcnt).station=seisdat.station(statindy).id;
        seisdat.arrival(arrivalcnt).event=seisdat.event.id;
        seisdat.arrival(arrivalcnt).phase=deblank(phaselist(arrivalindy,:));
        
        %%% compute arrival time, add origin time
        if ~isempty(dt)
            tt=mklssinglett(lsopt,...
                            seisdat.arrival(arrivalcnt).phase,...
                            hlon,hlat,hdep,...
                            seisdat.station(statindy).lon,...
                            seisdat.station(statindy).lat,...
                            seisdat.station(statindy).elev);
            tt=min(tt); % use only the earliest of possible multiples!
            seisdat.arrival(arrivalcnt).at=[tt-dt(statindy)+tbias(statindy),...
                                             tt+dt(statindy)+tbias(statindy),...
                                             tt+tbias(statindy)]+ot;
        end; % if ~isempty(dt)
        
        
        %%% back azimuth
        if ~isempty(dbaz)
            %disp('MKLSLOCATIONDEMO: generating BAZ data.');
            baz=mklsbazv(hlon,hlat,...
                         seisdat.station(statindy).lon,seisdat.station(statindy).lat);
            seisdat.arrival(arrivalcnt).baz=[baz-dbaz(statindy)+bazbias(statindy),...
                                             baz+dbaz(statindy)+bazbias(statindy),...
                                             baz+bazbias(statindy)];
        end; % if ~isempty(dbaz);
        
        %%% emergence angle
        % (not yet)
        
        %%% slowness
        %(not yet)
        
        arrivalcnt=arrivalcnt+1;
    end; % for arrivalindy
end; % for statindy
disp('MKLSLOCATIONDEMO: ... synthetic data generation done.');

%%% now we can run MKLOCSMITH!
figure(lsopt.verbosity_plot_fig);
clf;
disp('MKLSLOCATIONDEMO: run MKLOCSMITH...');
[exitstatus,incomp,loc,mygrid]=mklocsmith(lsopt,[],seisdat);
disp('... done.');
if strcmp(exitstatus,'no data')==0
    
    
%     %%% plot final grid map
%     figure(lsopt.verbosity_plot_fig);
%     clf;
%     %%% creates base map
%     mkmakemap(lsopt.mapopt);
%     %%% overlay grid nodes
%     hold on
%     %mklsgridplot(lsopt,mygrid,lsopt.verbosity_plot_mode); 
%     mklsgridresplot(lsopt,mygrid,'fitcloudmap',0);
%     hold off
%     %%% plot true epicenter into final grid plot
%     evdat.statcnt=1;
%     evdat.station.lon=hlon;
%     evdat.station.lat=hlat;
%     evdat.station.elev=-hdep;
%     evdat.station.id='Q';
%     hold on
%     shandle=mklsnetplot(lsopt,evdat,'fitcloudmap');
%     set(shandle,'Marker','p');
%     set(shandle,'MarkerSize',18);
%     set(shandle,'MarkerFaceColor','w');
%     set(shandle,'MarkerEdgeColor','k');
%     hold off
%     %%% plot station locations
%     hold on
%     shandle=mklsnetplot(lsopt,seisdat,'fitcloudmap');
%     set(shandle,'Marker','o');
%     set(shandle,'MarkerSize',10);
%     set(shandle,'MarkerFaceColor','r');
%     set(shandle,'MarkerEdgeColor','r');
%     hold off


    %%% plot final grid in 3D
    figure(3);
    clf;
    gridplotmode='fitsurface';
    mklsgridplot(lsopt,mygrid,gridplotmode);
    hold on
    mklsnetplot(lsopt,seisdat,gridplotmode);
    %%% plot true epicenter into final grid plot
    evdat.statcnt=1;
    evdat.station.lon=hlon;
    evdat.station.lat=hlat;
    evdat.station.elev=-hdep;
    evdat.station.id='Q';
    shandle=mklsnetplot(lsopt,evdat,gridplotmode);
    %%% some beautification
    set(shandle,'Marker','p');
    set(shandle,'MarkerSize',18);
    set(shandle,'MarkerFaceColor','w');
    set(shandle,'MarkerEdgeColor','k');
    hold off
    drawnow;
    
    
%     %%% plot returned location cloud in 3D
%     figure(4);
%     clf;
%     mode='otcloud';
%     %%% plot best fitting nodes
%     mklsgridresplot(lsopt,mygrid,mode,[]);
%     %%% add stations
%     hold on
%     mklsnetplot(lsopt,seisdat,mode);
%     hold off




%     %%% for comparison: plot BAZ map using the old LOCSMITH-2D package
%     % figure(3);
%     % for indy=1:seisdat.arrivalcnt
%     %     event(indy).name=seisdat.arrival(indy).id;
%     %     event(indy).slat=seisdat.station(indy).lat; 
%     %     event(indy).slon=seisdat.station(indy).lon;
%     %     event(indy).azi(1).mini=min(seisdat.arrival(indy).baz);
%     %     event(indy).azi(1).maxi=max(seisdat.arrival(indy).baz);
%     % end; % for indy
%     % mode='map';
%     % confidence=100;
%     % lsopt2D=mklocsmithopt('maxrec',75,'conf',confidence);
%     % figure(3);
%     % [rescode,incomp,elat,elon,counter,cntlat,cntlon]=mkbazloc(event,mode,lsopt2D);
%     % title('Data Compatibility Map from LOCSMITH-2D');


end; % if strcmp(exitstatus,'no data')==0


%%% LSOPT, MYGRID and SEISDAT are returned as output.

