function [reliability,efficiency,evalcnt]=mklszoptimdrv(matchthresh,zets,stepx,stepy,drmin,plotmode);
% mklszoptimdrv......driver for MKLSGRIDNEWVERTICAL
%
% call: [reliability,efficiency,evalcnt]=mklszoptimdrv(matchthresh,zets,stepx,stepy,drmin,plotmode);
%
% matchthresh: (number) [percent]
%              lsopt.iter_matchthresh value used by MKLSGRIDNEWVERTICAL
%
%       zets: (numeric array) [km]
%             z values for the initial sampling
%             The smallest z value in here must be zero.
%
%       stepx: (numeric array) [km]
%              list of depth values, the step function has its jumps at
%              these depths
%
%       stepy: (numeric array) [km]
%              function values of the step function.
%              At depth stepx(i) the function jumps to value stepy(i)
%
%       drmin: (number) [km]
%              The minimum allowed distance between function samples: the
%              search is stopped when the shortest sample distance is shorter
%              than this. 
%
%    plotmode: (string) [flag]
%              string flag to swtich on and off the production of
%              control plots during tests.
%              possible values: 'on' and 'off'
%
%
%
% result: reliability: (number) [percent]
%                      number of samples with maximum function value found
%                      by MKLSGRIDNEWVERTICAL algorithm in relation to the
%                      number found by unform sampling
%                      A reliability of e.g. 40% means that
%                      MKLSGRIDNEWVERTICAL returned only 40% as many
%                      maximum points as the uniform grid search did. This
%                      means that it missed some solutions.
%                      The reliability therefore has to be maximized.
%
%         efficiency: (number) [percent]
%                     number of function evaluations used by MKLSGRIDNEWVERTICAL
%                     divided by the number of function evaluations in a uniform
%                     sampling with the same maximal grid density.
%                     An efficiency of e.g. 40% means that
%                     MKLSGRIDNEWVERTICAL used only 100%-40%=60% of the function
%                     evaluations that were used in the uniform grid
%                     search. This means it saved 40% of the computational
%                     cost.
%                     The efficiency therefore has to be maximized.
%
%          evalcnt:   (number) [counter]
%                     number of function evaluations.
%
%        Obviously, there is a trade off between raliability and efficieny:
%        you can be very efficient when you miss a lot of the solutions...
%
% This function drives the optimization of a step function by
% MKLSGRIDNEWVERTICAL and evaluates the reliability and efficieny of the sampling
% algorithm.
% For the evaluation, a uniform grid search is conducted. The sampling
% density of this search is equal to the highest density achieved in the
% non-uniform search. The number of samples with maximal function value and
% the number of function evaluations are compared between the two methods.
%
% Martin Knapmeyer, 30.09.2005, 28.04.2006


%%% ckeck input
zets=sort(zets);
if zets(1)~=0
   error('MKLSZOPTIMRV: first z sample must be zero!');
end; %

%%% step function evaluation mode
%%% MUST be 'absolute'. MK28042006
stepmode='absolute';

%%% set LOCSMITH options, necessary to execute MKLSGRIDNEWVERTICAL
rp=6371; % planet radius in km
range=[0 360 -90 90];
mapopt=mkmapopt(...
                'proj_type','platecarree',...
                'proj_range',range,...
                'topo_mode','off',...
                'frame_mode','on',...
                'grid_mode','on',...
                'grid_range',range,...
                'grid_lonmesh',30,...
                'grid_latmesh',30,...
                'grid_style',':'...
                );
lsopt=mklsopt(...
              'planet_radius',rp,...
              'grid_type','triangular',...
              'grid_base','icosahedron',...
              'grid_init_minr',0,...
              'grid_init_maxr',6371,...
              'grid_init_dr',500,... % only one layer for testing!
              'grid_init_wdt',8000,...
              'iter_vertical_matchthresh',matchthresh,...
              'iter_circmin',200,...
              'iter_drmin',drmin,...
              'iter_max',200,...
              'iter_vertical_max',200,...
              'eval_baz','on',...
              'eval_at','on',...
              'eval_dt','off',...
              'verbosity',0,...
              'verbosity_plot_mode','fitcloudmap',...
              'verbosity_plot_fig',1,...
              'mapopt',mapopt...
              );

%%% dense samplin of function for plot purposes
if strcmp(plotmode,'on')==1
    fullx=0:rp;
    fully=mkstepfunction(fullx,stepx,stepy,stepmode);
end; % if strcmp(plotmode,'on')==1
          

%%% counter for function evaluations
evalcnt=0;

          
%%% evaluation of initial interval
x=zets;           
y=mkstepfunction(x,stepx,stepy,stepmode);
evalcnt=evalcnt+length(x);
             
              
%%% optimization loop
done=0;
cnt=0;
maxcnt=lsopt.iter_vertical_max;
while done==0
    
         
     %%% plot current function sampling
     if strcmp(plotmode,'on')==1
         figure(lsopt.verbosity_plot_fig);
         clf;
         plot(fullx,fully,'k');
         hold on
         plot(x,y,'r.-');
         hold off
         axis([min(fullx) max(fullx) min(fully)-1 max(fully)+1]);
         drawnow;
     end; % if strcmp(plotmode,'on')==1
          
     
     %%% construct new x vector
     [extendedx,newdr]=mklsgridnewvertical(lsopt,x,y);
     drawnow
     
     %%% which point in EXTENDEDX are new? evaluate only those!
     newx=setdiff(extendedx,x);
     
     
     %%% are there any new points? If not, cancel loop!
     if isempty(newx)
        %disp('Grid does not change any more!');
        done=1;
     end; % if isempty
     
     
     %%% evaluate new sampling
     newy=mkstepfunction(newx,stepx,stepy,stepmode);
     evalcnt=evalcnt+length(newx);
     
     
     %%% plot results for new samples
     if strcmp(plotmode,'on')==1
         figure(lsopt.verbosity_plot_fig);
         hold on
         h=plot(newx,newy,'mo');
         hold off
         %set(h,'LineWidth',3);
         drawnow;
     end; % if strcmp(plotmode,'on')==1
     
     %%% join n, newx, y, and newy
     x=[x newx];
     y=[y newy];
     [x,sorter]=sort(x);
     y=y(sorter);
     
     
     %%% increment loop counter
     cnt=cnt+1;
          
     %%% loop emergency brake
     if cnt>maxcnt
         disp('Max number of iterations reached');
         done=1;
     end; % if cnt>maxcnt
     
     
end; % while


%%% now do a uniform sampling for result comparison
%%% in this uniform sampling, we use the minimum sample distance of the
%%% non-uniform sampling as sampling distance. If the first sample in ZETS
%%% is zero, then the non-uniform sampling will be a subste of the uniform
%%% sampling.
%%% the uniform sampling is tuned such that at least one sample has maximum
%%% function value

%%% at which defining sample is the function max?
%%% maxy: max function value
%%% maxindy: number of defining sample at which max ist reached first time
[maxy,maxindy]=max(cumsum(stepy));

%%% construct uniform sequence of samples
%fullx=0:min(diff(x)):max(zets);
uniformdx=min(diff(x));
fullx=unique([stepx(maxindy):-uniformdx:min(zets) (stepx(maxindy)+uniformdx):uniformdx:max(zets)]);

%%% determine function values at given sample positions
fully=mkstepfunction(fullx,stepx,stepy,stepmode);
% disp(['MKLSZOIPTIMDRV: matchthresh=' int2str(matchthresh) ', init int. ' int2str(length(zets-1)),...
%        ', uniform sampling ' int2str(length(fullx)-1) ' intervals']);
%    figure(5); hold on; plot(length(zets-1),(length(fullx)-1),'.');  hold off

%%% determine max function value
maxval=max(stepy);
%%% how many points of the uniform sampling are max?
maxpos=find(fully==maxval);
maxcnt=length(maxpos);

%%% reliability: number of identified max-samples in the non-uniform
%%% sampling in relation to that of the uniform sampling
reliability=100*length(find(y==maxval))/maxcnt;

%%% efficiency: ratio of function evaluation count between both mehtods
efficiency=100-100*evalcnt/length(fullx);
%disp([int2str(evalcnt) ' ' int2str(length(fullx))]);


