function [efficiency,reliability]=mklszoptimtest(sampleanz,maxstepsize,matchthresh,zets,drmin,runs,plotmode);
% mklszoptimtest.......test efficiency of vertical sampling algorithm
%
% call: mklszoptimtest(sampleanz,maxstepsize,matchthresh,zets,drmin,runs,metaruns,plotmode);
%
%         sampleanz: (number) [counter]
%                    number of depth samples that define the step function
%
%       maxstepsize: (number) [counter]
%                    maximum size of vertical steps in function: at the
%                    position of the depth samples, the function will
%                    change by no more than this, in integer steps.
%
%       matchthresh: (number) [percent]
%                    value for lsopt.iter_matchthresh, to be used in tests
%
%              zets: (numeric array) [km]
%                    List of initial depth samples to be used. The sampling
%                    algorithm will start with these samples.
%                    Thee values are depths inside the earth, i.e. these
%                    must be values from 0 to 6371.
%                    The first value MUST be zero.
%                    Enter (0:1/n:1)*6371 to start with an n samples
%                    uniform sampling.
%
%              drmin: (number) [km]
%                    The minimum allowed distance between function samples: the
%                    search is stopped when the shortest sample distance is shorter
%                    than this. 
%
%              runs: (number) [counter]
%                    number of test runs to be made
%                    After RUNS runs of the vertical optimization
%                    algorithm, the number of failures and the cost saving
%                    (relative to a uniform sampling) will be determined
%
%          plotmode: (number) [flag]
%                    numerical flag to swtich on and off the production of
%                    control plots during tests.
%                    possible values:
%                    'off': no plot
%                    'histogram': histogram plot after each set of RUNS runs
%                    'function': as 1, but additionally plots the step function and
%                       the progress of the sampling
%
% result: efficiency: (numeric array) [percent]
%                     RUNS elements array, containing a list of the individual
%                     efficincy results of each optimization run (i.e. the entire distribution)
%
%         reliability: (numeric array) [percent]
%                     RUNS element array, containing a list of the
%                     individual reliability results of each optimization
%                     run (i.e. the entire distribution)
%
%                   
%
% This routine estimates the reliability of the vertical optimization
% algorithm used in LOCSMITH.
% This algorithm is used to determine the possible focal depths of the quake
% in consideration. To do so, all maxima of the degree-of-compatibility
% function (as function of depth) have to be determined. The values of this
% function are always positive integers, the function is thus a step
% function.
% A uniform depth sampling could be used, but is too expensive in terms of
% CPU time. Therefore the algorithm in MKLSGRIDNEWVERTICAL is used to
% construct a non-uniform sampling at lesser cost.
%
% The questions to be answered are:
% - Is this algorithm reliable?
%   This means: in how many cases does it NOT find all possible solutions?
%
% - Is this algorothm efficient?
%   This means: how many evaluations of the target function are saved
%               compared to the uniform sampling?
%
% To answer these questions, test runs wih random step functions are done.
% Each run of the algorithm produces a certain depth sampling in which a
% certain number of points is at the upper most step of the function. For
% comparison, a uniform sampling with sample distance equal to the smallest
% sample distance of the non-uniform sampling is carried out. This is done
% in a way that the non-uniform sampling produces a subset of the uniform
% sampling.
% Reliability can then be expressed as the number of missing
% maximum-samples in the non-uniform sampling.
% Efficiency can be expressed as ratio of function evaluations in the two
% samplings.
%
% RUNS runs of this procedure are made to get a statistical estimate, which
% is necessary since the step functions are generated randomly.
% These runs produce empirical distributions of both reliability and efficiency,
% which are returned.
%
%
% Martin Knapmeyer, 30.09.2005, 06.10.2005


if nargout==0
   tic;
end; % if nargout

%%% sometimes, divisions by zero occur. No warning please.
warning off


%%% evaluate plotmode
switch plotmode
    case {'off'}
        drvplotmode='off';
        histplotmode='off';
    case {'histogram'}
        drvplotmode='off';
        histplotmode='on';
    case {'function'}
        drvplotmode='on';
        histplotmode='on';
end; % switch


%%% build some arrays to store results
reliability=zeros(runs,1);
efficiency=zeros(runs,1);
evalcnt=zeros(runs,1);





%%% runs loop
for run=1:runs

    %%% construct random step function
    %%% 1. construct function in gradient mode that never jumps by more
    %%%    than +/- n, n integer, and that has minimum sample distance of DRMIN
    %%% 2. determine minimum function value
    %%% 3. evaluate function at given samples and add minimum
    %%% 4. the resulting function is then to be evaluated in absolute
    %%%    mode.
    %%% This is necessary, since the real functions appearing in
    %%% location problems are always positive and do probably not make
    %%% big jumps. MK28042006
    %sampleanz=10; % function has so many samples
    %maxstepsize=4; % maximum step size of function.
    %%% construct function with small jumps
    stepx=0:(drmin*2):710; % simple way to guarantee sample distance
    indies=unique(fix(rand(1,sampleanz)*(length(stepx)-1)+1));
    stepx=stepx(indies); 
    stepdirection=[sign(rand(1,sampleanz-1)-0.5) 0]; 
    stepsize=[round(rand(1,sampleanz-1)*maxstepsize) 0];
    gradienty=stepdirection.*stepsize;
    % gradienty=[sign(rand(1,sampleanz-1)*2-1) 0];
    %%% compute absolute function values
    stepy=mkstepfunction(stepx,stepx,gradienty,'gradient');
    %%% shift absolute values by function minimum
    stepy=stepy-min(stepy);
    %%% set last value to 0
    stepy(end)=0;

    %%% unrestricted random stepping
    %stepy=fix(rand(1,length(stepx))*4)+1;

    %%% run optimization
    [reliability(run),efficiency(run),evalcnt(run)]=mklszoptimdrv(matchthresh,...
                                                     zets,stepx,stepy,drmin,...
                                                     drvplotmode);


end; % for run

%%% control plot of reliability and costsaving distributions
if strcmp(histplotmode,'on')==1
    
    binwidth=2.5;

    figure(2);
    [n,xout]=hist(reliability,0:binwidth:100);
    bar(xout,100*n/runs);
    xlabel('Reliability [%]');
    ylabel('Occurence [%]');
    title('Reliability, Compared to Uniform Sampling');
    axis([0 100 0 100]);
    drawnow;
    

    figure(3);
    [n,xout]=hist(efficiency,0:binwidth:100);
    bar(xout,100*n/runs);
    xlabel('Efficiency [%]');
    ylabel('Occurence [%]');
    title('Efficiency, Compared to Uniform Sampling');
    axis([0 100 0 100]);
    drawnow;
    
    figure(4);
    [n,xout]=hist(evalcnt,0:binwidth:100);
    bar(xout,100*n/runs);
    xlabel('Function Evaluations');
    ylabel('Occurence [%]');
    title('Total Function Evaluations in Non-Uniform Sampling');
    axis([0 max(evalcnt(:)) 0 100]);
    drawnow;
    
    
    figure(5);
    [n,xout]=hist(reliability,0:binwidth:100);
    plot(xout,cumsum(100*n/runs),'k-');
    xlabel('Reliability [%]');
    ylabel('Cumulative Fraction [%]');
    title('Reliability, Compared to Uniform Sampling');
    axis([0 100 0 100]);
    drawnow;
    
    figure(6);
    [n,xout]=hist(efficiency,0:binwidth:100);
    plot(xout,cumsum(100*n/runs),'k-');
    xlabel('Efficiency [%]');
    ylabel('Cumulative Fraction [%]');
    title('Efficiency, Compared to Uniform Sampling');
    axis([0 100 0 100]);
    drawnow;

end; % if strcmp(plotmode,'on')==1
    
    
    



%%% write result to screen
if nargout==0
    disp(['MKLSZOPTIMTEST: initial intervals:  ' int2str(length(zets)-1)]);
    disp(['MKLSZOPTIMTEST: matchthresh value:  ' num2str(matchthresh)]);
    disp(['MKLSZOPTIMTEST: Median Evaluations: ' num2str(median(evalcnt))]);
    disp(['MKLSZOPTIMTEST: Median Efficiency:  ' num2str(median(efficiency)) '%']);
    disp(['MKLSZOPTIMTEST: Mean Efficiency:    ' num2str(mean(efficiency)) '%']);
    disp(['MKLSZOPTIMTEST: Std Err Efficiency: ' num2str(std(efficiency)/sqrt(length(efficiency))) '%']);
    disp(['MKLSZOPTIMTEST: Mean Reliability:   ' num2str(mean(reliability)) '%']);
    disp(['MKLSZOPTIMTEST: Median Reliability:   ' num2str(median(reliability)) '%']);
    disp(['MKLSZOPTIMTEST: Std Err Reliability: ' num2str(std(reliability)/sqrt(length(reliability))) '%']);
    toctime=toc;
    disp(['MKLSZOPTIMTEST: elapsed time: ' num2str(toctime) 's']);
end; % if nargin





%%% return results
%%% result are already in the output variables.


%%% but we want to have warnings from other contexts
warning on

