function [map,lat,lon]=mklsgapmap(slon,slat,elon,elat);
% mklsgapmap.......compute azimuthal gap for each point of surface
%
% call: [map,lat,lon]=mklsgapmap(slon,slat);
%       [map,lat,lon]=mklsgapmap(slon,slat,elon,elat);
%
%       slat: vector of station latitudes [deg N]
%       slon: vector of station longitudes [deg E]
%
%       elat: event latitudes [deg N]
%       elon: event longitudes [deg E]
%
% result: map: array containing the azimuthal gap for each point of surface
%         lat: latitudes for plotting of MAP
%         lon: longitudes for plotting of MAP
%         plot result via surf(lon,lat,map); or similar commands.
%
% The azimuthal gap is the largest angular gap between azimtuhally adjacent
% stations, measured at the point of the epicenter. The larger tha gap, the larger
% the errors in horizontal and vertical event localization
%
% longitues should be in the range 0...360
% longitudes between 180 and 360 will be corrected. others will not be corrected.
% latitudes should be in the range -90...90
% Latitudes outside this range are in the sole responsibility of the user.
%
% (This program is derived from an earlier one which used a coordinate
% convention different of the LOCSMITH conventions. sorry for the
% confusion)
%
% If only one station is given, the program terminates without plotting a
% map.
%
%
% Martin Knapmeyer, 14.08.2002, 02.06.2003, 08.03.2007, 11.12.2007



%%%  just to be sure...
slat=slat(:);
slon=slon(:);

%%% terminate if only one station is given MK08032007
if length(slat)==1
    map=[];
    lat=[];
    lon=[];
    return;
end; % if length(slat)==1



%%% correct SLON for values outsied -180...180
indies=find(slon>180);
if ~isempty(indies)
   slon(indies)=-(360-slon(indies));
end; % if ~isempty

%%% number of stations
anz=length(slat);

%%% to collect BAZ angles during scan of a point
baze=zeros(anz+1,1);


%%% prepare array for map
switch nargin
   case {2}
		dlat=5;
		dlon=5;
		szlon=fix(360/dlon);
		szlat=fix(180/dlat);
		map=zeros(szlon,szlat);
		%disp(['mklsgapmap: testing ' int2str(prod(size(map))) ' points']);
   case {4}
     % dlat=elat;
     % dlon=elon;
      szlon=length(elon);
      szlat=1;
      map=0;
   otherwise
      error('MKLSGAPMAP: wrong number of input arguments');
end; % switch nargin


%%% scan through map
if anz==1
   %%% if there is only one station, the gap is 360deg everywhere, but
   %%% my algorithm doesn't work then.
   map=map+360;
else; 
	latcnt=1;
	for lat=1:szlat
       loncnt=1;
       for lon=1:szlon
           
           %%% compute azimuths to all stations
           baze=baze*0;
           if nargin==2
              baze=real(mklsbaz(slat,slon,...
                              dlat*latcnt-90,dlon*loncnt-180));
           else
              baze=real(mklsbaz(slat,slon,...
                              elat(loncnt),elon(loncnt)));
           end; % if nargin==4
           baze=[0 baze(:)']';

                      
           %%% replace NaNs by zero
           %%% BAZ is NaN if epicenter is at station location, but a NaN
           %%% makes the gap computation impossible
           indies=find(isnan(baze));
           baze(indies)=zeros(size(indies));
           
           %%% sort stations by BAZ
           baze(2:(anz+1))=sort(baze(2:(anz+1)));
           
           %%% correct from BAZ to angles between stations
           baze(1)=(360-baze(anz+1))+baze(2);
           
           %%% find max difference
           gap=max([baze(1) abs(diff(baze(2:end)))']);
           
           %%% reduce to interval 0...360
           %%% no reduction shall be attempted for values
           %%% below 360, and 360 shall not be reduced to 0!
%            indies=find((gap>360)&((gap-fix(gap/360))~=0));
%            gap(indies)=gap(indies)-fix(gap(indies)/360)*360;
           if (gap>360)&((gap-fix(gap/360))~=0)
              gap=gap-fix(gap/360)*360;
           end; % if
           
           %%% save GAP to map
           map(loncnt,latcnt)=gap;
           
           %%% increase longitude counter
           loncnt=loncnt+1;
       end; % for lon
       latcnt=latcnt+1;
	end; % for lat
end; % if anz==1


%%% plot or display result
if (nargin==2)&(nargout==0)
   % base map and distances
%    basepath='/home/europa/mknapmey/planets/mars';
%    mapopt=mkmapopt('planet_type','uni',...
%                 'planet_radius',3389,...
%                 'topo_mode','off',...                
%                 'topo_path',[basepath '/topo/' 'mkearthtopo050.mat'],...
%                 'topo_exag',0.05,...
%                 'proj_type','hammer',...
%                 'proj_range',[-180 180 -90 90],...
%                 'frame_mode','on',...
%                 'grid_mode','on',...
%                 'grid_range',[-180 180 -90 90],...
%                 'grid_lonmesh',30,...
%                 'grid_latmesh',30,...
%                 'grid_style',':',...
%                 'grid_color','k',...
%                 'colorbar_mode','off');
%    basemap=mkmakemap(mapopt);
%    set(basemap.surf,'CData',map'); %fliplr(map));
%    set(basemap.surf,'Facecolor','texturemap');
%    set(basemap.surf,'CDatamapping','scaled');
%    %%% apply map projection to epicenters and stations
%    [epilon,epilat]=meshgrid((dlon:dlon:360)-180,(dlat:dlat:180)-90);
%    [epilon,epilat]=mklonlat2xy(epilon,epilat,mapopt,'forward');
%    [slon,slat]=mklonlat2xy(slon,slat,mapopt,'forward');
%    %%% add contours
%    maxsurf=max(max(get(basemap.surf,'ZData')));
%    hold on
%    contour3(epilon,epilat,map'+maxsurf,(0:10:180)+maxsurf,'k--');
%    hold off
%    %%% plot station network
% 	exagg=1/1000;
% 	hold on
% 	h=plot3(slon,slat,slon*0+basemap.maxz*basemap.exag*1.1,'^');
% 	hold off
% 	set(h,'MarkerSize',10);
% 	set(h,'MarkerFaceColor','w');
% 	set(h,'MarkerEdgeColor','k');
%    %%% beautification
%    %%% 125.26deg is the max azimuthal gap of a tetraeder config
%    maxmapval=max(max(map(:,2:89))); % leave out the poles
%    rgb=mkgoodbad(256,360/length(slon),125.26,maxmapval,'color');
%    colormap(rgb);
%    caxis([360/length(slon) maxmapval]);
%    %error('stopped');
%    colorbar('hor');
%    xlabel('Longitude');
%    ylabel('Latitude');
%    title(['Azimuthal Gap [deg]']);
%    %%% add light
%    light('position',[-1 1 2]);
%    material dull
%    figure(1);
   colanz=360; %360-360/anz;
   rgb=jet(colanz); %mktricolor([0 1 0],[1 1 0],[1 0 0],colanz,0.5);
   %rgb=rgb(fix(colanz/2):colanz,:);
   colormap(rgb);
   lon=(dlon:dlon:360)-180;
   lat=(dlat:dlat:180)-90;
   %contourf(lon-dlon/2,lat-dlat/2,map',0:10:360);
   surf(lon-dlon/2,lat-dlat/2,map');
   shading flat
   view(2);
   axis equal
   axis([-180 180 -90 90]);
   box on
   set(gca,'Xtick',-180:30:180);
   set(gca,'YTick',-90:30:90);
   set(gca,'Layer','Top');
   caxis([360/anz max(map(:))]);
   colorbar('hor');
   %%% add stations
   hold on
   h=plot3(slon,slat,slat*0+400,'ws');
   set(h,'MarkerSize',14);
   set(h,'MarkerFaceColor','w');
   set(h,'MarkerEdgeColor','k');
   hold off
   xlabel('Longitude');
   ylabel('Latitude');
   title('Azimuthal Gap [deg]');
%    figure(1);
else
   if nargout==0
      disp(['mklsgapmap: azimuthal gap is ' num2str(map)]);
   end; % if nargin
end; % if nargin


% %%% plot coordinates result
% lon=(dlon:dlon:360)-180;
% lat=(dlat:dlat:180)-90;

%%% return result
map=map';
if nargin==2
	%%% plot coordinates result
	lon=(dlon:dlon:360)-180;
	lat=(dlat:dlat:180)-90;

   lon=lon-dlon/2;
   lat=lat-dlat/2;
else
   lon=elon;
   lat=elat;
end; % if nargin