function [compatibility,earliestot,latestot,testcnt,incompatible]=mklsfindorigintime(lsopt,traveltimes,fxd);
% mklsfindorigintime.......find origin time compatible with computed TT
%
% call: [compatibility,earliest,latest,testcnt,incompatible]=mklsfindorigintime(lsopt,traveltimes,fxd);
%
%       lsopt: (structure)
%              LOCSMITH options structure as returned by MKLSOPT
%
%       traveltimes: (structure)
%              travel time structure as returned by MKLSEMPTYTRAVELTIMES
%              containing computed travel times for a single node and all phases
%              at all stations.
%
%       fxd: (structure)
%            Fast Acess Data structure as returned by MKLSLSD2FXD
%
%
% result: compatibility: (number) [counter]
%                        max number of compatible data
%                        The node for whicht the TRAVELTIMES were computed
%                        is compatible with no more as this number of
%                        phases.
%                        0 if no compatibility found.
%
%         earliestot: (numeric array) [sec]
%                     earliest origin time for which the node is compatible
%                     with the max number of phases.
%                     There might be several fitting time intervals. This
%                     is al list of the beginnings of all of these
%                     intervals.
%                     Empty if no compatibility found.
%
%         latestot: (numeric array) [sec]
%                   latest origin time for which the node is compatible
%                   with the max number of phases.
%                   There might be several fitting time intervals. This
%                   is al list of the endings of all of these
%                   intervals.
%                   Empty if no compatibility found.
%
%                   The i-th possible origin time interval starts at
%                   earliestot(i) and ends at latestot(i).
%
%         testcnt: (number) [counter]
%                  counter for the number of tests applied - in case of
%                  full compatibility, COMPATIBILITY equals TESTCNT.
%
%         incompatible: (structure array)
%                  In this structure, a list of incompatible data is given
%                  for each time interval. there is only one field:
%                  .names: (string matrix)
%                          a list of station identifier / phase name pairs
%                          in the syntax StationIdentifier:PhaseName.
%                          The corresponding arrival times are NOT
%                          compatible with the corresponding origin time
%                          interval.
%                 incompatible(i).names is the list for the interval
%                 defined by ealiestot(i) and latestot(i).
%
%
% This routine finds out with how many of the observed arrival times a
% certain node is compatible and for which origin times this compatibility
% holds.
%
% Martin Knapmeyer, 18.05.2005, 23.05.2005, 25.05.2005, 15.06.2005

% 05022007 comparecnt commented out


%%% init result
compatibility=0;
earliestot=[];
latestot=[];
testcnt=0;

%%% local verbosity
verbosity=4;


%%% loop over all origin times
refday=mklsconstants('ReferenceDay');

%%% prepare plotting
% figure(3);
% clf;
% title(['Compatible Origin Time Intervals']);
% xlabel(['Origin Time minus LOCSMITH Reference Time']);
% ylabel(['Comparison #']);
% box on
    
    
%%% loop over stations
% comparecnt=1; % counts comparisons carried out
timelist=[]; % array that serves as OT time axis later
mincompot=[]; % array to collect compatible origin time's beginnings
maxcompot=[]; % array to collect compatible origin time's endings
phasecompot=[]; % array to collect the stations and phases for which compatibility is possible
datalist=[]; % array to collect names for ALL existing data, needed to identify incompatible data
statcnt=length(fxd);
for statindy=1:statcnt
   
    %%% current station ID
    currentstation=fxd(statindy).id;
   
    %%% loop over phases
    phasecnt=size(fxd(statindy).phase,1); % number of phases for this station
    for phaseindy=1:phasecnt
       
        %%% current phase name
        currentphase=deblank(fxd(statindy).phase(phaseindy,:));
        
        %%% append datum to data list
        datalist=strvcat(datalist,[currentstation ':' currentphase]);
        
        %%% find data values that belong to this phase and are arrival times
        valueindy=find((fxd(statindy).value(:,2)==phaseindy)&...
                       (fxd(statindy).value(:,1)==1));
        if ~isempty(valueindy)
            %%% data values exist for current station and phase
            

           
            
            %%% find computed values for this phase and station
            %%% identifiy travel times for the current station
            thisstat=find(ismember(traveltimes.station,fxd(statindy).id,'rows')==1);
            thisphase=find(ismember(traveltimes.phase(thisstat,:),currentphase,'rows')==1);
            indies=thisstat(thisphase); % this is an index to traveltimes.tt !
%             disp(['       ' int2str(length(valueindy)) ' data values, ',...
%                   int2str(length(indies)) ' computed values']);
            mklsverbose(verbosity,lsopt,'MKLSFINDORIGINTIME: Station ',currentstation,': ',...
                        length(valueindy),' ',currentphase,...
                        ' data values, ',length(indies),' computed arrivals.');

            %%% The question now is: for which origin times does the
            %%% computed travel time fit into the observed arrival time window?
            %%% and then: for which origin times does the largest number of computed
            %%% times fit into the observations?
            
            %%% loop over data values
            for dataindy=valueindy
                
                %%% current arrival time interval
                minarrival=fxd(statindy).value(dataindy,3); % minimm arrival time of this phase
                maxarrival=fxd(statindy).value(dataindy,4); % maximum arrival time of this phase
                
                
                %%% loop over computed travel times
                for comptdindy=1:length(indies)
                    
                    %%% determine compatible origin times
                    currenttt=traveltimes.tt(indies(comptdindy)); % current travel time
                    mincompot=[mincompot minarrival-currenttt]; % earliest compatible origin time
                    maxcompot=[maxcompot maxarrival-currenttt]; % latest compatible origin time
                    timelist=[timelist NaN minarrival-currenttt maxarrival-currenttt];
                    phasecompot=strvcat(phasecompot,[currentstation ':' currentphase]); % station/phase of this interval
                    
%                     disp([currentstation ':' currentphase ' ',...
%                           num2str(currenttt) ' ',...
%                           num2str(minarrival-currenttt) ' ',...
%                           num2str(maxarrival-currenttt) ' ',...
%                           num2str(mean([minarrival maxarrival]))]);
%                     
                    %%% plot compatible origin time interval
%                     hold on
%                     plot([mincompot(comparecnt) maxcompot(comparecnt)],[1 1]*comparecnt,'.-');
%                     text(maxcompot(comparecnt),comparecnt,[' ' deblank(phasecompot(comparecnt,:))]);
%                     hold off
                    
%                     %%% incremet tests counter
%                     comparecnt=comparecnt+1;
                    
                end; % for comptdindy
                
                
                %%% increment counter for tests carried out
                testcnt=testcnt+1;
                
                
            end; % for dataindy
        else
            %%% no data for this station/phase combination!
            mklsverbose(verbosity,lsopt,'MKLSFINDORIGINTIME: Station ',currentstation,...
                        ': no ', currentphase,' data');
                    
            %%% dummy origin times
            mincompot=[mincompot NaN];
            maxcompot=[maxcompot NaN];
            timelist=[timelist NaN NaN NaN];
            phasecompot=strvcat(phasecompot,[currentstation ':' currentphase]);
            
%             %%% increment tests counter
%             comparecnt=comparecnt+1;
                    
            %%% incremet tests counter
            testcnt=testcnt+1;
        end; % if ~isempty(valueindy)
          
    end; % for phaseindy
    
end; % for statindy

%%% some result info
mklsverbose(verbosity,lsopt,...
            'MKLSFINDORIGINTIME: node #',traveltimes.nodeid(1),...
            ' tested against ',testcnt,' data.');


%%% now we have a list of intervals, each of which denotes an origin time
%%% interval compatible with a single phase at a single station.
%%% beginnings of these intervals are stored in MINCOMPOT
%%% endings of these intervals are stored in MAXCOMPOT,
%%% station/phase associations in PHASECOMPOT.
%%% Now we have to find out which origin time interval for which the number
%%% of different station/phase combinations is max.
%%% And we have to find out which data are not compatible (by name, to 
%%% enable the user to check the data!)
intervalcnt=length(mincompot); % number of intervals determined
earliest=min(mincompot); % earliest compatible origin time
latest=max(maxcompot); % latest compatible origin time

%%% array to collect compatibility measures for each time
compatible=zeros(size(timelist)); 
%%% structure to collect data item names for each time
acceptphase=repmat(struct('names',''),size(timelist),1);

%%% loop over all allowed origin times
for indy=1:length(timelist)
    %%% current candidate time
    candidateot=timelist(indy);
    %%% identify intervals compatible with the current candidate time
    intervalindy=find((mincompot<=candidateot)&...
                      (maxcompot>=candidateot));
    if ~isempty(intervalindy)
        namelist=phasecompot(intervalindy,:);
        if size(namelist,1)>1
            %%% list of station/phase combinations at this candidate time
            %%% contains more than one entry - but are these unique?
            namelist=unique(namelist,'rows');
            compatible(indy)=size(namelist,1);
            acceptphase(indy).names=namelist;
        else
            compatible(indy)=1;
            acceptphase(indy).names=namelist;
        end; % if size(namelist,1)>1
    else
        compatible(indy)=0;
    end; % if ~isemptty(intervalindy)
    
end; % for testot

% hold on
% plot(timelist,compatibility,'k-');
% hold off

%%% now, in which time interval does the compatibility reach its max?
compatibility=max(compatible);
%%% decompose besttimes into separate intervals - note that ther may be
%%% more than one
indies=find(compatible~=compatibility);
compatible(indies)=compatible(indies)*0;
%%% compatible is now nonzero only for time intervals with max compatiblity
timelist(indies)=timelist(indies)*NaN; % only times for max compatibility remain
nonnan=find(~isnan(timelist));
%timelist=timelist(nonnan);
earliestot=timelist(nonnan(1:2:end));
latestot=timelist(nonnan(2:2:end));
acceptphase=acceptphase(nonnan(1:2:end));

%%% write list of compatible data for each interval to screen
% for indy=1:length(nonnan)
%     outline='';
%     for j=1:compatibility
%         outline=[outline ', ' deblank(acceptphase(nonnan(indy)).names(j,:))];
%     end; % for j
%     disp(['MKLSFINDORIGINTIME: ' int2str(compatibility) ' compatible arrivals: ' outline]);
% end; % for indy

% disp('===================================');

%%% "invert" the list of compatible data given in ACCEPTPHASE into a list of
%%% incompatible data. To do so, we compare the content of ACCEPTPHASE with
%%% the DATALIST - those elements of DATALIST that are _NOT_ contained in
%%% ACCEPTPHASE are possibly incompatible.
incompatible=repmat(struct('names',''),length(acceptphase),1);
for indy=1:length(acceptphase)
    members=ismember(datalist,acceptphase(indy).names,'rows');
    nonmembers=find(members==0);
    incompatible(indy).names=datalist(nonmembers,:);
    
    
%     if (compatibility>3)&(length(nonmembers)==length(datalist)-compatibility)
%         outline='';
%         for j=1:length(nonmembers)
%             outline=[outline ' ' deblank(incompatible(indy).names(j,:))];
%         end; % for j
%         disp(['MKLSFINDORIGINTIME: ' int2str(length(nonmembers)) ' incompatible arrivals: ' outline]);
%     end; % if length(nonmembers)==length(datalist)-compatibility
    
end; % for indy


% hold on
% plot(earliestot,earliestot*0+compatibility,'r>');
% plot(latestot,latestot*0+compatibility,'r<');
% hold off


%%% return results
%%% compatible
%%% earliestot
%%% latestot
%%% testcnt
%%% are already in the returned variables.
return;



