function [exitstatus,mygrid,ttsupport]=mklsgrideval(lsopt,mygrid,simpledata,cumdatacnt,ttsupport);
% mklsgrideval..........iterates the current grid until convergence
%
% call: [exitstatus,mygrid,ttsupport]=mklsgrideval(lsopt,mygrid,simpledata,cumdatacnt,ttsupport);
%
%           lsopt: (structure)
%                  LOCSMITH options structure as returned by MKLSOPT
%
%           mygrid: (structure)
%                   LOCSMITH search grid
%
%           simpledata: (strcuture)
%                   LSD structure containing seismic arrival data and
%                   station data.
%
%           cumdatacnt: (numeric array)
%                   The IDs of all nodes of MYGRID that were evaluated for less
%                   or equal CUMDATACNT data values.
%
%           ttsupport: (structure)
%                   TTSUPPORT structure as needed by the used travel time
%                   software package.
%
% result:
% 
%         exitstatus: (string)
%                  a string denoting the reason why MKLOCSMITH came to a
%                  halt. This string should be considered when evaluating
%                  the output. The follwiong values are possible:
%
%                  'unexpected': the exit status is initialized with this
%                                value. At any legal exit condition, a new
%                                value should be written into the string.
%                                If this is not the case, the init value
%                                remains, because the stopping of the
%                                routine was indeed due to an unexpected
%                                reason. This should never occur.
%
%                  'no data':    There is no data to evaluate. No grid is
%                                generated and no location produced.
%
%                  'horizontally converged':  The desired spatial grid resolution for the
%                                location has been reached. The routine
%                                quits because the job is done.
%
%                  'horizontal iteration timeout': 
%                                The maximum number of iterations to 
%                                determine the horizontal extent of the 
%                                location has been reached. 
%                                This limit is defined in lsopt.iter_max.
%                                If this occurs, the job is NOT
%                                done and you should probably repeat the
%                                location with a higher iter_max limit.
%                                lsopt.iter_max is intended to be an
%                                emergency brake to avoid infinite
%                                iteration, not a standard tool to finish
%                                the program!
%
%         mygrid: (structure array)
%                 The search grid as it is at the end of the location
%                 process. This is a LOCSMITH search grid structure as
%                 returned by MKLSGRIDNEW.
%
%         ttsupport: (structure)
%                  TTSUPPORT structure. This structure might be as returned
%                  by MKLSEMPTYTTSUPPORT, i.e. unspecific, or with fields
%                  that are specific for the used travel time software.
%
%
% This routine evaluates the grid as given in MYGRID. The individual layers
% are iterated until convergence or timeout, but no new layers are
% generated. This is location on a fixed vertical sampling.
%
% Martin Knapmeyer, 04.07.2005, 05.02.2007

% 05022007 save everything after each horizontal iteration


%%% init result
exitstatus='unexpected';



%%% local verbosity level
verbosity=1;

%%% say hello
mklsverbose(verbosity,lsopt,'MKLSGRIDEVAL: starting at ',mkclock);


%%% main iteration loop
%%% the following loop ends if the maximum number of iterations is reached,
%%% or when MKLSGRIDMODIFY refuses to modify the grid any further.
done=0;
itercnt=0;
while done==0
    
    %%% say hello, iteration
    mklsverbose(verbosity,lsopt,'MKLSGRIDEVAL: horizontal iteration ',itercnt);


    %%% a dummy
    idlist=NaN;
    iterations=1;
    
    
    %%% get all nodes which need to be evaluated
    evalthese=mklsgridnonready(lsopt,mygrid,cumdatacnt);



    %%% evaluate grid for compliance with BAZ
    mygrid=mklsgridevalbaz(lsopt,mygrid,simpledata,evalthese);

    %%% evaluate grid for compliance with arrival times
    [mygrid,ttsupport]=mklsgridevalat(lsopt,mygrid,simpledata,evalthese,ttsupport);

    %%% evaluate grid for compliance with differential travel times
    [mygrid,ttsupport]=mklsgridevaldt(lsopt,mygrid,simpledata,evalthese,ttsupport);
    
    
    
    %%% control plot of current grid and its compatibilities (not recommended for large grids...)
%     mklsgridcontrolplot(lsopt,mygrid,simpledata);
    gridplotmode=lsopt.verbosity_plot_mode;
    if strcmp(gridplotmode,'none')==0
        figure(lsopt.verbosity_plot_fig);
        cla;
        %%% creates base map
        mkmakemap(lsopt.mapopt);
        %%% overlay grid nodes
        hold on
        mklsgridplot(lsopt,mygrid,gridplotmode); 
        hold off
        %%% overlay stations
        % needs a seisdat structure which is not available here...
        %hold on
        %mklsnetplot(lsopt,seisdat,gridplotmode);
        %hold off
        drawnow;
    end; % if strcmp(gridplotmode,'none')==0
    
    
    
%     figure(3);
%     clf;
%     gridplotmode='fitsurface';
%     mklsgridplot(lsopt,mygrid,gridplotmode);
%     alpha(1);
%     view(30,-45);
%     mkfiatlux('off');
%     mkfiatlux('on','position',[1 -1 -1]);
%     lighting phong
%     set(gca,'CameraViewAngle',6.7);
%     caxis([0 12]);
%     drawnow;
    
    
    
    %%% identify triangles that need to be iterated: those where .fit
    %%% matches .tested, i.e. those whose nodes match all data.
    idlist=mklsgridfindmatches(lsopt,mygrid);

    %%% iterate grid
    [mygrid,modifycnt]=mklsgridmodify(lsopt,mygrid,idlist,'iterate',iterations);
        
    %%% increment iteration counter
    itercnt=itercnt+1;
    
    
    %%% save everything MK05022007
    save ./mklsgrideval_save.mat mygrid lsopt simpledata cumdatacnt ttsupport
    
    
    
    %%% has the grid been modified at all?
    if (modifycnt==0)&&(done==0)
        %%% grid has not been modified - are we ready?
        %%% MKLSGRIDMODIFY does not modify the grid any further, but
        %%% this concerns only the horizontal position of nodes. 
            
        %%% exit condition checked only of no earlier exit condition
        %%% already applied - otherwise the true exit status could be
        %%% overwritten!

        exitstatus='horizontally converged';
        done=1;
        mklsverbose(verbosity,lsopt,'MKLSGRIDEVAL: MKLSGRIDMODIFY does not modify grid any further. Quit iteration loop. (Exit 1)');
         
       
    end; % if modifycnt
    
    
    %%% check if timeout reached
    if (itercnt>lsopt.iter_max)&(done==0)
        %%% TIMEOUT!
        %%% exit condition checked only of no earlier exit condition
        %%% already applied - otherwise the true exit status could be
        %%% overwritten!
        exitstatus='horizontal iteration timeout';
        done=1;
        mklsverbose(verbosity,lsopt,'MKLSGRIDEVAL: Max number of iterations applied. Quit iteration loop. (Exit 2)');
    end; % if itercnt>lsopt.maxiter
    
    
end; % while


%%% echo horizontal exit status
mklsverbose(verbosity,lsopt,'MKLSGRIDEVAL: horizontal exit status: ',exitstatus);


%%% say goodbye
mklsverbose(verbosity,lsopt,'MKLSGRIDEVAL: finished at ',mkclock);
