function [newgrid,newttsupport]=mklsgridevalattri(lsopt,mygrid,fxd,evalthese,ttsupport);
% mklsgridevalattri........evaluate arrival times of nodes in tri grids
%
% call: [newgrid,newttsupport]=mklsgridevalattri(lsopt,mygrid,simpledata,evalthese,ttsupport);
%
%       lsopt: (structure)
%              LSOPT structure as returned by MKLSOPT
%
%       mygrid: (structure)
%              grid structure as returned by MKLSGRIDNEW
%
%       fxd: (strcuture)
%                   FXD structure as returned by MKLSLS2FXD, containing all
%                   data.
%
%       evalthese: (structure)
%              structure denoting grid nodes to be evaluated, as returned
%              by MKLSGRIDNONREADY.
%              The following fields are expected:
%              .layer: (numeric)
%                      layer ID, only one per element of EVALTHESE
%              .nodes: (numeric array)
%                      list of nodes. This is a sublist of the
%                      nodes list in the mygrid.layer field,
%                      contining node IDs and coordinates formatted
%                      as in the layer structure.
%              .listpos: (numeric array)
%                      list of positions of the nodes in the
%                      original .nodes field of the input layer.
%                      evalthese.listpos(i) is the array indes of
%                      the node evalthese.nodes(i,:) wihtin the
%                      original layer structure.
%
%
%       ttsupport: (structure)
%                  TTSUPPORT structure. This structure might be as returned
%                  by MKLSEMPTYTTSUPPORT, i.e. unspecific, or with fields
%                  that are specific for the used travel time software.
%
%
% result: newgrid: (structure)
%                  as MYGRID, but with the tested-counters and fit-field actualized.
%
%
%         newttsupport: (structure)
%                  as TTSUPPORT, but with actualized contents.
%
%
%
% This routine computes the arrival times of nodes listed in EVALTHESE and
% finds out which nodes fit into the desired arrival time ranges of all stations.
% the .fit and .tested fields of the grid structure are updated
% accordingly.
%
% Searching for compatibility with arrival times is more complicated than
% searching for back azimuth compatibility: not only the 3D spatial
% coordinates have to be considered, but also the origin time coordinate.
% Travel Times have to be computed and then possible origin times have to
% be found.
% Fortunately, the modification of the origin time can be done very
% quickly, so that the search for the origin time can be done at very low
% computational cost: compute travel times and just add the trial origin
% time.
% Travel time computation itself is done by some external software.
% LOCSMITH does not contain any Travel time computation routines, but only
% a generic interface. Many of the work to be done here is in that
% interface routine.
%
% Martin Knapmeyer, 31.03.2005, 17.05.2005, 05.07.2005, 20.03.2007,
%                   22.03.2007, 02.04.2008

%%% 02042008: incompatibiliy messages only if number of incompatibles >0



%%% init result
newgrid=mygrid;
atidlist=NaN;

%%% verbosity
verbosity=3;


%%% compute travel times from all nodes specified in EVALTHESE to all stations
%%% Note that there might be more than one arrival for on phase (triplications!)
[traveltimes,newttsupport]=mklstttri(lsopt,mygrid,fxd,evalthese,ttsupport);


%%% now we have to find out which nodes are possible hypocenters. I don't
%%% know how, but somehow...
%%% OK. We have a list of layers with nodes (MYGRID), and we have a list of travel
%%% times for these nodes (TRAVELTIMES). And we have a range of possible
%%% origin times (LSOPT).
%%% so we have to loop over all layers for which new times are computed and
%%% then loop through the node list of TRAVELTIMES and see for which origin
%%% times the computed travel times are valid arrival times.
evaldlayercnt=length(traveltimes); % number fo layers for which new evaluations were made
layercnt=length(mygrid.layers); % number of layers in grid


%%% generate ID list of all layer with new evaluations
evaldlayers=zeros(evaldlayercnt,1);
for indy=1:evaldlayercnt
    evaldlayers(indy)=traveltimes(indy).layerid;
end; % for indy

%%% determine current maximum fitness of the whole grid
%currentmaxfit=max(mygrid.layers(layerindy).fit(:,2)); % current max compatibility in this layer
currentmaxfit=-inf;
fitlist=zeros(layercnt,1);
for layerindy=1:layercnt
    fitlist(layerindy)=max(mygrid.layers(layerindy).fit(:,2));
end; % for layerindy
currentmaxfit=max(fitlist);


%%% loop over all layers in grid
for layerindy=1:layercnt
    
    %%% say hello to this layer
    mklsverbose(verbosity-1,lsopt,'MKLSGRIDEVALATTRI: ',...
                mkclock,' ',...
                ' evaluating layer #',layerindy,...
                ' at R=',mygrid.layers(layerindy).radius,'km.');
    
    %%% is this layer's ID in the list of new evaluations?
    ttindy=find(evaldlayers==mygrid.layers(layerindy).id);
    if ~isempty(ttindy)
        %%% we have new travel times for this layer!
        
        
        %%% now loop over all nodes and compare all travel times of each
        %%% individual node with the observed arrivals and the allowed
        %%% range of origin times.
        nodecnt=size(mygrid.layers(layerindy).fit,1);
        for nodeindy=1:nodecnt
            
            %%% travel times for the current node are at which positions in list?
            ttlistpos=find(traveltimes(ttindy).nodeid==mygrid.layers(layerindy).fit(nodeindy,1));
            if ~isempty(ttlistpos)
                
                %%% from multiple positions we need only one MK20032007
                %%% NO! WE NEED THEM ALL!!! MK22032007
                %ttlistpos=ttlistpos(1);
                
                %%% new travel times exist!
                mklsverbose(verbosity+1,lsopt,'MKLSGRIDEVALATTRI: ',...
                            'layer ID',mygrid.layers(layerindy).id,': ',...
                            length(ttlistpos),' times for node ID ',...
                            mygrid.layers(layerindy).fit(nodeindy,1),...
                            ', current fitness = ',mygrid.layers(layerindy).fit(nodeindy,2),...
                            ', current testlevel = ',mygrid.layers(layerindy).tested(nodeindy,2));
                        
                %%% now we have to find out for which origin times the
                %%% travel times yield acceptable arrival times...
                %%% NOTE THAT the entries denoted by ttlistpos are the
                %%% results for all stations, and that triplications may
                %%% cause multiple arrivals. ttlistpos might contain a
                %%% quite large number of entries that have to be handled
                %%% somehow!!
                %%% ALSO NOTE THAT the traveltimes might be for different
                %%% phases!
                
                %%% At this point, we have to find out to how many of the
                %%% data the node is compatible and for which origin times.
                %%% We do this in a subroutine.
                nodetimes=mklsemptytraveltimes(1);
                nodetimes.layerid=traveltimes(ttindy).layerid;
                nodetimes.nodeid=traveltimes(ttindy).nodeid(ttlistpos);
                nodetimes.phase=deblank(traveltimes(ttindy).phase(ttlistpos,:));
                nodetimes.station=deblank(traveltimes(ttindy).station(ttlistpos,:));
                nodetimes.d=traveltimes(ttindy).d(ttlistpos);
                nodetimes.tt=traveltimes(ttindy).tt(ttlistpos);
           %disp(['node nr.' int2str(nodeindy) ', ID ' num2str(nodetimes.nodeid(end))]);
                [compatibility,earliestot,latestot,testcnt,incompatible]=mklsfindorigintime(lsopt,nodetimes,fxd);
                
                %%% update current max compatibility
                currentmaxfit=max(currentmaxfit,compatibility);
                
                
                %%% write list of incompatible data to screen
                %%% but only in cases where compatibility is at least as high as
                %%% the current max, since nobody is interested in nodes that turn
                %%% out to be poor hypocenter candidates.
                if compatibility>=currentmaxfit
                    for indy=1:length(incompatible)
                        namesanz=size(incompatible(indy).names,1);
                        outline='';
                        for j=1:namesanz
                            outline=[outline ' ' deblank(incompatible(indy).names(j,:))];
                        end; % for j
                        if namesanz>0
                           mklsverbose(verbosity,lsopt,'MKLSGRIDEVALATTRI: ',...
                                         'R=',mygrid.layers(layerindy).radius,'km',...
                                         ', node ',nodetimes.nodeid(1),...
                                         ': ',namesanz,' incomp. AT: ',...
                                         ' OT ',indy,' of ',length(incompatible),...
                                         ': ',outline);
                        end; % if namesanz
                    end; % for indy
                end; % if compatibility>=currentmaxfit
        

                
                %%% update fit field corresponding to the current node
                newgrid.layers(layerindy).fit(nodeindy,2)=newgrid.layers(layerindy).fit(nodeindy,2)+compatibility;
                
                %%% update test counter for the current node
                newgrid.layers(layerindy).tested(nodeindy,2)=newgrid.layers(layerindy).tested(nodeindy,2)+testcnt;
                
                %%% store compatible origin time values in grid
                newgrid.layers(layerindy).times(nodeindy).id=newgrid.layers(layerindy).fit(nodeindy,1);
                newgrid.layers(layerindy).times(nodeindy).starts=[newgrid.layers(layerindy).times(nodeindy).starts earliestot];
                newgrid.layers(layerindy).times(nodeindy).ends=[newgrid.layers(layerindy).times(nodeindy).ends latestot];
                
            else
                %%% no new travel times for this node
                %%% nothing happens
            end; % if ~isempty(ttlistpos)
            
        end; % for nodeindy
        
    else
        %%% no new travel times for this layer
        %%% nothing happens
    end; % if ~isempty(ttindy)
end; % for layerindy

return;

