function newgrid=mklsgridevalbaztri(lsopt,mygrid,fxd,evalthese);
% mklsgridevalbaztri......evaluate back azimuths of nodes in tri grids
%
% call: newgrid=mklsgridevalbaztri(lsopt,mygrid,simpledata,evalthese);
%
%       lsopt: (structure)
%              LSOPT structure as returned by MKLSOPT
%
%       mygrid: (structure)
%              grid structure as returned by MKLSGRIDNEW
%
%       fxd: (strcuture)
%                   FXD structure as returned by MKLSLS2FXD, containing all
%                   data.
%
%       evalthese: (structure)
%              structure denoting grid nodes to be evaluated, as returned
%              by MKLSGRIDNONREADY.
%              The following fields are expected:
%              .layer: (numeric)
%                      layer ID, only one per element of EVALTHESE
%              .nodes: (numeric array)
%                      list of nodes. This is a sublist of the
%                      nodes list in the mygrid.layer field,
%                      contining node IDs and coordinates formatted
%                      as in the layer structure.
%              .listpos: (numeric array)
%                      list of positions of the nodes in the
%                      original .nodes field of the input layer.
%                      evalthese.listpos(i) is the array indes of
%                      the node evalthese.nodes(i,:) wihtin the
%                      original layer structure.
%
% result: newgrid: (structure)
%                  as MYGRID, but with the tested-counters and fit-field actualized.
%
%
% This routine computes the back azimuths of nodes listed in EVALTHESE and
% finds out which nodes fit into the desired BAZ ranges of all stations.
% the .fit and .tested fields of the grid structure are updated
% accordingly.
%
% Martin Knapmeyer, 23.02.2005, 28.02.2005, 16.03.2005


%disp('mklsgridevalbaztri kann hier direkt getestet werden, ob daten inkompatibel sind, d.h. die hinzufuegung');
%disp('von daten nicht das maximum von .fit erhoehen?');


%%% init result
newgrid=mygrid;
bazidlist=NaN;

%%% verbosity
verbosity=4;

%%% extract layer ID list from EVALTHESE
layercnt=length(evalthese);
layerlist=zeros(1,layercnt);
for indy=1:layercnt
    layerlist(indy)=evalthese(indy).layer;
end; % for indy


%%% loop over stations
statcnt=length(fxd); % number of stations
for statindy=1:statcnt
    %%% loop over layers listed in EVALTHESE
    layercnt=length(mygrid.layers);
    for layerindy=1:layercnt
        
        
        %%% do we have to analyze this layer?
        layerpos=find(mygrid.layers(layerindy).id==layerlist);
        if ~isempty(layerpos)
            %%% layer is in the EVALTHESE list
            
            %%% temporary result ID list
            idlist.layer=newgrid.layers(layerindy).id;
            idlist.triangle=[];
        
            %%% compute BAZ from current station to all nodes
            [baz,dist]=mklsbazv(evalthese(layerpos).nodes(:,2),evalthese(layerpos).nodes(:,3),...
                                fxd(statindy).slon,fxd(statindy).slat);
%             mkbaztestplot(evalthese(layerpos).nodes(:,3),evalthese(layerpos).nodes(:,2),...
%                           fxd(statindy).slat,fxd(statindy).slon,...
%                           baz,dist);
                            
            %%% loop over all BAZ data for the current station
            bazdata=find(fxd(statindy).value(:,1)==2); % all entries in .value that are BAZ
            maxfit=max(newgrid.layers(layerindy).fit(:,2)); % max fit quality before appliying data
            for datacnt=1:length(bazdata)

                %%% identify nodes that have BAZ in range of current BAZ value
                acceptindy=find(...
                                (baz>=fxd(statindy).value(bazdata(datacnt),3))&... % larger/equal min value
                                (baz<=fxd(statindy).value(bazdata(datacnt),4))... % smaller/equal max value
                               );

                if ~isempty(acceptindy)
                   %%% acceptindy points to those entries in the baz list
                   %%% that are in desired range. the fit fields for the
                   %%% corresponding nodes need to be updated.
                   
                   %%% actualize fit fields
                   indy4layer=evalthese(layerpos).listpos(acceptindy);
                   newgrid.layers(layerindy).fit(indy4layer,2)=newgrid.layers(layerindy).fit(indy4layer,2)+1;
                   mklsverbose(verbosity,lsopt,'MKLSGRIDEVALBAZTRI: layer #',...
                               layerindy,': found ',...
                               length(indy4layer),...
                               ' nodes that fit BAZ to station #',statindy);
                else
                    indy4layer=[];
                end; % if ~isempty
                
                
                %%% actualize tested-counter for ALL tested nodes
                whichtested=evalthese(layerpos).listpos;
                newgrid.layers(layerindy).tested(whichtested,2)=newgrid.layers(layerindy).tested(whichtested,2)+1;
                
                %%% test if maximum fit is larger than maximum number of tests
                %%% incompatible data ("outliers") do not increase the max
                %%% fitness of the grid!
                maxtested=max(newgrid.layers(layerindy).tested(:,2));
                if maxfit<maxtested
                    mklsverbose(verbosity,lsopt,...
                          'MKLSGRIDEVALBAZTRI: possible outlier data: layer: #',...
                          layerindy,' (id=', num2str(newgrid.layers(layerindy).id),...
                          ' r=',num2str(newgrid.layers(layerindy).radius),'km)',...
                          ', datum: #',datacnt,...
                          ' Station ',fxd(statindy).id,...
                          ' Phase ',fxd(statindy).phase,...
                          ' Type ',fxd(statindy).value(1));
                end; % if maxfit
            
            end; % for datacnt
            %%% looped over BAZ values of all nodes in EVALTHESE
            
 

        end; % if ~isempty(layerpos)

        
    end; % for layerindy
    
end; % for statindy


%         %%% at this place, bazidlist ma contain multiple entries for each layers,
%         %%% since each layer is tested multiple times (once for each station!)
%         %%% Therefore we need to make bazidlist unique, but this is done outside,
%         %%% since the combination of idlists from different tests also requires a
%         %%% unification step.


return; 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function mkbaztestplot(lat,lon,slat,slon,baz);
% mkbaztestplot.......plot nodes with BAZ as color info
% just a quick-and-dirty plot routine for testing

figure(gcf);
clf;
minbaz=min(baz);
rgb=jet(fix(abs(min(baz)-max(baz)))+1);

%%% plot station
plot(slon,slat,'k^');
axis([0 360 -90 90]);
%%% plot nodes
hold on
for indy=1:length(lat)
    h=plot(lon(indy),lat(indy),'.');
    set(h,'MarkerFaceColor',rgb(fix(baz(indy)-minbaz)+1,:));
    set(h,'MarkerEdgeColor',rgb(fix(baz(indy)-minbaz)+1,:));
end; % for indy
h=plot(slon,slat,'ko');
set(h,'MarkerFaceColor','k');
hold off

caxis([minbaz max(baz)]);
colormap(rgb);
colorbar('hor');
axis equal
axis([0 360 -90 90]);

return;
