function fxd=mklslsd2fxd(lsd);
% mklslsd2fxd.......convert LSD structure into FXD structure
%
% call: fxd=mklslsd2fxd(lsd);
%
%       lsd: (structure)
%            LSD structure as returned by MKLSREADLSD
%
% result: fxd: (structure array)
%
%               fxd(i) is the data for the i-th station:
%
%               .id:   (string)
%                      station identifier string
%               .slat: (number) [deg]
%                      station latitude
%               .slon: (number) [deg]
%                      station longitude
%               .selv: (number) [km]
%                      station elevation above reference sphere
%
%               .phase: (string matrix)
%                       seismic phase name
%                       .phase(j,:) is the seismic phase for which the j-th
%                       datum is valid
%
%               .event: (string)
%                       ID of event to which data is associated.
%                       The FXD structure can handle ony one event at a
%                       time. If the LSD structure refers to more than one
%                       event, the conversion is terminated with an error!
%
%
%               .value: (numeric array) [unit type dependent]
%                       value is an n-by-4 array in which all data is compiled. It
%                       contains a data type label, a pointer to the phase
%                       name list and the data itself, which is to be
%                       interpreted according to the data type label.
%                       .value(j,1): data type label. The following labels
%                                    are defined:
%                                    1: arrival time
%                                    2: back azimuth
%                                    3: emergence angle
%                                    4: slowness
%                       .value(j,2): phase name pointer
%                       .value(j,3): minimum data value
%                       .value(j,4): maximum data value
%
%                       The data is represented as a numeric interval, i.e.
%                       as a smallest possible value and a largest possible
%                       value. The physical units depend on the data type:
%                       arrival times: seconds
%                       back azimuths: degrees
%                       emegrence angles: degrees
%                       slowness: seconds per degree
%
%               .atcnt: (number)
%                       number of arrival time data for this station
%
%               .bazcnt: (number)
%                       number of back azimuth data for this station
%
%               .emergcnt: (number)
%                       number of emergence angle data for this station
%
%               .slocnt: (number)
%                       number of slowness data for this station
%               
%
%
% FXD (Fast Access Data) format is a structure variable format that allows
% a faster access to station specific data than LSD structures, but is
% probably more confusing for humans and therefore not recommended as input
% or exchange format.
% The data type counters show for which type of data the grid needs to be
% evaluated.
%
% Martin Knapmeyer, 04.02.2005



%%% how many referenceable events in input?
evanz=size(lsd.event); % do not trust the event counter!
if prod(double(evanz~=1))
    error('MKLSLSD2FXD: there must be exactly one event defined in LSD input!');
end; % if prod(evanz)


%%% create an empty FXD structure
fxd=mklsemptyfxd(lsd.statcnt);


%%% copy station and event data into FXD
for indy=1:lsd.statcnt
    %%% station data
    fxd(indy).id=lsd.station(indy).id;
    fxd(indy).slat=lsd.station(indy).lat;
    fxd(indy).slon=lsd.station(indy).lon;
    fxd(indy).selv=lsd.station(indy).elev;
    
    %%% transform station longitude from -180...180 range to 0...360 range
    %%% if necessary (i.e. if negative)
%     if fxd(indy).slon<0
%         fxd(indy).slon=360+fxd(indy).slon;
%     end; % if 
    fxd(indy).slon=mklslon180to360(fxd(indy).slon);
    
    %%% copy event ID into FXD
    %%% at this point, we know that there is only one event data set!
    fxd(indy).event=lsd.event.id;
end; % for indy




%%% copy arrival data into FXD
for indy=1:lsd.arrivalcnt
    
    %%% determine index of station in FXD
    fxdindex=mklsfxdgetstat(fxd,lsd.arrival(indy).station);
    
    if isempty(fxdindex)
       error(['MKLSLSD2FXD: no station information for station ' upper(lsd.arrival(indy).station)]);
    end; % 
        
    if strcmp(lsd.arrival(indy).event,fxd(fxdindex).event)==1
        %%% arrival data refers to the event in lsd.event, proceed
        
        %%% determinme index of phase in FXD phase list
        phaseindex=mklsfxdgetphase(fxd(fxdindex),lsd.arrival(indy).phase);
        if isempty(phaseindex)
            %%% empty means: we're referring to a phase which is not yet in
            %%% the phase list. so we append it.
            fxd(fxdindex).phase=strvcat(fxd(fxdindex).phase,lsd.arrival(indy).phase);
            phaseindex=size(fxd(fxdindex).phase,1);
        end; % if isempty(phaseindex)
        
        
        %%% copy arrival time data
        if sum(isnan(lsd.arrival(indy).at))<2
            %%% This condition means: at least two non-NaN values
            fxd(fxdindex).value=[fxd(fxdindex).value;...
                                 1,... % type code for arrival time
                                 phaseindex,... % phase name pointer
                                 min(lsd.arrival(indy).at),...
                                 max(lsd.arrival(indy).at)];
        end; % if sum(isnan(lsd.arrival(indy).at))==0
        
        %%% copy azimuth data
        if sum(isnan(lsd.arrival(indy).baz))<2
            %%% This condition means: at least two non-NaN values
            fxd(fxdindex).value=[fxd(fxdindex).value;...
                                 2,... % type code for back azimuth
                                 phaseindex,... % phase name pointer
                                 min(lsd.arrival(indy).baz),...
                                 max(lsd.arrival(indy).baz)];
        end; % if sum(isnan(lsd.arrival(indy).baz))==0
        
        %%% copy emergence angle data
        if sum(isnan(lsd.arrival(indy).emerg))<2
            %%% This condition means: at least two non-NaN values
            fxd(fxdindex).value=[fxd(fxdindex).value;...
                                 3,... % type code for emergence angle
                                 phaseindex,... % phase name pointer
                                 min(lsd.arrival(indy).emerg),...
                                 max(lsd.arrival(indy).emerg)];
        end; % if sum(isnan(lsd.arrival(indy).emerg))==0
        
        %%% copy slowness data
        if sum(isnan(lsd.arrival(indy).slo))==0
            %%% This condition means: three non-NaN values present
            fxd(fxdindex).value=[fxd(fxdindex).value;...
                                 4,... % type code for slowness
                                 phaseindex,... % phase name pointer
                                 min(lsd.arrival(indy).slo),...
                                 max(lsd.arrival(indy).slo)];
        end; % if sum(isnan(lsd.arrival(indy).slo))==0
        
        
        
        
    else
        %%% arrival data refers to the wrong element
        error(['MKLSLSD2FXD: arrival data refers to event ''',...
               lsd.arrival(indy).event,...
               ''' instead of ''',...
               fxd(indy).event ''' !']);
    end; % if strcmp()==1
end; % for indy


%%% deterine how many data of each type are present
for indy=1:length(fxd)
    
    indies=find(fxd(indy).value(:,1)==1);
    fxd(indy).atcnt=fxd(indy).atcnt+length(indies);
    
    indies=find(fxd(indy).value(:,1)==2);
    fxd(indy).bazcnt=fxd(indy).bazcnt+length(indies);
    
    indies=find(fxd(indy).value(:,1)==3);
    fxd(indy).emergcnt=fxd(indy).emergcnt+length(indies);
    
    indies=find(fxd(indy).value(:,1)==4);
    fxd(indy).slocnt=fxd(indy).slocnt+length(indies);
    
end; % for indy



return;

