function loc=mklsgridextractloctri(lsopt,mygrid,level);
% mklsgridextractloctri........extract best fitting nodes from triangular grid
%
% call: loc=mklsgridextractloctri(lsopt,mygrid,level);
%
%               lsopt: (struct)
%                      LOCSMITH options structure as returned by MKLSOPT
%              mygrid: (struct)
%                      LOCSMITH search grid as returned by MKGRIDNEW
%               level: (number) [counter]
%                      All nodes are extracted that are compatible with
%                      at least LEVEL data.
%                      If LEVEL is the empty matrix [], LEVEL is set to the
%                      maximum achieved compatibility internally.
%
%
% result: loc: (structure array)
%
%              structure containing the nodes with maximum compatibility
%              with the evaluated data. The structure contains the
%              following fields:
%               .type: (string)
%                      grid type (from mygrid.type)
%             .maxfit: (number) [counter]
%                       the degree of compatibility of the returned nodes
%                       (i.e. the number of data to which they are
%                       compatible)
%           .nodelist: (numeric array) [deg deg km]
%                       List of 3D coordinates of nodes.
%                       nodelist(i,1) is the node longitude, in deg
%                       nodelist(i,2) is the node latitude, in deg
%                       nodelist(i,3) is the node radius, in km
%
%          .origintime: (structure) [sec]
%                       list containing the origin time intervals for each
%                       node, conatining the following fields:
%                       
%                       .id: node ID in the grid structure
%                       .starts: start times of the origin time intervals
%                       .ends: end times of the origin time intervals
%
%                       the i-th node is a possible hypocenter in the
%                       time intervals given by .origintime(i).starts and
%                       .origintime(i).ends.
%                       Not that ther might be more than one interval for
%                       each node (unlikely, but possible)
%
%           .nodecnt: (number) [counter]
%                      The total number of nodes in MYGRID.
%
% Martin Knapmeyer, 31.05.2005, 13.09.2005

%%% init result
loc.type=mygrid.type;
loc.maxfit=0;
loc.nodelist=[];
loc.origintime=[];
loc.nodecnt=0;

%%% determine max fitness of each layer
if isempty(level)
    [idlist,maxfit]=mklsgridfindmatches(lsopt,mygrid);
    loc.maxfit=max(maxfit); % and this is the max fitness in the whole grid
else
    loc.maxfit=level;
end; % if isempty(level)


%%% loop through all layers and collect them nodes
lyrcnt=length(mygrid.layers); % number of layers in grid
for lyrindy=1:lyrcnt
    
    %%% determine number of nodes in this layer and add it to NODECNT
    loc.nodecnt=loc.nodecnt+size(mygrid.layers(lyrindy).nodes,1);
    
    %%% sort nodelist and origin time list of layer by node ID
    %%% This is necessary to be sure that nodes and times are "parallel"
    %%% with respect to array index.
    %%% 1.) sort nodes
    [mygrid.layers(lyrindy).nodes(:,1),sorter]=sort(mygrid.layers(lyrindy).nodes(:,1));
    mygrid.layers(lyrindy).nodes(:,2)=mygrid.layers(lyrindy).nodes(sorter,2);
    mygrid.layers(lyrindy).nodes(:,3)=mygrid.layers(lyrindy).nodes(sorter,3);
    %%% 2.) sort origin times
    idlist=zeros(length(mygrid.layers(lyrindy).times),1);
    for indy=1:length(idlist)
        idlist(indy)=mygrid.layers(lyrindy).times(indy).id;
    end; % for indy
    [idlist,sorter]=sort(idlist);
    mygrid.layers(lyrindy).times=mygrid.layers(lyrindy).times(sorter);
    %%% 3.) sort fitness list
    [mygrid.layers(lyrindy).fit(:,1),sorter]=sort(mygrid.layers(lyrindy).fit(:,1));
    mygrid.layers(lyrindy).fit(:,2)=mygrid.layers(lyrindy).fit(sorter,2);
    
    
    %%% identify nodes with max fitness
    indies=find(mygrid.layers(lyrindy).fit(:,2)>=loc.maxfit);
    %%% cpoy node data into loc
    if ~isempty(indies)
        
        %%% extract node coordinates
        nodecnt=length(indies); % number of nodes to extract
        newnodes=zeros(nodecnt,3);
        
        newnodes(:,1:2)=mygrid.layers(lyrindy).nodes(indies,2:3); % lon, lat
        newnodes(:,3)=newnodes(:,3)+mygrid.layers(lyrindy).radius;
        loc.nodelist=[loc.nodelist; newnodes];
        
        %%% extract origin times
        newtimes=mygrid.layers(lyrindy).times(indies);
        loc.origintime=[loc.origintime; newtimes];
        
        
    end; % if ~isempty(indies)
end; % for lyrindy