function [idlist,maxfit]=mklsgridfindmatchtri(lsopt,mygrid);
% mklsgridfindmatches......find triangles whose nodes match most data
%
% call: [idlist,maxfit]=mklsgridfindmatchtri(lsopt,mygrid);
%
%               lsopt: (struct)
%                      LOCSMITH options structure as returned by MKLSOPT
%              mygrid: (struct)
%                      LOCSMITH search grid as returned by MKGRIDNEW
%
% result: 
%              idlist: (struct array)
%                      A struture variable that defines the layers and
%                      triangles to be modified. The following fields are
%                      expected:
%                      .layer: (numeric)
%                              layer ID, only one per element of IDLIST.
%                      .triangle: (numeric array)
%                                 list of triangle IDs
%                                 if this is the empty matrix [], all
%                                 triangles are modified.
%
%                      idlist(i).layer defines a layer in which
%                      modifications on triangles shall be made. Then
%                      idlist(i).triangle is a list of triangle IDs that
%                      defines which triangles of the speicifed layer are to
%                      be modified.
%                      If n layers are to be modified, IDLIST must consist
%                      of n elements, each having its own .layer and
%                      .triangle fields.
%
%              maxfit: (array) [counter]
%                      list containing the maximum compatibility for each
%                      layer. maxfit(1) is for the first layer in the grid,
%                      maxfit(end) for the last one. (this is not
%                      necessarily in sequence of layer radius!)
%
% This routine identifies all triangles that have nodes which match most of
% the data. These are the nodes at which the .fit field reaches its max.
% Requiring that .fit matches .tested exactly yields problems with data
% outliers.
%
% Martin Knapmeyer, 01.03.2005, 08.03.2005, 13.09.2005

%%% verbosity
verbosity=5;

%%% number of layers in grid
lyrcnt=length(mygrid.layers); 

%%% init result
idlist=repmat(struct('layer',[],'triangle',[]),lyrcnt);

%%% initial number of triangles in grid with this base body
initialtrianz=mklsconstants(['Triangles' lsopt.grid_base]);

maxfit=zeros(lyrcnt,1);
%%% loop over all layers
for layerindy=1:lyrcnt
    
    %%% copy layer ID into result
    idlist(layerindy).layer=mygrid.layers(layerindy).id;
    
    
    
    %%% find all nodes at which .fit exceeds lsopt.iter_matchthresh percent
    %%% of max(.fit). 
    %%% if lsopt.iter_matchthresh is empty, find all nodes that are
    %%% compatible with all data.
    
    %%% find nodes that are compatible to certain percentage
    maxfit(layerindy)=max(mygrid.layers(layerindy).fit(:,2));
    matchingnodes=find(mygrid.layers(layerindy).fit(:,2)>=maxfit(layerindy)*lsopt.iter_matchthresh/100);
    matchingnodes=mygrid.layers(layerindy).fit(matchingnodes,1); % IDs of these nodes

   % %%% find nodes compatible to all data tested so far
   % matchingnodes=find(mygrid.layers(layerindy).fit(:,2)==mygrid.layers(layerindy).tested(:,2));
    
    
    
    
    if ~isempty(matchingnodes)
       %%% now find out which triangles use these nodes
       
       %%% produce a FXS-layer-struct in which IDs are also indexes to the arrays
       %%% this is needed for the neighbourhood search
       fxslayer=mklsgridfxslayertri(mygrid.layers(layerindy));
       trilist=zeros(1,initialtrianz);
       for indy=1:initialtrianz
           trilist(indy)=mygrid.layers(layerindy).tri(indy).id;
       end; % for indy
    
       tricnt=length(mygrid.layers(layerindy).tri); % number of triangles in layer
       for triindy=1:tricnt
           childtype=lower(mygrid.layers(layerindy).tri(triindy).childtype);
           switch childtype
               case {'triangle'}
                   %%% nothing happens in this case
               case {'node'}
                   %%% find which node IDs are common to triangle node list
                   %%% and matchingnodes list
                   common=intersect(mygrid.layers(layerindy).tri(triindy).nodes,matchingnodes);
                   cnt=length(common);
                   searchmode=lsopt.iter_searchmode;
                   switch searchmode
                       case {'interior'}
                           if cnt~=0 %~isempty(common)
                               %%% triangle uses at least one of the fitting nodes,
                               %%% therefore append it to result and search for its
                               %%% neighbours

                               %%% append triangle ID list to result
                               idlist(layerindy).triangle=[idlist(layerindy).triangle; mygrid.layers(layerindy).tri(triindy).id];

                               %%% search for neighbours of this triangle and
                               %%% append them to result list. To do so, we need
                               %%% the FXS layer structure produced above.                       
                               neighbours=mklsgridoverlapccrtri(fxslayer,triindy,trilist);
                               %%% append identified neighbours to result
                               idlist(layerindy).triangle=[idlist(layerindy).triangle; neighbours(:)];

                           end; % if ~isempty(common)
                       case {'boundary'}
                           if (cnt==1)|(cnt==2) %~isempty(common)
                               %%% not all nodes of triangle fit. By
                               %%% iterating only these, we find the
                               %%% boundary of the solution area!
                               %%% neighbour triangles will be iterated
                               %%% only if not all nodes are in the
                               %%% solution!

                               %%% append triangle ID list to result
                               idlist(layerindy).triangle=[idlist(layerindy).triangle; mygrid.layers(layerindy).tri(triindy).id];

                               %%% search for neighbours of this triangle and
                               %%% append them to result list. To do so, we need
                               %%% the FXS layer structure produced above.                       
                               neighbours=mklsgridoverlapccrtri(fxslayer,triindy,trilist);
                               %%% append identified neighbours to result
                               idlist(layerindy).triangle=[idlist(layerindy).triangle; neighbours(:)];

                           end; % if ~isempty(common)
                       case {'existence'}
                           %%% in this mode, the whole program should stop
                           %%% as soon as the existence of a solution is
                           %%% proven, i.e. as soon as a single node is
                           %%% found which is compatible to all data.
                           mklsnotyet('crash');
                       otherwise
                           error(['MKLSGRIDFINDMATCHTRI: unknown search mode: ' upper(searchmode)]);
                   end; % swtich searchmode
                   
               case {'empty'}
                   error(['MKLSGRIDFINDMATCHTRI: unexpected child type ' upper(childtype)]);
               otherwise
                   error(['MKLSGRIDFINDMATCHTRI: unknown child type ' upper(childtype)]);
           end; % switch
       end; % for tricnt
       %%% make triangle ID list unique
       idlist(layerindy).triangle=unique(idlist(layerindy).triangle);
    else
       %%% no nodes fit all data - this means that all triangles have to be
       %%% iterated!
       idlist(layerindy).triangle=[];
    end; % if ~isempty(matchingnodes)
    
    %%% say something
    mklsverbose(verbosity,lsopt,'MKLSGRIDFINDMATCHTRI: found ',...
                length(idlist(layerindy).triangle),...
                ' triangles in layer # ',layerindy,...
                ' with fit < ',maxfit(layerindy));
    
end; % for layerindy
