function fxslayer=mklsgridfxslayertri(layer);
% mklsgridfxslayertri......construct Fast Access (FXS-) Layer
%
% call: fxslayer=mklsgridfxslayertri(layer);
%
%              layer: (struct)
%                     layer structure as returned by MKLSGRIDNEWTRILAYER
%
% result: fxslayer: (struct)
%                     a fast access layer structure.
%                     as LAYER, but all node IDs are now replaced by the
%                     array index in the node list for faster access.
%                     ADDITIONAL FIELDS in FXS layer:
%                     .maxfit: the maximum value of al .fit values.
%                     .triids: list of triangle IDs, for fast search for
%                              triangles by ID
%
% Searching the neighbours of a given triangle requires a lot of calls to
% MKLSGRIDGETTRIANGLE, making this a very slow process.
% This routine produces a layer structure in which the index to the node
% list is used as ID in triangles and fitness lists, and where triangle IDs
% are given in a separate list which may be searched fast.
% It is not very elegant to use two versions of the layer structure, but
% the need for speed makes it necessary. :-(
%
% Triangle IDs are not changed, which is important since Triangle IDs are
% what the neighbour-tests are searching for.
%
% Martin Knapmeyer, 23.11.2004, 08.12.2004



%%% how many triangles?
trianz=length(layer.tri);
triangleids=zeros(1,trianz);


%%% in preparation, we have to sort everything by size of node ID number to
%%% make sure that nodes are in the same sequence in all fields - this
%%% allows to speed up the replacing procedure considerably!
%%% 1. sort nodes
[layer.nodes(:,1),sorter]=sort(layer.nodes(:,1));
layer.nodes(:,2)=layer.nodes(sorter,2);
layer.nodes(:,3)=layer.nodes(sorter,3);
%%% sort fit array
[layer.fit(:,1),sorter]=sort(layer.fit(:,1));
layer.fit(:,2)=layer.fit(sorter,2);
%layer.fit=layer.fit(sorter,:);
%%% sort tested list
[layer.tested(:,1),sorter]=sort(layer.tested(:,1));
layer.tested(:,2)=layer.tested(sorter,2);
%layer.tested=layer.tested(sorter,:);
%%% sort times list
timeanz=length(layer.times);
timeids=zeros(1,timeanz);
for indy=1:timeanz
    timeids(indy)=layer.times(indy).id;
end; % for indy
[timeids,sorter]=sort(timeids);
layer.times=layer.times(sorter);
%layer.times=layer.times(sorter);

%%% init result
fxslayer=layer;

%%% loop over all triangles
for indy=1:trianz
    
    triangleids(indy)=layer.tri(indy).id;
    
    childtype=layer.tri(indy).childtype;
    switch childtype
        case {'node'}
            
            %%% loop over all three nodes of triangle
            for cnt=1:3
                
                %%% get row numbers of node CNT
                nodenr(cnt)=mklsgridgetnodetri(layer.tri(indy).nodes(cnt),layer);

                %%% replace IDs in triangle
                fxslayer.tri(indy).nodes(cnt)=nodenr(cnt);

                %%% replace IDs in nodes
                fxslayer.nodes(nodenr(cnt),1)=nodenr(cnt);

                %%% replace IDs in fitness list
                fxslayer.fit(nodenr(cnt),1)=nodenr(cnt); %fxslayer.fitness(nodenr(cnt),1)=nodenr(cnt);
                
                %%% replace IDs in tested list
                fxslayer.tested(nodenr(cnt),1)=nodenr(cnt);
                
                %%% replace IDs in times field
                fxslayer.times(nodenr(cnt)).id=nodenr(cnt);
            
            end; % for cnt
            
        case {'triangle'}
            %%% nothing happens with triangles that have triangle childs
        case {'empty'}
            %%% there shouldn't be any empty triangles!
            error(['MKLSGRIDTRINODEID2ROW: triangle #',...
                   int2str(indy),...
                   ' has no children!'] );
        otherwise
            error(['MKLSGRIDROWLAYERSTRI: unknown child type ',...
                   upper(childtype)]);
    end; % switch childtype
    
end; % for indy


%%% introduce new .max field
fxslayer.maxfit=max(fxslayer.fit(:,2:end));
fxslayer.triids=triangleids;