function newidlist=mklsgrididexpandtri(lsopt,grid,idlist);
% mklsgrididexpand.......follows ID tree down to decendants with nodes
%
% call: newidlist=mklsgrididexpandtri(lsopt,grid,idlist);
%
%               lsopt: (struct)
%                      LOCSMITH options structure as returned by MKLSOPT
%                grid: (struct)
%                      LOCSMITH search grid as returned by MKGRIDNEW
%              idlist: (struct array)
%                      A struture variable that defines the layers and
%                      triangles to be modified. The following fields are
%                      expected:
%                      .layer: (numeric)
%                              layer ID, only one per element of IDLIST.
%                      .triangle: (numeric array)
%                                 list of triangle IDs
%                                 If this is the empty matrix [], all
%                                 triangles with node-children will be
%                                 returned in NEWIDLIST.
%
%                      IDLIST is not allowed to be an array: it may define
%                      only one layer. Handling multiple layers must be
%                      done in an external loop.
%
% result: newidlist: (struct array)
%                 structure variable of the same kind as IDLIST, but the
%                 triangles specified here all have node as children.
%                 .layer: (numeric)
%                         The same ID as in idlist.layer.
%                 .triangle: (numeric array)
%                            IDs of triangles that have nodes as children
%                            are copied directly from idlist.triangle.
%                            IDs of triangles that have other triangles as
%                            children are replaced by the IDs of those
%                            triangles that are at the end of the triangle 
%                            tree and also have only nodes as children.
%
% This routine transforms a list of triangles of both types into a list of
% triangles to which iteration may be applied directly by constructing new
% nodes. This routine in non-recursive, although the problem suggests to
% write a reursive algorithm.
%
% Martin Knapmeyer, 24.09.2004

%%% local verbosity
verbosity=4;

%%% say hello
mklsverbose(verbosity,lsopt,'MKLSGRIDIDEXPANDTRI: starting...');


%%% identify the layer within grid
layerindex=[];
cnt=1;
done=0;
while done==0
    if grid.layers(cnt).id==idlist.layer
        layerindex=cnt;
        done=1;
        mklsverbose(verbosity,lsopt,...
                    'MKLSGRIDIDEXPANDTRI: layer ID ',idlist.layer,'...',...
                    ' corresponds to grid.layers(',cnt,').');
    else
        cnt=cnt+1;
    end; % if grid.layers(cnt).id==idlist.layer else
end; % while


%%% construct simplified triangle list: only ID and child type
trianz=length(grid.layers(layerindex).tri);
trilist=zeros(trianz,2);
for indy=1:trianz
    trilist(indy,1)=grid.layers(layerindex).tri(indy).id;
    childtype=(grid.layers(layerindex).tri(indy).childtype);
    mklsverbose(verbosity,lsopt,...
                        'MKLSGRIDEXPANDTRI: layer ',layerindex,...
                        ', triangle ',indy,...
                        ' type: ',childtype);
    switch childtype
        case {'node'}
            trilist(indy,2)=1; % childs are nodes
        case {'triangle'}
            trilist(indy,2)=0; % childs are triangles
        otherwise
            error(['MKLSGRIDIDEXPANDTRI: unknown child type ' upper(childtype)]);
    end; % swtich
end; % for indy


if isempty(idlist.triangle)
    %%% triangle list is empty - return all triangles that have nodes as
    %%% children
    
    %%% identify triangles with nodes as children?
    indies=find(trilist(:,2)==1);
    anz=length(indies); % number of such triangles
    
    %%% create raw output array
    newids=zeros(anz,1);
    
    %%% collect the desired triangles
    newids=trilist(indies,1);
    
else
    %%% triangle list is not empty - follow the tree toward the nodes.

    %%% add second column with child types to ID list
    idanz=length(idlist.triangle);
    newids=zeros(idanz,3);
    newids(:,1)=idlist.triangle;
    for indy=1:idanz
        triindex=find(trilist(:,1)==newids(indy,1));
        newids(indy,2)=trilist(triindex,2);
    end; % for indy


    %%% loop over newids and replace triangles that have triangles as children
    %%% by their children, until all triangles in list have nodes as children
    done=0;
    while done==0
        %%% identify all IDs in newids that pecify triangles that have
        %%% triangles as children
        childistri=find((newids(:,2)==0)&(newids(:,3)==0));
        mklsverbose(verbosity,lsopt,'MKLSGRIDIDEXPANDTRI: ',...
                        length(childistri),'triangles with triangle childs.');
        %%% mark these as completed, for later removal
        newids(childistri,3)=newids(childistri,3)*0-1;
        if isempty(childistri)
            %%% all triangles have node as children and we're done
            done=1;
        else
            %%% loop over entries specified by childistri and replace IDs by
            %%% IDs of child triangles
            anz=length(childistri); % so many entries
            mklsverbose(verbosity,lsopt,'MKLSGRIDIDEXPANDTRI: ',...
                        anz,' triangles to replace.');
            expander=zeros(4*anz,3);
            expander(:,3)=expander(:,3)-1; % set them to -1 because not all will be filled!
            for indy=1:anz
                %%% determine index of triangle in grid:
                gridindex=find(trilist(:,1)==newids(childistri(indy),1));
                if ~isempty(gridindex)
                    %%% append ID and chiltype of found triangle to expander list
                    childtype=(grid.layers(layerindex).tri(gridindex).childtype);
                    switch childtype
                        case {'node'}
                            indy2=(indy-1)*4+1;
                            expander(indy2,1)=grid.layers(layerindex).tri(gridindex).id;
                            expander(indy2,2)=1; % childs are nodes
                            expander(indy2,3)=0;
                        case {'triangle'}
                        %%% children are triangles: make a list to be appended
                        %%% to newisd list
                        for j=1:4
                            indy2=(indy-1)*4+j;
                            expander(indy2,1)=grid.layers(layerindex).tri(gridindex).child(j);
                            expander(indy2,2)=0; % childs are triangles
                            expander(indy2,3)=0;
                        end; % for j
                        otherwise
                            error(['MKLSGRIDIDEXPANDTRI: cannot handle child type ' upper(childtype)])
                    end; % switch
                end; % if ~isempty(gridindex)
            end; % for indy
            %%% append new rtiangles to list
            newids=[newids; expander];
        end; % if isempty(childistri)
    end; % while done

    %%% remove all entries of newids that have type -1
    indies=find(newids(:,3)~=-1);
    newids=newids(indies,:);

end; % if ismepty(idlist.triangle) else

%%% construct result variable
newidlist.layer=idlist.layer;
newidlist.triangle=newids(:,1);



%%% say goodbye
mklsverbose(verbosity,lsopt,'MKLSGRIDIDEXPANDTRI: done.');