function [newgrid,modifycnt]=mklsgridmodify(lsopt,grid,idlist,mode,iterations);
% mklsgridmodify........apply modifications to LOCSMITH search grid
%
% call: [newgrid,modifycnt]=mklsgridmodify(lsopt,grid,idlist,mode);
%       [newgrid,modifycnt]=mklsgridmodify(lsopt,grid,idlist,'iterate',iterations);
%
%              lsopt: (struct)
%                     LOCSMITH options structure as returned by MKLSOPT
%            
%               grid: (struct)
%                     a grid structure as returned by MKLSGRIDNEW.
%                     This grid will be refined according to the settings
%                     in the other parameters.
%
%             idlist: this parameter has different meanings for different
%                     grid types.
%
%                     For triangular grids:
%
%                     (struct array)
%                     A struture variable that defines the layers and
%                     triangles to be modified. The following fields are
%                     expected:
%                     .layer: (numeric)
%                             layer ID, only one per element of IDLIST.
%                     .triangle: (numeric array)
%                                list of triangle IDs
%
%                     idlist(i).layer defines a layer in which
%                     modifications on triangles shall be made. Then
%                     idlist(i).triangle is a list of triangle IDs that
%                     defines which triangles of the speicifed layer are to
%                     be modified.
%                     If n layers are to be modified, IDLIST must consist
%                     of n elements, each having its own .layer and
%                     .triangle fields.
%
%                     For cartesian grids:
%             
%                     (not yet implemented)
%
%               mode: (string)
%                     The MODE of operation defines what to do with the
%                     speciifed layers and triangles. The following modes
%                     are available:
%                     'kill': removes triangles from the triangle list of a
%                             layer. To remove means that treangles are
%                             irreversibly deleted from the list!
%                     'deactivate': deactivates triangles by setting their
%                                   active-flag, but leaving it in the list.
%                     'activate': re-activates deactivated triangles by
%                                 setting their active flag.
%                     'iterate': iterates the triangles (if they are
%                                active). This means that new triangles and
%                                nodes are generated.
%
%              iterations: (number)
%                          the number of grid iterations to be applied to
%                          the grid. This argument is relevant in 'iterate'
%                          mode only (grid refinement).
%
% result: grid: (structure array)
%               modified LOCSMITH search grid.
%         modifycnt: (number)
%                    A number counting how may modifications were applied.
%                    For precise definition, see MKLSGRIDMODIFYTRI and the
%                    respective routine for cartesian grids.
%
% In triangular grids, this routine only modifies the triangles of layers,
% but it does not remove existing layers or construct new layers.
%
% Martin Knapmeyer, 21.09.2004

%%% init result
newgrid=[];

%%% verbosity
verbosity=4;

%%% say hello!
mklsverbose(verbosity,lsopt,'MKLSGRIDMODIFY: start grid modifcation.');

%%% dispatch work to grid specific routines
gridtype=lower(grid.type);
switch gridtype
    case {'cartesian'}
        newgrid=grid;
        mklsnotyet('warn');
    case {'triangular'}
        nin=nargin;
        switch nin
            case {4}
                [newgrid,modifycnt]=mklsgridmodifytri(lsopt,grid,idlist,mode,1);
            case {5}
                [newgrid,modifycnt]=mklsgridmodifytri(lsopt,grid,idlist,mode,iterations);
            otherwise
                error('MKLSGRIDMODIFY: illegal number of input arguments!');
        end; % switch nin
    otherwise
        error(['MLSGRIDMODIFY: unknown grid type ' upper(gridtype)]);
end; % switch gridtype

%%% say goodbye!
mklsverbose(verbosity,lsopt,'MKLSGRIDMODIFY: grid modifcation completed.');

