function [newgrid,modifycnt]=mklsgridmodifytri(lsopt,grid,idlist,mode,iterations);
% mklsgridmodifytri........modify triangular LOCSMITH search grid
%
% call: [newgrid,modifycnt]=mklsgridmodifytri(lsopt,grid,idlist,'iterate',iterations);
%       [newgrid,modifycnt]=mklsgridmodifytri(lsopt,grid,idlist,mode);
%
%               lsopt: (struct)
%                      LOCSMITH options structure as returned by MKLSOPT
%                grid: (struct)
%                      LOCSMITH search grid as returned by MKGRIDNEW
%              idlist: (struct array)
%                      A struture variable that defines the layers and
%                      triangles to be modified. The following fields are
%                      expected:
%                      .layer: (numeric)
%                              layer ID, only one per element of IDLIST.
%                      .triangle: (numeric array)
%                                 list of triangle IDs
%                                 if this is the empty matrix [], all
%                                 triangles are modified.
%
%                      idlist(i).layer defines a layer in which
%                      modifications on triangles shall be made. Then
%                      idlist(i).triangle is a list of triangle IDs that
%                      defines which triangles of the speicifed layer are to
%                      be modified.
%                      If n layers are to be modified, IDLIST must consist
%                      of n elements, each having its own .layer and
%                      .triangle fields.
%
%                      If idlist==NaN, nothing is modified.
%
%               mode: (string)
%                     The MODE of operation defines what to do with the
%                     speciifed layers and triangles. The following modes
%                     are available:
%                     'kill': removes triangles from the triangle list of a
%                             layer. To remove means that treangles are
%                             irreversibly deleted from the list!
%                             KILL mode affects the specified triangle and
%                             all its descendant triangles.
%                             Nodes are not affected.
%                     'deactivate': deactivates triangles by setting their
%                                   active-flag, but leaving it in the
%                                   list.
%                                   DEACTIVATE affects only the specified
%                                   triangles, and NOT its descendants!
%                                   Nodes are not affected.
%                     'activate': re-activates deactivated triangles by
%                                 setting their active flag.
%                                 ACTIVATE affects onlt the specified
%                                 triangles and NOT it descendants.
%                                 Nodes are not affected.
%                     'iterate': iterates the triangles (if they are
%                                active). This means that new triangles and
%                                nodes are generated. (if new triangles use
%                                existing nodes, no new nodes are
%                                generated!)
%                                ITERATE mode affects the specified
%                                triangle and all its descendant
%                                triangles, and also generates new nodes.
%                                triangles are not iterated any further if 
%                                their circumcircle radius is smaller than
%                                lsopt.iter_circmin.
%
%                      NOTE: deactivated triangles can not be iterated, but
%                            can be killed!
%
%               iterations: (number)
%                          the number of grid iterations to be applied to
%                          the grid. This argument is relevant in 'iterate'
%                          mode only (grid refinement), in all other modes
%                          this parameter may be omitted (and defaults to
%                          one)
%
% result: newgrid: as GRID, but with the modifications applied as described
%                  above.
%
%         modifycnt: number of modified triangles.
%                    This number tells you how many triangles were killed,
%                    deactivated, activated or iterated.
%                    In iteration mode, it does NOT tell you how many new
%                    triangles were created, but how many of the existing
%                    triangles were iterated.
%                    if MODIFYCNT is zero, NEWGRID is the same as GRID.
%
% This routine applies standard operations on triangular grid: local and
% global refinement, triangle removal, and activation and deactivation of
% triangles.
% It can be used to work on all triangles of all layers of a given grid but
% also to work on single triangles of single layers.
% 
%
% Martin Knapmeyer, 21.09.2004, 24.09.2004, 25.10.2004, 29.10.2004,
%                   28.10.2005, 03.03.2005



%%% how many input?
nin=nargin;

%%% a useful constant, once more
radian=pi/180;


%%% init result
%%% the routine works on NEWGRID only.
newgrid=grid;
modifycnt=0;


%%% default number of iterations, if not specified in input
if nin==4
    iterations=1;
end; % if nargin==4

%%% local verbosity thresh
verbosity=5;


%%% immediate return if idlist is NaN!
if ~isstruct(idlist)
    if isnan(idlist)
        return;
    end; % if isnan(idlist)
end; % if ~isstruct


%%% say hello
mklsverbose(verbosity,lsopt,...
            'MKLSGRIDMODIFYTRI: starting in ',...
            upper(mode),' mode.');

        


%%% make mode string lowercase
mode=lower(mode);



%%% number of layers in grid
layeranz=length(newgrid.layers);

%%% easy-to-search list of layer IDs
%%% at this point, we catch the case of an empty IDLIST
layerids=[];
modifyalllayers=0;
if isempty(idlist)
    %%% empty list: modify all layers
    modifyalllayers=1;
    for indy=1:layeranz
           layerids=[layerids newgrid.layers(indy).id];
    end; % for indy
else
    %%% non-empty list: modify only specified layers
    modifyalllayers=0;
    for indy=1:length(idlist)
           layerids=[layerids idlist(indy).layer];
    end; % for indy
end; % if isempty(idlist) else


%if stack>0
%    disp([int2str(iterations)]);
%end; % if stack>0

%%% iterations loop: several consecutive iterations are applied
for iter=1:iterations

    %%% loop over layers
    for layercnt=1:layeranz
        
        
        %%% minimum allowed circumcircle radius for this layer, in deg
        %%% This needs to be in degrees for later fast comparison with the
        %%% circumcircle radii of triangles
        circmin=lsopt.iter_circmin/(newgrid.layers(layercnt).radius*radian);
        
        
        %%% do we need to modify this layer?
        modifylayer=0;
        if modifyalllayers==1
            modifylayer=1;
        else
            indies=find(layerids==newgrid.layers(layercnt).id);
            if isempty(indies)
                %%% layer ID is not in list
                modifylayer=0;
            else
                %%% layer ID is in list
                modifylayer=1;
                idpointer=indies(1); % points to this layers elment if IDLIST
                                     % since layerids is parallel to a non-empty idlist,
                                     % this is also the
                                     % index of the current layer.
            end; % if isempty(indies)
        end; % if modifyalllayers==1
        
        
        %%% start layer modifcation if requested
        if modifylayer==1

            mklsverbose(verbosity+1,lsopt,...
                        'MKLSGRIDMODIFY: modifying layer ',...
                        int2str(layercnt));
                    
            %%% expand triangles whose childs are triangles into triangles
            %%% whose childs are nodes: got to ends of triangle tree
            %%% This needs only be done if idlist is non-empty
            if ~isempty(idlist)
                indies=find(layerids==newgrid.layers(layercnt).id);
                expandedid=mklsgrididexpandtri(lsopt,newgrid,idlist(indies));
            else
                %%% idlist is empty, we have to create a dummy to obtain
                %%% the expanded triangle id list
                dummy.layer=newgrid.layers(layercnt).id;
                dummy.triangle=[]; % this can be handled
                expandedid=mklsgrididexpandtri(lsopt,newgrid,dummy);
            end; % if ~isempty

            %%% number of triangles in grid
            trianz=length(newgrid.layers(layercnt).tri);

            %%% loop over triangles
            %%% themnewnodes: we collect new triangles here and append the
            %%% completed list after finishing this loop. It's faster.
            if strcmp(mode,'iterate')
                themnewtricnt=0; % counter for new generated triangles
                themnewtriangles=mklsgridnewtrilist(4*length(expandedid.triangle),0); %[];
            end; % if strcmp(mode,'iter')
            for tricnt=1:trianz
                
                %%% the current triangle
                currenttri=newgrid.layers(layercnt).tri(tricnt);
                
                %%% do we need to midify this triangle?
                modifytri=0;
                
                
                if isempty(expandedid.triangle)
                    %%% triangle list is empty -> modify all triangles
                    modifytri=1;
                else
                    %% dummy id list is not empty, but does it contain
                    %%% the current triangle?
                    indies=find(expandedid.triangle==currenttri.id);
                    if isempty(indies)
                        %%% this triangle is not in list
                        modifytri=0;
                    else
                        %%% this triangle is in list
                        modifytri=1;
                    end; % if ismepty(indies)
                end; % if isempty(expandedid.triangle)
                
%                 if isempty(idlist)
% %                     %%% IDLIST is empty -> modify all triangles!
% %                     modifytri=1;
%                     %%% dummy id list is not empty, but does it contain
%                     %%% the current triangle?
%                     indies=find(expandedid.triangle==currenttri.id);
%                     if isempty(indies)
%                         %%% this triangle is not in list
%                         modifytri=0;
%                     else
%                         %%% this triangle is in list
%                         modifytri=1;
%                     end; % if ismepty(indies)
%                 else
%                     %%% IDLIST is not empty, but does it contain the
%                     %%% current triangle?
%                     if isempty(expandedid.triangle)
%                         %%% triangle list is empty -> modify all triangles
%                         modifytri=1;
%                     else
%                         %%% trilist is not empty, but does it contain
%                         %%% the current triangle?
%                         indies=find(expandedid.triangle==currenttri.id);
%                         if isempty(indies)
%                             %%% this triangle is not in list
%                             modifytri=0;
%                         else
%                             %%% this triangle is in list
%                             modifytri=1;
%                         end; % if ismepty(indies)
%                     end; % isempty(idlist(idpointer).triangle)
%                 end; % if isempty(idlist)
                
                
                %%% start triangle modification if requested
                if (modifytri==1)
                    
                    mklsverbose(verbosity+2,lsopt,...
                                    'MKLSGRIDMODIFYTRI: modifying triangle ',...
                                    tricnt,...
                                    ' of layer ',...
                                    layercnt);
                    
                    if (strcmp(currenttri.childtype,'node')==1)
                        %%% direct modification of child nodes
                        switch mode
                            case {'kill'}
                                %%% declare this triangle "empty"
                                currenttri.childtype='empty';
                                newgrid.layers(layercnt).tri(tricnt)=currenttri;
                                modifycnt=modifycnt+1;
                            case {'iterate'}
                                
                                if (currenttri.active==1)
                                    %%% get nodes from triangle
                                    nodeids=currenttri.nodes;
                                    [rownum,node1]=mklsgridgetnodetri(nodeids(1),newgrid.layers(layercnt));
                                    [rownum,node2]=mklsgridgetnodetri(nodeids(2),newgrid.layers(layercnt));
                                    [rownum,node3]=mklsgridgetnodetri(nodeids(3),newgrid.layers(layercnt));
                                    
                                    %%%% old condition, based on edge
                                    %%%% length, replaced MK06122004
                                    %%% determine max edge length of current
                                    %%% triangle, in km
                                    %edgelen=mklsgridtriedgelen([node1(2) node2(2) node3(2)],...
                                    %                           [node1(3) node2(3) node3(3)],...
                                    %                           newgrid.layers(layercnt).radius);
                                    %edgelen=max(edgelen); % look at longest edge only
                                    %
                                    %if edgelen>lsopt.iter_edgemin
                                    
                                    if currenttri.cr>circmin
                                        %%% Do not modify triangle if its
                                        %%% circumcircle radius is smaller
                                        %%% than the minimim radius!
                                        
                                        %%% increase modification counter
                                        modifycnt=modifycnt+1;

                                        %%% compute midpoints
                                        mid1=[NaN NaN];
                                        [mid1(1) mid1(2)]=mkmidpoint(node1(2),node1(3),node2(2),node2(3));
                                        mid2=[NaN NaN];
                                        [mid2(1) mid2(2)]=mkmidpoint(node2(2),node2(3),node3(2),node3(3));
                                        mid3=[NaN NaN];
                                        [mid3(1) mid3(2)]=mkmidpoint(node3(2),node3(3),node1(2),node1(3));

                                        %%% generate new nodes
                                        newnodes=[mklscreateid(1) NaN NaN;...
                                                  mklscreateid(1) NaN NaN;...
                                                  mklscreateid(1) NaN NaN;...
                                                  ];
                                        newnodes(1,2)=mid1(1);
                                        newnodes(1,3)=mid1(2);
                                        newnodes(2,2)=mid2(1);
                                        newnodes(2,3)=mid2(2);
                                        newnodes(3,2)=mid3(1);
                                        newnodes(3,3)=mid3(2);
                                        
                                        %%% look if nodes at the same position already exist. append
                                        %%% new nodes only if not. MK04112004
                                        reallynew=1:3; % to remember which were new and which already existed
                                        for nodeindy=1:3
                                            existwhere=find((newgrid.layers(layercnt).nodes(:,2)==newnodes(nodeindy,2))&...
                                                            (newgrid.layers(layercnt).nodes(:,3)==newnodes(nodeindy,3))); 
                                            if isempty(existwhere)
                                                %%% node not yet in the list: append it
                                                newgrid.layers(layercnt).nodes=[newgrid.layers(layercnt).nodes;...
                                                                                newnodes(nodeindy,:)];
                                            else
                                                %%% node with same position already exists, use
                                                %%% this one! (only ID is copied because coordinates are ident)
                                                newnodes(nodeindy,1)=newgrid.layers(layercnt).nodes(existwhere(1),1);
                                                reallynew(nodeindy)=NaN;
                                            end; % if isempty(existwhere)
                                        end; % for nodeindy
                                        reallynew=reallynew(find(~isnan(reallynew)));
                                        
                                        %%% reallynew now contains indexes to newnodes that show which
                                        %%% nodes are really new, and its length corresponds to the
                                        %%% number of really new nodes.

%                                         %%% append all new nodes to layer node list
%                                         newgrid.layers(layercnt).nodes=[newgrid.layers(layercnt).nodes;...
%                                                                         newnodes];


%                                         %%% generate new fitness list for new nodes
%                                         %%% here we need the reallynew-information, because
%                                         %%% we dont want to create double fitness entries with same ID!
%                                         newfit=zeros(length(reallynew),...
%                                                      size(newgrid.layers(layercnt).times,2)+1);
%                                         %%% copy node IDs into fitness list
%                                         newfit(:,1)=newnodes(reallynew,1);
%                                         %%% append new fitness list to layers fitness list
%                                         newgrid.layers(layercnt).fit=[newgrid.layers(layercnt).fit;...
%                                                                         newfit];

                                        %%% generate new triangles
                                        newtri=mklsgridnewtrilist(4,currenttri.iter+1);
                                        %%% New Triangle 1
                                        newtri(1).parent=currenttri.id;
                                        newtri(1).childtype='node';
                                        newtri(1).nodes=[newnodes(1,1) newnodes(2,1) newnodes(3,1)];
                                        %%% New Triangle 2
                                        newtri(2).parent=currenttri.id;
                                        newtri(2).childtype='node';
                                        newtri(2).nodes=[newnodes(1,1) node2(1) newnodes(2,1)];
                                        %%% New Triangle 3
                                        newtri(3).parent=currenttri.id;
                                        newtri(3).childtype='node';
                                        newtri(3).nodes=[newnodes(3,1) newnodes(2,1) node3(1)];
                                        %%% New Triangle 4
                                        newtri(4).parent=currenttri.id;
                                        newtri(4).childtype='node';
                                        newtri(4).nodes=[node1(1) newnodes(1,1) newnodes(3,1)];
                                        
                                        
                                        %%%%%% compute circumcircle parameters for new triangles MK03122004
                                        %%% New Triangle 1
                                        [cc,cr]=mkcircumcirclesph(newnodes(1,2:3),newnodes(2,2:3),newnodes(3,2:3));
                                        newtri(1).cc=cc;
                                        newtri(1).cr=cr;
                                        %%% New Triangle 2
                                        [cc,cr]=mkcircumcirclesph(newnodes(1,2:3),node2(2:3),newnodes(2,2:3));
                                        newtri(2).cc=cc;
                                        newtri(2).cr=cr;
                                        %%% New triangle 3
                                        [cc,cr]=mkcircumcirclesph(newnodes(3,2:3),newnodes(2,2:3),node3(2:3));
                                        newtri(3).cc=cc;
                                        newtri(3).cr=cr;
                                        %%% New Triangle 4
                                        [cc,cr]=mkcircumcirclesph(node1(2:3),newnodes(1,2:3),newnodes(3,2:3));
                                        newtri(4).cc=cc;
                                        newtri(4).cr=cr;
                                        %%%%%% circumcircles done
                                        

                                        %%% append new triangles to layer triangle list
                                        themnewtriangles((themnewtricnt+1):(themnewtricnt+4))=newtri;
                                        themnewtricnt=themnewtricnt+4;
                                        %themnewtriangles=[themnewtriangles, newtri];
                                        %newgrid.layers(layercnt).tri=[newgrid.layers(layercnt).tri, newtri];

                                        %%% modify original triangle to be of new childtype
                                        currenttri.nodes=[];
                                        currenttri.child=[newtri(1).id newtri(2).id newtri(3).id newtri(4).id];
                                        %currenttri.childtype='triangle';

                                        %%% replace the original triangle by the modified version
                                        newgrid.layers(layercnt).tri(tricnt)=currenttri;
                                    end; % if edgelen>lsopt.iter_edgemin
                                end; % if currentri.active==1
                            case {'activate'}
                                %%% declare this triangle active
                                currenttri.active=1;
                                newgrid.layers(layercnt).tri(tricnt)=currenttri;
                                modifycnt=modifycnt+1;
                            case {'deactivate'}
                                %%% declare this triangle inactive
                                currenttri.active=0;
                                newgrid.layers(layercnt).tri(tricnt)=currenttri;
                                modifycnt=modifycnt+1;
                            otherwise
                                error(['MKLSGRIDMODIFYTRI: unknown mode ' upper(mode)]);
                        end; % switch mode
                    else
                        if (strcmp(currenttri.childtype,'triangle')==1)
                            %%% in an earlier version, a recursive call was
                            %%% done here to handle child triangles. This
                            %%% is no longer necessary. The error below
                            %%% should never occur, since all triangular
                            %%% child objects are removed before coming
                            %%% here. MK24092004
                            error('MKLSGRIDMODIFYTRI: unexpected triangular child objects in triangle tree!');
                        end; % if (strcmp(currenttri.childtype,'triangle')==1)&(recursive==1)
                    end; % if (strcmp(currenttri.childtype,'node')==1)
                end; % if modifytri
                
                
            end; % for tricnt
            
            %%% afterburner: some modes require additional work after
            %%% looping through all triangles
            switch mode
                case {'kill'}
                    %%% remove all triangles that are declared "empty"
                    
                    remain=[];
                    for tricnt=1:trianz
                        if strcmp(newgrid.layers(layercnt).tri(tricnt).childtype,'empty')==0
                            remain=[remain tricnt];
                        end; % if strcmp
                    end; % for tricnt
                    newgrid.layers(layercnt).tri=newgrid.layers(layercnt).tri(remain);
                case {'iterate'}
                    
                    %%% append all new triangles to grid structure
                    newgrid.layers(layercnt).tri=[newgrid.layers(layercnt).tri, themnewtriangles(1:themnewtricnt)];
                    
                    %%% extend fitness list for new nodes
                    [currentfitlen,currentfitwdt]=size(newgrid.layers(layercnt).fit); % current size of list
                    currentnodecnt=size(newgrid.layers(layercnt).nodes,1); % total number of nodes
                    %%% generate empty fitness list
                    newfit=zeros(currentnodecnt-currentfitlen,currentfitwdt); 
                    %%% copy node IDs into fitness list
                    newfit(:,1)=newgrid.layers(layercnt).nodes(currentfitlen+1:currentnodecnt,1);
                    %%% append fitness entries to layers fitness list
                    newgrid.layers(layercnt).fit=[newgrid.layers(layercnt).fit;  newfit];
                    
                    %%% extend tested-counter list for new nodes
                    %%% generate empty tested-counter list
                    newtested=zeros(currentnodecnt-currentfitlen,2);
                    %%% copy node IDs into tested-list
                    newtested(:,1)=newgrid.layers(layercnt).nodes(currentfitlen+1:currentnodecnt,1);
                    %%% append tested entries to layers tested list
                    newgrid.layers(layercnt).tested=[newgrid.layers(layercnt).tested;  newtested];
                    
                    %%% extend origin times list for new nodes
                    %%% generate empty entries
                    newtimes=mklsgridtimeslist(newfit(:,1));
                    %%% append times lists to layers times lists
                    newgrid.layers(layercnt).times=[newgrid.layers(layercnt).times; newtimes];
                    
                    %%% set child type of all modified triangles
                    %%% a triangle is considered to be modified if its node
                    %%% list is empty. Then set childtype to 'triangle'. We
                    %%% only have to loop over the initial number of
                    %%% triangles as given in TRIANZ, since all new
                    %%% triangles are 'node'-triangles.
                    for tricnt=1:trianz
                        if isempty(newgrid.layers(layercnt).tri(tricnt).nodes)
                            newgrid.layers(layercnt).tri(tricnt).childtype='triangle';
                        end; % if isempty
                    end; % for tricnt
                    

                case {'activate','deactivate'}
                    %%% nothing happens
                otherwise
                    error(['MKLSGRIDMODIFYTRI: unknown mode ' upper(mode)]);
            end; % switch mode
            
        end; % if modifylayer==1
        
        
    end; % for layercnt
    
end; % for iter





%%% say goodbye
mklsverbose(verbosity,lsopt,...
            'MKLSGRIDMODIFYTRI: done.');