function mygrid=mklsgridnewtri(lsopt,radiuslist);
% mklsgridnewtri.........create new triangular grid
%
% call: mygrid=mklsgridnewtri(lsopt);
%       mygrid=mklsgridnewtri(lsopt,radiuslist);
%
%       lsopt: (struct)
%              LOCSMITH options structure as returned by MKLSOPT
%
%       radiuslist: (numeric array) [km]
%              List of radius values: grid layers with the radii specified
%              in this list are produced instead of evaluatin the radius
%              limits given in LSOPT.
%              If no RADIUSLIST is given, the respective definitions in
%              LSOPT will be used.
%              It is assumed that RADIUSLIST is a unique list of radius
%              values.
%
% result: mygrid: a structure variable representing a triangular grid
%               with properties as defined in LSOPT.
%               The structure has the following fields:
%
%               .type: (string)
%                      defines the grid type.
%                      this is the same as LSOPT.GRID_TYPE
%               .layers: (numeric array)
%                        Array of layer structures. Each element of the
%                        .layers field is a complete LAYER structure that
%                        defines a triangular grid for a single radius.
%                        .layers(1) is the innermost layer.
%                        For details of LAYER structures see the 
%                        MKLSGRIDNEWTRILAYER routine.
%
%               in case of problems, the grid variable will remain empty.
%
% Generates a grid according to specifications in LSOPT.
% The initial layers will be iterated until the maximum circumcircle radius is
% smaller than the node spacing defined by LSOPT.GRID_INIT_WDT.
% (To save time, the necessary number of iterations is estimated from precomputed
% values given by MKLSCONSTANTS, grid mesh width is not evaluated during construction.
% This means that the initial mesh width may be realized only approximatively.)
%
% NOTE that the value of the grid_init_wdt field overrides the value of the
% iter_circmin field: the inital grid will be iterated until the
% grid_init_wdt value of circumcircle radii is met, iter_circmin is
% ignored (in fact, it is set to 0 for this purpose)!
%
% Martin Knapmeyer, 08.09.2004, 26.01.2005, 05.07.2005


%%% a usefu constant
radian=pi/180;

%%% verbosity
verbosity=2;

%%% init result
mygrid=[];
output=[];


%%% determine call mode
%%% 'autoradii': one input argument, use LSOPT to define layer radii
%%% 'explicitradii': two input arguments, use second as radius list
switch nargin
    case {1}
         radiusmode='autoradii';
         layerno=1+fix(abs(lsopt.grid_init_minr-lsopt.grid_init_maxr)/lsopt.grid_init_dr); % number of layers to initialize
    case {2}
         radiusmode='explicitradii';
         layerno=length(radiuslist);
         if layerno==0
            %%% an explicit radius list is given, but empty. Very funny.
            return;
         end; % if layerno==0
    otherwise
        error(['MKLSGRIDNEWTRI: illegal number of input arguments!']);
end; % switch nargin


%%% set grid type
output.type=lsopt.grid_type;


%%% STEP 0
%%% (doing this here implies that all layers use the same grid base!)
switch lsopt.grid_base
    case {'tetrahedron'}
        whichlist='MaxCCRTetra';
    case {'octahedron'}
        whichlist='MaxCCROcta';
    case {'icosahedron'}
        whichlist='MaxCCRIcosa';
    otherwise
        error(['MKLSGRIDNEWTRI: unknown grid base ' upper(lsopt.grid_base)]);
end; % switch lsopt.grid_base
ccrlist=mklsconstants(whichlist); % max circumcircle radii in degrees
redfact=cumprod(ccrlist(1:end-1)./ccrlist(2:end)); % reduction factors per iteration
requiredccr=lsopt.grid_init_wdt; % required circumcircle radius in km
ccrlist=ccrlist*radian; % max circumcircle radii in radians

%%% construct the necessary number of layers
output.layers=[]; % init layer field

for indy=layerno:-1:1
    
    mklsverbose(verbosity,lsopt,...
                    'MKLSGRIDNEWTRI: creating triangular grid layer ',indy);
                
    %%% construct new layer
    switch radiusmode
        case {'autoradii'}
           radius=lsopt.grid_init_maxr-(indy-1)*lsopt.grid_init_dr; % layer radius
        case {'explicitradii'}
            radius=radiuslist(indy); % layer radius
        otherwise
            error(['MKLSGRIDNEWTRI: unknown radius determination mode ' upper(radiusmode)]);
    end; % switch autoradii
    
    if (radius>0)&(radius<=lsopt.planet_radius)
        %%% do not proceed if layer has negative or zero radius, or if
        %%% layer radius is larger than planet radius!
        
        %%% create new layer
        newlayer=mklsgridnewtrilayer(lsopt,radius);

        %%% iterate new layer as far as necessary:
            %%% 0. get precomputed circumcircle radius list for current grid base
            %%% 1. determine max circumcircle radius in new layer from values given
            %%%    in MKLSCONSTANTS (we currently have an uniteraded grid, the mesh
            %%%    width of which is stored there)
            %%% 2. estimate number of iterations necessary to meet the desired mesh
            %%%    width
            %%% 3. Apply iterations.
            %%%    At this point, the iter-circmin field is overwritten with
            %%%    zero to guarantee that the generated grid has the resolution
            %%%    required by the grid_init_wdt field!

            %%% STEP 0 has been done outside the loop already.

            %%% STEP 1
            initialccr=ccrlist(1)*newlayer.radius; % [km]


            %%% STEP 2
            if initialccr<=requiredccr
                %%% mesh is already fine enough, don't iterate any further
                smallenough=0;
            else
                %%% mesh is not fine enough, determine necessary number of
                %%% iterations to be applied
                reducedccr=initialccr./redfact;
                smallenough=find(reducedccr<=requiredccr);

                if isempty(smallenough)
                    %%% the desired mesh width is so small that the
                    %%% precomputed list does not contain enough shrink
                    %%% factors. By construction of that list, we can assume a
                    %%% constant factor of 2 to extend it.
                    neededfactor=min(reducedccr(:))/requiredccr;
                    addsteps=ceil(log2(neededfactor));
                    smallenough=length(redfact)+addsteps;
                end; % if isempty(smallenough)

            end

            %%% STEP 3
            dummygrid.type=output.type;
            dummygrid.layers=newlayer;
            idlist.layer=dummygrid.layers(1).id;
            idlist.triangle=[];
            iterations=smallenough(1);
            lsopt.iter_circmin=0; % override setting from caller routine!
            mklsverbose(verbosity-1,lsopt,'MKLSGRIDNEWTRI: applying ',iterations,...
                        ' iterations to match initial mesh width.');
            dummygrid=mklsgridmodifytri(lsopt,dummygrid,idlist,'iterate',iterations);

        %%% initial iteration done



        %%% append new layer to grid structure
        output.layers=[output.layers dummygrid.layers];
    
    end; % if radius>0
    
end; % for indy


%%% return result
mygrid=output;