function layer=mklsgridnewtrilayer(lsopt,radius);
% mklsgridnewtrilayer.........construct new layer for triangular grid
%
% call: layer=mklsgridnewtrilayer(lsopt,radius);
%
%             lsopt: LOCSMITH options structure as returned by MKLSOPT
%            radius: desired layer radius [km]
%
% result: layer: a new layer structure, representing a single layer of
%                a triangular grid at the zero-th iteration.
%                The structure contains the following fields:
%
%               .id:  (number)
%                     layer ID.
%                     this may be used to uniquely identify the layer.
%           .radius:  layer radius (number) [km]
%                     each layer consists of a number of points located on
%                     a sphere with this radius
%            .times:  (array structure)
%                     See MKLSGRIDTIMESLIST for details.
%              .tri:  (array structure)
%                     triangle list structure as returned by MKLSGRIDNEWTRILIST.
%                     See there for a description.
%                     The triangle iteration counter will be initialized as
%                     zero. All triangles will be active.
%            .nodes:  (numeric array) [ID deg deg]
%                     three column array containing the longitude and
%                     latituide coordinates of all nodes used by this
%                     layer.
%                     nodes(i,1) is the ID of the i-th node in the list
%                     nodes(i,2) is the longitude of the i-th node [deg]
%                     nodes(i,3) is the latitude of the i-th node [deg]
%              .fit:  (numeric array)
%                     Array containing the maximum quality measure for each node
%                     that is reached in the origin time intervals specified
%                     by the .TIMES field. This is a 2D array consisting
%                     of an ID that points to the corresponding node in
%                     space and a quality value that corresponds
%                     to the origin times given in the .TIMES field.
%                     fit(i,1) is the ID of the node for which the quality
%                              values in fit(i,2:end) are valid.
%                     fit(i,2) is the quality value of the node with ID
%                              fit(i,1) at origin times in the element of
%                              .TIMES with the same node ID.
%                     Upon initilaization, all quality values are set to 0.
%           .tested: (numeric array) [ID counter]
%                    two column array showing with how many of the
%                    available data values a node has been tested for
%                    compatibility.
%                    .tested(i,1) is the ID of th i-th node in the list
%                    .tested(i,2) is the number of tests applied to the
%                                 node with ID tested(i,1).
%                    Since this is a counter, elements for new nodes are
%                    initialized with 0, and new noes may be identified by
%                    searching for .tested(:,2)==0.
%                     
%
%          In case of problems, the result is empty (not an empty structure,
%          but simply empty: layer=[].
%
% Martin Knapmeyer, 09.09.2004, 03.12.2004, 10.02.2005

% When adding new grid base type, do not forget to add maximum edge length
% lists in MKLSCONSTANTS!

%%% verbosity level
verbosity=3;

%%% init result
layer=[];

%%% a useful constant
radian=pi/180;

%%% create emtpy structure
output.id=[];
output.radius=[];
output.times=[];
output.nodes=[];
output.fit=[];
output.tri=[];
output.tested=[];


%%%%%% now fill the empty layer structure with information.

%%% create layer ID
output.id=mklscreateid;


%%% store layer radius in output struct
output.radius=radius;

% %%% time coordinate vector
% refday=mklsconstants('ReferenceDay');
% mintime=mkymdhms2rjd(lsopt.grid_init_otmin,refday);
% maxtime=mkymdhms2rjd(lsopt.grid_init_otmax,refday);
% timestep=lsopt.grid_init_dt;
% output.times=mintime:timestep:maxtime;



%%% construct triangles and nodes
base=lower(lsopt.grid_base);
switch base
    case {'tetrahedron'}
        
        mklsverbose(verbosity,lsopt,...
                    'MKLSGRIDNEWTRILAYER: creating ',base,'.');
        
        %%% elementary parameters
        nodecnt=4; % number of nodes
        facecnt=4; % number of faces
        
        %%% Node positions
        output.nodes(1,2)=0; % lon 
        output.nodes(1,3)=90; % lat
        output.nodes(2,2)=0;
        output.nodes(2,3)=90-(2*acos(1/sqrt(3)))/radian;
        output.nodes(3,2)=120;
        output.nodes(3,3)=output.nodes(2,3);
        output.nodes(4,2)=240;
        output.nodes(4,3)=output.nodes(2,3);
        
        %%% Node IDs
        output.nodes(:,1)=mklscreateid(size(output.nodes(:,1)));
        %%% triangles
        
        tri=mklsgridnewtrilist(facecnt,0); % create list of empty triangles
        
        %%% perliminary list of nodes, referring to position in above 
        %%% definition of node coordinates
        %%% The sequence of corners of each face is important for correct
        %%% shading of the resulting grids! Ordering has been repaired for
        %%% tetrahedron, MK22.12.2004
        nodelist=[1 2 3;...
                  1 3 4;...
                  1 4 2;...
                  2 4 3;...
                  ];
      
    case {'octahedron'}
        
        mklsverbose(verbosity,lsopt,...
                    'MKLSGRIDNEWTRILAYER: creating ',base,'.');
        
        %%% elementary parameters
        nodecnt=6; % number of nodes
        facecnt=8; % number of faces
        
        %%% Node positions
        output.nodes(1,2)=0; % lon 
        output.nodes(1,3)=90; % lat
        output.nodes(2,2)=0;
        output.nodes(2,3)=0;
        output.nodes(3,2)=90;
        output.nodes(3,3)=0;
        output.nodes(4,2)=180;
        output.nodes(4,3)=0;
        output.nodes(5,2)=270;
        output.nodes(5,3)=0;
        output.nodes(6,2)=0;
        output.nodes(6,3)=-90;
        %%% Node IDs
        output.nodes(:,1)=mklscreateid(size(output.nodes(:,1)));
        %%% triangles
        tri=mklsgridnewtrilist(facecnt,0); % create list of empty triangles
        
        %%% perliminary list of nodes, referring to position in above 
        %%% definition of node coordinates
        %%% The sequence of corners of each face is important for correct
        %%% shading of the resulting grids! Ordering has been repaired for
        %%% octahedron, MK22.12.2004
        nodelist=[1 2 5;...
                  1 5 4;...
                  1 4 3;...
                  1 3 2;...
                  6 5 2;...
                  6 4 5;...
                  6 3 4;...
                  6 2 3];
        
    case {'icosahedron'}
        
        mklsverbose(verbosity,lsopt,...
                    'MKLSGRIDNEWTRILAYER: creating ',base,'.');
        
        %%% elementary parameters
        nodecnt=12; % number of nodes
        facecnt=20; % number of faces
        delta=2*asin(sqrt(2/(5+sqrt(5))))/radian;
        
        %%% Node positions
        output.nodes(1,2)=0; % lon 
        output.nodes(1,3)=90; % lat
        
        output.nodes(2,2)=0;
        output.nodes(2,3)=90-delta;
        output.nodes(3,2)=1*72;
        output.nodes(3,3)=90-delta;
        output.nodes(4,2)=2*72;
        output.nodes(4,3)=90-delta;
        output.nodes(5,2)=3*72;
        output.nodes(5,3)=90-delta;
        output.nodes(6,2)=4*72;
        output.nodes(6,3)=90-delta;
        
        output.nodes(7,2)=0+36;
        output.nodes(7,3)=-90+delta;
        output.nodes(8,2)=1*72+36;
        output.nodes(8,3)=-90+delta;
        output.nodes(9,2)=2*72+36;
        output.nodes(9,3)=-90+delta;
        output.nodes(10,2)=3*72+36;
        output.nodes(10,3)=-90+delta;
        output.nodes(11,2)=4*72+36;
        output.nodes(11,3)=-90+delta;
        
        output.nodes(12,2)=0;
        output.nodes(12,3)=-90;
        
        %%% Node IDs
        output.nodes(:,1)=mklscreateid(size(output.nodes(:,1)));
        %%% triangles
        tri=mklsgridnewtrilist(facecnt,0); % create list of empty triangles
        
        %%% perliminary list of nodes, referring to position in above 
        %%% definition of node coordinates
        %%% The sequence of corners of each face is important for correct
        %%% shading of the resulting grids! Ordering has been repaired for
        %%% ikosahedron, MK11112004
        nodelist=[1 2 3;...
                  1 3 4;...
                  1 4 5;...
                  1 5 6;...
                  1 6 2;...
                  
                  11 7 2;...
                  
                  2 7 3;...
                  3 7 8;...
                  
                  4 3 8;...
                  8 9 4;...
                  4 9 5;...
                  
                  5 9 10;...
                  6 5 10;...
                  
                  6 10 11;...
                  11 2 6;...
                  
                  12 8 7;...
                  12 9 8;...
                  12 10 9;...
                  12 11 10;...
                  12 7 11;...
                 ];
        
    case {'random'}
        mklsverbose(verbosity,lsopt,...
                    'MKLSGRIDNEWTRILAYER: creating ',base,' grid.');
        
        mklsnotyet;
    otherwise
        error(['MKLSGRIDNEWTRILAYER: unknown grid base ' upper(base)]);
end; % switch base




%%% transform preliminary list into final list of node IDs
for indy=1:facecnt
   tri(indy).childtype='node';
   tri(indy).nodes=output.nodes(nodelist(indy,:),1);
   tri(indy).nodes=tri(indy).nodes(:)'; % make it a row vector!
end; % for indy


%%% insert completed triangle list into layer structure
output.tri=tri;



%%% compute circumcircle parameters for each triangle MK03122004
for indy=1:facecnt
    
    %%% get node indexes and nodes
    rownum=mklsgridgetnodetri(output.tri(indy).nodes(1),output);
    a=output.nodes(rownum,2:3);
    rownum=mklsgridgetnodetri(output.tri(indy).nodes(2),output);
    b=output.nodes(rownum,2:3);
    rownum=mklsgridgetnodetri(output.tri(indy).nodes(3),output);
    c=output.nodes(rownum,2:3);
    
    %%% compute circumcircle parameters
    [cc,r]=mkcircumcirclesph(a,b,c);
    
    %%% store circumcircle parameters in triangle struct
    output.tri(indy).cc=cc;
    output.tri(indy).cr=r;
    
    %disp(['MKLSGRIDNEWTRILAYER: circumcircle radius triangle # ' int2str(indy) ': ' num2str(r) 'deg.']);
    
end; % for indy
%%% circumcircles completed.

%%% construct fit array and tested array
nodeanz=size(output.nodes,1);  % number of nodes
%timeanz=length(output.times);  % number of times
output.fit=zeros(nodeanz,2); % init with 0
output.tested=zeros(nodeanz,2); % init with 0 (it's a counter)
for indy=1:nodeanz
    output.fit(indy,1)=output.nodes(indy,1);
    output.tested(indy,1)=output.nodes(indy,1);
end; % for indy

%%% construct times field
output.times=mklsgridtimeslist(output.fit(:,1));



%%%%%% layer construction done.


%%% return result
layer=output;



