function [newz,newdr]=mklsgridnewvertical(lsopt,z,f);
% mklsgridnewvertical......construct improved depth/radius sampling
%
% call: [newz,newdr]=mklsgridnewvertical(lsopt,z,f);
%
%            lsopt: (structure)
%                   LSOPT structure as returned by MKLSOPT
%
%                z: (numeric array) [km]
%                   current depth/radius sampling. This vector contains the
%                   current depths/radii of the grid layers.
%
%                   In case of cartesian grids, this is a depth below the
%                   surface. In case of spherical grids, this is a radius
%                   (which will be converted into a depth internally)
%
%                f: (numeric array) [km]
%                   maximum compatibility reached at the given depths/radii
%                   (the function value to be optimized)
%
% result: newz: (numeric array) [km]
%               new list of depth/radius values the hopefully will bracket
%               th maxima of cofz(z) better than the previous ones.
%               This list is unique and does not contain values that are
%               also contained in Z.
%               If Z contains only one value, NEWZ will be empty.
%
%         newdr: (number) [km]
%                new depth/radius sampling interval. This must be compared
%                to lsopt.iter_drmin to stop the iteration.
%                If Z contains only one value, NEWDR will be NaN.
%
% This routine essentially performs a multimodal maximization of a 1D
% function f(z). It is applied to the layer sequences of both cartesian and
% triangular grids. The routine does not evaluate the target function, but
% constructs a new sampling of the input variable that will hopefully give
% a better solution for the function value, which has to be computed
% outside.
% The optimization result, however, depends on the input. A finer initial
% sampling results in a better chance to get all maxima (in case of multimode functions).
% The search _can_ do multimodal optimization, but unfortunately I can not 
% guarantee that all maxima are found. (you will hardly find a method that 
% can give such a guarantee!)
%
% The search does not leave the interval min(z)...max(z). So if a maximum
% is identified that contains the interval margin or is very close to it,
% it is _your_ responsibility to re-run the search with a wider initial
% interval.
%
% Note: this routine is implemented for the special case of a positive 
%       stair-like function and performs poor for other functions.
%       (the target function for this problem is integer-valued and positive)
%
% Martin Knapmeyer, 28.06.2005, 04.07.2005, 01.09.2005
%                   completely rewritten 06.09.2005
%                   completely rewritten 29.09.2005




%%% verbosity
verbosity=2;


%%% init result
newz=[];
newdr=NaN;





%%% say hello
mklsverbose(verbosity,lsopt,'MKLSGRIDNEWVERTICAL: construct new vertical sampling');


%%%%%% some preparations

%%% convert radii into depths if necessary
if strcmpi(lsopt.grid_type,'triangular')==1
    z=lsopt.planet_radius-z; % z is now a depth below the surface
end; % if strcmp(lower(lsopt.grid_type),'triangular')==1

%%% sort z and f to make z increasing with index
[z,sorter]=sort(z);
f=f(sorter);

%%% determine the allowed interval
%%% new z values below LOWBOUND or above HIBOUND are not acceptable!
lowbound=min(z);
hibound=max(z);

%%%%%% preparations done




%%%%%% construct new sampling

%%% we have to distinguish several cases:
%%% The input sampling might consist of a single depth sample.
%%% If the input sampling consists of seceral samples, the sampling of the
%%% maximum might be one of several cases:
%%%     there might be several samples that hav a common max value
%%%     there might be only a single sample with a max value
%%%     the max value of the given samling might be only a local, not the
%%%     global max.
%%% And we have to keep in mind that we are not allowed to leave the
%%% interval defined by Z!

% %%% mark allowed interval BEGIN
% ax=axis;
% hold on
% plot(lsopt.planet_radius-[1 1]*lowbound,[ax(3) ax(4)],'r');
% plot(lsopt.planet_radius-[1 1]*hibound,[ax(3) ax(4)],'r');
% hold off
% drawnow;
% %%% mark allowd interval END

% %%% plot function as it currently appears BEGIN
% hold on
% plot(lsopt.planet_radius-z,f,'m');
% hold off
% %%% plot function as it currently appears END


%%% determine value and position of maxima
%%% MAXVAL is the maximal value of function f(z)
%%% From the iteration of triangular grids we learned that it is
%%% sometimes a good idea to consider all points where the function
%%% exceeds some percentage of the actual max value. Therefore we
%%% multiply the max by lsopt.iter_vertical_matchthresh, as done in the
%%% triangular grid case. In the following, we treat this "reduced"
%%% maximum as if it was the real maximum.
maxval=max(f)*lsopt.iter_vertical_matchthresh/100;


% %%% mark max threshold BEGIN
% %%% draw horizontsal line at lsopt.iter_matchthresh threshold value
% hold on; 
% plot(lsopt.planet_radius-[lowbound hibound],[1 1]*maxval,'m:');
% hold off;
% drawnow;
% %%% mark max threshold END


%%% decompose current sampling into intervals and introduce new samples
%%% where necessary
zcnt=length(z); % current number of samples
if zcnt==1
   %%% current sampling consists of only one sample - do not construct
   %%% anything.
   newz=[];
   newdr=NaN;
else
   %%% there is more than one sample
   %%% => there are zcnt-1 intervals between zcnt samples.
   
   %%% interval decomposition
   %%% This is an explicit decomposition which produces a list of interval
   %%% start and end points and the functions values at these points. This
   %%% explicit construction wastes a lot of memory, but is better readable
   %%% than an implicit evaluation of the z and f lists.
   intervals=zeros(zcnt-1,5);
   for indy=1:zcnt-1
       interval(indy,1)=z(indy); % z at interval start
       interval(indy,2)=z(indy+1); % z at interval end
       interval(indy,3)=abs(interval(indy,2)-interval(indy,1)); % interval width
       interval(indy,4)=f(indy); % function value at interval start
       interval(indy,5)=f(indy+1); % function value at interval end
   end; % for indy
   
   
   if sum(abs(diff(f)))==0
      %%% all function samples have the same value: function appears to be
      %%% flat. => apply rule 3
      
      
      %%% loop over all intervals to insert middle points everywhere
      for indy=1:zcnt-1
          if interval(indy,3)>=lsopt.iter_drmin
             %%% but only to intervals that ar elong enough!!
             mklsverbose(verbosity,lsopt,'MKLSGRIDNEWVERTICAL: applying rule 3 to interval ',indy,' of ',zcnt-1);
             
             %%% new point in the middle of this interval
             middle=mean(interval(indy,1:2));
             newz=[newz middle];
             
          else
             %%% interval is shorter than minimum allowed length, don't
             %%% subdivide it any further!
          end; % if interval(indy,3)>=lsopt.iter_drmin
      end; % for indy
      
   else
       %%% function samples have different values
   
       


       %%% analyze intervals
       %%% loops over intervals. if one endpoint is above the max-threshold and
       %%% the interval is not already shorter than the lsopt.iter_drmin threshold,
       %%% a new point in the middle of the interval is generated.
       %%% Additionally, new points in the middles of the to neighbouring
       %%% intervals are generated.
       %%% This loop might construct double entries in the new samples list. In
       %%% a second step, it will be made unique.
       for indy=1:zcnt-1

    %        %%% mark current interval in plot BEGIN
    %        hold on
    %        handle=plot(lsopt.planet_radius-interval(indy,1:2),[-0.5 -0.5],'k');
    %        hold off
    %        drawnow;
    %        %%% mark current interval in plot END

           %%% if interval is long enough, look for possible maxima
           if interval(indy,3)>=lsopt.iter_drmin
               %%% interval is long enough for further subdivision!

    %            %%% mark current interval as suitable for subdivision
    %            set(handle,'Color','g');
    %            %%% mark current interval as suitable for subdivision

               %%% does is contain a maximum?
               if (interval(indy,4)>=maxval)||(interval(indy,5)>=maxval)
                   %%% one of the endpoints is on a maximum
                   %%% construct subdivisions!
                   mklsverbose(verbosity,lsopt,'MKLSGRIDNEWVERTICAL: applying rule 1 to interval ',indy,' of ',zcnt-1);

                   %%% new point in the middle of this interval
                   middle=mean(interval(indy,1:2));
                   newz=[newz middle];

    %                %%% mark new point as being set because of max BEGIN
    %                hold on
    %                plot(lsopt.planet_radius-middle,-0.4,'v');
    %                hold off
    %                drawnow;
    %                %%% mark new point as being set because of max END

                   %%% new point in the middle of the previous interval
                   if indy>1
                      mklsverbose(verbosity,lsopt,'MKLSGRIDNEWVERTICAL: applying rule 2 to interval ',indy-1,' of ',zcnt-1);
                      middle=mean(interval(indy-1,1:2));
                      newz=[newz middle];
    %                    %%% mark new point as being set because of neighbor BEGIN
    %                    hold on
    %                    plot(lsopt.planet_radius-middle,-0.6,'<');
    %                    hold off
    %                    drawnow;
    %                    %%% mark new point as being set because of neighbor END
                   end; % if indy>1

                   %%% new point in the middlow of the next interval
                   if indy<zcnt-1
                      mklsverbose(verbosity,lsopt,'MKLSGRIDNEWVERTICAL: applying rule 2 to interval ',indy+1,' of ',zcnt-1);
                      middle=mean(interval(indy+1,1:2));
                      newz=[newz middle];
    %                   %%% mark new point as being set because of neighbor BEGIN
    %                    hold on
    %                    plot(lsopt.planet_radius-middle,-0.6,'>');
    %                    hold off
    %                    drawnow;
    %                    %%% mark new point as being set because of neighbor END
                   end; % if indy


               else
                   %%% endpoints are below maximum
                   %%% Do not construct anything!
               end; % if (interval(indy,4)>=maxval)|(interval(indy,5)>=maxval)

           else
               %%% interval is too short for further subdivision!
               %%% Do not construct anything!

    %            %%% mark current interval as NOT suitable for subdivision
    %            set(handle,'Color','r');
    %            %%% mark current interval as NOT suitable for subdivision

           end; % if interval(indy,3)>lsopt.iter_drmin

       end; % for indy
       
   end; % sum(abs(diff(f)))==0
   
   
end; % if length(z)
   

%%% make list of new samples unique
%%% This is necessary since more than one subdivision rule may apply to an
%%% interval, but only one sample shlould be generated.
newz=unique(newz);

%%% convert depths into radii if necessary
if strcmpi(lsopt.grid_type,'triangular')==1
    newz=lsopt.planet_radius-newz; % z is now a radius
end; % if strcmp(lower(lsopt.grid_type),'triangular')==1

%%%%%% new sampling completed.


%%% say goodbye!
mklsverbose(verbosity,lsopt,'MKLSGRIDNEWVERTICAL: created ',length(newz),' new vertical samples. Done.');












% % %%% plot all samples (all plotting is done in radius coord for triang. grid
% % hold on; 
% % handle=plot(lsopt.planet_radius-z,f,'g.-'); 
% % hold off;
% % drawnow;
% 
% 
% %%%%%% construct new sampling
% 
% %%% we have to distinguish several cases:
% %%% The input sampling might consist of a single depth sample.
% %%% If the input sampling consists of seceral samples, the sampling of the
% %%% maximum might be one of several cases:
% %%%     there might be several samples that hav a common max value
% %%%     there might be only a single sample with a max value
% %%%     the max value of the given samling might be only a local, not the
% %%%     global max.
% %%% And we have to keep in mind that we are not allowed to leave the
% %%% interval defined by Z!
% 
% 
% %%% determine the allowed interval
% %%% new z values below LOWBOUND or above HIBOUND are not acceptable!
% lowbound=min(z);
% hibound=max(z);
% 
% % %%% mark allowed interval
% % ax=axis;
% % hold on
% % plot(lsopt.planet_radius-[1 1]*lowbound,[ax(3) ax(4)],'r');
% % plot(lsopt.planet_radius-[1 1]*hibound,[ax(3) ax(4)],'r');
% % hold off
% % drawnow;
% 
% 
% %%% analyse the max and construct new samples
% zlen=length(z); % current number of samples
% if length(z)==1
%    %%% current samping contains only one sample - do not construct anything
%    newz=[];
%    newdr=NaN;
% else
%    %%% current sampling contains more than one sample
%    
%    %%% determine value and position of maxima
%    %%% MAXVAL is the maximal value of function f(z)
%    %%% MAXPOS is a list of index values: f(z) is max for z values on index
%    %%%        positions given in MAXPOS
%    maxval=max(f);
%    maxpos=find(f==maxval);
%    
% %    %%% mark max samples
% %    hold on; 
% %    plot(lsopt.planet_radius-z(maxpos),f(maxpos),'ms');
% %    plot(lsopt.planet_radius-z(maxpos),f(maxpos)*0,'m.');
% %    hold off;
% %    drawnow;
%    
%    %%% cases: how many points are max?
%    if length(maxpos)==1
%       %%% There is only a single max point
%       switch maxpos
%           case {1}
%                %%% the first sample is max
%                %%% construct a new sample half way to the second sample
%                newdr=abs(z(2)-z(1))/2;
%                newz=z(1)+newdr;
%           case {zlen}
%                %%% the last sample is max
%                %%% construct a new sample half way from the second last sample
%                newdr=abs(z(zlen)-z(zlen-1))/2;
%                newz=z(zlen)-newdr;
%           otherwise
%                %%% some interior sample is max
%                %%% construct two new samples approx. half way to the two neigbours
%                %%% newdr is the max of the two distances to the two
%                %%% neighbors
%                leftdist=abs(z(maxpos)-z(maxpos+1));
%                rightdist=abs(z(maxpos)-z(maxpos-1));
%                newz=[z(maxpos)+leftdist/2 z(maxpos)-rightdist/2];
%                newdr=max([leftdist rightdist]);
%                
% %                %%% mark position of meanz and delimiting samples
% %                hold on
% %                plot(lsopt.planet_radius-z(maxpos),z*0,'k+');
% %                plot(lsopt.planet_radius-z(maxpos-1),0,'k<');
% %                plot(lsopt.planet_radius-z(maxpos+1),0,'k>');
% %                hold off
% %                drawnow;
%                 
%       end; % switch maxpos
%       %%% done with the single max point case
%    else
%       %%% There are more than one max points. These might belong to several
%       %%% disconnected plateaus!
%       
%       
%       
%       %%% to make further work easier, we make all non-max samples zero
%       %%% and all max samples one.
%       f=f*0; % everything is flat now
%       f(maxpos)=f(maxpos)+1; % maxima restored but scaled to 1
%       
%       
% % %           %%% plot modified function graph
% % %           hold on
% % %           plot(lsopt.planet_radius-z,f,'g.-');
% % %           delete(handle);
% % %           hold off
% % %           drawnow;
%       
%             
%       
%       %%% determine limiting samples of all plateaus in which function is
%       %%% max. This also determines the number of plateaus.
%       %%% STARTS contains the first samples of all plateaus
%       %%% ENDS contains the last samples of all plateaus.
%       %%% f(starts(i)) and f(ends(i)) is one for all i.
%       %%% to further simplify identification of plateaus, we append
%       %%% two samples before and after the current sampling. This makes it
%       %%% easier to identify maxima that begin at the first or end at the
%       %%% last sample. The values of the new dummy samples are of minor
%       %%% importance, what counts is their existence.
%       difff=diff([0 f 0]);
%       starts=find(difff==1);
%       ends=find(difff==-1)-1;
% 
%       
% %       %%% mark positions of STARTS and ENDS
% %       hold on
% %       plot(lsopt.planet_radius-z(starts),z(starts)*0,'k<');
% %       plot(lsopt.planet_radius-z(ends),z(ends)*0,'k>');
% %       hold off
% %       drawnow;
%       
%       
%       %%% construct new samples
%       %%% The new sampling must do two things:
%       %%% - the sampling on the plateau as we know it must become more dense
%       %%% - it must be tested if the plateau is more wide as we currently see.
%       %%% For each plateau, the two samples with the largest distance in
%       %%% between is identified. If this distance is larger than the
%       %%% allowed minimum distance in lsopt.iter_drmin, a new sample in the
%       %%% middle between the two is inserted.
%       %%% Additionally, two samples are set left and right of the current
%       %%% margins of the plateau (the STARTS and ENDS samples).
%       
%       %%% loop over all plateaus
%       plateaucnt=length(starts); % number of plateaus
%       for plateauindy=1:plateaucnt
%           
%           %%% if there is only one sample on the current plateau, some
%           %%% extra handling is needed
%           if ends(plateauindy)-starts(plateauindy)==0
%               %%% plateau currently has only one sample
%               %%% This is the same situaton (within this plateau) as if
%               %%% there were only one max valued sample at all, as above.
%               
%               
%               %%% first we have to determine where the sample is in
%               %%% relation to the other samples in z
%               samplepos=starts(plateauindy);
%               
%               %%% now we can do essentially the same case distinction as
%               %%% with a single max sample. The code, however, must be a bit
%               %%% different since we are in a loop over all plateaus and do
%               %%% not want to overwrite new points from previous plateaus!
%               
%               switch samplepos
%                   case {1}
%                        %%% the first sample is max
%                        %%% construct a new sample half way to the second sample
%                        newdr=abs(z(2)-z(1))/2;
%                        newz=[newz z(1)+newdr];
%                   case {zlen}
%                        %%% the last sample is max
%                        %%% construct a new sample half way from the second last sample
%                        newdr=abs(z(zlen)-z(zlen-1))/2;
%                        newz=[newz z(zlen)-newdr];
%                   otherwise
%                       
%                        %%% some interior sample is max
%                        %%% construct two new samples approx. half way to the two neigbours
%                        %%% newdr is the max of the two distances to the two
%                        %%% neighbors
%                        leftdist=abs(z(samplepos)-z(samplepos+1));
%                        rightdist=abs(z(samplepos)-z(samplepos-1));
%                        newz=[newz z(samplepos)+leftdist/2 z(samplepos)-rightdist/2];
%                        newdr=max([newdr leftdist rightdist]);
% 
% %                        %%% mark position of meanz and delimiting samples
% %                        hold on
% %                        plot(lsopt.planet_radius-z(samplepos),z*0,'k+');
% %                        plot(lsopt.planet_radius-z(samplepos-1),0,'k<');
% %                        plot(lsopt.planet_radius-z(samplepos+1),0,'k>');
% %                        plot(lsopt.planet_radius-newz,newz*0,'rv');
% %                        hold off
% %                        drawnow;
% 
%               end; % switch samplepos
%               
%           else
%               %%% plateau currently has more than one sample
%           
%               %%% determine distances between current samples
%               %%% sampledistance(i) is the distance between z(i) and z(i+1)
%               sampledistance=diff(z(starts(plateauindy):ends(plateauindy)));
% 
%               %%% determine largest distance between neigbourhing samples
%               %%% the largest distance is between z(sampledistance(maxindy)) and its
%               %%% successor
%               [maxdistance,maxindy]=max(sampledistance);
% 
%               %%% if the max distance is larger than lsopt.iter_drmin, insert a
%               %%% new sample at the middle, and two samples right and left
%               %%% of the STARTS and ENDS positions.
%               if maxdistance>lsopt.iter_drmin
%                   
%                   %%% new point in the middle
%                   %%% distance is larger than lower limit, insert new point
%                   insertthis=z(starts(plateauindy)+maxindy)-maxdistance/2;
%                   newz=[newz insertthis];
%                   
%                            
%                 
%                   
%               end; % if maxdistance>lsopt.iter_drmin
%               
%               
%                %%% add new points at the plateau margins
%                margindistance=max(maxdistance,lsopt.iter_drmin);
%                insertthis=z(starts(plateauindy))-margindistance/2;
%                newz=[newz max(lowbound,insertthis)];
%                insertthis=z(ends(plateauindy))+margindistance/2;
%                newz=[newz insertthis];
%                
%                
%                
% %              %%% plot new samples on x axis
% %              hold on
% %              plot(lsopt.planet_radius-newz,0,'rv');
% %              hold off
% %              drawnow
%                
%           
%           end; % if ends(plateauindy)-starts(plateauindy)==1
%       end; % for plateauindy=1:plateaucnt
%       
%       
%    end; % if length(maxpos)
%    
% end; % if length(z);
% 
% 
% 
% 
% 
% 
% % %%%%%% create some additional random samples
% % 
% % %%% the deterministic part of the algorithm does usually not find plateaus
% % %%% which were not sampled in the very beginning. INtroduction of some
% % %%% random samples may reduce thi problem. MK07092005
% % 
% % %%% we will create as many random samples as we created determinsitic
% % %%% samples. They will be distributed uniformly over the entire allowed
% % %%% interval.
% % randomanz=length(newz);
% % randomz=rand(min([length(newz) 1]),1); %size(newz)); % uniform in [0 1]
% % randomz=(hibound-lowbound)*randomz+lowbound; % uniform in [lowbound hibound]
% % newz=[newz randomz];
% % 
% % %%%%%% random sample creation done 
% % 
% % 
% % %%%%%% new sampling complete
% 
% 
% 
% 
% 
% %%%%%% finally some polishing
% 
% 
% %%% remove all new samples outside the LOWBOUND-HIBOUND interval
% keep=find((newz>=lowbound)&(newz<=hibound));
% newz=newz(keep);
% 
% %%% make new sample list unique
% newz=unique(newz);
% %hold on; plot(lsopt.planet_radius-newz,newz*0,'mo'); hold off
% 
% 
% %%% remove z values from NEWZ that are already contained in the input z
% newz=setdiff(newz,z);
% 
% 
% % %%% plot z positions of old and new sampling
% % hold on
% % plot(lsopt.planet_radius-z,z*0,'b.');
% % plot(lsopt.planet_radius-newz,newz*0,'bo');
% % hold off
% % drawnow;
% 
% 
% %%%%%% place additional samples into sparsely sampled area(s)
% 
% %%% By introducing new samples in sparsely sampled areas, I hope to
% %%% discover currently unsampled plateaus.
% %%% ONE new sample is placed in the middle of the longest unsampled
% %%% interval.
% %%% This is done only if newz contains new samples, and only if the most
% %%% sparsely sampled interval is longer than lsopt.iter_drmin.
% 
% if length(newz)>0
% 
%     %%% the current sampling, including the new samples so far
%     allsamples=unique([z] ); % newz]);
% 
%     %%% the local sampling sparsity is given by the distance between samples
%     samplesparsity=diff(allsamples);
% 
%     %%% determine where sampling is most sparse
%     [maxsparse,maxsparsepos]=max(samplesparsity);
%     
%     
%     %%% is the sparsest interval longer than the min threshold?
%     if maxsparse>lsopt.iter_drmin
%     
%         %%% the new sample is to be placed between allsamples(maxsparsepos) and
%         %%% its successor. In the middle.
%         desparser=allsamples(maxsparsepos)+maxsparse/2;
% 
% %         %%% mark desparser position
% %         hold on
% %         plot(lsopt.planet_radius-desparser,-0.5,'c^');
% %         hold off
% %         drawnow;
% 
%         %%% add it to the new samples list
%         newz=[newz desparser];
%     
%     end; % if maxsparse>lsopt.iter_drmin
%     
%     %%% make new sample list unique again
%     newz=unique(newz);
% 
% end; % if length(newz)>0
% 
% %%%%%% additional samples completed.
% 
% 
% %%% convert depths into radii if necessary
% if strcmp(lower(lsopt.grid_type),'triangular')==1
%     newz=lsopt.planet_radius-newz; % z is now a radius
% end; % if strcmp(lower(lsopt.grid_type),'triangular')==1
% 
% %%%%%% final polishing done
% 
% 
% 
% 
% 
% %%% say goodbye!
% mklsverbose(verbosity,lsopt,'MKLSGRIDNEWVERTICAL: created ',length(newz),' new vertical samples. Done.');