function handle=mklsgridplottrilayer(lsopt,layer,mode);
% mklsgridplottrilayer.......plot single layer of triangular search grid
%
% call: handle=mklsgridplottrilayer(lsopt,layer,mode);
%
%       lsopt: (struct)
%              LOCSMITH options structure as returned by MKLSOPT     
%       layer: (struct)
%              one layer of a triangular LOCSMITH search grid 
%        mode: (string)
%              plot mode
%              This string defineshow to plot the grid. Several options
%              exist:
%              'none': no plot is generated.
%              'cloud': point cloud: just the nodes themselves, using
%                       the dot-marker, creates a line object.
%              'mesh':  wire frame plot, creates a patch object
%              'surface':  surface plot, creates a patch object
%              'meshmap':   map wire frame mesh as map plot according,
%                       to map options in lsopt.mapopt.
%                       Just plots the node mesh. It is assumed that a base
%                       map with topography etc. already exists!
%              'cloudmap': as 'meshmap', but does not plot edges between
%                       nodes. All nodes are plotted with the same color.
%              'fitcloudmap': as 'cloudmap', but color of nodes is
%                             determined according to the .fit field 
%                             of the layer.
%                             uses open symbols for inacceptable nodes and
%                             filled symbols for acceptable ones.
%              'fitsurface': as 'surface', but color of nodes is determined
%                            according to the .fit field of the layer.
%              'fitcloud': as 'cloud', but color of nodes is determined
%                          according to the .fit field of the layer.
%                          uses open symbols for inacceptable nodes and
%                          filled symbols for acceptable ones.
%
%              Map plots use the layer radius (in km) as vertical
%              coordinate. This guarantees that layers are separated and in
%              the correct order.
% 
% result: handle: (numeric array) handles to the created graphics objects.
%                 The type of the generated graphics object depends on the
%                 MODE parameter. please check the types before using the
%                 handles!
%
% Note that this routine does NOT set hold "on"!
%
% Plot appearance is controlled by several values defined in MKLSCONSTANTS:
% Transparency, FaceColor, EdgeColor, SphereFacets
%
% Martin Knapmeyer, 10.09.2004, 20.03.2007, 09.05.2007

% 09052007: open symbols for inacceptable nodes, filled for acceptable

%%% a useful constant
radian=pi/180;

%%% verbosity level
verbosity=5;

%%% init result
handle=[];


%%% add node numbers to plot? MK20032007
addnodenumbers=0;
nodenumberfont=10;


mode=lower(mode);
switch mode
    case {'none'}
        %%% do nothing!
    case {'cloud'}
        
        mklsverbose(verbosity,lsopt,'MKLSGRIDPLOTTRILAYER: plotting ',...
                    mode,...
                    ' into figure ',gcf);
                
        %%% some specs for the plot
        nodecolor=mklsconstants('NodeColor');
        nodemarker=mklsconstants('NodeMarker');
                
       %%% transform lon/lat coordinates into cartesian plot co'
       [x,y,z]=sph2cart(layer.nodes(:,2)*radian,...
                 layer.nodes(:,3)*radian,...
                 layer.radius);

        
        %%% just plot the points
        cloudhandle=plot3(x,y,z,nodemarker);
        set(cloudhandle,'color',nodecolor);
        
        %%% plot a sphere with layer radius
        transparency=mklsconstants('Transparency');
        spherefacets=mklsconstants('SphereFacets');
        facecolor=mklsconstants('FaceColor');
        edgecolor=mklsconstants('EdgeColor');
        spherelighting=mklsconstants('Lighting');
        [x,y,z]=sphere(spherefacets);
        x=x*layer.radius;
        y=y*layer.radius;
        z=z*layer.radius;
        hold on
        spherehandle=surf(x,y,z);
        hold off
        set(spherehandle,'FaceColor',facecolor);
        set(spherehandle,'EdgeColor',edgecolor);
        set(spherehandle,'FaceAlpha',transparency);
        set(spherehandle,'EdgeAlpha',transparency);
        set(spherehandle,'FaceLighting',spherelighting);
        set(spherehandle,'Edgelighting',spherelighting);
        
        %%% compose output handle list
        handle=[cloudhandle spherehandle];
        
        %%% special plot beautification
        mkfiatlux;
        material dull
        
    case {'mesh'}
        
        mklsverbose(verbosity,lsopt,'MKLSGRIDPLOTTRILAYER: plotting ',...
                    mode,...
                    ' into figure ',gcf);
                
        %%% transform lon/lat coordinates into cartesian plot co'
        [x,y,z]=sph2cart(layer.nodes(:,2)*radian,...
                 layer.nodes(:,3)*radian,...
                 layer.radius);

        
        %%% some specs for the plot
        edgecolor=mklsconstants('EdgeColor');
        
        %%% create triangle list     
        trilist=mklsgridtrinodeid2row(layer);
              
        %%% plot edges
        handle=trimesh(trilist,x,y,z,'FaceColor','None','EdgeColor',edgecolor);
        
        %%% add node numbers
        if addnodenumbers==1
            for nodecnt=1:length(x)
            text(x(nodecnt),y(nodecnt),z(nodecnt),...
                 int2str(nodecnt),...
                 'Color','r',...
                 'FontSize',nodenumberfont);
            end; % for nodecnt
        end; % if addnodenumbers==1
        
        
    case {'surface'}
        
        mklsverbose(verbosity,lsopt,'MKLSGRIDPLOTTRILAYER: plotting ',...
                    mode,...
                    ' into figure ',gcf);
        
        %%% transform lon/lat coordinates into cartesian plot co'
        [x,y,z]=sph2cart(layer.nodes(:,2)*radian,...
                 layer.nodes(:,3)*radian,...
                 layer.radius);

        
        %%% some specs for the plot
        facecolor=mklsconstants('FaceColor');
        edgecolor=mklsconstants('EdgeColor');
        surflighting=mklsconstants('Lighting');
        
        %%% create triangle list  
        trilist=mklsgridtrinodeid2row(layer);
              
        %%% the plotting itself
        handle=trisurf(trilist,x,y,z);
        set(handle,'FaceColor',facecolor);
        set(handle,'EdgeColor',edgecolor);
        set(handle,'FaceLighting',surflighting);
        set(handle,'Edgelighting',surflighting);
        
        
        
        %%% add node numbers
        if addnodenumbers==1
            for nodecnt=1:length(x)
             text(x(nodecnt),y(nodecnt),z(nodecnt),...
                  int2str(nodecnt),...
                  'Color','r',...
                  'FontSize',nodenumberfont);
            end; % for nodecnt
        end; % if addnodenumbers==1
        
        %%% special plot beautification
        mkfiatlux;
        colormap(white);
        material dull
        
    case {'meshmap'}
        
        %mklsnotyet('crash');
        
        mklsverbose(verbosity,lsopt,'MKLSGRIDPLOTTRILAYER: plotting ',...
                    mode,...
                    ' into figure ',gcf);
                
        %%% apply map projection to node coordinates
        [x,y]=mklonlat2xy(layer.nodes(:,2),...
                          layer.nodes(:,3),...
                          lsopt.mapopt,'forward');
        z=x*0+layer.radius;
                
        %%% some specs for the plot
        edgecolor=mklsconstants('EdgeColor');
        
        %%% create triangle list     
        trilist=mklsgridtrinodeid2row(layer);
        %trilist=delaunay(x,y);
              
        %%% plot edges
        handle=trimesh(trilist,x,y,z,'FaceColor','None','EdgeColor',edgecolor);
        
        view(2);
        
    case {'cloudmap'}
        
        mklsverbose(verbosity,lsopt,'MKLSGRIDPLOTTRILAYER: plotting ',...
                    mode,...
                    ' into figure ',gcf);
                
        %%% apply map projection to node coordinates
        [x,y]=mklonlat2xy(layer.nodes(:,2),...
                          layer.nodes(:,3),...
                          lsopt.mapopt,'forward');
        z=x*0+layer.radius;
                
        %%% some specs for the plot
        nodecolor=mklsconstants('NodeColor');
        nodemarker=mklsconstants('NodeMarker');
              
        %%% plot nodes
        handle=plot3(x,y,z,nodemarker); 
        set(handle,'Color',nodecolor);
        
        view(2);
        
    case {'fitcloudmap'}
        
        mklsverbose(verbosity,lsopt,'MKLSGRIDPLOTTRILAYER: plotting ',...
                    mode,...
                    ' into figure ',gcf);
                
        %%% some specs for the plot       
        nodemarker=mklsconstants('NodeMarker');
                
        %%% apply map projection to node coordinates
        [x,y]=mklonlat2xy(layer.nodes(:,2),...
                          layer.nodes(:,3),...
                          lsopt.mapopt,'forward');
        z=x*0+layer.radius;
                
        %%% construct a color table to be used
        %%% has as many elements as tests were carried out for nodes
        maxtest=max(layer.tested(:,2));
        rgb=jet(1+maxtest);
        colormap(rgb);
        
              
        %%% plot nodes
        maxfit=max(layer.fit(:,2)); % maximum fitness value
        hold on
        for indy=0:maxfit
            subset=find(layer.fit(:,2)==indy);
            z=x(subset)*0+layer.radius;
            pointhandle=plot3(x(subset),y(subset),z,nodemarker);
            if indy==maxtest
                %%% use filled symbol for acceptable (100% fit) nodes MK09052007
                set(pointhandle,'MarkerFaceColor',rgb(indy+1,:));
            else
               %%% use open symbol for not acceptable nodes  MK09052007
               set(pointhandle,'MarkerFaceColor','none');
            end; % if indy==maxtest
            set(pointhandle,'MarkerEdgeColor',rgb(indy+1,:));
            handle=[handle pointhandle(:)'];
        end; % for indy
        hold off
%         hold on
%         for indy=1:length(x);
%             vertexcolor=layer.fit(indy,2)+1;
%             pointhandle=plot(x(indy),y(indy),'.');
%             set(pointhandle,'MarkerFaceColor',rgb(vertexcolor,:));
%             set(pointhandle,'MarkerEdgeColor',rgb(vertexcolor,:));
%             handle=[handle pointhandle];
%         end; % for indy
%         hold off
        %vertexcolor=layer.fit(:,2)+1;
        %handle=fill(x,y,vertexcolor);
        %set(handle,'Marker','.');
        %set(handle,'LineStyle','none');
        caxis([0 maxfit]);
        %colorbar('hor');
        view(2);
        
        
    case {'fitsurface'}
        
        
        mklsverbose(verbosity,lsopt,'MKLSGRIDPLOTTRILAYER: plotting ',...
                    mode,...
                    ' into figure ',gcf);
                 
        
        %%% transform lon/lat coordinates into cartesian plot co'
        [x,y,z]=sph2cart(layer.nodes(:,2)*radian,...
                 layer.nodes(:,3)*radian,...
                 layer.radius);
             
                 
        %%% construct a color table to be used
        %%% has as many elements as tests were carried out for nodes
        rgb=jet(1+max(layer.tested(:,2)));
        colormap(rgb);
        
       
        %%% some specs for the plot
        facecolor=mklsconstants('FaceColor');
        edgecolor=mklsconstants('EdgeColor');
        surflighting=mklsconstants('Lighting');
        
        %%% create triangle list  
        trilist=mklsgridtrinodeid2row(layer);
              
        %%% the plotting itself
        handle=trisurf(trilist,x,y,z,layer.fit(:,2));
        set(handle,'EdgeColor',edgecolor);
        set(handle,'FaceLighting',surflighting);
        set(handle,'Edgelighting',surflighting);
        set(handle,'FaceColor','interp');
        set(handle,'FaceAlpha',0.3);
        set(handle,'EdgeAlpha',0.1);
        
        %%% add node numbers
        if addnodenumbers==1
            for nodecnt=1:length(x)
             text(x(nodecnt),y(nodecnt),z(nodecnt),...
                  int2str(nodecnt),...
                  'Color','r',...
                  'FontSize',nodenumberfont);
            end; % for nodecnt
        end; % if addnodenumbers
        
        
        %%% set view direction to show solution area
        %%% first determine where solution area is (or which is currently the
        %%% most interesting area, according to fit), then set view
        fitting=find(layer.fit(:,2)==max(layer.fit(:,2)));
        az=layer.nodes(fitting(1),2);
        elv=layer.nodes(fitting(1),3);
        if ~isempty(fitting)
           view(az+90,elv);
        end; % if ~isempty(fitting)
        
        %%% switch on a light right "above" the solution area
        %%% (tricky to determine the right coordinates... these are not 
        %%%  quite correct, but are sufficient for now)
        mkfiatlux('off'); % first switch all light off
        [lx,ly,lz]=mkfastsph2cart(az+180,-elv);
        mkfiatlux('on','Position',[lx ly lz],'Style','infinite');
        
        %%% special plot beautification
        material dull
        
    case {'fitcloud'}
        
        mklsverbose(verbosity,lsopt,'MKLSGRIDPLOTTRILAYER: plotting ',...
                    mode,...
                    ' into figure ',gcf);
                
        %%% define symbol to be used for nodes
        nodemarker=mklsconstants('NodeMarker');
                
                
        %%% transform lon/lat coordinates into cartesian plot co'
        [x,y,z]=sph2cart(layer.nodes(:,2)*radian,...
                  layer.nodes(:,3)*radian,...
                  layer.radius);
              
        %%% construct a color table to be used
        %%% has as many elements as tests were carried out for nodes
        maxtest=max(layer.tested(:,2));
        rgb=jet(1+maxtest);
        colormap(rgb);          
        
        %%% plot nodes
        maxfit=max(layer.fit(:,2)); % maximum fitness value
        hold on
        for indy=0:maxfit
            subset=find(layer.fit(:,2)==indy);
            pointhandle=plot3(x(subset),y(subset),z(subset),nodemarker);
            set(pointhandle,'MarkerEdgeColor',rgb(indy+1,:));
            if indy==maxtest
                %%% use filled symbol for acceptable (100% fit) nodes MK09052007
                set(pointhandle,'MarkerFaceColor',rgb(indy+1,:));
            else
               %%% use open symbol for not acceptable nodes  MK09052007
               set(pointhandle,'MarkerFaceColor','none');
            end; % if indy==maxtest
            handle=[handle pointhandle];
        end; % for indy
        hold off
        
%         %%% plot nodes
%         hold on
%         for indy=1:length(x);
%             vertexcolor=layer.fit(indy,2)+1;
%             pointhandle=plot3(x(indy),y(indy),z(indy),'.');
%             set(pointhandle,'MarkerFaceColor',rgb(vertexcolor,:));
%             set(pointhandle,'MarkerEdgeColor',rgb(vertexcolor,:));
%             handle=[handle pointhandle];
%         end; % for indy
%         hold off
        
        %%% plot a sphere with layer radius
        transparency=mklsconstants('Transparency');
        spherefacets=mklsconstants('SphereFacets');
        facecolor=mklsconstants('FaceColor');
        edgecolor=mklsconstants('EdgeColor');
        spherelighting=mklsconstants('Lighting');
        [x,y,z]=sphere(spherefacets);
        x=x*layer.radius;
        y=y*layer.radius;
        z=z*layer.radius;
        hold on
        spherehandle=surf(x,y,z);
        hold off
        set(spherehandle,'FaceColor',facecolor);
        set(spherehandle,'EdgeColor',edgecolor);
        set(spherehandle,'FaceAlpha',transparency);
        set(spherehandle,'EdgeAlpha',transparency);
        set(spherehandle,'FaceLighting',spherelighting);
        set(spherehandle,'Edgelighting',spherelighting);
        
        %%% compose output handle list
        handle=[handle spherehandle];
        
        %%% set view direction to show solution area
        %%% first determine where solution area is (or which is currently the
        %%% most interesting area, according to fit), then set view
        fitting=find(layer.fit(:,2)==max(layer.fit(:,2)));
        az=layer.nodes(fitting(1),2);
        elv=layer.nodes(fitting(1),3);
        if ~isempty(fitting)
           view(az+90,elv);
        end; % if ~isempty(fitting)
        
        %%% switch on a light right "above" the solution area
        %%% (tricky to determine the right coordinates... these are not 
        %%%  quite correct, but are sufficient for now)
        mkfiatlux('off'); % first switch all light off
        [lx,ly,lz]=mkfastsph2cart(az+180,-elv);
        mkfiatlux('on','Position',[lx ly lz],'Style','infinite');
        
        %%% special plot beautification
        material dull
        
        

        
    otherwise
        error(['MKLSGRIDPLOTTRILAYER: unknown plot mode ' upper(mode)]);
end; % switch mode


%%% general plot beautification
if strcmp(mode,'none')~=1
    axis tight
    axis equal
    axis vis3d
    axis off
end; % if strcmp(mode
