function [ccrlist,ratios]=mklsgridreftri(gridbase,maxiter,doplot);
% mklsgridreftri.......determine max circumcircle radii at different iterations
%
% call: [ccrlist,ratios]=mklsgridreftri(gridbase,maxiter,doplot);
%
%                     gridbase: (string)
%                               this string deterines which grid base is 
%                               used. This must be one of the possible
%                               values of the .grid_base field of the
%                               LOCSMITH options structure (see
%                               MKLSDEFAULTS).
%                      maxiter: (numeric)
%                               maximum number of iterations applied to the
%                               grid.
%                       doplot: (string)
%                               if present, this must be a string that is
%                               accepted as plotmode by MKLSGRIDPLOT.
%
% result: ccrlist: (numeric array) [degree]
%                  List of the determined circumcircle radii.
%                  ccrlist(i) is the maximum circumcircle radius in degrees
%                  of the grid after i-1 iterations (the 0-th iteration is 
%                  the initial grid).
%                  Length is measured as great circle arc length.
%
%         ratios: (numeric array) []
%                 List of ratios of circumcircle radii:
%                 ratios(i) is the ratio of the max circumcircle radius
%                 after i iterations ot the max circumcircle radius after
%                 i-1 iterations. This list is derived from CCRLIST.
%
% This routine determines how the maximum circumcircle radius of triangles
% in a grid of % specified type changes with iteration, for a specified number of
% iterations. These factors are needed for the grid construction, when the
% number of iterations necesssary to reach a certain spatial resolution has
% to be estimated.
% The grid is iterated step by step until the specified max number of
% iterations is reached, and all circumcircles are evaluated at each
% iteration.
%
% If no output argument is given, the max circumcircle radii are
% written to the screen
%
% Martin Knapmeyer, 23.09.2004 totally rewritten 17.01.2005

%%% to measure the wasted time
tic;

%%% how many input/output parms?
nin=nargin;
nout=nargout;

%%% construct a test grid
lsopt=mklsopt('grid_base',gridbase,...
              'iter_circmin',0,...
              'verbosity',0);
grid=mklsgridnew(lsopt);

% %%% km-to-degree factor for outermost layer
% kmfactor=2*pi*grid.layers(end).radius/360;

%%% factor controlling roundoff precision
rounding=mklsconstants('ReftriRoundoff');


%%% init list to collect all max edge lengths
ccrlist=[];

%%% loop over grid iterations
for iter=0:maxiter
    
    %%% apply one iteration to one triangle of one layer of grid
    %%% This saves much time and memory!
    if iter>0
       idlist.layer=newgrid.layers(end).id;
       idlist.triangle=newgrid.layers(end).tri(1).id;
       newgrid=mklsgridmodify(lsopt,newgrid,idlist,'iterate');
    else
        %%% this is to have the original grid in the evaluation!
        newgrid=grid;
    end; % if iter>0
    
    %%% plot outermost layer
    if nin==3
        mklsgridplottrilayer(lsopt,newgrid.layers(end),doplot);
        light;
        drawnow;
    end; % if nin==3
    
    %%% determine all edge lengths of outermost layer
    lengthlist=mklsgridstattrilayer(lsopt,newgrid.layers(end),'circle');
    
    %%% reduce edge lengths from km to degrees
    %lengthlist=lengthlist/kmfactor;
    
    %%% the largest triangle is one of the triangles that were not iterated.
    %%% to find the largest of the iterated triangles, we simply have
    %%% to search the second largest circumcircle. Due to roundoff problems, we
    %%% first round the radii, then construct a unique list and take
    %%% the second largest element of this. Note that unique() sorts the
    %%% list in ascending order!
    lengthlist=fix(lengthlist*rounding)/rounding;
    lengthlist=unique(lengthlist);
    %maxlength=max(lengthlist);
    if length(lengthlist)>1
       maxlength=lengthlist(end-1);
    else
       maxlength=lengthlist;
    end; % if length(lengthlist)>1
    
    %%% add max edge length to list
    ccrlist=[ccrlist maxlength];
    
    %%% if no output argument given, write maxlist to screen
    if nout==0
       if iter==0
           ratio=1;
       else
           ratio=ccrlist(end-1)/ccrlist(end);
       end; % if iter==0
       disp(['MKLSGRIDREFTRI: iteration ',...
              int2str(iter),...
              ', max cc radius ',...
              sprintf('%13.10f',ccrlist(end)),...
              ' ratio last/this: ',...
              sprintf('%13.10f',ratio)]);
    end; % if nout==0
    
    
end; % for iter


%%% determine the desired ratios
ratios=ccrlist(1:(end-1))./ccrlist(2:end);

%%% how long did it take?
toctime=toc;
disp(['MKLSGRIDREFTRI: elapsed time ' num2str(toctime) 'sec.']);

