function handle=mklsgridresplottri(lsopt,mygrid,mode,complevel);
% mklsgridresplottri...........plot LOCSMITH result
%
% call: handle=mklsgridresplottri(lsopt,mygrid,mode,complevel);
%
%       lsopt: (struct)
%              LOCSMITH options structure as returned by MKLSOPT            
%        mygrid: (struct)
%              LOCSMITH search grid
%        mode: (string)
%              plot mode
%              This string defines how to plot the grid. 
%              
%              'none': no plot is generated.
%              'fitcloudmap': plots map of all nodes, but uses vertical
%                             axis for the achieved compatibility level.
%                             Only points with compatibility > COMPLEVEL
%                             are plotted. This map shows you which are the
%                             best epicenters, and which compatibility
%                             levels should be used with the other modes.
%                             It is therefore recommended to use
%                             complevel=0 for 'fitcloudmap'.
%              'otcloudmap': plots map of acceptable nodes, with color determined
%                            by origin time.
%              'otcloud': plot 3D sphere with acceptable nodes inside. Color is
%                         determined by origin time.
%              'depthcloudmap': plots map of acceptable nodes, with color determined
%                         by node depth.
%
%        complevel: (number) [counter]
%                   compatibility level. All nodes with compatibility equal
%                   or larger than this will be plotted.
%                   Enter empty to get only nodes with (unknown) max
%                   compatibility.
%
% result: handle: list of handles of all created graphics objects
%                  The type of the created objects depends on the MODE
%
% Martin Knapmeyer, 31.05.2005

%%% a useful constant
radian=pi/180;

%%% verbosity level
verbosity=5;

%%% init result
handle=[];


%%% general preparation: 


%%% general preparations
mode=lower(mode);
switch mode
    case {'none'}
         %%% nothing happens
    case {'fitcloudmap'}
        
        %%% extract all nodes with .fit>COMPLEVEL from MYGRID
        %%% loop through all layers and collect them nodes
        %%% NODELIST will be a list of nodes
        %%%          nodelist(:,1): fitness of node
        %%%          nodelist(:,2): longitude of node
        %%%          nodelist(:,3): latitude of node
        nodelist=[]; % to collect all nodes
        lyrcnt=length(mygrid.layers); % number of layers in grid
        for lyrindy=1:lyrcnt
            
            %%% sort nodes by ID
            [idlist,sorter]=sort(mygrid.layers(lyrindy).fit(:,1));
            mygrid.layers(lyrindy).fit=mygrid.layers(lyrindy).fit(sorter,:);
            mygrid.layers(lyrindy).nodes=mygrid.layers(lyrindy).nodes(sorter,:);
            %%% identify nodes with acceptable fitness
            accept=find(mygrid.layers(lyrindy).fit(:,2)>=complevel);
            
            if ~isempty(accept)
               newnodes=mygrid.layers(lyrindy).nodes(accept,:);
               newnodes(:,1)=mygrid.layers(lyrindy).fit(accept,2);
               nodelist=[nodelist; newnodes];
            end; % is ~isempty(accept)
            
        end; % for lyrindy
         
        
    case {'otcloudmap','otcloud'}
        %%% extract nodes with maximum compatibility from grid
        loc=mklsgridextractloc(lsopt,mygrid,complevel);
        
        if isempty(loc.nodelist)
            
            error(['MKLSGRIDRESPLOTTRI: no acceptable nodes found!']);
            
        else
        
            %%% sort nodes, fitnesses and times by node ID. This
            %%% makes later access easier.
            lyrcnt=length(mygrid.layers);
            for indy=1:lyrcnt
                %%% sort nodelist and origin time list of layer by node ID
                %%% This is necessary to be sure that nodes and times are "parallel"
                %%% with respect to array index.
                %%% 1.) sort nodes
                [mygrid.layers(indy).nodes(:,1),sorter]=sort(mygrid.layers(indy).nodes(:,1));
                mygrid.layers(indy).nodes(:,2)=mygrid.layers(indy).nodes(sorter,2);
                mygrid.layers(indy).nodes(:,3)=mygrid.layers(indy).nodes(sorter,3);
                %%% 2.) sort origin times
                idlist=zeros(length(mygrid.layers(indy).times),1);
                for timeindy=1:length(idlist)
                    idlist(timeindy)=mygrid.layers(indy).times(timeindy).id;
                end; % for indy
                [idlist,sorter]=sort(idlist);
                mygrid.layers(indy).times=mygrid.layers(indy).times(sorter);
                %%% 3.) sort fitness list
                [mygrid.layers(indy).fit(:,1),sorter]=sort(mygrid.layers(indy).fit(:,1));
                mygrid.layers(indy).fit(:,2)=mygrid.layers(indy).fit(sorter,2);     
            end; % for indy

            %%% identify earliest and latest possible origin time, needed to
            %%% scale the colormap
       disp('verwechsele ich hier earliest und latest??');
            rgblen=mklsconstants('ColorMapLength'); % colormap length
            rgb=jet(rgblen);
            colormap(rgb);
            earliest=inf;
            latest=-inf;
            plottime=[];
            for indy=1:length(loc.origintime)
                if ~isempty(loc.origintime(indy).starts)
                    plottime=[plottime min(loc.origintime(indy).starts)];
                else
                    plottime=[plottime NaN];
                end; % if ~isempty
                earliest=min([earliest loc.origintime(indy).starts]);
                latest=max([latest loc.origintime(indy).ends]);
            end % for indy
            dt=(latest-earliest)/rgblen;
        
        end;
        
    case {'depthcloudmap'}
        
        %%% extract nodes with maximum compatibility from grid
        loc=mklsgridextractloc(lsopt,mygrid,complevel);
        
        if isempty(loc.nodelist)
            
            error(['MKLSGRIDRESPLOTTRI: no acceptable nodes found!']);
            
        else
     
        
            %%% identify deepest and shallowest node, needed to
            %%% scale the colormap
            rgblen=mklsconstants('ColorMapLength'); % colormap length
            rgb=jet(rgblen);
            colormap(rgb);
            deepest=lsopt.mapopt.planet_radius-min(loc.nodelist(:,3));
            shallowest=lsopt.mapopt.planet_radius-max(loc.nodelist(:,3));
            if deepest==shallowest
               deepest=shallowest+10;
            end; % if deepest==shallowest
            dr=abs(shallowest-deepest)/rgblen;
        
        end; % if isempty(loc.nodelist)
        
        
    otherwise
        error(['MKLSGRIDPLOTTRI: unknown plot mode ' upper(mode)]);
end; % switch



%%% the plotting itself
mode=lower(mode);
switch mode
    case {'none'}
         %%% nothing happens
    case {'fitcloudmap'}
        
        %%% plot base map
        mkmakemap(lsopt.mapopt);

        %%% define symbol to be used for nodes
        nodemarker=mklsconstants('NodeMarker');
        
        
         %%% loop over compatibility levels existing in nodelist
         complevels=unique(nodelist(:,1));
         complevelcnt=length(complevels);
         rgb=jet(complevelcnt);
         for complevelindy=1:complevelcnt
             
             %%% identify nodes with this compatibility
             accept=find(nodelist(:,1)==complevels(complevelindy));
             if ~isempty(accept)
                 
                %%% extract coordinates 
                lon=nodelist(accept,2);
                lat=nodelist(accept,3);
                lvl=nodelist(accept,1);
                
                %%% apply map projection
                [x,y]=mklonlat2xy(lon,lat,lsopt.mapopt,'forward');
                
                %%% plot nodes
                hold on
                pointhandle=plot3(x,y,lvl,'.');
                hold off
                handle=[handle pointhandle];
                set(pointhandle,'MarkerFaceColor',rgb(complevelindy,:));
                set(pointhandle,'MarkerEdgeColor',rgb(complevelindy,:));
                set(pointhandle,'UserData',complevels(complevelindy));
                
             end; % if ~isempty
             
         end; % for complevelindy
         
    case {'otsolution'}
         mklsnotyet('warn');
    case {'otcloudmap'}
        
        mklsverbose(verbosity,lsopt,'MKLSGRIDRESPLOTTRI: plotting ',...
                    mode,...
                    ' into figure ',gcf);
        
                    
        %%% plot base map
        mkmakemap(lsopt.mapopt);
        
        %%% define symbol to be used for nodes
        nodemarker=mklsconstants('NodeMarker');
                    
        %%% apply map projection to node coordinates
        [x,y]=mklonlat2xy(loc.nodelist(:,1),...
                          loc.nodelist(:,2),...
                          lsopt.mapopt,'forward');
        z=loc.nodelist(:,3);
        
               
        %%% plot nodes, colored by earlies possible origin time
        hold on
        for indy=0:(rgblen-1)
            subset=find((plottime>=indy*dt+earliest)&(plottime<=(indy+1)*dt+earliest));
            if ~isempty(subset)
                %z=x(subset)*0+layer.radius;
                pointhandle=plot3(x(subset),y(subset),z(subset),nodemarker);
                set(pointhandle,'MarkerFaceColor',rgb(indy+1,:));
                set(pointhandle,'MarkerEdgeColor',rgb(indy+1,:));
                handle=[handle pointhandle];
            end; % if ~isempty(subset)
        end; % for indy
        hold off
                       
        %%% plot beautification
        caxis(sort([earliest latest]));
        cbarhandle=colorbar('hor');
        namehandle=get(cbarhandle,'XLabel');
        set(namehandle,'string','Origin Time Offset [s]');
        view(2);
        xlabel('Longitude');
        ylabel('Latitude');
        
        
  case {'otcloud'}
        
        mklsverbose(verbosity,lsopt,'MKLSGRIDPLOTTRILAYER: plotting ',...
                    mode,...
                    ' into figure ',gcf);
          
        
        %%% plot a sphere with planet radius
        transparency=mklsconstants('Transparency');
        spherefacets=mklsconstants('SphereFacets');
        facecolor=mklsconstants('FaceColor');
        edgecolor=mklsconstants('EdgeColor');
        spherelighting=mklsconstants('Lighting');
        [x,y,z]=sphere(spherefacets);
        x=x*lsopt.planet_radius;
        y=y*lsopt.planet_radius;
        z=z*lsopt.planet_radius;
        hold on
        spherehandle=surf(x,y,z);
        hold off
        set(spherehandle,'FaceColor',facecolor);
        set(spherehandle,'EdgeColor',edgecolor);
        set(spherehandle,'FaceAlpha',transparency);
        set(spherehandle,'EdgeAlpha',transparency);
        set(spherehandle,'FaceLighting',spherelighting);
        set(spherehandle,'Edgelighting',spherelighting);
        
        
        %%% define symbol to be used for nodes
        nodemarker=mklsconstants('NodeMarker');
        
        %%% transform lon/lat coordinates of nodes into cartesian plot co'
        [x,y,z]=sph2cart(loc.nodelist(:,1)*radian,...
                        loc.nodelist(:,2)*radian,...
                        loc.nodelist(:,3));
                    
                        
        %%% plot nodes, colored by earlies possible origin time
        hold on
        for indy=0:(rgblen-1)
            subset=find((plottime>=indy*dt+earliest)&(plottime<=(indy+1)*dt+earliest));
            if ~isempty(subset)
                %z=x(subset)*0+layer.radius;
                pointhandle=plot3(x(subset),y(subset),z(subset),nodemarker);
                set(pointhandle,'MarkerFaceColor',rgb(indy+1,:));
                set(pointhandle,'MarkerEdgeColor',rgb(indy+1,:));
                handle=[handle pointhandle];
            end; % if ~isempty(subset)
        end; % for indy
        hold off
        
        %%% compose output handle list
        handle=[handle spherehandle];
        
        %%% special plot beautification
        if (~isinf(earliest))&(~isinf(latest))
            caxis(sort([earliest latest])); % not correct, but stable
            cbarhandle=colorbar('hor');
            namehandle=get(cbarhandle,'XLabel');
            set(namehandle,'string','Origin Time Offset [s]');
        end; % if ~isinf
        axis vis3d
        axis off
        mkfiatlux;
        material dull
        
        
    case {'depthcloudmap'}
        
        mklsverbose(verbosity,lsopt,'MKLSGRIDRESPLOTTRI: plotting ',...
                    mode,...
                    ' into figure ',gcf);
        
                    
        %%% plot base map
        mkmakemap(lsopt.mapopt);
        
        %%% define symbol to be used for nodes
        nodemarker=mklsconstants('NodeMarker');
                    
        %%% apply map projection to node coordinates
        [x,y]=mklonlat2xy(loc.nodelist(:,1),...
                          loc.nodelist(:,2),...
                          lsopt.mapopt,'forward');
        z=lsopt.mapopt.planet_radius-loc.nodelist(:,3);
        
               
        %%% plot nodes, colored by z coordinate
        hold on
        for indy=0:(rgblen-1)
            subset=find((z>=indy*dr+shallowest)&(z<=(indy+1)*dr+shallowest));
            if ~isempty(subset)
                %z=x(subset)*0+layer.radius;
                pointhandle=plot3(x(subset),y(subset),...
                            lsopt.mapopt.planet_radius-z(subset),...
                            nodemarker);
                set(pointhandle,'MarkerFaceColor',rgb(indy+1,:));
                set(pointhandle,'MarkerEdgeColor',rgb(indy+1,:));
                handle=[handle pointhandle];
            end; % if ~isempty(subset)
        end; % for indy
        hold off
                       
        %%% plot beautification
        caxis([shallowest deepest]);
        cbarhandle=colorbar('hor');
        namehandle=get(cbarhandle,'XLabel');
        set(namehandle,'string','Focal Depth [km]');
        view(2);
        xlabel('Longitude');
        ylabel('Latitude');
        
    otherwise
         error(['MKLSGRIDRESPLOTTRI: unknown plot mode ' upper(mode)]);
end; % switch