function reslist=mklsgridstattrilayer(lsopt,layer,mode);
% mklsgridstattrilayer.......statistical analysis of triangular layer
%
% call: reslist=mklsgridstattrilayer(lsopt,layer);
%       reslist=mklsgridstattrilayer(lsopt,layer,mode);
%
%               lsopt: (struct)
%                      LOCSMITH options structure as returned by MKLSOPT
%               layer: (struct)
%                      layer structure as returned by MKGRIDNEWTRILAYER
%                mode: (string)
%                      evaluation mode. The foloowing modes are available:
%                      'length': edge lengths in km are determined
%                                (only for triangles that have nodes as
%                                children)
%                      'circle': circlumcircle radii in degrees are determined
%                                (only for triangles that have nodes as
%                                children)
%                      'circlekm': as 'circle', but with radii in km
%                      'iterations': the number of iterations applied to
%                                    each triangle
%                      DEFAULT: 'circle'
%
% result: reslist: (numeric array)
%                  vector containing lenghts or radii (depending on MODE)
%                  of all edges/triangles. Lengths are in km or degrees 
%                  depending on MODE.
%                  From this, histograms may be produced by hist(reslist);
%
% performs statistical analyses of the point distribution in a layer:
% - triangle area (spherical triangle, in km^2)
% - point distance (great circle distance in km)
%
% Analyzes only triangles with childtype=='node'. Triangles with childtype
% 'triangle' are ignored. Other childtypes cause errors.
%
% Plots are produced if no output parameter is specified.
%
% Martin Knapmeyer, 13.09.2004, 23.09.2004, 21.12.2004


%%% local verbosity
verbosity=3;

%%% another useful constant
radian=pi/180;

%%% understand input
if nargin==2
    mode='circle';
end; % if nargin
mode=lower(mode);

%%% number of triangles to analyze
tricnt=length(layer.tri);

%%% init vectors to collect edge length and area
reslist=[];

%%% init counter for child types
nodechildcnt=0;
trianglechildcnt=0;

%%% collect data
%%% loop over triangles
for indy=1:tricnt
    
    childtype=(layer.tri(indy).childtype);
    switch childtype
        case {'node'}
            
            nodechildcnt=nodechildcnt+1;
            
            % get triangle nodes
            lonlist=[];
            latlist=[];
            for nodecnt=1:3
                id=layer.tri(indy).nodes(nodecnt);
                [rownum,nodes]=mklsgridgetnodetri(id,layer);
                lonlist=[lonlist nodes(2)];
                latlist=[latlist nodes(3)];
            end; % for nodecnt
        
            switch mode
                case {'length'}
                    % edge lenghts
                    edgelen=mklsgridtriedgelen(lonlist,latlist,layer.radius);
                    reslist=[reslist edgelen];
                case {'circle'}
                    reslist=[reslist layer.tri(indy).cr];
                case {'circlekm'}
                    reslist=[reslist layer.tri(indy).cr*radian*layer.radius];
                case {'area'}
                    % areas
                    %area=mklsgridtriarea(lonlist,latlist);
                    mklsnotyet('warn');
                case {'iterations'}
                    reslist=[reslist layer.tri(indy).iter];
                otherwise
                    error(['MKLSGRIDSTATTRILAYER: unknown evaluation mode ' upper(mode)]);
            end; % switch mode
        
        case {'triangle'}
            trianglechildcnt=trianglechildcnt+1;
            %%% nothing happens
        case {'empty'}
            error(['MKLSGRIDSTATTRILAYER:  triangle #',...
                   int2str(indy),...
                   ' has no children!']);
        otherwise
            error(['MKLSGRIDSTATTRILAYER: unknown child type ',...
                    upper(childtype)]);
    end; % switch childtype
    
end; % for indy


%%% evaluate child type counters
mklsverbose(verbosity,lsopt,'MKLSGRIDSTATTRILAYER: triangles with node childs: ',...
            nodechildcnt);
mklsverbose(verbosity,lsopt,'MKLSGRIDSTATTRILAYER: triangles with triangle childs: ',...
            trianglechildcnt);
mklsverbose(verbosity,lsopt,'MKLSGRIDSTATTRILAYER: min max median result: ',...
            min(reslist),' ',...
            max(reslist),' ',...
            median(reslist));


%%% evaluate edge  length data if no output parm specified
if nargout==0
    switch mode
    case {'length'}
        edgeanz=length(reslist);
        [n,xout]=hist(reslist,length(unique(reslist)));
        stem(xout,100*n/edgeanz);
        xlabel('Edge Length [km]');
        ylabel(['Number of Edges (percent of: ' int2str(edgeanz) ')']);
        title('Edge Length Distribution');
        box on
        grid on
    case {'circle'}
        circleanz=length(reslist);
        [n,xout]=hist(reslist,length(unique(reslist)));
        stem(xout,100*n/circleanz);
        xlabel('Circumcircle Radius [deg]');
        ylabel(['Number of Circles (percent of: ' int2str(circleanz) ')']);
        title('Circumcircle Radius Distribution');
        box on
        grid on
    case {'circlekm'}
        mklsnotyet('warn');
    case {'iterations'}
        mklsnotyet('warn');
    case {'area'}
        mklsnotyet('warn');
    otherwise
        error(['MKLSGRIDSTATTRILAYER: unknown plot mode ' upper(mode)]);
    end; % switch mode
end; % if nargout==0


