function mklsplotloctri(lsopt,loc,statdat,setlabels);
% mklsplotloctri.......plot resulting location for triangular grids
%
% call: mklsplotloctri(lsopt,loc);
%       mklsplotloctri(lsopt,loc,statdat);
%       mklsplotloctri(lsopt,loc,statdat,setlabels);
%
%
%          lsopt: (structure)
%                  LOCSMITH options structure as returned by MKLSOPT
%
%          loc: (strcuture array)
%
%                  This sturcture contains those nodes that are compatible with
%                  most of the evaluated data. The details of the structure
%                  depend on the grid type. See MKLSGRIDEXTRACTLOCTRI for more info.
%
%          statdat: (structure array)
%
%                   LSD structure containing the position sof the stations
%                   If this input parameter is present, the stations will
%                   be drawn into the plots. Otherwise, no stations will be
%                   drawn.
%
%           setlabels: (string)
%                   flag to decide if stations are labelled with their
%                   names or not. allowed values are:
%                   'label': apply labels
%                   'nolabel: do not apply labels
%                   DEFAULT: do not apply labels
%
%
% result: none
%
% This routine plots the result nodes given in the LOC structure in an
% unfolded-cube-plot, showing a map view, a longitude-depth-view and a
% latitude-depth view.
%
% Martin Knapmeyer, 04.07.2007, 14.08.2007, 21.08.2007


%%% a useful constant
radian=pi/180;

%%% clear current figure
plotfig=gcf;
figure(plotfig);
clf;
set(plotfig,'Color','w');
drawnow;


%%% understand input
statlon=[];
statlat=[];
nin=nargin;
switch nin
    case {2}
       drawstats=0;
    case {3,4}
       drawstats=1;
       if nin==3
          setlabels='nolabel';
       end; %  if nin==3
       stationmarker=mklsconstants('StationMarker');
       %%% collect station locations for inclusion in axis limitation
       %%% determination MK21082007
       statcnt=length(statdat.station);
       for indy=1:statcnt
           statlon=[statlon statdat.station(indy).lon];
           statlat=[statlat statdat.station(indy).lat];
       end; % for indy 
    otherwise
        error('MKLSPLOTLOCTRI: illegal number of input arguments!')
end; % switch nin


%%% convert node coordinates to -180...180 coordinates, which is often more
%%% conventient for mapping purposes.
%%% If you do not wish this conversion to be applied, just change to
%%% DOCONVERT flag MK21082007
doconvert='yes';
if strcmp(doconvert,'yes')
   beyond180=find(loc.nodelist(:,1)>180); % nodes with longitudes in 180...360
   loc.nodelist(beyond180,1)=loc.nodelist(beyond180,1)-360;
end; % if strcmp(doconvert,'yes')


%%% determine ranges
minlon=min([statlon(:); loc.nodelist(:,1)]);
maxlon=max([statlon(:); loc.nodelist(:,1)]);
minlat=min([statlat(:); loc.nodelist(:,2)]);
maxlat=max([statlat(:); loc.nodelist(:,2)]);
mindepth=min(lsopt.planet_radius-loc.nodelist(:,3));
maxdepth=max(lsopt.planet_radius-loc.nodelist(:,3));

%%% add some margin to the ranges
marginwidth=10; % percent of width/height
lonmarg=(maxlon-minlon)*marginwidth/100;
latmarg=(maxlat-minlat)*marginwidth/100;
depmarg=(maxdepth-mindepth)*marginwidth/100;
minlon=minlon-lonmarg;
maxlon=maxlon+lonmarg;
minlat=minlat-latmarg;
maxlat=maxlat+latmarg;
mindepth=mindepth-depmarg;
maxdepth=maxdepth+depmarg;


%%%%%% do the plot
markertype=mklsconstants('NodeMarker');
nodecolor=mklsconstants('NodeColor');


%%% map view
figure(plotfig);
subplot(2,2,1);
set(gca,'Units','centimeters');
handle=plot(loc.nodelist(:,1),loc.nodelist(:,2),markertype);
set(handle,'MarkerFaceColor',nodecolor);
set(handle,'MarkerEdgeColor',nodecolor);
if drawstats
    hold on
    statcnt=length(statdat.station);
    for indy=1:statcnt
        plot(statlon(indy),...
             statlat(indy),...
             stationmarker);
        if strcmp(setlabels,'label')
            text(statlon(indy),...
                 statlat(indy),...
                 statdat.station(indy).id);
        end; % strcmp(setlabels,'label')
    end; % for indy
    hold off
end; % if drawstats
%%% beautification
ylabel('Latitude [deg]');
set(gca,'XTickLabel',[]);
axis([minlon maxlon minlat maxlat]);
axis equal
pos221=get(gca,'Position');
grid on
handle221=gca;
ax221=axis;


%%% latitude-depth view
figure(plotfig);
subplot(2,2,2);
set(gca,'Units','centimeters');
handle=plot(lsopt.planet_radius-loc.nodelist(:,3),loc.nodelist(:,2),markertype);
set(handle,'MarkerFaceColor',nodecolor);
set(handle,'MarkerEdgeColor',nodecolor);
xlabel('Depth [km]');
set(gca,'YTickLabel',[]);
axis([mindepth maxdepth minlat maxlat]);
pos222=get(gca,'Position');
set(gca,'Position',[pos221(1)+pos221(3) pos222(2) pos222(3) pos222(4)]);
grid on
ax222=axis;


%%% longitude-depth-view
figure(plotfig);
subplot(2,2,3);
set(gca,'Units','centimeters');
handle=plot(loc.nodelist(:,1),lsopt.planet_radius-loc.nodelist(:,3),markertype);
set(handle,'MarkerFaceColor',nodecolor);
set(handle,'MarkerEdgeColor',nodecolor);
xlabel('Longitude [deg]');
ylabel('Depth [km]');
axis([minlon maxlon mindepth maxdepth]);
axis ij
pos223=get(gca,'Position');
set(gca,'Position',[pos223(1) pos221(2)-pos222(3) pos223(3) pos222(3)]);
grid on


%%% plot true axis ratio 3D view in spherical coordinates MK14082008
figure(plotfig+1);
drawnow;
clf;
grayshade=0.75;
%%% hypocenters
[locx,locy,locz]=sph2cart(loc.nodelist(:,1)*radian,...
                          loc.nodelist(:,2)*radian,...
                          loc.nodelist(:,3));
handle=plot3(locx,locy,locz,'.');
set(handle,'MarkerFaceColor',nodecolor);
set(handle,'MarkerEdgeColor',nodecolor);
%%% spherical surface element
% minlon=floor(min(loc.nodelist(:,1)));
% maxlon=ceil(max(loc.nodelist(:,1)));
% minlat=floor(min(loc.nodelist(:,2)));
% maxlat=ceil(max(loc.nodelist(:,2)));
% lonwidth=abs(maxlon-minlon);
% latwidth=abs(maxlat-minlat);
% lonrange=linspace(minlon,...
%                   maxlon,...
%                   10);
% latrange=linspace(minlat,...
%                   maxlat,...
%                   10);
minlon=ax221(1);
maxlon=ax221(2);
if abs(maxlon-minlon)>360
   minlon=0;
   maxlon=360;
end; % if abs(maxlon-minlon)>360
minlat=ax221(3);
maxlat=ax221(4);
maxrad=lsopt.planet_radius;
minrad=maxrad-ax222(2);
% lonrange=sort([minlon maxlon get(handle221,'XTick')]);
% latrange=sort([minlat maxlat get(handle221,'YTick')]);
nodecnt=100;
lonrange=linspace(minlon,...
                  maxlon,...
                  nodecnt);
latrange=linspace(minlat,...
                  maxlat,...
                  nodecnt);
[longrid,latgrid]=meshgrid(lonrange,latrange);
[sphx,sphy,sphz]=sph2cart(longrid*radian,latgrid*radian,maxrad);
hold on
handle=surf(sphx,sphy,sphz);
shading interp
set(handle,'FaceAlpha',0.5);
set(handle,'EdgeAlpha',0.5);
if abs(maxlon-minlon)~=360
    plot3(sphx(:,1),sphy(:,1),sphz(:,1),'Color',[1 1 1]*grayshade);
    plot3(sphx(:,end),sphy(:,end),sphz(:,end),'Color',[1 1 1]*grayshade);
end; % if abs(maxlon-minlon)~=360
plot3(sphx(1,:),sphy(1,:),sphz(1,:),'Color',[1 1 1]*grayshade);
plot3(sphx(end,:),sphy(end,:),sphz(end,:),'Color',[1 1 1]*grayshade);
hold off
%%% spherical bottom element
[sphx,sphy,sphz]=sph2cart(longrid*radian,latgrid*radian,minrad);
hold on
handle=surf(sphx,sphy,sphz);
shading interp
set(handle,'FaceAlpha',0.5);
set(handle,'EdgeAlpha',0.5);
if abs(maxlon-minlon)~=360
    plot3(sphx(:,1),sphy(:,1),sphz(:,1),'Color',[1 1 1]*grayshade);
    plot3(sphx(:,end),sphy(:,end),sphz(:,end),'Color',[1 1 1]*grayshade);
end; % if abs(maxlon-minlon)~=360
plot3(sphx(1,:),sphy(1,:),sphz(1,:),'Color',[1 1 1]*grayshade);
plot3(sphx(end,:),sphy(end,:),sphz(end,:),'Color',[1 1 1]*grayshade);
hold off
%%% spherical intermediate  elements, every 100km in depth
if maxrad-minrad>100
   for radius=maxrad:-100:minrad
       [sphx,sphy,sphz]=sph2cart(longrid*radian,latgrid*radian,radius);
       hold on
       if abs(maxlon-minlon)~=360
           plot3(sphx(:,1),sphy(:,1),sphz(:,1),'Color',[1 1 1]*grayshade);
           plot3(sphx(:,end),sphy(:,end),sphz(:,end),'Color',[1 1 1]*grayshade);
       end; % if abs(maxlon-minlon)~=360
       plot3(sphx(1,:),sphy(1,:),sphz(1,:),'Color',[1 1 1]*grayshade);
       plot3(sphx(end,:),sphy(end,:),sphz(end,:),'Color',[1 1 1]*grayshade);
       hold off
   end; % for radius
end; % if lsopt.planet_radius-min(loc.nodelist(:,3))>100
%%% vertical edges
if abs(maxlon-minlon)~=360
    [edgx,edgy,edgz]=sph2cart([minlon maxlon minlon maxlon,...
                               minlon maxlon minlon maxlon]*radian,...
                              [minlat maxlat maxlat minlat,...
                               minlat maxlat maxlat minlat]*radian,...
                              [minrad minrad minrad minrad,...
                               maxrad maxrad maxrad maxrad]);
    hold on
    plot3(edgx([1 5]),edgy([1 5]),edgz([1 5]),'Color',[1 1 1]*grayshade); % minlon/minlat -> minlon/minlat
    plot3(edgx([2 6]),edgy([2 6]),edgz([2 6]),'Color',[1 1 1]*grayshade); % maxlon/maxlat -> maxlon/maxlat
    plot3(edgx([3 7]),edgy([3 7]),edgz([3 7]),'Color',[1 1 1]*grayshade); % minlon/maxlat -> minlon/maxlat
    plot3(edgx([4 8]),edgy([4 8]),edgz([4 8]),'Color',[1 1 1]*grayshade); % maxlon/minlat -> maxlon/minlat
    hold off
end; %
%%% coordinate axes
elongate=1.5;
hold on
plot3([0 maxrad*elongate],[0 0],[0 0],'k-');
plot3([0 0],[0 maxrad*elongate],[0 0],'k-');
plot3([0 0],[0 0],[0 maxrad*elongate],'k-');
text([maxrad*elongate],[0],[0],'X');
text([0],[maxrad*elongate],[0],'Y');
text([0],[0],[maxrad*elongate],'Z');
hold off


%%% plot stations
if drawstats
    hold on
    statcnt=length(statdat.station);
    for indy=1:statcnt
        [x,y,z]=sph2cart(statlon(indy)*radian,statlat(indy)*radian,lsopt.planet_radius);
        plot3(x,y,z,...
             stationmarker);
        if strcmp(setlabels,'label')
            text(x,y,z,...
                 statdat.station(indy).id);
        end; % strcmp(setlabels,'label')
    end; % for indy
    hold off
end; % if drawstats


%%% plot beautification
axis equal;
axis off;
axis vis3d;
set(gcf,'Color','w');
colormap(bone);
drawnow;

%%% back to fig 1, to avoid creation of more new windows
figure(plotfig);








