function [numparms,stringparms]=mklsparmconsolid(keyword,modifier,parmlist,silent);
% mklsparmconsolid..........analysis & consolidation of parameter list
%
% call: [numparms,stringparms]=mklsparmconsolid(keyword,modifier,parmlist,silent);
%
%       keyword: the identified keyword (without then "!")
%       modifier: the identified modifier (without the "!")
%       parmlist: the raw parameter list
%       silent: verbosity flag:
%               1: run silent
%               0: run verbose
%
%       Input parameters are as returned by MKLSSPLITTOKENLIST
%
% result: numparms: numeric column vector containing all numeric parameters
%         stringprams: string row vector containing the string parameter (only one possible)
%
% The input matrix PARMLIST is a very raw version of a parameter list. It is split
% up into the parameter values as intended by the file.
% To do so, names of keywords and modifiers are hard-wired within this routine.
%
% What this routine does is essentially the following: look at which keyword and
% modifier are given and thencut and paste the lines of PARMLIST into the suitable
% formats.
% Additionally, it checks many syntactical and semantical issues.
%
% Martin Knapmeyer, 02.09.2003, 09.09.2003


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                                                                    %%
%% the following routine is format-specific: keywords and modifiers and their         %%
%% properties are hardwired here. future versions may use a description file, I hope. %%
%%                                                                                    %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                              

%%% init result
numparms=[];
stringparms=[];


if ~isempty(keyword)
   %%% there is a keyword, so there might be a modifier and parameters
   
   %%% make things not case sensitive
   keyword=lower(keyword);
   modifier=lower(modifier);

   switch keyword
%       case {'universal'}
%            if isempty(modifier)
%               error(['MKLSPARMCONSOLID: modifier required for keyword '...
%                      upper(keyword) '.']);
%            else
%               buffy=mklsstrmat2line(parmlist);
%               numparms=str2num(buffy);
%            end; % if isempty(modifier)
%            
%       case {'planet'}
%            if isempty(modifier)
%               error(['MKLSPARMCONSOLID: modifier required for keyword '...
%                      upper(keyword) '.']);
%            else
%               switch modifier
%                  case {'name'}
%                       %%% purely string parameter
%                       stringparms=mklsstrmat2line(parmlist);
%                  case {'radius'}
%                       buffy=mklsstrmat2line(parmlist);
%                       numparms=str2num(buffy);
%                  case {'gm','mif'}
%                       %%% purely numeric interval parameter
%                       buffy=mklsstrmat2line(parmlist);
%                       numparms=str2num(buffy);
%                  otherwise
%                     error(['MKLSPARMCONSOLID: modifier ' upper(modifier)...
%                            'illegal for keyword' upper(keyword)]);
%               end; % switch modifier
%            end; % if isempty(modifier)
           
      case {'station'}
           if isempty(modifier)
              error(['MKLSPARMCONSOLID: modifier required for keyword '...
                     upper(keyword) '.']);
           else
              switch modifier
                 case {'start'}
                      %%% purely string parameter
                      stringparms=mklsstrmat2line(parmlist);
                 case {'lon','lat','elev'}
                      %%% purely numeric interval parameter
                      buffy=mklsstrmat2line(parmlist);
                      numparms=str2num(buffy);
                 case {'end'}
                      %%% no parameter allowed
                      if ~isempty(parmlist)
                         error(['MKLSPARMCONSOLID: no parameter allowed for modifier '...
                                upper(modifier) ' of keyword ' upper(keyword) '.']);
                      end; % if ~isempty(parmlist)
                 otherwise
                    error(['MKLSPARMCONSOLID: modifier ' upper(modifier)...
                           ' illegal for keyword ' upper(keyword)]);
              end; % switch modifier
           end; % if isempty(modifier)
           
      case {'event'}
           if isempty(modifier)
              error(['MKLSPARMCONSOLID: modifier required for keyword '...
                     upper(keyword) '.']);
           else
              switch modifier
                 case {'start','info'}
                      %%% purely string parameter
                      stringparms=mklsstrmat2line(parmlist);
                 case {'lon','lat','depth','ot'}
                      %%% purely numeric interval parameter
                      buffy=mklsstrmat2line(parmlist);
                      numparms=str2num(buffy);
                 case {'end'}
                      %%% no parameter allowed
                      if ~isempty(parmlist)
                         error(['MKLSPARMCONSOLID: no parameter allowed for modifier '...
                                upper(modifier) ' of keyword ' upper(keyword) '.']);
                      end; % if ~isempty(parmlist)
                 otherwise
                    error(['MKLSPARMCONSOLID: modifier ' upper(modifier)...
                           'illegal for keyword' upper(keyword)]);
              end; % switch modifier
           end; % if isempty(modifier)
           
%       case {'tt'}
%            if isempty(modifier)
%               error(['MKLSPARMCONSOLID: modifier required for keyword '...
%                      upper(keyword) '.']);
%            else
%               switch modifier
%                  case {'start','station','event','phase'}
%                       %%% purely string parameter
%                       stringparms=mklsstrmat2line(parmlist);
%                  case {'dist','tt','depth'}
%                       %%% purely numeric interval parameter
%                       buffy=mklsstrmat2line(parmlist);
%                       numparms=str2num(buffy);
%                       if strcmp(modifier,'tt')
%                          numparms=numparms; %+1500;
%                       end; % if strcmp
%                  case {'end'}
%                       %%% no parameter allowed
%                       if ~isempty(parmlist)
%                           error(['MKLSPARMCONSOLID: no parameter allowed for modifier '...
%                                  upper(modifier) ' of keyword ' upper(keyword) '.']);
%                       end; % if ~isempty(parmlist)
%                  otherwise
%                     error(['MKLSPARMCONSOLID: modifier ' upper(modifier)...
%                            'illegal for keyword' upper(keyword)]);
%               end; % switch modifier
%            end; % if isempty(modifier)
           
      case {'arrival'}
           if isempty(modifier)
              error(['MKLSPARMCONSOLID: modifier required for keyword '...
                     upper(keyword) '.']);
           else
              switch modifier
                 case {'start','station','event','phase'}
                      %%% purely string parameter
                      stringparms=mklsstrmat2line(parmlist);
                 case {'at','baz','emerg','slo'}
                      %%% purely numeric interval parameter
                      buffy=mklsstrmat2line(parmlist);
                      numparms=str2num(buffy);
                 case {'end'}
                      %%% no parameter allowed
                      if ~isempty(parmlist)
                         error(['MKLSPARMCONSOLID: no parameter allowed for modifier '...
                                upper(modifier) ' of keyword ' upper(keyword) '.']);
                      end; % if ~isempty(parmlist)
                 otherwise
                    error(['MKLSPARMCONSOLID: modifier ' upper(modifier)...
                           'illegal for keyword' upper(keyword)]);
              end; % switch modifier
           end; % if isempty(modifier)
           
      otherwise
          error(['MKLSPARMCONSOLID: unknown keyword ' upper(keyword)]);
   end; % switch keyword
   
   %%% format output
   numparms=mklsnumparmsformat(keyword,modifier,numparms,stringparms);
   stringparms=stringparms(:)';

   %%% tell them what we've found
   if ~silent
      if ~isempty(numparms)
         disp(['MKLSPARMCONSOLID: numeric parameters [' num2str(numparms) ']']);
      end; % if ~isempty
      if ~isempty(stringparms)
         disp(['MKLSPARMCONSOLID: string parameters "' stringparms '"']);
      end; % if ~isempty
   end; % if ~silent
   
else
   %%% there is no keyword   
end; % if ~isempty(keyword)




return;