function lsd=mklsreadlsd(pfad,silent);
% mklsreadsid........read Seismic Data file
%
% call: lsd=mklsreadlsd(pfad);
%       lsd=mklsreadlsd(pfad,silent);
%
%           pfad: full pth to the .lsd file to be read
%           silent: if present, nothing is written to the screen.
%                   otherwise, the programm produces some output on read progress.
%
% result: lsd: a LSD structure representing the file content.
%              This structure has the following fields:
%
%             .statcnt: number of stations defined in structure
%             .eventcnt: number of events defined in structure
%             .arrivalcnt: number of arrival data records defined in structure
%
%             .station: an array of sub structures containing positions of all stations.
%                       each element of the array has the following fields:
%                       .id: a string containing the station id
%                       .lon: three elements vector defining station longitude
%                       .lat: threee elements vector defining station latitude
%                       .elev: three elements vector defining station elevation
%
%             .event: an array of sub structures containing informations about all events
%                     each element of the array has the following fields:
%                     .id: a string containing the event ID
%                     .info: a string containing informal additional info
%                     .lon: three elements vector defining constraints for event longitude
%                           (event location is constrainded to this range by external evidence)
%                     .lat: three elements vector defining constraints for event latitude
%                           (event location is constrainded to this range by external evidence)
%                     .depth: three elements vector defining constraints for event focal depth
%                           (event location is constrainded to this range by external evidence,
%                            this is of special importance for meteoroid impacts!)
%                     .ot: three elements vector defining constraints for event focal depth
%                           (event location is constrainded to this range
%                           by external evidence)
%
%            .arrival: an array of arrival data (which does not imply a source localization)
%                  each element is a sub structure with the following fields:
%                  .id: string containing data record ID
%                  .station: string, containing station identifier
%                  .event: string, containing event identifier
%                  .phase: string, containing seismic phase name
%                  .at: arrival time
%                  .baz: back azimuth angle
%                  .emerg: emergence angle
%                  .slo: slowness
%
%           An undefined string will be empty.
%           An undefined numeric quantity will be [NaN NaN NaN]: even if a quantity is undefined,
%           it will be possible to access the respective field of the result lsd structure with
%           indices 1,2,3.
%
%
%
%
% Martin Knapmeyer, 09.09.2003, adapted for LOCSMITH 02.02.2005

%disp('1500s laufzeitshift aus MKREADLSD wieder entfernen!');

%%% init result
lsd.statcnt=0;
lsd.eventcnt=0;
lsd.arrivalcnt=0;
lsd.station=[];
lsd.event=[];
lsd.arrival=[];



%%% run silent?
if nargin==2
   silent=1;
else
   silent=0;
end; % if nargin==2


%%% open file
[fid,msg]=fopen(pfad,'r');
if fid==-1
   error(['MKLSREADLSD: ' msg]);
end; % if fid

%%% read & interpret lines
done=0;
linecnt=1; % line counter
while done==0
   if (feof(fid)) %|(linecnt>25)
      %%% end of file reached. quit file loop.
      done=1;
   else
      %%% we gonna get another line from the file!
      %%% read the next line
      line=fgetl(fid);
      
      %disp(['line    :' line]);
      
      %%% strip off comments
      line=mklsstripcomments(line);
      %disp(['stripped:' line]);
      
      %%% split line into tokens
      [tokcnt,tokenlist]=mklssplitline(line);
      
      %%% identify keyword and modifier
      [keyword,modifier,parmlist]=mklssplittokenlist(tokenlist,linecnt,silent);
      
      %%% consolidation of parmlist
      [numparms,stringparms]=mklsparmconsolid(keyword,modifier,parmlist,silent);
      
      %%% build identified parameters into field of lsd structure
      lsd=mklsparm2lsd(lsd,keyword,modifier,numparms,stringparms);
      
      %%% increment line counter
      linecnt=linecnt+1;
   end; % if feof(fid)
end; % while done

%%% close file
fclose(fid);


%%% leave MKLSREADLSD
return;


% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %                                 HELPER FUNCTIONS                                 %
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% function newline=mkstripcomments(oldline)
% % mkstripcomments.......strip off .sid file comments from line
% %
% % call: newline=mkstripcomments(oldline)
% %
% %       oldline: string containing a line from a .sid file
% %
% % result: newline: the same als OLDLINE but all comments stripped off.
% %
% % comments begin with '#', '//', or '/*' and end at the end of the line.
% %
% % Martin Knapmeyer, 01.09.2003
% 
% %%% find all comment start sequences
% indies=[findstr(oldline,'#') findstr(oldline,'//') findstr(oldline,'/*')];
% 
% %%% remove all comments
% if ~isempty(indies)
%    %%% a comment exists
%    indies=sort(indies);
%    commentstart=indies(1);
%    newline=oldline(1:(commentstart-1));
% else
%    %%% no comment exists
%    newline=oldline;
% end; % if ~isempty
% 
% return;
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% 
% function [tokcnt,tokenlist]=mksplitline(line);
% % mksplitline.......split text line into tokens used in .sid files
% %
% % call: [tokcnt,tokenlist]=mksplitline(line);
% %
% %       line: string containing a line from a .sid file
% %
% % result: tokcnt: number of tokens found
% %         tokenlist: strin matrix, containing on token from LINE per row
% %
% % A token is defined as a sequence of non-whitespace characters, where
% % whitespace are ASCII characters 9 and 32.
% %
% % Martin Knapmeyer, 01.09.2003
% 
% %%% init result
% tokenlist=[];
% 
% %%% split line
% done=0;
% while done==0
%     [tok,rem]=strtok(line);
%     if ~isempty(tok)
%        %%% add token to tokenlist
%        tokenlist=strvcat(tokenlist,tok);
%        line=rem;
%     else
%        %%% no more tokens
%        done=1;
%     end; % if ~isempty
% end; % while done
% 
% tokcnt=size(tokenlist,1);
% 
% %disp(['MKSPLITLINE: ' int2str(tokcnt) ' tokens found.']);
% 
% 
% return;
% 
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% 
% function [keyword,modifier,parmlist]=mksplittokenlist(tokenlist,linecnt,silent);
% % mksplittokenlist.........divide token list into keyword, modifier, and parameter list
% %
% % call: [keyword,modifier,parmlist]=mksplittokenlist(tokenlist,linecnt,silent);
% %
% %       tokenlist: string matrix containing token list, as returned by MKSPLITLINE
% %       linecnt: line counter, needed in error messages
% %       silent: verbosity flag:
% %               1: run silent
% %               0: run verbose
% %
% % result: keyword: string containing the keyword (without "!")
% %                   empty if no keyword present
% %         modifier: string containing the modifier to the keyword (without "!"),
% %                   empty if no modifier present
% %         parmlist: string matrix, containing all parameters
% %                   empty if no parameters present
% %
% % Keywords and Modifiers are recognized by 
% % a) position in TOKENLIST and
% % b) preceeding exclamation mark
% %
% % Martin Knapmeyer, 01.09.2003
% 
% 
% %%% init output
% keyword=[];
% modifier=[];
% parmlist=[];
% 
% tokcnt=size(tokenlist,1);
% if tokcnt==0
%    %%% token list is empty
% else
%    %%% token list is not empty
%    keyword=deblank(tokenlist(1,:));
%    if strcmp(keyword(1),'!')~=1
%       error(['MKSPLITTOKENLIST: Exclamation mark missing at begin of line '...
%              int2str(linecnt) ' of file.']);
%    else
%       keyword=keyword(2:end); % strip off exclamation mark
%       if tokcnt>1
%          %%% other tokens might be modifier and parameters
%          modifier=deblank(tokenlist(2,:));
%          if strcmp(modifier(1),'!')~=1
%             %%% not identified as modifier ("!" missing)
%             modifier=[];
%             parmlist=tokenlist(2:tokcnt,:);
%          else
%             modifier=modifier(2:end); % strip off exclamation mark
%             if tokcnt>2
%                %%% remaining tokens are parameters
%                parmlist=tokenlist(3:tokcnt,:);
%             end; % if tokcnt>2
%          end; % strcmp(modifier(1),'!')~=1 else
%       else
%          %%% no more tokens
%          modifier=[];
%          parmlist=[];
%       end; % if tokcnt>1 else
%    end; % if strcmp((keyword(1),'!')~=1
% end; % if tokcnt
% 
% %%% tell them what we've found
% if ~silent
%    if ~isempty(keyword)
%       disp(['MKSPLITTOKENLIST: keyword ' upper(keyword) ' identified.']);
%    end; % if ~isempty(keyword)
%    if ~isempty(modifier)
%       disp(['MKSPLITTOKENLIST: modifier ' upper(modifier) ' identified.']);
%    end; % if ~isempty(modifier)
% end; % if ~silent
% 
% %%% if parmlist is empty, make an error
% if (tokcnt>0)&(isempty(parmlist))&(strcmp(modifier,'end')==0)
%    error(['MKSPLITTOKENLIST: line ' int2str(linecnt) ': illegally empty parameter list!']);
% end; % if isempty(parmlist)
% 
% 
% %%% case insensitive output of keywords and modifiers
% keyword=lower(keyword);
% modifier=lower(modifier);
% 
% return;
% 
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% function line=mkstrmat2line(strmat);
% % mkstrmat2line.......concatenates all lines of a string matrix into a single line
% %
% % call: line=mkstrmat2line(strmat);
% %
% %       strmat: string matrix as generated by strvcat()
% %
% % result: line: a single-line string in which all rows of STRMAT are concatenated
% %
% % Rows of STRMAT are deblanked and then separated by a single ASCII 32 character.
% %
% % Martin Knapmeyer 03.09.2003
% 
% %%% init result
% line=[];
% 
% %%% how many lines?
% linecnt=size(strmat,1);
% 
% %%% loop over all lines
% if linecnt==0
%    error('MKSTRMAT2LINE: parameters illegally empty! (allowed only for !end modifier)');
% else
%    for indy=1:linecnt
%        if indy==1
%           line=[deblank(strmat(indy,:))];
%        else
%           line=[line char(32) deblank(strmat(indy,:))];
%        end; % if indy==1
%    end; % for indy
% end; % if linecnt
% 
% return;
% 
% 
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%                                                                                    %%
% %% the following routines are format-specific: keywords and modifiers and their       %%
% %% properties are hardwired here. future versions may use a description file, I hope. %%
% %%                                                                                    %%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% 
% function [numparms,stringparms]=mkparmconsolid(keyword,modifier,parmlist,silent);
% % mkparmconsolid..........analysis & consolidation of parameter list
% %
% % call: [numparms,stringparms]=mkparmconsolid(keyword,modifier,parmlist,silent);
% %
% %       keyword: the identified keyword (without then "!")
% %       modifier: the identified modifier (without the "!")
% %       parmlist: the raw parameter list
% %       silent: verbosity flag:
% %               1: run silent
% %               0: run verbose
% %
% %       Input parameters are as returned by MKSPLITTOKENLIST
% %
% % result: numparms: numeric column vector containing all numeric parameters
% %         stringprams: string row vector containing the string parameter (only one possible)
% %
% % The input matrix PARMLIST is a very raw version of a parameter list. It is split
% % up into the parameter values as intended by the file.
% % To do so, names of keywords and modifiers are hard-wired within this routine.
% %
% % What this routine does is essentially the following: look at which keyword and
% % modifier are given and thencut and paste the lines of PARMLIST into the suitable
% % formats.
% % Additionally, it checks many syntactical and semantical issues.
% %
% % Martin Knapmeyer, 02.09.2003, 09.09.2003
%                               
% 
% %%% init result
% numparms=[];
% stringparms=[];
% 
% 
% if ~isempty(keyword)
%    %%% there is a keyword, so there might be a modifier and parameters
%    
%    %%% make things not case sensitive
%    keyword=lower(keyword);
%    modifier=lower(modifier);
% 
%    switch keyword
%       case {'universal'}
%            if isempty(modifier)
%               error(['MKPARMCONSOLID: modifier required for keyword '...
%                      upper(keyword) '.']);
%            else
%               buffy=mkstrmat2line(parmlist);
%               numparms=str2num(buffy);
%            end; % if isempty(modifier)
%            
%       case {'planet'}
%            if isempty(modifier)
%               error(['MKPARMCONSOLID: modifier required for keyword '...
%                      upper(keyword) '.']);
%            else
%               switch modifier
%                  case {'name'}
%                       %%% purely string parameter
%                       stringparms=mkstrmat2line(parmlist);
%                  case {'radius'}
%                       buffy=mkstrmat2line(parmlist);
%                       numparms=str2num(buffy);
%                  case {'gm','mif'}
%                       %%% purely numeric interval parameter
%                       buffy=mkstrmat2line(parmlist);
%                       numparms=str2num(buffy);
%                  otherwise
%                     error(['MKPARMCONSOLID: modifier ' upper(modifier)...
%                            'illegal for keyword' upper(keyword)]);
%               end; % switch modifier
%            end; % if isempty(modifier)
%            
%       case {'station'}
%            if isempty(modifier)
%               error(['MKPARMCONSOLID: modifier required for keyword '...
%                      upper(keyword) '.']);
%            else
%               switch modifier
%                  case {'start'}
%                       %%% purely string parameter
%                       stringparms=mkstrmat2line(parmlist);
%                  case {'lon','lat','elev'}
%                       %%% purely numeric interval parameter
%                       buffy=mkstrmat2line(parmlist);
%                       numparms=str2num(buffy);
%                  case {'end'}
%                       %%% no parameter allowed
%                       if ~isempty(parmlist)
%                          error(['MKPARMCONSOLID: no parameter allowed for modifier '...
%                                 upper(modifier) ' of keyword ' upper(keyword) '.']);
%                       end; % if ~isempty(parmlist)
%                  otherwise
%                     error(['MKPARMCONSOLID: modifier ' upper(modifier)...
%                            'illegal for keyword' upper(keyword)]);
%               end; % switch modifier
%            end; % if isempty(modifier)
%            
%       case {'event'}
%            if isempty(modifier)
%               error(['MKPARMCONSOLID: modifier required for keyword '...
%                      upper(keyword) '.']);
%            else
%               switch modifier
%                  case {'start','info'}
%                       %%% purely string parameter
%                       stringparms=mkstrmat2line(parmlist);
%                  case {'lon','lat','depth','ot'}
%                       %%% purely numeric interval parameter
%                       buffy=mkstrmat2line(parmlist);
%                       numparms=str2num(buffy);
%                  case {'end'}
%                       %%% no parameter allowed
%                       if ~isempty(parmlist)
%                          error(['MKPARMCONSOLID: no parameter allowed for modifier '...
%                                 upper(modifier) ' of keyword ' upper(keyword) '.']);
%                       end; % if ~isempty(parmlist)
%                  otherwise
%                     error(['MKPARMCONSOLID: modifier ' upper(modifier)...
%                            'illegal for keyword' upper(keyword)]);
%               end; % switch modifier
%            end; % if isempty(modifier)
%            
%       case {'tt'}
%            if isempty(modifier)
%               error(['MKPARMCONSOLID: modifier required for keyword '...
%                      upper(keyword) '.']);
%            else
%               switch modifier
%                  case {'start','station','event','phase'}
%                       %%% purely string parameter
%                       stringparms=mkstrmat2line(parmlist);
%                  case {'dist','tt','depth'}
%                       %%% purely numeric interval parameter
%                       buffy=mkstrmat2line(parmlist);
%                       numparms=str2num(buffy);
%                       if strcmp(modifier,'tt')
%                          numparms=numparms; %+1500;
%                       end; % if strcmp
%                  case {'end'}
%                       %%% no parameter allowed
%                       if ~isempty(parmlist)
%                           error(['MKPARMCONSOLID: no parameter allowed for modifier '...
%                                  upper(modifier) ' of keyword ' upper(keyword) '.']);
%                       end; % if ~isempty(parmlist)
%                  otherwise
%                     error(['MKPARMCONSOLID: modifier ' upper(modifier)...
%                            'illegal for keyword' upper(keyword)]);
%               end; % switch modifier
%            end; % if isempty(modifier)
%            
%       case {'raw'}
%            if isempty(modifier)
%               error(['MKPARMCONSOLID: modifier required for keyword '...
%                      upper(keyword) '.']);
%            else
%               switch modifier
%                  case {'start','station','event','phase'}
%                       %%% purely string parameter
%                       stringparms=mkstrmat2line(parmlist);
%                  case {'at','baz','emerg','slo'}
%                       %%% purely numeric interval parameter
%                       buffy=mkstrmat2line(parmlist);
%                       numparms=str2num(buffy);
%                  case {'end'}
%                       %%% no parameter allowed
%                       if ~isempty(parmlist)
%                          error(['MKPARMCONSOLID: no parameter allowed for modifier '...
%                                 upper(modifier) ' of keyword ' upper(keyword) '.']);
%                       end; % if ~isempty(parmlist)
%                  otherwise
%                     error(['MKPARMCONSOLID: modifier ' upper(modifier)...
%                            'illegal for keyword' upper(keyword)]);
%               end; % switch modifier
%            end; % if isempty(modifier)
%            
%       otherwise
%           error(['MKPARMCONSOLID: unknown keyword ' upper(keyword)]);
%    end; % switch keyword
%    
%    %%% format output
%    numparms=mknumparmsformat(keyword,modifier,numparms,stringparms);
%    stringparms=stringparms(:)';
% 
%    %%% tell them what we've found
%    if ~silent
%       if ~isempty(numparms)
%          disp(['MKPARMCONSOLID: numeric parameters [' num2str(numparms) ']']);
%       end; % if ~isempty
%       if ~isempty(stringparms)
%          disp(['MKPARMCONSOLID: string parameters "' stringparms '"']);
%       end; % if ~isempty
%    end; % if ~silent
%    
% else
%    %%% there is no keyword   
% end; % if ~isempty(keyword)
% 
% 
% 
% 
% return;
% 
% 
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% function [newsid]=mkparm2sid(oldsid,keyword,modifier,numparms,stringparms);
% % mkparm2sid.........builds sid structure field from identified keywords & parameters
% %
% % call: [newsid]=mkparm2sid(oldsid,keyword,modifier,numparms,stringparms);
% %
% %       oldsid: existing sid structure
% %       keyword: keyword identified by MKSPLITTOKENLIST
% %       modifer: modifier identified by MKSPLITTOKENLIST
% %       numparms: numeric paramater list returned by MKPARMCONSOLID
% %       stringparms: string parameter returned by MKPARMCONSOLID
% %
% % result: newsid: new sid structure with additional fields as defined by input.
% %
% % For definition of the sid structure see help lines of MKREADSID.
% %
% %
% % Martin Knapmeyer 03.09.2003, 09.09.2003
% 
% % MKPARMCONSOLID has already checked syntax. Here we can assume aeverything is OK!
% % we assume that
% % - KEYWORD and MODIFIER are lowercase
% % - NUMPARMS and STRINGPARMS are not empty when numeric or string parms are required
% 
% %%% init output
% newsid=oldsid;
% 
% %%% init empty dummy station, event, tt, raw structures
% newstation.id=[];
% newstation.lon=[1 1 1]*NaN;
% newstation.lat=[1 1 1]*NaN;
% newstation.elev=[1 1 1]*NaN;
% 
% newevent.id=[];
% newevent.info=[];
% newevent.lon=[1 1 1]*NaN;
% newevent.lat=[1 1 1]*NaN;
% newevent.depth=[1 1 1]*NaN;
% newevent.ot=[1 1 1]*NaN;
% 
% newtt.id=[];
% newtt.station=[];
% newtt.event=[];
% newtt.phase=[];
% newtt.dist=[1 1 1]*NaN;
% newtt.tt=[1 1 1]*NaN;
% newtt.depth=[1 1 1]*NaN;
% 
% newraw.id=[];
% newraw.station=[];
% newraw.event=[];
% newraw.phase=[];
% newraw.at=[1 1 1]*NaN;
% newraw.baz=[1 1 1]*NaN;
% newraw.emerg=[1 1 1]*NaN;
% newraw.slo=[1 1 1]*NaN;
% 
% %%% dispatch input
% if ~isempty(keyword)
% 
%    %%% dispatch input to respective fields
%    switch keyword
%        case {'universal'}
%             switch modifier
%                 case {'newton'}
%                      newsid.newton=numparms;
%                 otherwise
%                      error(['MKPARM2SID: keyword ' upper(keyword) ' with illegal modifier ' upper(modifier)]);
%             end; % switch modifier
%        case {'planet'}
%             switch modifier
%                 case {'name'}
%                      newsid.planet.name=stringparms;
%                 case {'radius'}
%                      newsid.planet.radius=numparms;
%                 case {'gm'}
%                      newsid.planet.gm=numparms;
%                 case {'mif'}
%                      newsid.planet.mif=numparms;
%                 otherwise
%                      error(['MKPARM2SID: keyword ' upper(keyword) ' with illegal modifier ' upper(modifier)]);
%             end; % switch modifier
%        case {'station'}
%             switch modifier
%                 case {'start'}
%                     newsid.station=[newsid.station newstation];
%                     newsid.statcnt=newsid.statcnt+1;
%                     newsid.station(newsid.statcnt).id=stringparms;
%                 case {'lon'}
%                     newsid.station(newsid.statcnt).lon=numparms;
%                 case {'lat'}
%                     newsid.station(newsid.statcnt).lat=numparms;
%                 case {'elev'}
%                     newsid.station(newsid.statcnt).elev=numparms;
%                 case {'end'}
%                     %%% nothing happens
%                 otherwise
%                      error(['MKPARM2SID: keyword ' upper(keyword) ' with illegal modifier ' upper(modifier)]);
%             end; % switch modifier
%             
%        case {'event'}
%             switch modifier
%                case {'start'}
%                     newsid.event=[newsid.event newevent];
%                     newsid.eventcnt=newsid.eventcnt+1;
%                     newsid.event(newsid.eventcnt).id=stringparms;
%                case {'info'}
%                     newsid.event(newsid.eventcnt).info=stringparms;
%                case {'lon'}
%                     newsid.event(newsid.eventcnt).lon=numparms;
%                case {'lat'}
%                     newsid.event(newsid.eventcnt).lat=numparms;
%                case {'depth'}
%                     newsid.event(newsid.eventcnt).depth=numparms;
%                case {'ot'}
%                     newsid.event(newsid.eventcnt).ot=numparms;
%                case {'end'}
%                     %%% nothing happens
%                otherwise
%                      error(['MKPARM2SID: keyword ' upper(keyword) ' with illegal modifier ' upper(modifier)]);
%             end; % switch modifier
%        
%        case {'tt'}
%             switch modifier
%                case {'start'}
%                     newsid.tt=[newsid.tt newtt];
%                     newsid.ttcnt=newsid.ttcnt+1;
%                     newsid.tt(newsid.ttcnt).id=stringparms;
%                case {'station'}
%                     newsid.tt(newsid.ttcnt).station=stringparms;
%                case {'event'}
%                     newsid.tt(newsid.ttcnt).event=stringparms;
%                case {'phase'}
%                     newsid.tt(newsid.ttcnt).phase=stringparms;
%                case {'dist'}
%                     newsid.tt(newsid.ttcnt).dist=numparms;
%                case {'tt'}
%                     newsid.tt(newsid.ttcnt).tt=numparms;
%                case {'depth'}
%                     newsid.tt(newsid.ttcnt).depth=numparms;
%                case {'end'}
%                     %%% nothing happens
%                otherwise
%                     error(['MKPARM2SID: keyword ' upper(keyword) ' with illegal modifier ' upper(modifier)]);
%             end; % switch modifier
%        
%        case {'raw'}
%            switch modifier
%                case {'start'}
%                     newsid.raw=[newsid.raw newraw];
%                     newsid.rawcnt=newsid.rawcnt+1;
%                     newsid.raw(newsid.rawcnt).id=stringparms;
%                case {'station'}
%                     newsid.raw(newsid.rawcnt).station=stringparms;
%                case {'event'}
%                     newsid.raw(newsid.rawcnt).event=stringparms;
%                case {'phase'}
%                     newsid.raw(newsid.rawcnt).phase=stringparms;
%                case {'at'}
%                     newsid.raw(newsid.rawcnt).at=numparms;
%                case {'baz'}
%                     newsid.raw(newsid.rawcnt).baz=numparms;
%                case {'emerg'}
%                     newsid.raw(newsid.rawcnt).emerg=numparms;
%                case {'slo'}
%                     newsid.raw(newsid.rawcnt).slo=numparms;
%                case {'end'}
%                     %%% nothing happens
%                otherwise
%                     error(['MKPARM2SID: keyword ' upper(keyword) ' with illegal modifier ' upper(modifier)]);
%             end; % switch modifier
%        
%        otherwise
%            error(['MKPARM2SID: unknown keyword ' upper(keyword)]);
%    end; % switch keyword
% 
% end; % if ~isempty
% 
% return;
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% function formatted=mknumparmsformat(keyword,modifier,unformt,stringparm);
% % mknumparmsformat........format numeric parameter vector into min-mean-max list
% %
% % call: formatted=mknumparmsformat(keyword,modifier,unformt,stringparm);
% %
% %       keyword: the keyword from the sid file
% %       modifier: the modifier from the sid file
% %       unformt: unformatted parameter list
% %                This is the numeric parameter list extracted from sid file
% %       stringparms: the stringparameters found for the current line
% %
% % result: formatted: formatted numeric parameter list
% %                    in most cases, this will be a 3 elements vector with
% %                    formatted(1): lower boundary
% %                    formatted(2): "likely to be true" value or NaN if not specified in file
% %                    formatted(3): upper boundary
% %
% %                    in some cases, depending on keyword and modifier, this will
% %                    be a single value, the true value. (example: the planets radius
% %                    is usually known so precise that giving boundaries is not neccessary.)
% %
% %                    In case of a non-empty STRINGPARM, FORMATTED will be empty.
% %
% % Since parameters aer either numeric _or_ string in SID files, a non-empty STRINGPARM
% % means that there is no numeric parameter to be formatted. Or vice versa, if the string
% % parameter is not empty, the formatting will be applied to the numeric parameter.
% %
% % Martin Knapmeyer, 09.09.2003
% 
% %%% init result
% formatted=[NaN NaN NaN];
% 
% if (isempty(stringparm))&(strcmp(modifier,'end')==0)
% 
%    %%% format
%    if (strcmp(keyword,'planet'))&(strcmp(modifier,'radius'))
%       %%% 1 element output
%       panz=length(unformt);
%       switch panz
%          case {1}
%              formatted=unformt;
%          otherwise
%              error(['MKNUMPARMSFORMAT: illegal number of parameters for ',...
%                     keyword '/' modifier '!']);
%       end; % switch panz
%    else
%       %%% 3 element output
%       panz=length(unformt);
%       switch panz
%           case {2}
%                formatted(1)=min(unformt);
%                formatted(3)=max(unformt);
%           case {3}
%                indies=find(isnan(unformt));
%                switch length(indies)
%                   case {0}
%                      formatted=sort(unformt);
%                   case {1}
%                      formatted(1)=min(unformt);
%                      formatted(2)=NaN;
%                      formatted(3)=max(unformt);
%                   otherwise
%                      error('MKNUMPARMSFORMAT only one NaN allowed in numeric parameters!');
%                end; % swith length(indies)
%                
%                
%           otherwise
%               error(['MKNUMPARMSFORMAT: illegal number of parameters for ',...
%                       keyword '/' modifier '!']);
%       end; % swtich panz
%    end; % if else
%    
% else
% 
%    formatted=[];
% 
% end; % if isempty(stringparm)