function lsdefaults=mklsdefaults;
% mklsdefaults........generate LOCSMITH options structure with defaults
%
% call: lsdefaults=mklsdefaults;
%
% result: lsdefaults: a structure containing the fields defined below.
%                   The following help text describes the meaning of fields
%                   and possible values, but not the actual defaults.
%                   To get these, execute MKLSDEFAULTS.
%
% The LSDEFAULTS are variables that the user might want to tune for his
% specific problem. There is another routine, MKLSCONSTANTS, in which
% values are defined that the user doesn't shouldn't change - except for
% a few ones, which have to be available in routines that do not have
% access to the LOCSMITH options. You might want to look into the help
% lines of MKLSCONSTANTS to see if there is anything you might wnat to
% change.
% MKLSCONSTANTS also contains declarations which do not affect the location
% process itself but things like plotting of grids and solutions.
%
% The following fields are defined:
% (note that there are different types of grids and that not all options
% apply to all grids!)
%
% PLANET DESCRIPTION
%
% planet_radius: (number) [km]
%               radius of the planet, in km
%               No layer of spherical grids may have points outside this.
%               This is used only for spherical grids.
%
% max_depth:    (number) [km]
%               maximum z coordinate of cartesian grids.
%               This is used only for cartesian grids.
%
% GRID GEOMETRY DEFAULTS:
%
% grid_type:    (string)
%               defines the type of grid to be used. possible values are:
%               'cartesian': a simple rectangular x/y/z grid
%               'triangular':  a triangular grid similar to Buckmister
%                              Fuller's domes
%
% grid_base:    (string)
%               the basic geometry for triangular grids. These are based
%               on some polyhedron and the refined recursively. The
%               starting geometry defines symmetry properties and the point
%               density that is reached after a given number of iterations.
%               This is ignored if grid_type is not 'triangle'.
%               possible values are:
%               'tetrahedron': 4 points in tetrahedron cofniguration
%               'octahedron': 6 points in octahedron configuration
%               'icosahedron': 12 points in icosahedron configuration
%
% INITIAL PROPERTIES OF GRID
%
% .grid_init_minr: (number) [km]
%                Radius of the inner most grid layer for triangle grids, in
%                km. This is used for initialization only, as the grid will
%                change during adaption.
%
% .grid_init_maxr: (number) [km]
%                Radius of the outermost grid layer for triangle grids, in
%                km. This is used for initialization only, as the grid will
%                change during adaption.
%
% .grid_init_dr: (number) [km]
%                initial spacing between layers fo triangle grids, in km
%                The outermost layer will have a radius equal to the
%                outermost layer's, and the next deeper layer will be this
%                number of kilometers deeper.
%
% .grid_init_wdt: (number) [km]
%                initial node spacing on the outermost layer, in km
%                The grid on the outermost layer will be iterated until the
%                average node spacing correpsonds to this value. Deeper
%                layers are then constructed to keep the node density
%                constant.
%
% .grid_init_dx: (number) [km]
%                initial grid spacing in x direction, in km
%                this defines the x spacing for cartesian grids
%
% .grid_init_dy: (number) [km]
%                initial grid spacing in y direction, in km
%                this defines the y spacing for cartesian grids
%
% .grid_init_dz: (number) [km]
%                initial grid spacing in z direction, in km
%                this defines the z spacing for cartesian grids
%
% ITERATION CONTROL
%
% .iter_matchthresh: (number) [percent]
%                compatibility threshold: a triangle is included in the
%                iteration when the compatibility at one of its nodes
%                exceeds this percentage of the current maximum compatibility. 
%                (or if it is a neighbour of such a triangle)
%                For example: if iter_matchthresh==75, then only those
%                triangles are iterated that have a node with equal or more
%                than 75% of the current maximum compatibility within the layer.
%                Set this to 0 to get a simple regular sampling of a
%                uniform triangle grid (be careful not to produce too many nodes!)
%
% .iter_circmin: (number) [km]
%                minimum circumcircle radius of triangles: if the circumcircle of
%                a triangle is smaller than this, it will not be iterated
%                any further.
%
% .iter_drmin:   (number) [km]
%                minimum radius difference between two neigbouring layers.
%                If the distance between to layers is smaller than this, no
%                layer will be constructed inbetween.
%
% .iter_max:     (number) [counter]
%                Maximum number of iterations that may be applied within a
%                horizontal layer.
%                This is just an emergency brake to guarantee that the iteration
%                will stop at some time.
%
% .iter_vertical_max: (number) [counter]
%                Maximum number of iterations that may be applied to
%                determine the vertical coordinate of the location.
%                This is just an emergency brake to guarantee that the
%                iteration will stop at some time.
%
% .iter_vertical_matchthresh: (number) [percent]
%                compatibility threshold analoguous to .iter_matchthresh,
%                but fort the iteration in vertical direction (z coordinate) 
%                
%
% .iter_searchmode (string) [predefined values]
%                  Search mode defines how to characterize the solution set of the location problem.
%                  The following modes are defined:
% 	               'interior': the whole interior of the solution set is returned, 
%                              discretized at the predefined grid solution
% 	               'boundary': only the boundary of the solution set is discretized at 
%                              full resolution, it's interior is discretized with a less dense grid.
% 	               The difference between these modi is best describe with the continents-demo 
%                  (see MKLSGRIDDEMO routine): in 'interior' mode, the whole land mass is returned, 
%                  whereas the 'boundary' mode returns coast lines only.
%
%
% EVALUATION METHOD CONTROL
%
% .eval_baz     (string)
%               Flag to switch the evaluation of back azimuth data on or
%               off. The following values are allowed:
%               'on': back azimuth data will be evaluated
%               'off': back azimuth data will NOT be evaluated
%
% .eval_at      (string)
%               Flag to switch the evaluation of arrival time data on or
%               off. The following values are allowed:
%               'on': arrival time data will be evaluated
%               'off': arrival time data will NOT be evaluated
%
% .eval_dt      (string)
%               Flag to switch the evaluation of arrival time difference data on or
%               off. The following values are allowed:
%               'on': arrival time difference data will be evaluated
%               'off': arrival time difference data will NOT be evaluated
%
% .eval_emerg   (string)
%               Flag to switch the evaluation of emergence angle data on or
%               off. The following values are allowed:
%               'on': emergence angle data will be evaluated
%               'off': emergence angle data will NOT be evaluated
%
% .eval_slo     (string)
%               Flag to switch the evaluation of slowness data on or
%               off. The following values are allowed:
%               'on': slowness data will be evaluated
%               'off': slowness data will NOT be evaluated
%
% TRAVEL TIME COMPUTATION
%
% .tt_package    (string)
%                string defininf the travel time software package to be
%                used for travekl time computation. Ths string given here
%                will be used to switch between different packages.
%
% .tt_options:   (undefined struct)
%                The contents of this field depend on the external travel
%                time software defined in the .tt_package field. See the
%                interface routines for more information.
%                THE OPTIONS RETURNED BY MKLSDEFAULTS ARE VALID FOR THE
%                TTBOX PACKAGE ONLY. OTHER SOFTWARE MIGHT USE SOMETHING
%                DIFFERENT.
%
%                For TTBOX, the following fields are defined:
%
%                tt_options.model: (strcuture)
%                                  CLR structure defining the iasp91 model
%                tt_options.interpdz: (number)
%                                     depth interpolation stepwidth used by MKCLR2MODEL
%                tt_options.interpmode: (string)
%                                     depth interpolation mode used by MKCLR2MODEL
%
% MAPPING OPTIONS
%
% .mapopt:       (mapopt stuct)
%                A complete mapopt structure as returned by MKMAPOPT.
%
% VERBOSITY
%
% .verbosity:    (number) []
%                Verbosity level. This controls the number of progress
%                messages printed during program run. The higher this number
%                is, the more messages are printed. Set this to zero to run in 
%                silence.
%
% .verbosity_plot_mode: (string) []
%                Control plot verbosity level. Some routines can produce
%                control plots according to the plot mode defined here.
%                Thsi must be a string parameter that is accepted by the
%                MKLSGRIDPLOT routine.
%                Control plots are drawn into the current figure.
%                'none': no plot is produced.
% .verbosity_plot_fig: (number) []
%                handle to the figure in which verbosity plots are made.
%                The figure is made active using figure(lsopt.verbosity_plot_fig);
%
% Martin Knapmeyer, 08.09.2004, 13.09.2005




%%%%%% Create Default Structure

%%% Planet Description
x.planet_radius=6371;
x.max_depth=2*x.planet_radius;

%%% Grid Geometry
x.grid_type='triangular';
x.grid_base='icosahedron';

%%% Initial Properties of Grid
x.grid_init_minr=0;
x.grid_init_maxr=x.planet_radius;
x.grid_init_dr=abs(x.grid_init_maxr-x.grid_init_minr);
x.grid_init_wdt=1000;
x.grid_init_dx=500;
x.grid_init_dy=500;
x.grid_init_dz=500;

%%% Iteration control
x.iter_matchthresh=100; % percent!
x.iter_circmin=500;
x.iter_drmin=500;
x.iter_max=20;
x.iter_vertical_max=20;
x.iter_vertical_matchthresh=100;
x.iter_searchmode='interior';

%%% evaluation method control
x.eval_baz='off';
x.eval_at='on';
x.eval_dt='off';
x.eval_emerg='off';
x.eval_slo='off';

%%% TRAVEL TIME COMPUTATION
x.tt_package='ttbox';
x.tt_options=[];
%%% The following declarations are valid for TTBOX only!
if exist('mkreadclr')==2
    %%% testing for the existence of the MKREADCLR routine is used as
    %%% testing for the existence of TTBOX. If MKREADCLR exists, we try to
    %%% initialize TTBOX settings here.
    fullpath=which('mkreadclr'); % path where MKREADCLR resides
    pathstr=fileparts(fullpath);
    iasp91path=[pathstr filesep 'data' filesep 'iasp91.clr'];
    x.tt_options.model=mkreadclr(iasp91path,'silent');
    x.tt_options.interpdz=50; % km
    x.tt_options.interpmode='flat'; % interpolation mode in MKCLR2MODEL
end; % if exist('mkreadclr')==2

%%% MAPPING OPTIONS
x.mapopt=mkmapopt('default');

%%% VERBOSITY
x.verbosity=0;
x.verbosity_plot_mode='none';
x.verbosity_plot_fig=1;







%%% return result
lsdefaults=x;
