function shandle=mklsnetplot(lsopt,seisdat,gridplotmode);
% mklsnetplot.......plot seismic station network
%
% call: shandle=mklsnetplot(lsopt,seisdat,gridplotmode);
%
%       lsopt: (structutre)
%              LSOPT structure as returned by MKLSOPT
%
%       seisdat: (strcuture)
%              LSD structure containing the station positions,
%              as returned by MKREADLSD
%
%       gridplotmode: (string)
%              string denoting the plot mode. This must be a plot mode
%              compatible to the modes of MKLSGRIDPLOT or MKLSGRIDRESPLOT,
%              since the station network will usually be plotted into a 
%              plot of the search grid.
%              'none' does not plot anything
%
% result: shandle: (numeric) [handle]
%                  handle to the line object created to plot the stations
%
% This routine plots the distribution of seismic stations. In case of a
% map plot, the settings in LSOPT.MAPOPT will be used.
%
% Martin Knapmeyer, 15.03.2005


%%% a useful constant
radian=pi/180;

%%% which color to use for stations?
stationcolor=mklsconstants('NodeColor');
stationmarker=mklsconstants('StationMarker');


%%% extract station coordinates from seisdat struct
lon=zeros(1,seisdat.statcnt);
lat=lon;
elv=lon;
id=[];
for indy=1:seisdat.statcnt
    lon(indy)=seisdat.station(indy).lon;
    lat(indy)=seisdat.station(indy).lat;
    elv(indy)=seisdat.station(indy).elev;
    id=strvcat(id,seisdat.station(indy).id);
end; % for indy


%%% transform longitudes into 0...360 degrees range
lon=mklslon180to360(lon);
        

%%% switch plotting modes
gridplotmode=lower(gridplotmode);
switch gridplotmode
    case {'none'}
        %%% no plot is generated.
    case {'cloud','mesh','surface','fitsurface','fitcloud','otcloud'}
        %%% 3D globe plots
        
        %%% transform lon/lat/elv coordinates into x/y/z
        [x,y,z]=sph2cart(lon*radian,lat*radian,(elv+lsopt.planet_radius));
        
        %%% plot station markers
        shandle=plot3(x,y,z,stationmarker);
        set(shandle,'MarkerFaceColor',stationcolor);
        set(shandle,'MarkerEdgeColor',stationcolor);
        
        %%% annotate stations with their names
        for indy=1:seisdat.statcnt
            text(x(indy),y(indy),z(indy),[' ' deblank(id(indy,:))])
        end; % for indy
        
    case {'meshmap','cloudmap','fitcloudmap','otcloudmap','depthcloudmap'}
        %%% map plots
        
        %%% transform lon/lat/elv into map x/y/z
        [x,y]=mklonlat2xy(lon,lat,lsopt.mapopt,'forward');
        z=lsopt.mapopt.planet_radius+elv;
        z=z*lsopt.mapopt.topo_exag;
        
        %%% plot station markers
        shandle=plot3(x,y,z,stationmarker);
        set(shandle,'MarkerFaceColor',stationcolor);
        set(shandle,'MarkerEdgeColor',stationcolor);
        
        %%% annotate stations with their names
        for indy=1:seisdat.statcnt
            text(x(indy),y(indy),z(indy),deblank(id(indy,:)))
        end; % for indy
        
    otherwise
        error(['MKLSNETPLOT: unknown plot mode ' upper(gridplotmode)]);
end; % switch gridplotmode