function options=mklsopt(varargin);
% mklsopt............get/set LocSmith OPTions structure
%
% call: options=mklsopt;
%       options=mklsopt('default');
%       options=mklsopt('Name1',Value1,'Name2',Value2,...);
%
%          'default': return default settings for all options
%                     This is the same as calling mklsopt without any
%                     input parameter.
%       'Name',Value: return complete options structure, in which
%                     the options specified by NAME has the value
%                     specified by VALUE.
%                     If a Name occurs more than once, only the last
%                     of the values specified will apply.
%                     Names used here must correspond to the field names
%                     of eviopt as given below.
%                     Option names are not case sensitive.
%                     One level of substructures can be handled to set
%                     MAPTOPT fields. To set a mapping option, use
%                     'mapopt.FieldName' as option name parameter, where
%                     FieldName is a valid field name of the MAPOPT
%                     structure.
%
% result: options: complete lsopt structure as generated by MKLSDEFAULTS.
%                  For an explanation of the fields see help lines for
%                  MKLSDEFAULTS. To obtain the default values, run
%                  MKLSDEFAULTS.
%
%
% Martin Knapmeyer 08.09.2004, completely rewritten 25.10.2004

%%% This is a generalized verion of the option handler used e.g. in my
%%% mapping toolbox or ion the EVI package. This version does not contain the
%%% construction of a default structure, but just invokes a routine that
%%% returns one. This change makes the option handler fully protable to other
%%% packages.

%%% To add new fields to the options-structure:
%%% Add the new field and its description to the routine which produces the
%%% default structure.

%%% To use this routine in a different software package:
%%% 1)change the name of the routine which produces the default structure as
%%%   well as the name of this routine (the latter is to avoid confusion
%%%   between different packages)
%%% 2) if necessary, change the name of the plausibility checker.

%%% To include the MAPTOP substructure into the LSOPT structure, it was
%%% necessary to rewrite the routine completely. It can now handle ONE
%%% level of substructures.
%%% MK25.10.2004


%%% name of this routine, used for error messages
myname='MKLSOPT';


%%% initialize structure with defaults
%%% it is necessary to do this here, because
%%% the input evaluation will check this for
%%% names of existing fields.
options=mklsdefaults;



%%% evaluate user input
%%% nin2 is number of input parameters
[nin1,nin2]=size(varargin);
switch nin2
   case {0,1}
      %%% return defaults
      %%% nothing needs to be done here.
   otherwise
      %%% if nin2 is even, nin2/2 options are modified and have
      %%% to be extraced from the VARARGIN array
      %%% if nin2 is odd, the input is illegal
      if nin2/2==fix(nin2/2)
         %%% even: evaluate input
         allnames=fieldnames(options);
         allnames=allnames(:);
         namesanz=length(allnames);
         for indy=1:2:nin2
             %%% current name and value
             optname=lower(varargin{indy});
             optvalue=varargin{indy+1};
             %%% is the name a legal name?
             %%% if yes, set the value, if no, ERROR!
             if isfield(options,optname)==1
                 %%% OPTNAME denotes an existing field: set value
                 options=setfield(options,optname,optvalue);
             else
                 %%% OPTNAME does not denote an existing field. Does it
                 %%% denote an existing substructure? If yes: set
                 %%% substructure field, if no: error.
                 if isempty(findstr(optname,'.'))
                     %%% OPTNAME does not contain a dot, so it does not
                     %%% denote a sub structure, therefore it does not
                     %%% denote anything.
                     error([myname ': illegal option name ' upper(optname)]);
                 else
                     %%% OPTNAME contains a dot, so it may denote a
                     %%% substructure. But does this substructure exist?
                     [substruct,substructfield]=strtok(optname,'.');
                     substructfield=substructfield(2:end); % strip leading dot
                     if isfield(options,substruct)==1
                         %%% SUBSTRUCT is a field of OPTIONS, so set value
                         %%% into field of substruct - if it exists...
                         if isfield(getfield(options,substruct),substructfield)==1
                             %%% field denoted by SUBSTRUCTFIELD exists,
                             %%% set value to OPTVALUE
                             evalstr=['options.' substruct '.' substructfield '=optvalue;'];
                             eval(evalstr);
                         else
                             %%% field denoted by SUBSTRUCTFIELD does not
                             %%% exists, cast an error
                             error([myname ': ' upper(substructfield),...
                                   'is not a field of sub structure ',...
                                   upper(substruct)]);
                         end; % if isfield
                     else
                         %%% SUBSTRUCT is not a field of OPTIONS
                         error([myname ': ' upper(substruct),...
                                'is not an existing field']);
                     end; % if isfield(substruct)
                 end; % if isempty(findstr))
                 
             end; % if isfield

         end; % for indy
      else
         %%% odd: illegal!
         error([myname ': number of input arguments must be even or 0 or 1!']);
      end; % if nin2/2
end; % switch nin2


%%%%%%%% plausibility tests
options=mklscheckopt(options);


