function incomp=mklspointchk(lsopt,statcat,seisdat,hlon,hlat,hdepth);
% mklspointchk.........determine degree of compatibility of a single point 
%
% call: incomp=mklspointchk(lsopt,statcat,seisdat,hlon,hlat,hdepth);
%
%           lsopt: (structure)
%                  LOCSMITH options structure as returned by MKLSOPT
%
%         statcat: (structure)
%                  Optional LSD structure containing Station descriptions
%                  This struct may contain station descriptions
%                  (coordinates) to be used when the SEISDAT structure does
%                  not contain all station information (or no station info
%                  at all)
%                  This might be empty [], then only the station
%                  information from the SEISDAT structure is used.
%
%                  IMPORTANT:
%                  It is not checked if STATCAT and SEISDAT contain
%                  contradictory informations, e.g. different coordinates
%                  for the same station. If such contradictory data is
%                  given, it is undefined which information is actually
%                  used, and phantastic things may happen!
%
%         seisdat: (structure)
%                  LSD structure containing seismic arrival data for a
%                  single event. The structure may optionally contain
%                  station information data.
%
%                  NOTE: _some_ station information must be given! If
%                  neither STATCAT nor SEISDAT contain station info, the
%                  location cannot be carried out!
%
%         hlon: (number) [deg]
%              candidate hypocenter longitude
%
%         hlat: (number) [deg]
%              candidate hypocenter latitude
%
%         hdepth: (number) [km]
%              candidate hypocenter depth below surface
%
%
%
% results:
%
%         incomp: (numeric array) [counter]
%                 final incompatiblity status as determined by
%                 MKLSGRIDEVALINCOMP. 
%
%                 incomp(1): This value shows how many of the data
%                           are not compatible with the returned solution.
%                           0: all data compatible. congratulations, you have located
%                           the quake!
%                           >0: a number of INCOMP data values is not compatible with
%                               the observations. You should look into MYGRID and
%                               into the data to find out the reason (outlier data,
%                               wrong velocity model or other problems)
%                               Running MKLOCSMITH at higher verbosity (level 4 or higher)
%                               will give hints which data are not compatible.
%                 incomp(2): This is just the total number of data against
%                            which the nodes were checked. You need this to
%                            find out how severe the incompatibility really is.
%
%
% This routine takes LSD input data as MKLOCSMITH itself, but tests only
% one single point in the planet for compatibility with the given data.
%
% The routine constructs a dummy grid which contains the desired point and
% then uses the same subroutines for evaluation as MKLOCSMITH does. To do
% so, it creates a dummy LSOPT structure which is partly derived from the
% LSOPT structure given by the user. However, it is assumed that a
% tringular grid is used.
%
% Martin Knapmeyer, 27.06.2007


%%% init result
incomp=[inf 0];


%%% local verbosity level
verbosity=1;
gridplotmode=lsopt.verbosity_plot_mode;

%%% say hello
mklsverbose(verbosity,lsopt,'MKLSPOINTCHK: starting at ',mkclock);

%%% start timer
tic;

%%% connect station descriptions from STATCAT and SEISDAT into a single
%%% structure - just concatenate STATCAT to the respective SEISDAT field.
if ~isempty(statcat)
   seisdat.station=[seisdat.station statcat.station];
   seisdat.statcnt=seisdat.statcnt+statcat.statcnt;
end;


%%% translate the extended SEISDAT structure into a simpler format that may
%%% be searched by "find()". This simpler format has to be sorted station
%%% by station.
simpledata=mklslsd2fxd(seisdat);


%%% give overview of amount of data and number of data
%%% STATCNT is the total number of stations
%%% CUMDATACNT is the cumulative number of data at all stations, to be
%%%            compared with the .tested-field of the layer structure
%%%            cumdatacnt(1) is the number of data _before_ the evaluation
%%%            of the first station and thus always zero.
%%%            cumdatacnt(2) is the number of data _before_ the evaluation
%%%            of the second station, i.e. the number of data given for the
%%%            first station.
%%%            cumdatacnt(3) is the number of data at the first station
%%%            plus the number of data at the second station.
%%%            So after the evaluation of a node at the n-th station, the
%%%            .tested-counter for that node must equal cumdatacnt(n). The
%%%            evaluation of a node is necessary if the .tested-counter is
%%%            smaller than cumdatacnt. A poit is completely evaluated if
%%%            its .tested-counter equals cumdatacnt(end).
%%% TOTALATCNT is the total number of arrival times at all stations together
statcnt=length(simpledata);
cumdatacnt=zeros(1,statcnt+1);
for stationindy=1:statcnt
    mklsverbose(verbosity,lsopt,'MKLSPOINTCHK: data type count for station: ',...
                simpledata(stationindy).id,char(13),...
                '            arrival times:  ',simpledata(stationindy).atcnt,char(13),...
                '            back azimuths:  ',simpledata(stationindy).bazcnt,char(13),...
                '            emergence ang.: ',simpledata(stationindy).emergcnt,char(13),...
                '            slownesses:     ',simpledata(stationindy).slocnt);
    cumdatacnt(stationindy+1)=simpledata(stationindy).atcnt+...
                              simpledata(stationindy).bazcnt+...
                              simpledata(stationindy).emergcnt+...
                              simpledata(stationindy).slocnt;
end; % for stationindy
cumdatacnt=cumsum(cumdatacnt); % makes cumdatacnt a cumulative sum



%%% second part of the data incompatibility measure:
%%% total number of available data
incomp(2)=cumdatacnt(end);


%%% dummy lsopt structure to create a simple grid
%%% this grid will have one layer with a radius corresponding to the
%%% desired point's depth
dmylsopt=lsopt; % in general, we use the options given by the user
dmylsopt.grid_base='tetrahedron'; % do no generate an unnecessary amount of nodes
dmylsopt.grid_init_minr=lsopt.planet_radius-hdepth; % only one layer at the desired depth
dmylsopt.grid_init_maxr=dmylsopt.grid_init_minr;
dmylsopt.grid_init_dr=10000; % a big number to guarantee that there is only one layer
dmylsopt.grid_init_wdt=1e5; % to guarantee that the initial grid is not iterated
dmylsopt.iter_max=1; % to guarantee that no iteration occurs at all
dmylsopt.iter_vertical_max=0; % to guarantee that no iteration occurs at all



%%% create the dummy grid
mklsverbose(verbosity,lsopt,'MKLSPOINTCHK: creating new search grid.');
mygrid=mklsgridnew(dmylsopt);


%%% replace one of the grid nodes by the desired point location
mygrid.layers.radius=lsopt.planet_radius-hdepth; % make sure layer is at correct depth
mygrid.layers.nodes(1,2)=hlon;
mygrid.layers.nodes(1,3)=hlat;
wantednodeid=mygrid.layers.nodes(1,1);


%%% we want to test only one single point, so we pretend that all others
%%% have already been tested
nodecnt=size(mygrid.layers.nodes(:,1)); % number of nodes
for nodeindy=2:nodecnt
    mygrid.layers.tested(nodeindy,2)=incomp(2);
end; % for nodeindy


%%%%%% evaluate the dummy grid


%%% to be able to call the time evaluation, we need a new TTSUPPORT
%%% structure.
ttsupport=mklsemptyttsupport(lsopt);


%%% evaluate current depth sampling
[horexitstatus,mygrid,ttsupport]=mklsgrideval(dmylsopt,mygrid,simpledata,cumdatacnt(statcnt),ttsupport);

%%% extract incompatibility of the desired node
indy=find(mygrid.layers.fit(:,1)==wantednodeid);
incomp(1)=incomp(2)-mygrid.layers.fit(indy,2);

%%% write result to screen
mklsverbose(verbosity-1,lsopt,'MKLSPOINTCHK: resulting incompatibility:',...
            incomp(1),' of ',incomp(2));
