function [exitstatus,incomp,loc,mygrid]=mklocsmith(lsopt,statcat,seisdat,mygrid);
% mklocsmith.......LOCSMITH main program: location of Earthquakes on grids
%
% call: [exitstatus,loc,mygrid]=mklocsmith(lsopt,statcat,seisdat);
%       [exitstatus,loc,mygrid]=mklocsmith(lsopt,statcat,seisdat,mygrid);
%
%           lsopt: (structure)
%                  LOCSMITH options structure as returned by MKLSOPT
%
%         statcat: (structure)
%                  Optional LSD structure containing Station descriptions
%                  This struct may contain station descriptions
%                  (coordinates) to be used when the SEISDAT structure does
%                  not contain all station information (or no station info
%                  at all)
%                  This might be empty [], then only the station
%                  information from the SEISDAT structure is used.
%
%                  IMPORTANT:
%                  It is not checked if STATCAT and SEISDAT contain
%                  contradictory informations, e.g. different coordinates
%                  for the same station. If such contradictory data is
%                  given, it is undefined which information is actually
%                  used, and phantastic things may happen!
%
%         seisdat: (structure)
%                  LSD structure containing seismic arrival data for a
%                  single event. The structure may optionally contain
%                  station information data.
%
%                  NOTE: _some_ station information must be given! If
%                  neither STATCAT nor SEISDAT contain station info, the
%                  location cannot be carried out!
%
%          mygrid: (structure)
%                  Optional grid structure: The output of a previous run of
%                  MKLOCSMITH can be used as input, i order to further
%                  refine the grid. This is useful if the previous solution
%                  is not compatible with all data or is suspect to be
%                  incomplete. You can also just produce a solution with
%                  better resolution.
%                  BUT: if this is the output of MKLOCSMITH, then
%                  MKLOCSMITH had some reason to quit. So if you restart
%                  with this grid and the same LSOPT as before, MKLOCSMITH
%                  will quit immediately, since the termination condition
%                  is still true. So you have to change at least one of the
%                  iteration control parameters in LSOPT!
%
%
% result: 
%         exitstatus: (string)
%                  a string denoting the reason why MKLOCSMITH came to a
%                  halt. This string should be considered when evaluating
%                  the output. The follwiong values are possible:
%
%                  'unexpected': the exit status is initialized with this
%                                value. At any legal exit condition, a new
%                                value should be written into the string.
%                                If this is not the case, the init value
%                                remains, because the stopping of the
%                                routine was indeed due to an unexpected
%                                reason. This should never occur.
%
%                  'no data':    There is no data to evaluate. No grid is
%                                generated and no location produced.
%
%                  'vertically converged':  
%                                The desired spatial grid resolution for the
%                                location has been reached. The routine
%                                quits because the job is done.
%
%                  'vertical iteration timeout':
%                                The maximum number of iterations to
%                                deterimine the focal depth has been
%                                reached. This limit is defined in
%                                lsopt.iter_vertical_max.
%                                If this occurs, the job is NOT done and
%                                you should probably retry the location 
%                                with a higher iter_layer_max limit.
%                                lsopt.iter_vertical_max is intended to be an
%                                emergency brake to avoid infinite
%                                iteration, not a standard tool to finish
%                                the program!
%
%                  'horizontal iteration timeout': 
%                                The maximum number of iterations to 
%                                determine the horizontal extent of the 
%                                location has been reached. 
%                                This limit is defined in lsopt.iter_max.
%                                If this occurs, the job is NOT
%                                done and you should probably repeat the
%                                location with a higher iter_max limit.
%                                lsopt.iter_max is intended to be an
%                                emergency brake to avoid infinite
%                                iteration, not a standard tool to finish
%                                the program!
%
%         incomp: (numeric array) [counter]
%                 final incompatiblity status as determined by
%                 MKLSGRIDEVALINCOMP. 
%
%                 incomp(1): This value shows how many of the data
%                           are not compatible with the returned solution.
%                           0: all data compatible. congratulations, you have located
%                           the quake!
%                           >0: a number of INCOMP data values is not compatible with
%                               the observations. You should look into MYGRID and
%                               into the data to find out the reason (outlier data,
%                               wrong velocity model or other problems)
%                               Running MKLOCSMITH at higher verbosity (level 4 or higher)
%                               will give hints which data are not compatible.
%                 incomp(2): This is just the total number of data against
%                            which the nodes were checked. You need this to
%                            find out how severe the incompatibility really is.
%
%         loc: (structure)
%              This structure contains the 4D coordinates of those node
%              which fit the data best. The details of the structure depend
%              on grid type, see MKLSGRIDEXTRACTLOC for details.
%              
%                   
%
%         mygrid: (structure array)
%                 The search grid as it is at the end of the location
%                 process. This is a LOCSMITH search grid structure as
%                 returned by MKLSGRIDNEW.
%
%
% MKLOCSMITH is the main routine of the LOCSMITH package. This routine is
% the one called by the user in order to locate a quake. It drives the
% construction and subsequent modifcation of the search grid and the
% evaluation of the seismoligcal data on this grid.
%
% ALGORITHM:
%
% In the outermost loop, the maximum attainable compatibility with the
% given data is computed as one dimensional integer function of the vertical
% coordinate (depth or radius, depending on grid type). The maxima of this
% function are searched.
% For each value of the vertical coordinate, a whole layer is evaluated to
% determine the compatiblity as function of the horizontal coordinates. The
% iteration of layers therefore serves as the method to determine the
% function values of compatibility as function of depth/radius.
%
%
% Martin Knapmeyer, 03.02.2005, 04.07.2005, 05.02.2007

% 05022007: save data after each mklsgrideval call





%%% how much input?
nin=nargin;



%%% init result
exitstatus='unexpected';
loc=[];
incomp=[NaN NaN];
if nin==3
   mygrid=[];
end; % if nin==3


%%% local verbosity level
verbosity=1;
gridplotmode=lsopt.verbosity_plot_mode;

%%% say hello
mklsverbose(verbosity,lsopt,'MKLOCSMITH: starting at ',mkclock);

%%% start timer
tic;

%%% connect station descriptions from STATCAT and SEISDAT into a single
%%% structure - just concatenate STATCAT to the respective SEISDAT field.
if ~isempty(statcat)
   seisdat.station=[seisdat.station statcat.station];
   seisdat.statcnt=seisdat.statcnt+statcat.statcnt;
end;


%%% translate the extended SEISDAT structure into a simpler format that may
%%% be searched by "find()". This simpler format has to be sorted station
%%% by station.
simpledata=mklslsd2fxd(seisdat);


%%% give overview of amount of data and number of data
%%% STATCNT is the total number of stations
%%% CUMDATACNT is the cumulative number of data at all stations, to be
%%%            compared with the .tested-field of the layer structure
%%%            cumdatacnt(1) is the number of data _before_ the evaluation
%%%            of the first station and thus always zero.
%%%            cumdatacnt(2) is the number of data _before_ the evaluation
%%%            of the second station, i.e. the number of data given for the
%%%            first station.
%%%            cumdatacnt(3) is the number of data at the first station
%%%            plus the number of data at the second station.
%%%            So after the evaluation of a node at the n-th station, the
%%%            .tested-counter for that node must equal cumdatacnt(n). The
%%%            evaluation of a node is necessary if the .tested-counter is
%%%            smaller than cumdatacnt. A poit is completely evaluated if
%%%            its .tested-counter equals cumdatacnt(end).
%%% TOTALATCNT is the total number of arrival times at all stations together
statcnt=length(simpledata);
cumdatacnt=zeros(1,statcnt+1);
for stationindy=1:statcnt
    mklsverbose(verbosity,lsopt,'MKLOCSMITH: data type count for station: ',...
                simpledata(stationindy).id,char(13),...
                '            arrival times:  ',simpledata(stationindy).atcnt,char(13),...
                '            back azimuths:  ',simpledata(stationindy).bazcnt,char(13),...
                '            emergence ang.: ',simpledata(stationindy).emergcnt,char(13),...
                '            slownesses:     ',simpledata(stationindy).slocnt);
    cumdatacnt(stationindy+1)=simpledata(stationindy).atcnt+...
                              simpledata(stationindy).bazcnt+...
                              simpledata(stationindy).emergcnt+...
                              simpledata(stationindy).slocnt;
end; % for stationindy
cumdatacnt=cumsum(cumdatacnt); % makes cumdatacnt a cumulative sum

%%% second part of the data incompatibility measure:
%%% total number of available data
incomp(2)=cumdatacnt(end);


%%%%%% here we begin with the real stuff


%%% if there is no data at all, do not waste time and finish immediately!
if incomp(2)==0
    incomp(1)=incomp(2);
    exitstatus='no data';
else
    
    %%% OK, there is data and we have to evaluate it. (So much for the idea
    %%% of a lazy afternoon ;-) )
    
    %%% generate the initial search grid if necessary
    switch nin
        case {3}
             %%% no search grid is given, create one
             mklsverbose(verbosity,lsopt,'MKLOCSMITH: creating new search grid.');
             mygrid=mklsgridnew(lsopt);
        case {4}
             %%% use the search grid given in input
             mklsverbose(verbosity,lsopt,'MKLOCSMITH: using input search grid.');
        otherwise
             error(['MKLOCSMITH: requires 3 or 4 input parameters!']);
    end; % switch nin
    
    
    %%% to be able to call the time evaluation, we need a new TTSUPPORT
    %%% structure.
    ttsupport=mklsemptyttsupport(lsopt);

    
    %%% control plot of initial grid (not recommended for large grids...)
%     mklsgridcontrolplot(lsopt,mygrid,simpledata);
    gridplotmode=lsopt.verbosity_plot_mode;
    if strcmp(gridplotmode,'none')==0
        figure(lsopt.verbosity_plot_fig);
        cla;
        %%% creates base map
        mkmakemap(lsopt.mapopt);
        %%% overlay grid nodes
        hold on
        mklsgridplot(lsopt,mygrid,gridplotmode); 
        hold off
        %%% overlay stations
        hold on
        mklsnetplot(lsopt,seisdat,gridplotmode);
        hold off
        drawnow;
    end; % if strcmp(gridplotmode,'none')==0
    
    
    %%% vertical iteration loop
    %%% This loop ends when the vertical resolution defined in lsopt.drmin
    %%% is reached or when the maximum number of iterations defined in
    %%% lsopt.iter_vertical_max is exceeded.
    done=0;
    vertitercnt=0;
    while done==0
        
        %%% say hello, iteration
        mklsverbose(verbosity,lsopt,'MKLOCSMITH: start vertical iteration #',vertitercnt);
        
        
        %%% evaluate current depth sampling
        [horexitstatus,mygrid,ttsupport]=mklsgrideval(lsopt,mygrid,simpledata,cumdatacnt(statcnt),ttsupport);
        
        
        %%% save everything MK05022007
        save ./locsmith_save.mat horexitstatus mygrid ttsupport lsopt simpledata cumdatacnt
        

        %%% handle different exit status options of MKLSGRIDEVAL
        switch horexitstatus
            
            case {'unexpected'}
                
                %%% don't know why MKLSGRIDEVAL has finished - something's
                %%% wrong. better stop everything.
                exitstatus='unexpected';
                done=1;
                
                %%%%%% end of case 'unexpected'
                
            case {'no data'}
                
                %%% There is no data to evaluate. we're done.         
                exitstatus='no data';
                done=1;
                
                %%%%%% end of case 'no data'
                
            case {'horizontal iteration timeout'}
                
                %%% the evaluation of ONE layer was stopped because of a
                %%% timeout. This means that the result for tht layer is
                %%% not reliable and the whole process should be restarted.
                exitstatus='horizontal iteration timeout';
                done=1;
                 
                %%%%%% end of case 'horizontal iteration timeout'
           
            case {'horizontally converged'}
                
                %%% ALL layers have been evaluated succesfully, now improve the
                %%% vertical sampling if neccessary/possible.
        
                %%% extract compatibility as function of depth from grid
                [z,cofz,nodecnt]=mklsgridgetcofz(lsopt,mygrid);


                %%% plot compatibility as function of depth (only for development purpose)
                figure(2);
                clf;
                [plotz,sorter]=sort(z);
                plot(cofz(sorter),lsopt.planet_radius-plotz,'k.-');
                axis([-1 cumdatacnt(end)+1 0 lsopt.planet_radius]);
                axis ij
                xlabel('Number of Satisfied Constraints');
                ylabel('Depth [km]');
                title(['Compatibility with ' int2str(cumdatacnt(end)) ' Data']);
                grid on
                drawnow;

                
                %%% generate new, improved depth sampling
                [extendedz,newdz]=mklsgridnewvertical(lsopt,z',cofz');


                %%% if the current sampling of the C(z) function is better than the
                %%% lsopt.iter_drmin, leave iteration loop, else improve sampling
                if ((isempty(extendedz))&(done==0)) % was: ((newdz<lsopt.iter_drmin)|(isempty(extendedz)))&(done==0) MK07092005
                   %%% vertical sampling is finished! We're done!
                   exitstatus='vertically converged';
                   done=1;
                   mklsverbose(verbosity,lsopt,'MKLOCSMITH: vertical sampling will not be modified any further. Quit iteration loop. (Exit 1)');
                else
                   %%% add new layers at the new depth values to the grid
                   
                   
                   %%% insert new layers into search grid
                   asbefore=length(mygrid.layers);
                   mygrid=mklsgridextend(lsopt,mygrid,extendedz);
                   if length(mygrid.layers)==asbefore
                      %%% MKLSGRIDEXTEND did not change the grid - finish.
                      exitstatus='vertically converged';
                      done=1;
                      mklsverbose(verbosity,lsopt,'MKLOCSMITH: vertical sampling will not be modified any further. Quit iteration loop. (Exit 2)');
                   end; % if length(mygrid.layers)==asbefore
                   
                   %%% insert markers for new layer depths into
                   %%% compatibility as function of depth plot
                   %figure(2);
                   %hold on
                   %for indy=1:length(mygrid.layers)
                   %    plot(0,mygrid.layers(indy).radius,'o');
                   %end; % for indy
                   %hold off
                   
                end; % if 


                %%% increment iteration counter
                vertitercnt=vertitercnt+1;


                %%% check if vertical timeout is reached
                if (vertitercnt>lsopt.iter_vertical_max)&(done==0)
                    %%% TIMEOUT!
                    %%% exit condition checked only of no earlier exit condition
                    %%% already applied - otherwise the true exit status could be
                    %%% overwritten!
                    exitstatus='vertical iteration timeout';
                    done=1;
                    mklsverbose(verbosity,lsopt,'MKLOCSMITH: Max number of vertical iterations applied. Quit iteration loop. (Exit 3)');
                end; % if itercnt>lsopt.maxiter


                %%% control plot of current grid (not recommended for large grids...)
        %         mklsgridcontrolplot(lsopt,mygrid,simpledata);
                gridplotmode=lsopt.verbosity_plot_mode;
                if strcmp(gridplotmode,'none')==0
                    figure(lsopt.verbosity_plot_fig);
                    cla;
                    %%% creates base map
                    mkmakemap(lsopt.mapopt);
                    %%% overlay grid nodes
                    hold on
                    mklsgridplot(lsopt,mygrid,gridplotmode); 
                    hold off
                    %%% overlay stations
                    hold on
                    mklsnetplot(lsopt,seisdat,gridplotmode);
                    hold off
                    drawnow;
                end; % if strcmp(gridplotmode,'none')==0

                %%%%%% end of case: horizontally converged
        
            otherwise
                error(['MKLOSMITH: unkown horizontal exit status ' upper(horexitstatus)]);
        end; % switch horexitstatus
        
    end; % while done
    
    
    
    %%% test data compatibility status: are there nodes that are compatible
    %%% with all data? If not, how many incompatible data are there?
    incomp(1)=mklsgridevalincomp(mygrid);
    
end; % if incomp(2)==0





%%%%%% finally, we're done.

%%% extract nodes with maximum compatibility from grid
loc=mklsgridextractloc(lsopt,mygrid);


%%% say goodbye
toctime=toc;
mklsverbose(verbosity,lsopt,'MKLOCSMITH: finished at ',mkclock,' total time ',toctime,'s');
mklsverbose(verbosity,lsopt,'MKLOCSMITH: total number of nodes generated: ',loc.nodecnt);
if incomp(2)~=0
    %%% how much of the data is compatible?
    incompstatus=100*incomp(1)/incomp(2);
else
    %%% This happens when there is no data to evaluate
    incompstatus=NaN;
end; % if incomp2~=0
mklsverbose(verbosity,lsopt,'MKLOCSMITH: final incompatiblity status: ',...
            incomp(1),' of ',incomp(2),' = ',incompstatus,'%');
mklsverbose(verbosity,lsopt,'MKLOCSMITH exit status: ',exitstatus);