function handle=mk3dcolorbar(direction,ticks,radius,cblength);
% mk3dcolorbar.........creates a 3D colorbar
%
% call: handle=mk3dcolorbar;
%       handle=mk3dcolorbar(direction);
%       handle=mk3dcolorbar(direction,ticks);
%       handle=mk3dcolorbar(direction,ticks,radius,length);
%
%            direction: direction of colorbar. possible values:
%                       'hor': horizontal colorbar
%                       'vert': vertical colorbar
%                       default: 'hor'
%            ticks: values at which labels are written
%                   ticks(1) is the leftmost/lowermost label
%                   ticks(length(ticks)) is the rightmost/uppermost label
%                   tick values are in units of caxis.
%                   default: []
%            radius: radius of the colorbar
%                    default: 1/40 of length
%            length: length of the colorbar
%                    default: 1/3 of axis qidth (or height, depending on direction)
%
% result: handle: list of handles to created objects
%                 which represent the colorbar
%
% the colorbar is realized as a colored cylinder in order
% to show not only the used colors but also the effect
% of lighting.
%
% Martin Knapmeyer, 02.07.2003


%%% a useful constant
radian=pi/180;


%%% determine current axis ranges
ax=axis;
cax=caxis;

%%% make ticks a vector
ticks=ticks(:);

%%% Defaults
switch nargin
   case {0}
        direction='hor';
        ticks=[];
        cblength=abs(ax(2)-ax(1))/2;
        radius=cblength/40;
   case {1}
        ticks=[];
        cblength=abs(ax(2)-ax(1))/2;
        radius=cblength/40;
   case {2}
        cblength=abs(ax(2)-ax(1))/2;
        radius=cblength/40;
   case {4}
        %%% everything is defined by user
   otherwise
       error('MK3DCOLORBAR: illegal number of input arguments!');
end; % switch nargin

%%% get colormap
rgb=colormap;
rgblen=size(rgb,1);

%%% generate x/y/z values and cdata matrix for the colorbar
[x,y,z]=cylinder((1:rgblen)*0+radius);

%%% generate x/y/z values for ticks
tickradius=1.1*radius;
tx=tickradius*cos((0:10:360)*radian);
ty=tickradius*sin((0:10:360)*radian);
tz=ty*0;


%%% generate CDATA matrix
cdata=z*rgblen; % this will be the color index


%%% evaluate direction switch
switch direction
   case {'hor'}
       %%% modify surface xy data
       plotx=z*cblength;
       ploty=x;
       %%% position
       xoffset=mean(ax(1:2))-cblength/2;
       yoffset=min(ax(3:4))-2*radius;
       %%% modify tick xy data
       tickpx=tz;
       tickpy=tx;
       tickxshift=cblength/abs(max(cax)-min(cax));
       tickyshift=0;
   case {'vert'}
       %%% modify surface xy data
       plotx=x;
       ploty=z*cblength;
       %%% modify tick xy data
       tickpx=tx;
       tickpy=tz;
       tickxshift=0;
       tickyshift=cblength/abs(max(cax)-min(cax));
       %%% position
       xoffset=max(ax(1:2))+2*radius;
       yoffset=mean(ax(3:4))-cblength/2;
   otherwise
       error(['MK3DCOLORBAR: unknown direction ' upper(direction)]);
end; % switch direction

%%% shift coordinates to centered plotting 
plotx=plotx+xoffset;
ploty=ploty+yoffset;
tickpx=tickpx+xoffset;
tickpy=tickpy+yoffset;

%%% coordinates of box around the cylinder
minx=min(plotx(:));
maxx=max(plotx(:));
miny=min(ploty(:));
maxy=max(ploty(:));
meanz=mean(y(:));

%%% draw colorbar
hold on
%%% cylinder
handle=surf(plotx,ploty,y);
set(handle,'CData',cdata);
set(handle,'CDataMapping','direct');
set(handle,'EdgeColor','interp');
set(handle,'FaceColor','interp');
set(handle,'FaceLighting','Flat');
set(handle,'EdgeLighting','Flat');
%%% horizontal box around cyliner
handle(2)=fill3([minx maxx maxx minx],[miny miny maxy maxy],[1 1 1 1]*meanz,'k');
set(handle(2),'FaceColor','none');
set(handle(2),'EdgeColor','k');
%%% ticks
ticks=[min(cax)' ticks' max(cax)']';
tickanz=length(ticks);
for indy=1:tickanz
    handle(3)=fill3(tickpx+tickxshift*abs(ticks(indy)-min(cax)),...
                   tickpy+tickyshift*abs(ticks(indy)-min(cax)),...
                   ty,'k');
end; % for indy
hold off
