function h=mkearth(varargin);
% mkearth...............make very impressive 3D plot of globe
%
% call:  mkearth;
%        mkearth('defaults');
%        mkearth('demo1');
%        mkearth('demo2');
%        mkearth('help');
%        h=mkearth(lon,lat,hgt);
%        h=mkearth(PropertyName,PropertyValue,...);
%
%        without arguments: demo1 plot
%
%        one argument version: return default settings of all globe
%                              properties or print help file or run
%                              one of the demos.
%
%        three arguments version: lon: center longitude [deg]
%                                 lat: center latitude [deg]
%                                 hgt: camera height above LON/LAT [km]
%                                 The three argument mode is a simlple way
%                                 to define Camera and Target positions (see
%                                 below): camera is at height HGT above Target
%
%        many arguments version: Allows to define many plot properties,
%                                see below for explanation
%
% results: none
%
%
% Globe Properties:
%
% If you wish to customize your plot, enter pairs of PropertyNames and Property-
% Values like for MatLab handle graphic objects. The following table lists
% all Properties and what they are good for. Some properties are defined by
% scalars, some are vectors, some are strings. To find out the default values,
% call MKEARTH without argument 'defaults'.
% String Properties are NOT case sensitive.
% Property names are NOT case sensitive.
% To make you able to edit created objects afterwards, any created surface will
% contain a specific string in the UserData field of its objects description.
% Use FINDOBJ to find objects created by MKEARTH. The specific strings identifying
% different types of surfaces are given below.
%
%     Mode      'string'      used to distinguish several modes. Available are:
%                                'help': print help file 
%                             'default': return default values of all Properties
%                               'demo1': run demo 1
%                               'demo2': run demo 2
%                             'regular': use given (or default) parameters to
%                                        plot the globe
%                             For obvious reasons, the routine will return
%                             'default' as default mode, although 'regular' is
%                             the default mode!
%                             call mkearth('mode','regular'); to see the result
%                             of default settings.
%                             If you enter a 'resolution', it will be transported
%                             into the demo mode, allowing to run it with high
%                             resolution data sets.
% 
%
%     Target    [lon lat]     Camera target on globe: defines the geographical
%                             location of the point in view
%                             will be transformed into xyz-coordinates and
%                             passed to the axes-object's CameraTarget
%                             LON and LAT are in degrees
%                             By default, this will be set to be directly
%                             on the earth's surface at longitue and latitude of
%                             the camera.
%
%     Camera    [lon lat hgt] Camera position above globe
%                             will be transformed into xyz-coordinates and
%                             passed to the axes object's CameraPosition
%                             LON and LAT in DEGREES, HGT in km
%
%     LightPos  [dlon dlat]   relative light position
%                             a light object will be created at position
%                             [lon+dlon lat+dlat hgt]
%                             DLON and DLAT are in degrees
%
%
%     Flame     'string'      switch light on or don't
%                             'on': a light object will be generated
%                             'off': do no generate light object
%                             it is recommended to set this to 'off' from the
%                             second of several subsequent calls to MKEARTH on
%                             to avoid generating many light sources.
%
%     Stuff     'string'      choose material properties for earth surface
%                             available are 'dull', 'shiny', 'metal' (see
%                             MatLab's MATERIAL for description)
%
%     Resolution 'string'     Elevation model resolution
%                             'preview': use the topo-variable shipped with
%                                        MatLab (what you get by 'load topo')
%                                'none': do not plot any topography
%                              pathname: path to a .mat-file containing a vari-
%                                        able named 'topo' which contains the
%                                        elevation model in the same format as
%                                        the topo dataset shipped with MatLab
%                                        may also contain arrays TOPOLON and
%                                        TOPOLAT describing the longitudes and
%                                        latitudes of all points in a way that
%                                        surf(topolon,topolat,topo) plots a map.
%                                        If TOPOLON and TOPOLAT are not given in
%                                        the file, they will be generated.
%                                        (MKEARTH loads the file using LOAD(),
%                                        so you may omit the .mat-suffix or
%                                        give a path to an ASCII file.)
%
%     Exaggeration  xagg      vertical exaggeration of topography
%                             is set to 1 if 'resolution' is 'none'
%
%     PlotArea   [swlon swlat nelon nelat]
%                             Area to plot, defined as rectangle from upper
%                             left corner (northwest, nw) to lower right (south-
%                             east, se) corner
%                             It is currently not possible to cut out areas that
%                             are crossed by the greenwich meridian. 
%
%     AreaMode     string     area plot mode
%                             'positive': plot topography inside PlotArea
%                             'negative': plot topography outside PlotArea
%                             'nocut': do not cut out any parts of the surface
%
%     MinAltitude    minalt   areas with altitude below MINALT will not be
%                             plotted
%
%     MaxAltitude   maxalt    areas with altitude above MAXALT will not be
%                             plotted
%
%     RadialLift    rlift     Add RLIFT km to topography in PlotArea to show
%                             earth's interior, in km
%
%     XYZShift     [dx dy dz] shift anything by DX,DY,DZ km
%
%
%     ColorResolution  colres change topography color any COLRES meters
%
%     WireFrame    depth      plot wire frame from PlotArea topography (plus
%                             RadialLift and XYZShift)
%                             if empty: do not plot wire frame
%                             else: downto DEPTH km, measured from the current
%                                   EarthRadius
%                             -1: plot wire frame of thickness equal to
%                                       RadialLift
%
%     Walls        depth      plot massive walls from PlotArea topography (plus
%                             RadialLift and XYZShift)
%                             if empty: do not plot walls
%                             else: downto DEPTH km, measured from the current
%                                   EarthRdaius
%                             -1: plot walls to depth equal to RadialLift
%
%     WallSize    [rad lonlat] size of matrices used to draw walls
%                              a matrix of size lonlat-by-rad elements will
%                              be used to draw vertical walls (independent of
%                              their angular or vertical real-world-size)
%
%
%     EarthRadius  re         Earth's Radius, in km
%                             changing this is not really the same as changing
%                             the exaggeration: the latter does not affect the
%                             size of the coordinate range.
%                             depths defined elsewhere (e.g. for walls) will
%                             be relative to this radius.
%
%     Coordinates [minlon maxlon minlat maxlat]
%                             coordinate range covered by topography data
%                             allows to use topo-files that conatain neither
%                             the whole earth nor separate lon and lat arrays.
%
%                             allows to stretch and quench topographies.
%
%     CoordMode   'string'    defines which coordinates to use
%                             'manual': use coordinate range given in
%                                       COORDINATES
%                             'auto':   use COORDINATES only if data file
%                                       did not contain lon and lat arrays
%
%                              will automatically be set to 'manual' if COORDI-
%                              NATES ared defined in the input.
%
%     Discontinuities [d1 d2 ... dn]  depths of discontinuities insides earth
%                                     (given as radius in km) to be considered
%                                     when plotting walls in cross sections
%                                     Walls will be defined at some depths given
%                                     by a linspace call and at these discontinuities.
%
%
% UserData strings used to designate created graphic objects:
%
% 'MKEARTH/topo' - topography surface
% 'MKEARTH/colorfix' - surface used to fix colors to a certain range
% 'MKEARTH/sphere' - spheres drawn with resolution 'none'
% 'MKEARTH/wall/W' - western wall of drawn plot area's walls
% 'MKEARTH/wall/E' - eastern wall of drawn plot area's walls
% 'MKEARTH/wall/S' - southern wall of drawn plot area's walls
% 'MKEARTH/wall/N' - northern wall of drawn plot area's walls
%
% This is a standalone-program: all you need to run it is the mkearth.m file and
% MatLab in Version 5 or higher (the program makes use of laguage constructions
% that were not available in earlier versions). The program does not make any
% assumptions on your platform (except the existence of color graphics) and
% should run on any platform for which MatLab is available.
%
%
% The routine plots into the current figure using HOLD ON, so be sure to apply 
% clf if you don't want to mix it up with other plots!
%
% Martin Knapmeyer, 07.01.2000, 18.02.2000, 03.03.2000, 16./17.03.2000, 26.03.2000
%                   27.03.2000, 29.03.2000, 19.04.2000, 19.06.2002

%%%%% define helper variables
radian=pi/180; % factor to transform degrees into radians
h=[]; % dummy for result variable


%%%%% analyze input
p=mkanalyzeinput(varargin);


%%%%% for faster rendering
set(gcf,'renderer','zbuffer');

%%%%% run something
switch p.mode
  case {'default','defaults'},
      disp(p);
  case {'help'},
      help mkearthhelp;
  case {'demo1'},
       disp('MKEARTH: running demo1');    
       area=[20 -20 80 65];
       xyzshift=[4000 1000 0];
       colres=50;
       mkearth('areamode','negative','plotarea',area,...
               'wireframe',2900,'walls',2900,...
               'lightpos',[100 -65],...
               'resolution',p.resolution);
       hold on
       mkearth('resolution','none','earthradius',3471,'wireframe',[],...
               'plotarea',area,'areamode','positive',...
               'flame','off');
       mkearth('areamode','positive','plotarea',area,...
               'xyzshift',xyzshift,...
               'resolution','none','earthradius',3471,'wireframe',[],...
               'flame','off');
       mkearth('areamode','positive','plotarea',area,...
               'xyzshift',xyzshift,...
               'wireframe',2900,'walls',2900,...
               'colorresolution',colres,...
               'camera',[90 30 90000],'lightpos',[100 -65],...
               'resolution',p.resolution,...
               'flame','off');
       hold off;
  case {'demo2'},
       clf;
       mkearth('plotarea',[180 -90 360 90],'areamode','positive',...
               'walls',2900,...
               'resolution',p.resolution);
       drawnow;
       hold on
       mkearth('plotarea',[180 -90 360 90],'areamode','positive',...
               'earthradius',3480,'resolution','none',...
               'flame','off');
       drawnow;
       mkearth('plotarea',[0 -90 360 0],'areamode','positive',...
               'earthradius',3480,'resolution','none',...
               'walls',2258.5,...
               'flame','off');
       drawnow;
       mkearth('plotarea',[0 0 360 90],'areamode','positive',...
               'earthradius',1221.5,'resolution','none',...
               'flame','off');
       drawnow;
       mkearth('plotarea',[0 -90 180 90],'areamode','positive',...
               'walls',6371,...
               'flame','off',...
               'colorresolution',50,...
               'xyzshift',[0 7000 0],...
               'resolution',p.resolution);
       drawnow;
       hold off
       view(128,17.5);
  case {'regular'},
      %%%%% choose and load dataset
      [topo,topolon,topolat]=mkloaddata(p);
      %%% cut out area (may be done in a subroutine in later versions)
      switch p.areamode
         case {'negative'},
             disp('MKEARTH: cutting out PlotArea...');
             indies=find((topolon>=p.plotarea(1))&...
                         (topolat>=p.plotarea(2))&...
                         (topolon<=p.plotarea(3))&...
                         (topolat<=p.plotarea(4)));
              topo(indies)=topo(indies)+NaN;
              topolat(indies)=topolat(indies)+NaN;
              topolon(indies)=topolon(indies)+NaN;
              disp(['         '...
                    int2str(length(indies)) ' of '...
                    int2str(length(topo(:))) ' elements set to NaN.']);
         case {'positive'},
              disp('MKEARTH: cutting out anything but PlotArea...');
              indies=find((topolon<p.plotarea(1))|...
                         (topolat<p.plotarea(2))|...
                         (topolon>p.plotarea(3))|...
                         (topolat>p.plotarea(4)));
              topo(indies)=topo(indies)+NaN;
              topolat(indies)=topolat(indies)+NaN;
              topolon(indies)=topolon(indies)+NaN;
              disp(['         '...
                    int2str(length(indies)) ' of '...
                    int2str(length(topo(:))) ' elements set to NaN.']);
         case {'nocut'},
              % nothing happens
         otherwise
              error(['MKEARTH: unknwon AreaMode' p.areamode]);
      end; % switch p.areamode
      %%% transform geographical coordinates into xyz,
      %%% apply exaggeration and radial lifting
      % topography
      [x,y,z]=sph2cart(topolon*radian,topolat*radian,p.re+topo*p.xagg+p.rlift);
      clear topolon topolat % not needed any longer
      % camera position
      [cpx,cpy,cpz]=sph2cart(p.camera(1)*radian,...
                             p.camera(2)*radian,...
                             p.re+p.camera(3));
      % light position, derived from camera position
      [lpx,lpy,lpz]=sph2cart((p.lightpos(1)+p.camera(1))*radian,...
                             (p.lightpos(2)+p.camera(2))*radian,...
                             p.re+p.camera(3));
      % camera target
      [ctx,cty,ctz]=sph2cart(p.target(1)*radian,...
                             p.target(2)*radian,...
                             p.re);
      %%% apply XYZshift
      x=x+p.xyzshift(1);
      y=y+p.xyzshift(2);
      z=z+p.xyzshift(3);
      %%% plot!
      hold on
      surfacehandle=surf(x,y,z,topo);
      hold off
      axis tight
      axis vis3d
      axis equal
      axis off
      %%% add wireframe
      mkwireframe(p);
      %%% add massive walls
      mkwalls(p);
      %%% colortable
      [rgb,minhgt,maxhgt]=mkearthcolor(p.colres);
      colormap(rgb);
      dummy=[0 0;0 0];
      hold on
      h=surf(dummy,dummy,dummy,[minhgt 0;0 maxhgt]); % forces colormap to span -12...19
      set(h,'UserData','MKEARTH/colorfix');
      hold off
      caxis([minhgt maxhgt]);
      %%% set camera and target
      set(gca,'CameraPosition',[cpx cpy cpz]);
      set(gca,'CameraTarget',[ctx cty ctz]);
      set(gca,'projection','orthographic');
      %%% define light
      if isequal(p.flame,'on')
         light('position',[lpx lpy lpz]);
      end; % if isequal(p.flame,'on')
      if isequal(p.resolution,'none')
         set(surfacehandle,'EdgeColor','none'); % better if no topo given
         set(surfacehandle,'FaceColor','interp');
         set(surfacehandle,'EdgeLighting','none'); % looks better...?
         set(surfacehandle,'FaceLighting','phong');
         set(surfacehandle,'UserData','MKEARTH/sphere');
         mkmaterial(surfacehandle,'metal');
      else
         set(surfacehandle,'EdgeColor','none'); % less time consumptive than interp,
                                                % and no visible difference with good data
         set(surfacehandle,'FaceColor','flat'); 
         set(surfacehandle,'EdgeLighting','none'); % looks better...?
         set(surfacehandle,'FaceLighting','flat');
         set(surfacehandle,'UserData','MKEARTH/topo');
         mkmaterial(surfacehandle,p.stuff);
      end;
  otherwise
     error(['MKEARTH: unknown mode ' p.mode]);
end; % switch p.mode
  
  
%%%%% occupy as much of figure window as possible
axunits=get(gca,'units');
set(gca,'units','normalized');
set(gca,'position',[0 0 1 1]);
set(gca,'units',axunits);


%%%%% end.
return;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                             %
%                         ---    SUBROUTINES   ---                            %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%






function parms=mkanalyzeinput(userinput);
% mkanalyzeinput.......analyze varargin input of MKEARTH and return parameters
%
% call: parms=mkanalyzeinput(userinput);
%
%       userinput: cell array containing the user's input as parameter list
%                  or as PropertyName/PropertyValue list
%
% result: parms: structure array containing ALL parameters neede by the main
%                program. Fields not given in the user's input are filled with
%                default values.
%
% Martin Knapmeyer, 18.02.2000


%%%%% set defaults
default=mkearthdefaults;


%%%%% analysis loop
[nin1,nin2]=size(userinput); % nin2 is the number of input paramneters
switch nin2
   case 0,
      %%% no input argument: run demo
      disp(['MKANALYZEINPUT: One Argument Mode encountered']);
      parms=default;
      parms.mode='demo1';
   case 1,
       %%% one input argument: return defaults
       singleinput=lower(char(userinput));
       switch singleinput
          case {'defaults','help','demo1','demo2'},
             disp(['MKANALYZEINPUT: ' singleinput ' Mode encountered']);
             parms=default;
             parms.mode=singleinput;
          otherwise 
             error(['MKANALYZEINPUT: unknown single argument ' singleinput]),
       end; % switch userinput
   case 3,
       %%% three input args: simple lon-lat-hgt mode
       disp(['MKANALYZEINPUT: Three Arguments Mode encountered']);
       lon=userinput{1};
       lat=userinput{2};
       hgt=userinput{3};
       parms=default;
       parms.target=[lon lat];
       parms.camera=[lon lat hgt];
   otherwise
       disp(['MKANALYZEINPUT: PropertyName/PropertyValue Mode encountered']);
       if fix(nin2/2)==(nin2/2)
          %%% even number of input args: decompose input into names and values
          parms=default;
          % loop over all given ParameterName/Value pairs
          % check legality/plausibility of input here!
          % it is probably possible to construct a loop that replaces the
          % switch-statement, but with such loop, it would be impossible
          % to set a property depending on the value of another (see 'camera')
          for i=1:2:nin2 
              propertyname=lower(userinput{i}); % to be case insensitive
              propertyvalue=userinput{i+1};
              switch propertyname
                 case {'mode'},
                     parms.mode=propertyvalue;
                 case {'target'},
                     parms.target=propertyvalue;
                     if isequal(parms.targetmode,'auto')
                        parms.targetmode='manual';
                     end; % if isequal
                 case {'camera'},
                     parms.camera=propertyvalue;
                     if isequal(parms.targetmode,'auto')
                        parms.target=propertyvalue(1:2);
                     end; % if isequal
                 case {'lightpos'},
                     parms.lightpos=propertyvalue;
                 case {'resolution'},
                     parms.resolution=propertyvalue;
                 case {'exaggeration'},
                     parms.xagg=propertyvalue;
                 case {'plotarea'},
                     parms.plotarea=propertyvalue;
                 case {'areamode'},
                     parms.areamode=propertyvalue;
                 case {'minaltitude'},
                     parms.minalt=propertyvalue;
                 case {'maxaltitude'},
                     parms.maxalt=propertyvalue;
                 case {'radiallift'},
                     parms.rlift=propertyvalue;
                 case {'xyzshift'},
                     parms.xyzshift=propertyvalue;
                 case {'colorresolution'},
                     parms.colres=propertyvalue;
                 case {'wireframe'},
                     parms.wireframe=propertyvalue;
                 case {'walls'},
                     parms.walls=propertyvalue;
                 case {'earthradius'},
                     parms.re=propertyvalue;
                 case {'coordinates'},
                     parms.coordinates=propertyvalue;
                     parms.coordmode='manual';
                 case {'coordmode'},
                     parms.coordmode=propertyvalue;
                 case {'discontinuities'},
                     parms.discontinuities=propertyvalue;
                 case {'flame'},
                     parms.flame=propertyvalue;
                 case {'stuff'},
                     parms.stuff=propertyvalue;
                 case {'wallsize'},
                     parms.wsz=propertyvalue;
                 case {'targetmode'},
                     parms.targetmode=propertyvalue;
                 otherwise
                     error(['MKANALYZEINPUT: unknown globe property ' propertyname]);
              end;
          end; % for i
       else
          %%% odd number of input args, but not 1 or 3
          error('MKANALYZEINPUT: illegal number of input arguments!');
       end; % if fix
end; % switch nin


%%%%% no vertical exaggeration if resolution is none
if isequal(parms.resolution,'none')
   parms.xagg=1;   
end; % if isequal

%%%%% make any string value lowercase
parms.mode=lower(parms.mode);
parms.areamode=lower(parms.areamode);


return; % end of analyzeinput

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%






function default=mkearthdefaults;
% mkearthdefaults...........defines defeault globe properties
%
% call: default=mkearthdefaults;
%
% result: default: structure containing default values of all properties
%
% martin Knapmeyer, 26.03.2000

default.mode='regular';
default.target=[23.5 35.5];
default.targetmode='auto';
default.camera=[23.5 35.5 60000];
default.lightpos=[-90 90];
default.resolution='preview';
default.xagg=25;
default.plotarea=[0 360 -90 90];
default.areamode='nocut';
default.minalt=-12;
default.maxalt=19;
default.rlift=0;
default.xyzshift=[0 0 0];
default.wireframe=[];
default.walls=[];
default.colres=121; % results in 256 colors if cpt table spans -12...19
default.re=6371;
default.coordinates=[0 360 -90 90];
default.coordmode='auto';
default.discontinuities=[1221.5 3480 3630 5701 5971 6151 6291 6346.6]; % from PREM
default.flame='on';
default.stuff='dull';
default.wsz=[30 100];
return; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%







function [topo,topolon,topolat]=mkloaddata(globe);
% mkloaddata.........load topography data from given pfad or resolution type
%
% call: t=mkloaddata(globe);
%
%       globe: globe property structure
%
% result: topo: array containing a topography model
%         topolon: array containing the longitudes of all points in TOPO
%         topolat: array conatinging the latitudes of all points in TOPO
%
%         surf(topolon,topolat,topo) will plot a map.
%
%
% martin knapmeyer, 18.02.2000


%%%%% extract needed globe properties, is a little mamory waste but makes
%%%%% the following code more portable
pfad=globe.resolution;
c=globe.coordinates;
pc=globe.plotarea;
pc=[min(pc(1),pc(3)) max(pc(1),pc(3))...
    min(pc(2),pc(4)) max(pc(2),pc(4))]; % necessary for later use with meshgrid
dlon=abs(c(2)-c(1)); % width of covered longitude range
dlat=abs(c(4)-c(3));  % width of covered latitude range
cm=globe.coordmode;
minalt=globe.minalt;
maxalt=globe.maxalt;
re=globe.re;
areamode=globe.areamode;


     %%%%% evaluate pfad
     switch pfad
     case {'preview','Preview','PREVIEW'},
           disp(['MKLOADDATA: loading default data']);
           load topo
        case {'none','None','NONE'},
           %%% since the main program needs something to plot, we return
           %%% an array of zeroes and coordinates here.
           disp('MKLOADDATA: constructing dummy data');
           s1=180; % was: 50 MK24012006
           s2=360; % was: 50 MK24012006
           topo=zeros(s1,s2)+mkdepthfun(re)*1000;
           if isequal(areamode,'positive')
              disp('MKLOADDATA: constructing coordinates for plotarea');
              dlon=abs(pc(2)-pc(1)); % width of covered longitude range
              dlat=abs(pc(4)-pc(3)); % width of covered latitude range
              [topolon,topolat]=meshgrid(pc(1):dlon/(s2-1):pc(2),...
                 pc(3):dlat/(s1-1):pc(4));
           else
              disp('MKLOADDATA: constructing coordinates for global area');
              [topolon,topolat]=meshgrid(c(1):dlon/(s2-1):c(2),...
                                         c(3):dlat/(s1-1):c(4));
           end; % if isequal
        otherwise
           if (exist(pfad,'file')==0)&(exist([pfad '.mat'],'file')==0)
              %%% does not exist as file: is not a topography file to be loaded
              error(['MKEARTH: resolution type ' pfad ' not recognized']);
           else
              %%% exists as file: try to load it via 'load'
              disp(['MKLOADATA: loading data from ' pfad]);
              load(pfad);
              %%% does variable TOPO exist in the workspace now?
              if exist('topo','var')~=1
                 error(['MKEARTH: could not load topo data from ' pfad]);
              end; % if exist
           end; % if exist(p.resolution)
      end; % switch p.resolution
      %%% we now have a variable named topo in the workspace.


      %%%%% generate coordinates
      %%% if loaded file did not contain arrays for longitude and latitude
      %%% named TOPOLON and TOPOLOAT, generate them
      [s1,s2]=size(topo);
      switch cm
         case {'manual'},
             disp('MKLOADDATA: constructing longitudes and latitudes MANUAL');
             [topolon,topolat]=meshgrid(c(1):dlon/(s2-1):c(2),...
                                        c(3):dlat/(s1-1):c(4));
         case {'auto'},
             if (~exist('topolon','var'))&(~exist('topolat','var'))
                %%% compute geographical coordinates for the dataset in topo
                %%% here we assume that it has the latitude-by-longitude structure
                disp('MKLOADDATA: constructing longitudes and latitudes AUTO');
                [topolon,topolat]=meshgrid(c(1):dlon/(s2-1):c(2),...
                                           c(3):dlat/(s1-1):c(4));
             end; % if exist
      end; % switch cm
      % we now have variables TOPOLON and TOPOLAT.    

      topo=topo/1000; % altitude is in km!
      
      % blot out areas below minalt and above maxalt
      indies=find((topo<minalt)|(topo>maxalt));
      topo(indies)=topo(indies)+NaN;
      if ~isempty(indies)
         disp(['MKLOADDATA: ' int2str(length(indies))...
               ' elements outside altitude range']);
      end; % if ~isempty(indies)
     

return; % end of mkloaddata


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%







function [rgb,minhgt,maxhgt,maxtopohgt]=mkearthcolor(colres);
% mkearthcolor..........generate colormap with certain height resolution
%
% call: rgb=mkearthcolor(colres);
%
%           colres: meters of altitude per color
%
% result: rgb: colormap array for use with colormap(rgb)
%         minhgt, maxhgt: min and max 'heights' for which
%                         colors are defined. In km. use this with CAXIS.
%         maxtopohgt: max height which is considered as topography,
%                     'heights' above this are considered to describe
%                     physical parameters used to color walls.
%                     Also in km.
%
% if mini==maxi, the current colormap will be returned.
%
% martin knapmeyer, 18.02.2000, 27.03.2000


% initialize dummy result
res=[0.5 0 0];

%%%%% define colortable (format adopted from GMT cpt-format)
cpt=[-12000.0   0   0   0  -4900.0  32  32 128;...
      -4900.0  32  32 128  -1500.0  96  96 255;...
      -1500.0  96  96 255    -40.0 192 192 255;...
        -40.0 192 192 255     +0.0  64 192  64;...
         +0.0  64 192  64   +250.0 150 150  75;...
       +250.0 150 150  75  +1000.0 200 200 100;...
      +1000.0 200 200 100  +4000.0 200 200 200;...
      +4000.0 200 200 200  +9000.0 255 255 255;... % here end the topography part
      +9000.0   0   0   0 +12750.0 255   0   0;... % here begins temperature
     +12750.0 255   0   0 +15853.0 255 128   0;...
     +15853.8 255 128   0 +16211.5 255 200   0;...
     +16211.5 255 200   0 +18540.4 255 255   0;...
     +18540.4 255 255   0 +19000.0 255 255 255];
% number of entries in cpt
anz=size(cpt,1);
% min and max defined heights and pseudoheights
minhgt=min(min(cpt(:,[1 5])));
maxhgt=max(max(cpt(:,[1 5])));
maxtopohgt=9; % max heigth which is considered as topography in cpt definition


% number of colors to generate
hgtanz=round(abs(maxhgt-minhgt)/colres); 


if hgtanz~=0
   disp(['MKEARTHCOLOR: constructing ' int2str(hgtanz) ' elements colormap']);
   
   % heights for which colors have to be generated
   height=[minhgt:abs(maxhgt-minhgt)/hgtanz:maxhgt];
   
   
   % compute color by finding the correct elements
   % and linear interpolation
   for i=1:hgtanz
      curhgt=height(i);
      % search entry
      done=0;
      cptcnt=1; % entry counter
      while done==0
          if (cpt(cptcnt,1)<=curhgt)&(cpt(cptcnt,5)>=curhgt)
             done=1; % height is between z(n-1) and z(n)
          else
             cptcnt=cptcnt+1;
             if cptcnt>anz
                cptcnt=anz;
                done=1;
                if cpt(1,1)>curhgt % height below lowest defined height
                   cptcnt=1;       % take lowermost color
                end; % if cpt(1,1)
                if cpt(anz,5)<curhgt % height above highest defined height
                   cptcnt=anz;       % take uppermost color
                end; % if cpt(anz,5)
             end; % if cptcnt
          end; % if cpt...
      end; % while done
      % interpolate
      % red
      res(i,1)=interp1([cpt(cptcnt,1) cpt(cptcnt,5)],...
                       [cpt(cptcnt,2) cpt(cptcnt,6)],curhgt,'linear');
      % green
      res(i,2)=interp1([cpt(cptcnt,1) cpt(cptcnt,5)],...
                       [cpt(cptcnt,3) cpt(cptcnt,7)],curhgt,'linear');
      % blue
      res(i,3)=interp1([cpt(cptcnt,1) cpt(cptcnt,5)],...
         [cpt(cptcnt,4) cpt(cptcnt,8)],curhgt,'linear');
   end; % for i
else
   res=[0.5 0 0];
end; % if hgtanz~=0


%%% clip rgb to [0,1]: sometimes, elements are slightly larger than 1 or lightlty smaller
%%% than 0 due to roundoff errors
res=res/255;
indies=find(res<0);
res(indies)=res(indies)*0;
indies=find(res>1);
res(indies)=res(indies)*0+1;

%%% replace all NaNs by 1
indies=find(isnan(res));
res(indies)=zeros(size(indies))+1;

% return result
rgb=res;
maxhgt=maxhgt/1000;
minhgt=minhgt/1000;



return; % end of mkearthcolor

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%







function mkwireframe(p);
%mkwireframe..........plot wireframe at edges of plotted area
%
% call: mkwireframe(p);
%
%       p: parameter structure conatining all globe properties
%
% result: none
%
% martin knapmeyer 19.02.2000


if ~isempty(p.wireframe) % do not waste any time if no wire frame is required...
   %%% some abbreviations
   radian=pi/180;
   lon1=p.plotarea(1)*radian;
   lon2=p.plotarea(3)*radian;
   lat1=p.plotarea(2)*radian;
   lat2=p.plotarea(4)*radian;
   dx=p.xyzshift(1);
   dy=p.xyzshift(2);
   dz=p.xyzshift(3);
   %%% construct edges
   % so many point along curvatured edges 
   howmuchlon=100;
   howmuchlat=100;
   howmuchrad=2;
   % coordinates for edges in lon direction
   lonedge=linspace(lon1,lon2,howmuchlon);
   % coordinates for edges in lat direction
   latedge=linspace(lat1,lat2,howmuchlat);
   % coordinates for edges in radial direction
   if p.wireframe==-1
      radedge=linspace(p.re,p.re+p.rlift,howmuchrad);
   else
      radedge=linspace(p.re-p.wireframe+p.rlift,...
                       p.re+p.rlift,howmuchrad);
   end; % if p. wireframe==-1
   dummyzero=zeros(size(latedge));
   %%% compute coordinates and plot!
   hold on
   [x,y,z]=sph2cart(lonedge,dummyzero+lat1,dummyzero+radedge(1)); 
   plot3(x+dx,y+dy,z+dz,'k');
   [x,y,z]=sph2cart(lonedge,dummyzero+lat2,dummyzero+radedge(1)); 
   plot3(x+dx,y+dy,z+dz,'k');
   [x,y,z]=sph2cart(lonedge,dummyzero+lat1,dummyzero+radedge(2)); 
   plot3(x+dx,y+dy,z+dz,'k');
   [x,y,z]=sph2cart(lonedge,dummyzero+lat2,dummyzero+radedge(2)); 
   plot3(x+dx,y+dy,z+dz,'k');
   [x,y,z]=sph2cart(dummyzero+lon1,latedge,dummyzero+radedge(1)); 
   plot3(x+dx,y+dy,z+dz,'k');
   [x,y,z]=sph2cart(dummyzero+lon2,latedge,dummyzero+radedge(1)); 
   plot3(x+dx,y+dy,z+dz,'k');
   [x,y,z]=sph2cart(dummyzero+lon1,latedge,dummyzero+radedge(2)); 
   plot3(x+dx,y+dy,z+dz,'k');
   [x,y,z]=sph2cart(dummyzero+lon2,latedge,dummyzero+radedge(2)); 
   plot3(x+dx,y+dy,z+dz,'k');
   [x,y,z]=sph2cart([0 0]+lon1,[0 0]+lat1,radedge); 
   plot3(x+dx,y+dy,z+dz,'k');
   [x,y,z]=sph2cart([0 0]+lon2,[0 0]+lat1,radedge); 
   plot3(x+dx,y+dy,z+dz,'k');
   [x,y,z]=sph2cart([0 0]+lon1,[0 0]+lat2,radedge); 
   plot3(x+dx,y+dy,z+dz,'k');
   [x,y,z]=sph2cart([0 0]+lon2,[0 0]+lat2,radedge); 
   plot3(x+dx,y+dy,z+dz,'k');
   hold off
end; % if ~isempty


return; % end of mkwireframe

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%







function [maxfun,minfun]=mkwalls(p);
%mkwalls..........plot massive walls at sides of plotted area
%
% call: mkwalls(p);
%
%       p: parameter structure conatining all globe properties
%
% result: MAXFUN, MINFUN: max and min values of the function
%                         used to color the walls.
%                         will be NaN if no walls are drawn.
%
% result: none
%
% martin knapmeyer 26.03.2000


maxfun=NaN;
minfun=NaN;


if ~isempty(p.walls) % do not waste any time if no wire frame is required...
   %%% some abbreviations
   radian=pi/180;
   lon1=p.plotarea(1)*radian;
   lon2=p.plotarea(3)*radian;
   lat1=p.plotarea(2)*radian;
   lat2=p.plotarea(4)*radian;
   dx=p.xyzshift(1);
   dy=p.xyzshift(2);
   dz=p.xyzshift(3);
   %%% construct edges
   % so many point along curvatured edges 
   howmuchlonlat=p.wsz(1);
   howmuchrad=p.wsz(2);
   % coordinates for edges in lon direction
   lonedge=linspace(lon1,lon2,howmuchlonlat);
   % coordinates for edges in lat direction
   latedge=linspace(lat1,lat2,howmuchlonlat);
   % coordinates for edges in radial direction
   if p.walls==-1
      radedge=linspace(p.re,p.re+p.rlift,howmuchrad);
   else
      radedge=linspace(p.re-p.walls+p.rlift,...
                       p.re+p.rlift,howmuchrad);
   end; % if p. wireframe==-1
   indies=find((p.discontinuities>min(radedge))&...
               (p.discontinuities<max(radedge)));              
   radedge=sort([p.discontinuities(indies) radedge]);              
   %%%%%%% generate and plot meshs
   [maxf1,minf1]=mksinglewall(p,lonedge,radedge,lat1,'ns','MKEARTH/wall/S'); % southern
   [maxf2,minf2]=mksinglewall(p,lonedge,radedge,lat2,'ns','MKEARTH/wall/N'); % northern
   [maxf3,minf3]=mksinglewall(p,latedge,radedge,lon1,'ew','MKEARTH/wall/W'); % western
   [maxf4,minf4]=mksinglewall(p,latedge,radedge,lon2,'ew','MKEARTH/wall/E'); % eastern
   maxfun=max([maxf1 maxf2 maxf3 maxf4]);
   minfun=min([minf1 minf2 minf3 minf4]);
end; % if ~isempty   


return;
   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%








function [maxfun,minfun]=mksinglewall(p,horedge,radius,p3,modus,wallname);
% mksinglewall...........plot single wall of spherical segment
%
% call: mksinglewall(horedge,radius,p3,modus,wallname);
%
%       horedge: defines the coordinates of the horizontal
%                edges (lon or lat, in degrees)
%       radius: defines the coordinates of the vertical
%               edges (in km)
%       p3: defines the coordinate value which is constant
%           on each edge (lon or lat, in degrees)
%       modus: 'ns': wall in north-south direction
%              'we': wall in west-east direction
%       wallname: string that will be written into the UserData field of
%                 the created surface object
%
% result: MAXFUN, MINFUN: max and min values of the function
%                         used to color the walls.
%
% martin Knapmeyer 26.03.2000

 [hor1,rads]=meshgrid(horedge,radius);
 hor2=zeros(size(hor1))+p3;
 switch modus
 case {'ns'},
    [x1,x2,x3]=sph2cart(hor1,hor2,rads+p.rlift);
 case {'ew'},
    [x1,x2,x3]=sph2cart(hor2,hor1,rads+p.rlift);
 end, % switch modus
 x1=x1+p.xyzshift(1);
 x2=x2+p.xyzshift(2);
 x3=x3+p.xyzshift(3);
 wallvalues=mkdepthfun(rads);
 hold on
 handle=surf(x1,x2,x3,wallvalues);
 hold off
 set(handle,'EdgeColor','none');
 set(handle,'FaceColor','interp');
 set(handle,'EdgeLighting','none');
 set(handle,'FaceLighting','phong');
 set(handle,'UserData',wallname);
 mkmaterial(handle,'metal');


maxfun=max(wallvalues(:));
minfun=min(wallvalues(:));

return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%







function fun=mkdepthfun(r);
% mkdepthfun............return physical function value for given radius
%
% call: fun=mkdepthfun(r);
%
%       r: radius value (in km)
%          may be amvector or a matrix
%
% result: fun: some physical value valid for radius R
%              will be a vector or matrix of the same
%              size and shape as R.
%              In the current implelentation, FUN is the temperature
%              at radius R in Kelvin, quoted from Stacey's "Physics
%              of the Earth", Brookfield Press (3rd ed., I think).
%
% Martin Knapmeyer 26.03.2000, 27.03.2000

% temperature as function of depth as given by Stacey
% at discontinuities, only the value below it is given.
radius=[   0  400  800 1200 1221.5 1600 2000 2400,...
        2800 3200 3480 3630 3800   4200 4600,...
        5000 5400 5701 6016 6221   6331 6356 6371];
  temp=[5101 5087 5041 4965 4961   4846 4693 4501,...
        4265 3981 3750 3564 3018   2712 2509,...
        2313 2109 1950 1750 1550    930  630  300 ];
  
  
% interpolate distribution to radius values in r
rsize=size(r); % save size and shape of r for later reshaping of FUN
r=r(:); % has to be a vector for interp1!
fun=interp1(radius,temp,r);

%%% replace NaNs by 5000
indies=find(isnan(fun));
fun(indies)=zeros(size(indies))+5000;

% reshape FUN to the same size and shape as r had
fun=reshape(fun,rsize);

% rescale to range 9...19 (10 is 0K, 20 is 5200K, compare this to the second
% part of the color definition in MKEARTHCOLOR)
fun=(10*(fun/5101)+9);


return; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%








function mkmaterial(handle,stuff)
%MKMATERIAL.......... Material reflectance mode.
%
% call: mkmaterial(handle,stuff);
%
%       handle: handle of the surface object which should
%               be "made" of the given material
%       stuff: a string defining the material
%              available are:
%               'dull'
%               'shiny'
%               'metal'
%               'default'
%
% result: none
%
% MKMATERIAL sets the ambient/diffuse/specular strength and the
% specular exponent of surface objects.
%
% derived from MatLab's MATERIAL, which sets these properties for
% all surfaces and patches within the current axes.
%
% Martin Knapmeyer, 28.03.2000

stuff=lower(stuff);

switch stuff
   case {'dull'},
      ka = 0.3; kd = 0.8; ks = 0.0; n = 10; sc = 1.0;
   case {'shiny'},
      ka = 0.3; kd = 0.6; ks = 0.9; n = 20; sc = 1.0;
   case {'metal'},
      ka = 0.3; kd = 0.3; ks = 1.0; n = 25; sc = 0.5;
   case {'default'},
      ka = 'default'; kd = 'default'; ks = 'default'; n = 'default'; sc = 'default';
   otherwise
      error(['MKMATERIAL: unknwon material ' stuff]);
end; % switch
   
set(handle,'amb', ka, 'diff', kd, 'speculars', ks, 'speculare', n, 'specularc', sc)


