function [x,y]=mkglobusmodelkit(lon,lat,mapopt,direction);
% mkglobusmodelkit......produce sheet of paper that can be glued on a ball 
%
% call: [x,y]=mkglobusmodelkit(lon,lat,mapopt,direction);
%
%       lon: nearly arbitrary matrix of longitude values [deg]
%            longitude is expected to range from -180deg to 180deg
%            (see description of LAT for assumptions made)
%       lat: nearly arbitrary matrix of latitude values [deg]
%            (expected to be a rectangular matrix, longitude counted in the
%             second index!)
%       mapopt: map options structure as returned by MKMAPOPT
%               The .proj_specops field is interpreted as folows:
%               .proj_specops(1): number of lobes
%               .proj_specops(2): ball radius [cm]
%       direction: projection direction:
%                  'forward': transform lon/lat to x/y
%                  'inverse': transform x/y to lon/lat
%
% result: x: matrix x coordinates
%         y: matrix of y coordinates
%
%         X and Y have same size and shape as LON and LAT (values with equal index correspond)
%         but are projected and scaled according to the settings in MAPOPT
%
% This routine produces a plot, which with some scaling, can be glued on
% a ball in order to get a globe from map data.
% The routine does not only compute map projection data, it also sets some
% figure properties to scale the map to a given ball's size\
%
% for .proj_specops(1)=1, this projection becomes the Mercator-Sanson projection.
%
%
% Before printing the figure, you have to set several figure properties in a way
% that one unit of the x and y axes correspond to 1cm on paper.
%
%
% Martin Knapmeyer, 02.07.2003

% removed:
% In order to get distinct tiles, a number of .proj_specops(1) columns of the lat
% matrix is set to NaN


% a useful constant
radian=pi/180;




switch lower(mapopt.planet_type)
    case {'sphere','uni'}
         switch direction
             case {'forward'}
                  lat=lat*radian;
                  lon=lon*radian;
                  lobeanz=mapopt.proj_specops(1); % number of lobes
                  ballrad=mapopt.proj_specops(2); % ball radius [cm]
                  deltalon=(360/lobeanz)*radian;
                  %%% compute the projection
                  x=lon*0;
                  for lobe=0:(lobeanz-1)
                      indies=find((lon>-pi+lobe*deltalon)&(lon<=-pi+(lobe+1)*deltalon));
                      middlemeridian=(-pi+(lobe+1/2)*deltalon);
                      x(indies)=middlemeridian+...
                                (lon(indies)-middlemeridian).*cos(lat(indies));
                  end; % for lobeanz
                  %%% set lat values of some parallels to NaN
                  %grdwdt=size(lat,2); %%% width of latitude grid
                  %for lobe=1:(lobeanz-1)
                  %    indy=lobe*fix(grdwdt/lobeanz);
                  %    if (~mkodd(grdwdt))&(~mkodd(lobeanz))&(lobe==lobeanz/2)
                  %       indy=indy+1;
                  %    end; % if
                  %    lat(:,indy)=lat(:,indy)+NaN;
                  %end; % for lobe
                  y=lat;
                  %%% scale x and y to fit ball circumference
                  x=x*ballrad/2;
                  y=y*ballrad/2;
                  %%% all you have to do now is scale the print in a way
                  %%% that one unit of x and y axes are 1cm!
             case {'inverse'}
                  error([': projection direction ' upper(direction)...
                         ' not yet available for planet type ' upper(mapopt.planet_type)]);
             otherwise
                  error([': unknown projection direction ' upper(direction)]);
         end; % switch direction
    case {'bi'}
         error([': planet type ' upper(mapopt.planet_type) ' not yet available!']);
         switch direction
             case {'forward'}
                  error([': projection direction ' upper(direction)...
                         ' not yet available for planet type ' upper(mapopt.planet_type)]);
             case {'inverse'}
                  error([': projection direction ' upper(direction)...
                         ' not yet available for planet type ' upper(mapopt.planet_type)]);
             otherwise
                  error([': unknown projection direction ' upper(direction)]);
         end; % switch direction
    case {'tri'}
         error([': planet type ' upper(mapopt.planet_type) ' not yet available!']);
         switch direction
             case {'forward'}
                  error([': projection direction ' upper(direction)...
                         ' not yet available for planet type ' upper(mapopt.planet_type)]);
             case {'inverse'}
                  error([': projection direction ' upper(direction)...
                         ' not yet available for planet type ' upper(mapopt.planet_type)]);
             otherwise
                  error([': unknown projection direction ' upper(direction)]);
         end; % switch direction
    otherwise
         error([': unknwon planet type ' upper(mapopt.planet_type)]);
end; %  switch mapopt.planet_type