function map=mkmakemap(mapopt);
% mkmakemap....... main map plotting routine
%
% call: map=mkmakemap(mapopt);
%
%       mapopt: map options structure as returned by MKMAPOPT
%
%       With no input parameters:
%                shaded relief map of Mars Topography
%                longitude east positive
%                center meridian 0
%                latitude -90...90
%                longitude -180...180
%                projection: plate caree
%                (these are not really the projection settings, this
%                is just a short description of what you get)
%       The no parameter mode is a historical artefact and is kept here
%       to be compatible with many of my older mars related programs.
%
% result: map: structure containing map description and handles to
%              generated map objects
%              map.figure: figure, in which map is drawn (gcf)
%              map.axes: axes in which map is drawn (gca)
%              map.surf: handle to surface or patch object, 
%                        containing the topography
%                        If mapopt.proj_pole specifies a rotation,
%                        trisurf is used to plot the topography as a
%                        number of patches instead of a surface.
%                        NOTE that you cannot add a texture map to a
%                        patch to define color!!
%              map.grid: handle to line object, containing the grid
%              map.frame: handle to the object that contains the map frame
%              map.bar: handles to objects of 3D colorbar
%              map.exag: vertical exaggeration of topography
%              map.minz: minimum topographic height [m]
%              map.maxz: maximum topographic height [m]
%              map.lon:  longitude matrix to plot surface [deg]
%              map.lat:  latitude matrix to plot surface [deg]
%
%              When no input arguments are specified, only the fields
%              figure, axes, aurf, exag, minz, maxz, lon, lat are returned.
%
% Martin Knapmeyer, 19.08.2002, 01.07.2003, 11.05.2004, 12.05.2004,
%                   26.11.2007
%
% BUGS: can currently not plot anything but the default...


%%% which mode?
%%% nin==0: compatibility mode
%%% nin==1: full featured mode
nin=nargin;


if nin==0
	%%% the following is not what I intend this routine to be.
	%%% Its only a quick and dirty solution for currend needs.
	%%% Kept here for compatibility with older programs that
	%%% use the zero-arguments call
	
	error(' The zero argument call of MKMAKEMAP is obsolete. Please replace it.');

	%%% load topography
	load /user/neptun/knapmeye/planets/mars/topo/topo050.mat lon lat topo
	topo=flipud(topo');
	lon=flipud(lon');
	lat=flipud(lat');
	%%% modify topography: from -180 to 180 longitude!
	sz=size(lon);
	for indy=1:sz(2);
      topo(:,indy)=mkshiftr(topo(:,indy),sz(1)/2);
      lon(:,indy)=mkshiftr(lon(:,indy),sz(1)/2);
	end; % for indy
	indies=find(lon>180);
	lon(indies)=-(360-lon(indies));
	%%% plot topo with gapmap as overlay
	%clf;
	%%% topo %%%%%%%%%%%%%%%%%%%
	exagg=1/1000; % to control vertical exaggeration (affects lighting!)
	handle=surf(lon,lat,topo*exagg);
	shading flat
	light('Position',[-1 1 0]);
	lighting phong
	material dull
	%%% beautification
	light('Position',[-1 1 3]);
	view(2);
	axis equal
	%axis normal
	axis([-180 180 -90 90]);
	set(gca,'Xtick',-180:30:180);
	set(gca,'YTick',-90:30:90);
	set(gca,'Layer','Top');
	%colorbar('hor');
	xlabel('Longitude');
	ylabel('Latitude');
	box on
	%%% return result structure
   map.figure=gcf;   
   map.axes=gca;
   map.surf=handle;
   map.exag=exagg;
   map.minz=min(topo(:));
   map.maxz=max(topo(:));
   map.lon=lon;
   map.lat=lat;
   %%% return
   return;
end; % if nin==0


%%%%%%% BUILD MAP ACCORDING TO MAPOPT STRUCTURE

%%% where do we plot?
currentfig=gcf;
currentaxe=gca;


%%% some settings
set(currentfig,'Renderer','zbuffer'); % OpenGL produces rubbish in stereographic projection!


%%% load topo data
if strcmp(mapopt.topo_mode,'on');
   topo=mkreadtopo(mapopt);
end; % if strcmp(mkmakemap.topo_mode,'on');

%%% generate grid
if strcmp(mapopt.grid_mode,'on')
   [gridlon,gridlat]=mkmakegrid(mapopt);
end; % if 

% %%% apply coordinate rotation to grid
% if strcmp(mapopt.grid_mode,'on')
%    if ~isempty(mapopt.proj_pole)
%       [gridlon,gridlat]=mkmaprotate(gridlon,gridlat,mapopt);
%    end; % if ~isempty(mapopt.proj_pole)
% end; % if 


% %%% apply coordinate rotation to topography
% if strcmp(mapopt.topo_mode,'on');
%     if ~isempty(mapopt.proj_pole)
%        [topo.lon,topo.lat]=mkmaprotate(topo.lon,topo.lat,mapopt);
%     end; % if ~isempty(mapopt.proj_pole)
% end; % if strcmp(mapopt.topo_mode,'on');

%%% apply map projection to topography according to MAPOPT
if strcmp(mapopt.topo_mode,'on');
    [plotx,ploty]=mklonlat2xy(topo.lon,topo.lat,mapopt,'forward');
    topo.topo=topo.topo*mapopt.proj_scale*mapopt.topo_exag;
    map.minz=min(topo.topo(:));
    map.maxz=max(topo.topo(:));
else
    map.minz=0;
    map.maxz=0;
end;

%%% generate frame
if strcmp(mapopt.frame_mode,'on')
   [framelon,framelat,frameelv]=mkmakeframe(mapopt,map.minz,map.maxz);
end; % if

% %%% apply coordinate rotation to frame
% % ??? is that a good idea?
% if strcmp(mapopt.frame_mode,'on')
%    if ~isempty(mapopt.proj_pole)
%       [framelon,framelat]=mkmaprotate(framelon,framelat,mapopt);
%    end; % if ~isempty(mapopt.proj_pole)
% end; % if


%%% apply map projection to grid
if strcmp(mapopt.grid_mode,'on')
   [gridx,gridy]=mklonlat2xy(gridlon,gridlat,mapopt,'forward');
end; % if 


%%% apply map projection to frame
if strcmp(mapopt.frame_mode,'on')
   [framex,framey]=mklonlat2xy(framelon,framelat,mapopt,'forward');
end; % if

%%% plot topography
if strcmp(mapopt.topo_mode,'on')
   if ~isempty(mapopt.proj_pole)
      tri=delaunay(plotx(:),ploty(:));
      surfhandle=trisurf(tri,plotx(:),ploty(:),topo.topo(:));
   else
      surfhandle=surf(plotx,ploty,topo.topo);
   end; % if isempty(mapopt.proj_pole)
   cax=[topo.minalt topo.maxalt]*mapopt.proj_scale*mapopt.topo_exag;
else
   surfhandle=[];
   cax=[0 1];
end; % if strcmp(mapopt.topo_mode,'on')
view(2);
axis equal
axis tight
shading flat;
caxis(cax);

%%% plot grid
if strcmp(mapopt.grid_mode,'on')
   hold on
   gridhandle=plot3(gridx,gridy,gridx*0+map.maxz,mapopt.grid_style);
   set(gridhandle,'Color',mapopt.grid_color);
   %switch mapopt.grid_style
   %   case {'+','o','*','.','x','s','d','^','v','>','<','p','h'}
   %      %%% this is a marker style
   %      set(gridhandle,'Marker',mapopt.grid_style);
   %      set(gridhandle,'LineStyle','none'
   %   case {'-','--',':','-.'}
   %      %%% this is a line style
   %      set(gridhandle,'LineStyle',mapopt.grid_style);
   %end; % switch mapopt.grid_style
   hold off
else
   gridhandle=[];
end; % if 

%%% plot frame
if strcmp(mapopt.frame_mode,'on')
   hold on
   framehandle=plot3(framex,framey,frameelv);
   set(framehandle,'Color',mapopt.frame_color);
   set(framehandle,'LineStyle',mapopt.frame_style);
   hold off
else
   framehandle=[];
end; % if 

%%% plot colorbar
%%% assumes that topo data is in meter!
if (strcmp(mapopt.colorbar_mode,'on'))&(strcmp(mapopt.topo_mode,'on'))
   barminalt=fix(topo.minalt/1000)*1000;
   barmaxalt=fix(topo.maxalt/1000)*1000;
   ticks=(barminalt:1000:barmaxalt)*mapopt.proj_scale*mapopt.topo_exag;
   barhandle=mk3dcolorbar(mapopt.colorbar_dir,ticks);
else
   barhandle=[];
end;

%%% instead of "axis tight": axis range has to be slightly larger than "tight"
%%% to avoid holes in the map frame
additionalspace=1/100; % percent
axis tight
ax=axis;
axwdt=ax(2)-ax(1);
axhgt=ax(4)-ax(3);
ax(1)=ax(1)-axwdt*additionalspace;
ax(2)=ax(2)+axwdt*additionalspace;
ax(3)=ax(3)-axhgt*additionalspace;
ax(4)=ax(4)+axhgt*additionalspace;
axis(ax);

%%% some axis properties
box off
grid off
axis off
title(lower(['Map in ' mapopt.proj_type ' Projection']));


%%%%%%% return result structure
map.figure=currentfig;
map.axes=currentaxe;
map.surf=surfhandle;
map.grid=gridhandle;
map.frame=framehandle;
map.bar=barhandle;
map.exag=mapopt.topo_exag;

if strcmp(mapopt.topo_mode,'on')
   map.lon=topo.lon;
   map.lat=topo.lat;
else
    map.lon=[]; % MK29092004
    map.lat=[];
end; % if strcmp(mapopt.topo_mode,'on')
