function mapopt=mkmapopt(varargin);
% mkmapopt............get/set MAPping OPTions structure
%
% call: mapopt=mkmapopt;
%       mapopt=mkmapopt('default');
%       mapopt=mkmapopt('Name1',Value1,'Name2',Value2,...);
%
%          'default': return default settings for all options
%                     This is the same as calling mkmapopt without any
%                     input parameter.
%       'Name',Value: return complete options structure, in which
%                     the options specified by NAME has the value
%                     specified by VALUE.
%                     If a Name occurs more than once, only the last
%                     of the values specified will apply.
%                     Names used here must correspond to the field names
%                     of eviopt as given below.
%                     Option names are not case sensitive.
%
% result: mapopt: complete mapopt structure as needed by MKMAKEMAP.
%                 The strucutre consists of the following fields:
%
%                 MAP PROJECTION STUFF
%
%                 .proj_type:    projection type
%                 .proj_scale:   projection scale
%                 .proj_range:   projected part of topography
%                                4 element vector [minlon maxlon minlat maxlat] that defines the
%                                area shown in the map.
%                                minlon might be negative, but abs(maxlon-minlon) must be <360deg.
%                                If not specified explicitly, this will be equal to [0 360 -90 90]
%                 .proj_specops: special options needed by the projection
%                                The content of this array is not defined here. Some projections
%                                need additional parameters like tangent-parallels and the like.
%                                Such data is stored here. See the projection routines to
%                                find details.
%                 .proj_pole:    Rotational parameters
%                                This is a two element vector [lon lat] which defines a
%                                rotation of the coordinate frame.
%                                Data will be rotated by lon degrees eastwward and then
%                                by 90-lat degrees northward
%                                DEFAULT: empty
%                                longitude has to be counted eastward from 0 to 360
%                                latitude has to be counted northward from -90 to 90
%
%                                If this is empty, no rotation will be applied. PLotting the
%                                map is much fastern then, because delaunay triangulation has
%                                to be applied to rotated topo data. Depending on the projection,
%                                the behaviouir of rotation may be unexpected. Try plotting maps
%                                with [0,0], [0,90], [90,0] to get an idea of what you have to choose.
%                                When using rotation, it is recommended to plot grid lines
%                                with a high number of points per line and in '.' style.
%
%                 PLANET SIZE STUFF
%
%                 .planet_type: Ellipsoid type
%                               Planets may be uniaxial, biaxial or triaxial ellipsoids
%                               possible values are:
%                               'uni', 'sphere': Uniaxial is a sphere, of course.
%                               'bi': Biaxial is a rotational ellipsoid.
%                               'tri': Triaxial is a triaxial ellipsoid.
%                               Axes of biaxial and triaxial planets are expected
%                               to be polar and in equatorial plane.
%
%                 .planet_radius: 1, 2, or 3 element vector of radius values, in km
%                               For uniaxial (spherical) planets, only the first element is used.
%                               For biaxial planets, elements 1 and 2 are used.
%                               For triaxial planets, all three elements are used.
%                               Axis orientation is defined by .PLANET_ORIENT substructure
%
%                 .planet_orient: Ellipsoid axis orientation for triaxial planets
%                                 This substructure defines the points at the surface to
%                                 which the three axes of the ellipsoi d point.
%                                 Not that axes are not necessarily polar/equatorial!
%                                 planet_orient(1,1): longitude of planet_radius(1) [deg]
%                                 planet_orient(1,2): latitude of planet_radius(1) [deg]
%                                 planet_orient(2,1): longitude of planet_radius(2) [deg]
%                                 planet_orient(2,2): latitude of planet_radius(2) [deg]
%                                 planet_orient(3,1): longitude of planet_radius(3) [deg]
%                                 planet_orient(3,2): latitude of planet_radius(3) [deg]
%
%                 TOPO DATA STUFF
%
%                 .topo_mode:    switch to enable or disable topography plotting
%                                'on': the topography will be plotted
%                                'off': the topography will not be plotted.
%                                DEFAULT: 'on'
%
%                 .topo_path:    path to .mat file containing topography data
%                                The topography file is expected to contain the following matrix:
%                                topo: matrix of elevation values, with topo(1,1) being the
%                                      elevation at 0degE, 90degS and topo(end,end) being the
%                                      elevation at 360degE, 90degN
%                                OPTIONAL:
%                                      a matrix LON with longitude values and a matrix LAT with
%                                      latitude values for each element of TOPO.
%                 .topo_range:   4 element matrix defining coordinate range of topo data
%                                [minlon maxlon minlat maxlat], all given in degrees,
%                                longitude positive to east, latitude positive to north
%                                if the data givn here is not compatible to LON and LAT data
%                                given in the topo file, these are overridden! To avoid this, set
%                                .TOPO_RANGE to empty!
%                                NOTE: this property defines the size of the topo data set, not
%                                      the area shown in the map! (that's .proj_range)
%                 .topo_exag:    topography exaggeration factor
%                                topography will be multiplied by map scale and then by this factor
%                                This results in a vertical exaggeration of topography withn respect
%                                to horizontal extent by factor .TOPO_EXAGG
%
%                 GRID STUFF
%
%                 .grid_mode:    grid plot mode
%                                'on': a grid will be drawn according to the grid parameters
%                                'off': no gridd will be drawn, parameters are irrelevant.
%                 .grid_range:   area of the map to be covered with lon/lat grid
%                                4 element vector [minlon maxlon minlat maxlat]
%                                If not specified explicitly, this will be equal to [0 360 -90 90]
%                 .grid_lonmesh: grid mesh width in longitude direction [deg]
%                 .grid_latmesh: grid mesh width in latitude direction [deg]
%                 .grid_points:  number of points per grid line
%                 .grid_color:   grid line color, specified as MatLab ColorSpec
%                                (name string or RGB triple)
%                 .grid_style:   grid line style, specified as MatLab LineSpec
%
%                 FRAME STUFF
%
%                 .frame_mode:   frame plot mode
%                                'on': a frame will be drawn around the map
%                                'off': no frame will be drawn
%                 .frame_color: frame line color, specified as MatLab ColorSpec
%                 .frame_style: frame line style, specified as MatLab LineSpec
%
%                 COLORBAR STUFF
%
%                 .colorbar_mode: colorbar plot mode
%                                 'on' plots a colorbar under or beside map
%                                 'off' does not plot a colorbar
%                 .colorbar_dir: colorbar orientation
%                                'hor': horizontal colorbar under map
%                                'vert': vertical colorbar beside map
%
%
%
%
%
% Martin Knapmeyer 01.07.2003, 11.05.2004

%%% To add new fields to the options-structure:
%%% 1) add the new field and its default in the initialization code below
%%% 2) add a description of the new field in the help lines.
%%% 3) if necessary, add code that tests for plausibility of user input
%%% 4) write code that evaluates the new field within the
%%%    other mapping routines (where appropriate)
%%% That's all. mkmapopt is able to handle input for the
%%% new field automatically.


%%% write mkmapbox version info to screen
%%% doing that here guarantees that the user will notice the version
%%% data sooner or later.
%mkmapversion;


%%% initialize structure with defaults
%%% it is necessary to do this here, because
%%% the input evaluation will check this for
%%% names of existing fields.
%%%
%%% Default map options are valid for spherical earth (and my file system)

%%% PLANET SIZE STUFF
mapopt.planet_type='sphere';     % ellipsoid type
mapopt.planet_radius=6371;       % radius [km]
mapopt.planet_orient(1,1)=0;     % axes orientations [deg]
mapopt.planet_orient(1,2)=90;
mapopt.planet_orient(2,1)=0;
mapopt.planet_orient(2,2)=0;
mapopt.planet_orient(3,1)=90;
mapopt.planet_orient(3,2)=0;

%%% TOPO DATA STUFF
mapopt.topo_mode='on';
mapopt.topo_path='/home/europa/mknapmey/planets/erde/topo/etopo20.mat'; % load topo from here
mapopt.topo_range=[0 360 -90 90];
mapopt.topo_exag=1;

%%% MAP PROJECTIONS STUFF
mapopt.proj_type='platecaree';   % default map projection
mapopt.proj_scale=1;             % default map scale
mapopt.proj_range=[0 360 -90 90]; % projection range
mapopt.proj_specops=[];          % projection special parameters
mapopt.proj_pole=[];           % coordinate frame rotation parameter

%%% GRID STUFF
mapopt.grid_mode='on';           % grid mode
mapopt.grid_range=[0 360 -90 90]; % grid range
mapopt.grid_lonmesh=10;          % mesh width [deg]
mapopt.grid_latmesh=10;          % mesh width [deg]
mapopt.grid_points=100;          % number of points per grid line
mapopt.grid_color='k';           % grid line color (colorspec)
mapopt.grid_style='-';           % grid kline style (linespec)

%%% FRAME STUFF
mapopt.frame_mode='on';
mapopt.frame_color='k';           % grid line color (colorspec)
mapopt.frame_style='-';           % grid kline style (linespec)

%%% COLORBAR STUFF
mapopt.colorbar_mode='on';
mapopt.colorbar_dir='hor';


%%% default initialization done.


%%% evaluate user input
%%% nin2 is number of input parameters
[nin1,nin2]=size(varargin);
switch nin2
   case {0,1}
      %%% return defaults
      %%% nothing needs to be done here.
   otherwise
      %%% if nin2 is even, nin2/2 options are modified and have
      %%% to be extraced from the VARARGIN array
      %%% if nin2 is odd, the input is illegal
      if nin2/2==fix(nin2/2)
         %%% even: evaluate input
         allnames=fieldnames(mapopt);
         allnames=allnames(:);
         namesanz=length(allnames);
         for indy=1:2:nin2
             %%% current name and value
             optname=lower(varargin{indy});
             optvalue=varargin{indy+1};
             %%% is the name a legal name?
             %%% if yes, set the value, if no, ERROR!
             done=0;
             namecnt=1;
             while done==0
                if strcmp(allnames(namecnt),optname)
                   mapopt=setfield(mapopt,optname,optvalue);
                   done=1;
                else
                   namecnt=namecnt+1;
                   if namecnt>namesanz
                      error(['MKMAPOPT: illegal option name ' upper(optname)]);
                   end; % if namecnt>namesanz
                end; % if strcmp
             end; % while done==0
         end; % for indy
      else
         %%% odd: illegal!
         error('MKMAPOPT: number of input arguments must be even or 0 or 1!');
      end; % if nin2/2
end; % switch nin2


%%% FIX PAPER UNITS TO CENTIMETER!
mapopt.paper_units='centimeter';

%%%%%%%% plausibility tests
