function topo=mkreadtopo(mapopt);
% mkreadtopo........read topo data from file
%
% call: topo=mkreadtopo(mapopt);
%
%       mapopt: map options structure as returned by MKMAPOPT
%
% result: topo: structure describing 3D elevation surface
%               .topo: elevation array, elevations in meter
%                      matrix of elevation values, with topo(1,1) being the
%                      elevation at 0degE, 90degS and topo(end,end) being the
%                      elevation at 360degE, 90degN
%               .lon: matrix of longitude values corrsponding to .TOPO
%               .lat: matrix of latitude values corresponding to .TOPO
%               .maxalt: maximum altitude in file, for use in caxis()
%               .minalt: minimum altitude in file, for use in caxis()
%
% If the topo file specified in MAPOPT does not contain LON and LAT data,
% these are constructed according to range data in MAPOPT.
%
% If lon/lat data in the file is not compatible with .topo_range, new lon/lat
% data is constructed as well!
%
% Martin Knapmeyer, 01.07.2003


%%% load topo from file and construct lon/lat data if necessary
if ~isempty(mapopt.topo_path)
   topo=load(mapopt.topo_path);
   doconstruct=0; % lon/lat construxtion neccessary flag
   if (~mkexistfield(topo,'lon'))|(~mkexistfield(topo,'lat'))
      %%% either the lon field or the lat field does not exist
      %%% - we will construct one accordings to mapopt.topo_range
      doconstruct=1;
   else
      %%% both lon and lat field exist
      %%% are they compatible with mapopt.topo_range?
      if (min(topo.lon(:))~=mapopt.topo_range(1))|...
         (max(topo.lon(:))~=mapopt.topo_range(2))|...
         (min(topo.lat(:))~=mapopt.topo_range(3))|...
         (max(topo.lat(:))~=mapopt.topo_range(4))
         %%% incompatible
         if ~isempty(mapopt.topo_range)
             doconstruct=1;
         end; % if ~isempty
      else
         doconstruct=0;
      end;
   end; % if (~mkexistfield(topo,'lon'))|(~mkexistfield(topo,'lat'))
   %%% so now construct lon lat fields if neccessary!
   if doconstruct
      if isempty(mapopt.topo_range)
         error('MKREADTOPO: have to construct LON and LAT, but MAPOPT.TOPO_RANGE is empty!');
      end; % if isempty
      minlon=mapopt.topo_range(1);
      maxlon=mapopt.topo_range(2);
      minlat=mapopt.topo_range(3);
      maxlat=mapopt.topo_range(4);
      deltalon=abs(maxlon-minlon)/(size(topo.topo,2)-1);
      deltalat=abs(maxlat-minlat)/(size(topo.topo,1)-1);
      [topo.lon,topo.lat]=meshgrid(minlon:deltalon:maxlon,...
                                   minlat:deltalat:maxlat);
   end; % if doconstruct
else
   topo=[];
end; % if ~isempty(topo_path)

%%% determine altitude rangegiven in topo file
topo.maxalt=max(topo.topo(:));
topo.minalt=min(topo.topo(:));

%%% reduce topo/lon/lat data matrices to the area specified in mapopt.proj_range
mapminlon=mapopt.proj_range(1);
mapmaxlon=mapopt.proj_range(2);
mapminlat=mapopt.proj_range(3);
mapmaxlat=mapopt.proj_range(4);
if abs(mapmaxlon-mapminlon)<=360
   if mapminlon<0
      %%% transform data from 0...360 into -180...180 representation
      indies=find(topo.lon>180);
      meri180=max(find(topo.lon(1,:)<180));
      meri360=size(topo.lon,2);
      topo.lon(indies)=-(360-topo.lon(indies));
      topo.lon=[topo.lon(:,(meri180+1):meri360), topo.lon(:,1:meri180)];
      topo.topo=[topo.topo(:,(meri180+1):meri360), topo.topo(:,1:meri180)];
   end;
   %%% now cut out the desired range
   lonindies=find((topo.lon(1,:)>=mapminlon)&(topo.lon(1,:)<=mapmaxlon));
   latindies=find((topo.lat(:,1)>=mapminlat)&(topo.lat(:,1)<=mapmaxlat));
   topo.topo=topo.topo(latindies,lonindies);
   topo.lon=topo.lon(latindies,lonindies);
   topo.lat=topo.lat(latindies,lonindies);
else
   error('MKREADTOPO: spanned longitude range must be <=360deg!');
end;