function [traveltimes,newttsupport]=mklstttri(lsopt,mygrid,fxd,evalthese,ttsupport);
% mklstttri........travel time computation (interface routine for triangular grids)
%
% call: [traveltimes,newttsupport]=mklstttri(lsopt,mygrid,fxd,evalthese,ttsupport);
%
%       lsopt: (structure)
%              LSOPT structure as returned by MKLSOPT
%
%       mygrid: (structure)
%              grid structure as returned by MKLSGRIDNEW
%
%       fxd: (strcuture)
%                   FXD structure as returned by MKLSLS2FXD, containing all
%                   data.
%
%       evalthese: (structure)
%              structure denoting grid nodes to be evaluated, as returned
%              by MKLSGRIDNONREADY.
%              The following fields are expected:
%              .layer: (numeric)
%                      layer ID, only one per element of EVALTHESE
%              .nodes: (numeric array)
%                      list of nodes. This is a sublist of the
%                      nodes list in the mygrid.layer field,
%                      contining node IDs and coordinates formatted
%                      as in the layer structure.
%              .listpos: (numeric array)
%                      list of positions of the nodes in the
%                      original .nodes field of the input layer.
%                      evalthese.listpos(i) is the array indes of
%                      the node evalthese.nodes(i,:) wihtin the
%                      original layer structure.
%
%
%       ttsupport: (structure)
%                  TTSUPPORT structure. This structure might be as returned
%                  by MKLSEMPTYTTSUPPORT, i.e. unspecific, or with fields
%                  that are specific for the used travel time software.
%
%
% result: traveltimes: (structure)
%                      A travel time structure as returned by
%                      mklsemptytraveltimes.
%
%                      If TTBOX is used, the travel times contained in here
%                      are corrected for the station elevation by computing
%                      the travel time of a ray in a layer with surface 
%                      velocity and thickness corresponding to the station
%                      elevation. The actual ray parameter will be used.
%
%
%         newttsupport: (structure)
%                  as TTSUPPORT, but with actualized contents.
%
%
% This routine computes traveltimes from all nodes specified in EVALTHESE
% to all stations (in FXD) at which travel time observations are given in FXD.
%
% This routine is the generic interface to external travel time computation
% software. In original form, it contains an interface to TTBOX (Knapmeyer,
% 2004, Seismological Research Letters, vol. 75, No. 6, 726-733).
% Interfaces to other software may be integrated. There are some LSOPT
% fields which can be defined according to the needs of the software used.
% You need to construct an appropriate input and you have to transform the
% output into the format of the TRAVELTIMES structure.
%
% Martin Knapmeyer, 31.03.2005, 17.05.2005, 24.01.2007, 25.01.2007,
%                   03.05.2007

%%% 24012007 use MKSHOOTRAY instead of MKFINDP
%%% 25012007 use MKPSAMPLER and apply MKIMPROVEMODEL
%%% 03052007 correction for station elevation using vertical rays


%%% init result
layercnt=length(mygrid.layers); % number of layers in grid
traveltimes=mklsemptytraveltimes(layercnt);
newttsupport=ttsupport; % in case of doubt, return this unchanged!

%%% verbosity
verbosity=4;



%%% extract layer ID list from EVALTHESE
layercnt=length(evalthese);
layerlist=zeros(1,layercnt);
for indy=1:layercnt
    layerlist(indy)=evalthese(indy).layer;
end; % for indy


%%% make a list of all phases for which the FXD contains arrival time data
statcnt=length(fxd); % number of stations
allphases=[]; % to collect all phase names
for statindy=1:statcnt
    allarrivals=find(fxd(statindy).value(:,1)==1);
    if ~isempty(allarrivals)
        for cnt=1:length(allarrivals)
            phasepointer=fxd(statindy).value(allarrivals(cnt),2);
            allphases=strvcat(allphases,fxd(statindy).phase(phasepointer,:));
        end; % for cnt
    end; % if ~isempty(allarivals)
end; % for statindy
%%% make phase list unique
allphases=unique(allphases,'rows');
allphasecnt=size(allphases,1); % total number of phases with times



%%% make sure the TTSUPPORT structure is for the specified travel time
%%% software
if strcmp(ttsupport.package,lsopt.tt_package)~=1
    error(['MKLSTTTRI: travel time packages specified in LSOPT and TTSUPPORT do not match!']);
end; %


%%% switch to the different travel time packages
switch lower(lsopt.tt_package)
    
    %%%%%% TTBOX INTERFACE
    case {'ttbox'}
        
        %%% prepare empty pscan structure
        pscan=mkemptypscan(1);
        
        %%% if newttsupport does not contain a pscan field, insert one
        if mklsexistfield(newttsupport,'pscan')==0
           newttsupport.pscan=mkemptypscan(1);
        end; % if mklsexistfield
        
        %%% distance comparison tolerance (to account for TTBOX shooting error)
        eps=1e-3; % [deg]
        
        %%% transform the CLR model struct given in the LSOPT into an ND
        %%% model acceptable for MKFINDP and MKTTQUICK
        velocitymodel=mkclr2model(lsopt.tt_options.model,...
                                  lsopt.tt_options.interpdz,...
                                  lsopt.tt_options.interpmode);
        velocitymodel=mkimprovemodel(velocitymodel); % MK25012007

        %%% loop over layers listed in EVALTHESE
        for layerindy=1:layercnt       
            
            %%% ID of this layer
            traveltimes(layerindy).layerid=mygrid.layers(layerindy).id;
            
            %%% do we have to analyze this layer?
            layerpos=find(mygrid.layers(layerindy).id==layerlist);
            if ~isempty(layerpos)
               %%% layer is in the EVALTHESE list, so evaluate it! 
               
                %%% depth of current layer below surface
                layerdepth=lsopt.planet_radius-mygrid.layers(layerindy).radius;
               
               
                %%% execute dist(p) scan to prepare MKFINDP-call
                for allphaseindy=1:allphasecnt
                    %%% determine which phase is to be scanned
                    phase=deblank(allphases(allphaseindy,:));
                    %%% has a pscan for this phase/depth combination been
                    %%% done before? If TTSUPPORT already contains a scan
                    %%% for this, do not waste more time!
                    scanexists=0;
                    scanindex=[];
                    [scanexists,scanindex]=mkfindpscan(newttsupport.pscan,phase,layerdepth);
                    if scanexists==1
                        %%% scan already exists, use that!
                        mklsverbose(verbosity,lsopt,'MKLSTTTRI: existing TTBOX PSCAN found for ',...
                                    phase,' from ',layerdepth,...
                                    'km source depth.');
                        pscan=[pscan newttsupport.pscan(scanindex)]; %pscan(allphaseindy)=newttsupport.pscan(scanindex);
                    else
                        %%% no scan existtin, do a new one!
                        mklsverbose(verbosity,lsopt,'MKLSTTTRI: running TTBOX PSCAN for ',...
                                    phase,' from ',layerdepth,...
                                    'km source depth at ',mkclock);
                        % mkscanp is obsolete! MK25012007 pscan=[pscan mkscanp(phase,layerdepth,velocitymodel)]; %pscan(allphaseindy)=mkscanp(phase,layerdepth,velocitymodel);
                        pscan=[pscan mkpsampler(phase,layerdepth,velocitymodel)];
                    end; % if scanexists
                end; % for allphaseindy=1:allphasecnt
                
                
                %%% loop over stations
                for statindy=1:statcnt

                    %%% unique catalogue of phases detected at current station
                    phasecatalogue=unique(fxd(statindy).phase,'rows');
                    phasecnt=size(phasecatalogue,1); % so many phases 

                    %%% compute epicentral distances from all requsted nodes
                    %%% of this layer to the current station
                    distlist=mklsepidistv(evalthese(layerpos).nodes(:,2),evalthese(layerpos).nodes(:,3),...
                                          fxd(statindy).slon,fxd(statindy).slat);

                   

                    %%% loop over all phases detected at the current station
                    for phaseindy=1:phasecnt

                        
                        %%% compute traveltimes for these distances and the
                        %%% depth of the current layer - this is the actual
                        %%% TTBOX calling
                        %%% d will containt the distances for which ray
                        %%% parameters have been calculated. NOTE that
                        %%% triplications cause multiple entries of the
                        %%% same d value (or of d values with very small differences)!
                        
                        %%% the current phase is...
                        phase=deblank(phasecatalogue(phaseindy,:));
                        mklsverbose(verbosity+1,lsopt,'MKLSTTTRI: running TTBOX for ',...
                                    'Station ',fxd(statindy).id,', Phase ',phase,...
                                    ' at ',length(evalthese(layerpos).nodes(:,2)),' nodes, start: ',mkclock);
                                
                        
                        %%% first identify the right PSCAN entry
                        [pscanfound,pscanindy]=mkfindpscan(pscan,phase,layerdepth);

                        
                        %%% now call MKSHOOTRAY for the nodes in question
                        [p,a,d]=mkshootray(phase,distlist,layerdepth,velocitymodel,pscan(pscanindy));
                        %%% and the compute travel times for the ray parms
                        %%% found by MKSHOOTRAY.
                        ttcurves=mkttquick(velocitymodel,layerdepth,p,a,phase,'silent');
                        
                        
                        %%%%%%%%%%%% station elevation correction MK03052007
                        
                            %%% elevation correction is executed only if
                            %%% elevation is not zero
                            if fxd(statindy).selv~=0
                                
                                %%% determine surface velocities of velocity model
                                surfvel=mkinterpmodel(velocitymodel,0,'simple');
                                surfindy=find(surfvel.z==0);
                                surfindy=surfindy(1); % just in case... usually there should be only one surface...
                        
                                %%% what velocoty to use? depends on wave type
                                %%% of the closing leg of the ray, i.e. on the
                                %%% last letter of the phase name
                                lastletter=phase(end);
                                switch lastletter
                                    case {'P','p'}
                                         %%% phase arrives as P wave
                                         vcorrect=surfvel.vp(surfindy);
                                    case {'S','s'}
                                         %%% phase arrives as S wave
                                         vcorrect=surfvel.vs(surfindy);
                                    otherwise
                                         error(['MKLSTTTRI: unexpected phase name ' phase '  in elevation correction']);
                                end; % switch lastletter
                                
                                %%% compute travel time for ray in constant velocity layer with velocity VCORRECT
                                %%% eqution can be found in TTBOX documentation. note that ray parameter is a vector here!
                                %%% correction is positive for positive station elevation
                                pcartesian=p./velocitymodel.rp; % ray parameter in sec/km! (cf MKPSFER2FLAT!)
                                ttcorrection=fxd(statindy).selv./sqrt(vcorrect*(1-pcartesian.*pcartesian.*vcorrect));
                                
                                
                                %%% apply correction to all times in TTCURVES
                                timecnt=length(ttcurves.ttc.rayp); % number of travel times in ttcurves struct
                                for timeindy=1:timecnt
                                    
                                    %%% find correction for corresponding ray parameter
                                    raypindy=find(p==ttcurves.ttc.rayp(timeindy));
                                    
                                    if ~isempty(raypindy)
                                        %%% the correction itself
                                        mklsverbose(verbosity+1,lsopt,...
                                              'MKLSTTTRI: station ',fxd(statindy).id,...
                                              ': elevation ',fxd(statindy).selv, 'km at ',...
                                              vcorrect,'km/s yields ',...
                                              phase,' correction: ',ttcorrection(raypindy(1)),'sec.');
                                        ttcurves.ttc.t(timeindy)=ttcurves.ttc.t(timeindy)-ttcorrection(raypindy(1)); % elements of raypindy are all equal!
                                    else
                                        %%% no travel time available for
                                        %%% the ray parameter in question!
                                    end; % if ~isempty(raypindy)
                                    
                                end; % for timeindy
                                
                            
                            end; % if fxd(statindy).selv~=0
                        
                        %%%%%%%%%%%% end of station elevation correction
                        

                        %%% add newly computed phase travel times to result
                        %%% structure
                        nodeanz=size(evalthese(layerpos).nodes,1); % number of nodes eval'd
                        for nodecnt=1:nodeanz
                            %%% find travel times for current node
                            indies=find(abs(distlist(nodecnt)-ttcurves.ttc.d)<eps);
                            %%% copy all travel times for this node into
                            %%% result structure
                            mklsverbose(verbosity+1,lsopt,'MKLSTTTRI: node #',evalthese(layerpos).nodes(nodecnt,1),...
                                        ': ',length(indies),' arrivals for ',phase,' found.');
                            for ttcnt=1:length(indies)
                                traveltimes(layerindy).nodeid=[traveltimes(layerindy).nodeid; evalthese(layerpos).nodes(nodecnt,1)];
                                traveltimes(layerindy).phase=strvcat(traveltimes(layerindy).phase,phase);
                                traveltimes(layerindy).station=strvcat(traveltimes(layerindy).station,fxd(statindy).id);
                                traveltimes(layerindy).d=[traveltimes(layerindy).d; ttcurves.ttc.d(indies(ttcnt))];
                                traveltimes(layerindy).tt=[traveltimes(layerindy).tt; ttcurves.ttc.t(indies(ttcnt))];
                            end; % for ttcnt
                            
                        end; % for nodecnt

                    end; % for phaseindy
                        
                end; % for statcnt
                
            end; % if ~isempty(layerpos)
                
        end; % for layerindy
        
        %%% return possibly extended pscan info
        newttsupport.pscan=pscan;
        
        %%% remove empty entries from the pscanlist
        nonempty=[];
        for indy=1:length(newttsupport.pscan)
            if ~isempty(newttsupport.pscan(indy).angles)
                nonempty=[nonempty indy];
            end; % if ~isempty(newttsupport.pscan.angles)
        end; % for indy
        newttsupport.pscan=newttsupport.pscan(nonempty);
        
    %%%%%%% END OF TTBOX INTERFACE
        
    
    %%%%%%% NO INTERFACE AVAILABLE ERROR
    otherwise
        error(['MKLSTTTRI: no interface to ' upper(lsopt.tt_package) 'available']);
end; % switch lsopt.tt_package


return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                       HELPER FUNCTIONS                                %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%% TTBOX HELPER FUNCTIONS %%%%%%%

function [existence,position]=mkfindpscan(pscan,phase,h);
% mkfindpscan........find out if a pscan for given phase and depth exists
%
% call: [existence,position]=mkfindpscan(pscan,phase,h);
%
%       pscan: (structure array)
%              PSCAN structure as returned by MKSCANP (TTBOX routine)
%       phase: (string)
%              seismic phase name string
%           h: (number) [km]
%              focal depth
%
% result: existence: (number) [flag]
%                    1 if PSCAN contains a scan for PHASE and H
%                    0 else
%
%         position: (number) [counter]
%                   index of the scan in PSCAN: pscan(position) is the
%                   requested scan
%
% Martin Knapmeyer, 23.06.2005

existence=0;
position=[];
cnt=1;
pscanlength=length(pscan);
done=0;
while done==0
    if cnt>pscanlength
       done=1;
    else
       if (strcmp(phase,pscan(cnt).phase)==1)&&(pscan(cnt).h==h)
          %%% this is the scan we're searching for
          existence=1;
          position=cnt;
          done=1;
       else
          %%% no... try next one.
          cnt=cnt+1;
       end; % if
    end; % if cnt>pscanlength
end; % while done

return;

%%%%%%% END OF TTBOX HELPER FUNCTIONS SECTION %%%%%%%


